<?php

class AutoreportModuleSchema extends CakeSchema {

    public function before($event = []) {
        $db = ConnectionManager::getDataSource($this->connection);
        $db->cacheSources = false;

        $CronjobModel = ClassRegistry::init('Cronjob');

        //Check if cronjob exists
        $result = $CronjobModel->find('first', [
            'conditions' => [
                'Cronjob.plugin' => 'AutoreportModule',
                'Cronjob.task'   => 'Autoreport',
            ],
        ]);
        if (empty($result)) {
            $data = [
                'Cronjob' => [
                    'task'     => 'Autoreport',
                    'plugin'   => 'AutoreportModule',
                    'interval' => '720',
                ],
            ];
            $CronjobModel->create();
            $CronjobModel->save($data);
        }

        $result = $CronjobModel->find('first', [
            'conditions' => [
                'Cronjob.plugin' => 'AutoreportModule',
                'Cronjob.task'   => 'AutoreportCalculateAvailability',
            ],
        ]);
        if (empty($result)) {
            $data = [
                'Cronjob' => [
                    'task'     => 'AutoreportCalculateAvailability',
                    'plugin'   => 'AutoreportModule',
                    'interval' => '60',
                ],
            ];
            $CronjobModel->create();
            $CronjobModel->save($data);
        }

        return true;
    }

    public function after($event = []) {
    }

    public $autoreports = [
        'id'                       => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'primary'],
        'name'                     => ['type' => 'string', 'null' => false, 'default' => null, 'length' => 200, 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'description'              => ['type' => 'string', 'null' => false, 'default' => null, 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'container_id'             => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'index'],
        'timeperiod_id'            => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'index'],
        'report_interval'          => ['type' => 'string', 'null' => false, 'default' => null, 'length' => 20, 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'report_send_interval'     => ['type' => 'string', 'null' => false, 'default' => null, 'length' => 20, 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'consider_downtimes'       => ['type' => 'boolean', 'null' => false, 'default' => null],
        'last_send_date'           => ['type' => 'datetime', 'null' => false, 'default' => null],
        'min_availability_percent' => ['type' => 'boolean', 'null' => false, 'default' => '1'],
        'min_availability'         => ['type' => 'float', 'null' => true, 'default' => null, 'length' => '8,3'],
        'check_hard_state'         => ['type' => 'boolean', 'null' => false, 'default' => '0'],
        'use_start_time'           => ['type' => 'integer', 'null' => false, 'default' => '0', 'length' => 4],
        'report_start_date'        => ['type' => 'date', 'null' => true, 'default' => null],
        'last_percent_value'       => ['type' => 'float', 'null' => false, 'default' => null],
        'last_absolut_value'       => ['type' => 'integer', 'null' => false, 'default' => null],
        'show_time'                => ['type' => 'integer', 'null' => false, 'default' => '1', 'length' => 4],
        'last_number_of_outages'   => ['type' => 'integer', 'null' => false, 'default' => '0'],
        'failure_statistic'        => ['type' => 'integer', 'null' => false, 'default' => '0', 'length' => 4],
        'consider_holidays'        => ['type' => 'integer', 'null' => false, 'default' => '0', 'length' => 4],
        'calendar_id'              => ['type' => 'integer', 'null' => false, 'default' => '0'],
        'max_number_of_outages'    => ['type' => 'integer', 'null' => true, 'default' => null],
        'created'                  => ['type' => 'datetime', 'null' => false, 'default' => null],
        'modified'                 => ['type' => 'datetime', 'null' => false, 'default' => null],
        'indexes'                  => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters'          => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $hosts_to_autoreports = [
        'id'                   => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'primary'],
        'autoreport_id'        => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'index'],
        'host_id'              => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'index'],
        'outage_duration'      => ['type' => 'integer', 'null' => true, 'default' => null],
        'configuration_option' => ['type' => 'integer', 'null' => false, 'default' => '0', 'length' => 4],
        'created'              => ['type' => 'datetime', 'null' => false, 'default' => null],
        'modified'             => ['type' => 'datetime', 'null' => false, 'default' => null],
        'indexes'              => [
            'PRIMARY'            => ['column' => 'id', 'unique' => 1],
            'autoreport_id_host' => ['column' => ['autoreport_id', 'host_id'], 'unique' => 1],
            'host_id'            => ['column' => 'host_id', 'unique' => 0],
        ],
        'tableParameters'      => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $services_to_autoreports = [
        'id'                   => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'primary'],
        'autoreport_id'        => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'index'],
        'host_id'              => ['type' => 'integer', 'null' => false, 'default' => null],
        'service_id'           => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'index'],
        'outage_duration'      => ['type' => 'integer', 'null' => true, 'default' => null],
        'configuration_option' => ['type' => 'integer', 'null' => false, 'default' => '0', 'length' => 4],
        'graph_settings'       => ['type' => 'string', 'null' => false, 'length' => 10, 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'created'              => ['type' => 'datetime', 'null' => false, 'default' => null],
        'modified'             => ['type' => 'datetime', 'null' => false, 'default' => null],
        'indexes'              => [
            'PRIMARY'               => ['column' => 'id', 'unique' => 1],
            'autoreport_id_service' => ['column' => ['autoreport_id', 'service_id'], 'unique' => 1],
            'service_id'            => ['column' => 'service_id', 'unique' => 0],
        ],
        'tableParameters'      => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $users_to_autoreports = [
        'id'              => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'primary'],
        'autoreport_id'   => ['type' => 'integer', 'null' => false, 'default' => null],
        'user_id'         => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'index'],
        'indexes'         => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
            'user_id' => ['column' => 'user_id', 'unique' => 0],
        ],
        'tableParameters' => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];


    public $autoreport_settings = [
        'id'                   => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'primary'],
        'configuration_option' => ['type' => 'string', 'null' => false, 'default' => null, 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'store_path'           => ['type' => 'string', 'null' => false, 'default' => null, 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'notification_emails'  => ['type' => 'string', 'null' => false, 'default' => null, 'length' => 255, 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'created'              => ['type' => 'datetime', 'null' => false, 'default' => null],
        'modified'             => ['type' => 'datetime', 'null' => false, 'default' => null],
        'indexes'              => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters'      => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $autoreport_quarterlies = [
        'id'              => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'primary'],
        'year'            => ['type' => 'integer', 'null' => false, 'default' => null],
        'quarter'         => ['type' => 'integer', 'null' => false, 'default' => null],
        'filename'        => ['type' => 'string', 'null' => false, 'default' => null, 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'created'         => ['type' => 'datetime', 'null' => false, 'default' => null],
        'indexes'         => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters' => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $autoreports_availability_log = [
        'id'                              => ['type' => 'biginteger', 'null' => false, 'default' => null, 'key' => 'primary'],
        'autoreport_id'                   => ['type' => 'integer', 'null' => false],
        'report_interval'                 => ['type' => 'string', 'null' => false, 'length' => 20, 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'last_update'                     => ['type' => 'biginteger', 'null' => false],
        'evaluation_start'                => ['type' => 'biginteger', 'null' => false],
        'evaluation_end'                  => ['type' => 'biginteger', 'null' => false],
        'availability_total_time'         => ['type' => 'integer', 'null' => false],
        'minimal_availability_time'       => ['type' => 'integer', 'null' => false],
        'determined_availability_time'    => ['type' => 'integer', 'null' => false],
        'determined_outage_time'          => ['type' => 'integer', 'null' => false],
        'minimal_availability_percent'    => ['type' => 'float', 'null' => true, 'default' => null, 'length' => '8,3'],
        'determined_availability_percent' => ['type' => 'float', 'null' => false],
        'maximal_number_outages'          => ['type' => 'integer', 'null' => true, 'default' => null],
        'determined_number_outages'       => ['type' => 'integer', 'null' => true, 'default' => null],
        'created'                         => ['type' => 'datetime', 'null' => false, 'default' => null],
        'indexes'                         => [
            'PRIMARY'                   => ['column' => 'id', 'unique' => 1],
            'autoreport_id_update_time' => ['column' => ['autoreport_id', 'last_update'], 'unique' => 0],
        ],
        'tableParameters'                 => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];
}
