<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.

use itnovum\openITCOCKPIT\Core\Interfaces\CronjobInterface;

class AutoreportCalculateAvailabilityTask extends AppShell implements CronjobInterface {

    public $uses = [
        'AutoreportModule.Autoreport',
        'Timeperiod'
    ];
    public function execute($quiet = false) {
        $this->params['quiet'] = $quiet;
        $this->stdout->styles('green', ['text' => 'green']);

        $this->out('Calculate autoreport availability ...', true);
        $this->calculateAvailabilityForAutoreports();
        $this->out('<green>   Ok</green>');
        $this->hr();
    }

    private function calculateAvailabilityForAutoreports() {
        $allAutoreports = [];
        $allTimeperiods = $this->Timeperiod->find('all');
        $validTimeperiods = array_filter($allTimeperiods, function ($elem) {
            return !empty($elem['Timerange']);
        });
        App::import('Controller', 'AutoreportModule.Autoreports');
        $AutoreportsController = new AutoreportsController();

        $validTimeperiodIds = Hash::extract($validTimeperiods, '{n}.Timeperiod.id');
        if (!empty($validTimeperiodIds)) {
            $allAutoreports = $this->Autoreport->find('all', [
                'recursive'  => -1,
                'contain'    => [
                    'AutoreportAvailabilityLog' => [
                        'fields' => [
                            'AutoreportAvailabilityLog.last_update'
                        ],
                        'order'  => [
                            'AutoreportAvailabilityLog.id' => 'desc'
                        ],
                        'limit'  => 1
                    ]
                ],
                'conditions' => [
                    'Autoreport.min_availability_percent' => 1,
                    'Autoreport.timeperiod_id'            => $validTimeperiodIds
                ],
                'fields'     => [
                    'Autoreport.id',
                    'Autoreport.name',
                    'Autoreport.report_interval'
                ]
            ]);
        }
        foreach ($allAutoreports as $autoreport) {
            $hasToBeCalculated = true;
            if (!empty($autoreport['AutoreportAvailabilityLog'][0])) {
                $lastUpdateDate = new DateTime(date('d.m.Y H:i:s', $autoreport['AutoreportAvailabilityLog'][0]['last_update']));
                switch ($autoreport['Autoreport']['report_interval']) {
                    case 'DAY':
                        $lastUpdateDate->modify('+1 hour');
                        if ($lastUpdateDate->getTimestamp() > time()) {
                            $hasToBeCalculated = false;
                        }
                        break;
                    default:
                        $lastUpdateDate->modify('+1 day');
                        if ($lastUpdateDate->getTimestamp() > time()) {
                            $hasToBeCalculated = false;
                        }
                        break;
                }
            }
            if (!$hasToBeCalculated) {
                continue;
            }
            $this->hr();
            $calculatedRanges = $AutoreportsController->calculateAvailability($autoreport['Autoreport']['id']);

            foreach ($calculatedRanges as $dateRange) {
                $this->out('"' . $autoreport['Autoreport']['name'] . '" ' . date('d.m.Y H:i:s', $dateRange['start']) . ' - ' . date('d.m.Y H:i:s', $dateRange['end']) . '<green>' . " \xE2\x9C\x93" . '</green>');
            }
        }
    }
}
