<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement. 
// The license agreement and license key were sent with the order confirmation.
App::uses('CakeEmail', 'Network/Email');
App::uses('Folder', 'Utility');
App::uses('CakeTime', 'Utility');

use itnovum\openITCOCKPIT\Core\Interfaces\CronjobInterface;
use Symfony\Component\Filesystem\Filesystem;

class AutoreportTask extends AppShell implements CronjobInterface {

    public $uses = ['AutoreportModule.Autoreport', 'AutoreportModule.AutoreportSetting', 'Systemsetting', 'Timeperiod'];

    public function execute($quiet = false) {
        $this->params['quiet'] = $quiet;
        $this->stdout->styles('green', ['text' => 'green']);
        $this->stdout->styles('red', ['text' => 'red']);
        $this->out('Delete temporary automatic report files...', false);
        $this->clenupAutoreportCharts();
        $this->out('<green>   Ok</green>');
        $this->out('Sending automatic reports ...');
        $this->sendAutoReports();
        $this->cleanUp();
        $this->hr();

        $all_autoreport_settings = $this->AutoreportSetting->currentAutoreportSettings();
        $default_autoreport_settings = $this->AutoreportSetting->defaultAutoreportSettings();
        $quarterlySend = (boolean)($all_autoreport_settings['AutoreportSetting']['configuration_option'] &
            $default_autoreport_settings['AUTOREPORTS_GENERATE_QUARTERLY_REPORTS']['value']);
        if ($quarterlySend) {
            $this->out('Generating automatic quarterly reports ...');
            $this->generateQuarterlyReports();
            $this->hr();
        }
    }

    public function clenupAutoreportCharts() {
        $path = App::pluginPath('AutoreportModule') . WEBROOT_DIR . DS . 'tmp';
        if (is_dir($path)) {
            App::uses('Folder', 'Utility');
            $folder = new Folder($path);
            foreach ($folder->find() as $file) {
                if (filemtime($path . DS . $file) < (time() - 120)) {
                    unlink($path . DS . $file);
                }
            }
        }
    }

    public function sendAutoReports() {
        $this->_systemsettings = $this->Systemsetting->findAsArray();
        $all_reports = [];
        $allTimeperiods = $this->Timeperiod->find('all');
        $validTimeperiods = array_filter($allTimeperiods, function ($elem) {
            return !empty($elem['Timerange']);
        });
        $validTimeperiodIds = Hash::extract($validTimeperiods, '{n}.Timeperiod.id');
        if (!empty($validTimeperiodIds)) {
            $all_reports = $this->Autoreport->find('all', [
                'conditions' => [
                    'Autoreport.timeperiod_id' => $validTimeperiodIds,
                ],
            ]);
        }
        $all_autoreport_settings = $this->AutoreportSetting->currentAutoreportSettings();
        $default_autoreport_settings = $this->AutoreportSetting->defaultAutoreportSettings();
        $nothing_to_do = true;
        $pdfSend = (boolean)($all_autoreport_settings['AutoreportSetting']['configuration_option'] &
            $default_autoreport_settings['AUTOREPORTS_SEND_REPORT_AS_PDF']['value']);
        $csvSend = (boolean)($all_autoreport_settings['AutoreportSetting']['configuration_option'] &
            $default_autoreport_settings['AUTOREPORTS_SEND_REPORT_AS_CSV']['value']);

        if (empty($all_autoreport_settings['AutoreportSetting']['store_path']) && !$pdfSend && !$csvSend) {
            $this->out(__(' no reports to send'), false);
            $this->out('<green>   Ok</green>');

            return;
        }

        foreach ($all_reports as $key => $autoreport_data) {
            $subject = ($all_autoreport_settings['AutoreportSetting']['configuration_option'] &
                $default_autoreport_settings['AUTOREPORTS_TENANT_AS_SUBJECT']['value']) ? $autoreport_data['Container']['name'] : $autoreport_data['Autoreport']['name'];

            $has_to_be_send = $this->Autoreport->hasToBeSend($autoreport_data['Autoreport']['last_send_date'], $autoreport_data['Autoreport']['report_send_interval']);
            if ($has_to_be_send) {
                $nothing_to_do = false;
                $report_start = date('d.m.Y', $this->Autoreport->reportStartTime($autoreport_data['Autoreport']['report_interval']));
                $report_end = date('d.m.Y', $this->Autoreport->reportEndTime($autoreport_data['Autoreport']['report_interval']));
                if ($autoreport_data['Autoreport']['use_start_time'] && $autoreport_data['Autoreport']['report_start_date'] != '0000-00-00') {
                    $start_date = CakeTime::toUnix($autoreport_data['Autoreport']['report_start_date']);
                    if ($start_date < $this->Autoreport->reportEndTime($autoreport_data['Autoreport']['report_interval'])) {
                        $report_start = date('d.m.o', $start_date);
                    }
                }
                if (empty($start_date)) {
                    $start_date = strtotime($report_start);
                }

                $baseFilename = $autoreport_data['Autoreport']['name'];
                $baseFilenameSuffix = '';
                if ($all_autoreport_settings['AutoreportSetting']['configuration_option'] & $default_autoreport_settings['AUTOREPORTS_USE_CREATE_DATE_IN_FILENAME']['value']) {
                    $baseFilenameSuffix .= '_' . date('dmY', time());
                }
                if ($all_autoreport_settings['AutoreportSetting']['configuration_option'] & $default_autoreport_settings['AUTOREPORTS_USE_YEAR_IN_FILENAME']['value']) {
                    if (strlen($baseFilenameSuffix) === 0) {
                        $baseFilenameSuffix .= '_' . date('Y', strtotime($report_start));
                    }
                }
                $baseFilename .= $baseFilenameSuffix;

                $checkForNoneExisting = (boolean)($all_autoreport_settings['AutoreportSetting']['configuration_option'] &
                    $default_autoreport_settings['AUTOREPORTS_GENERATE_ONLY_FOR_EXISTING']['value']);

                $createdAfterText = '';
                if ($checkForNoneExisting) {
                    $this->loadModel('Service');
                    $connectedServiceIds = Set::extract($autoreport_data, 'AutoreportServiceMembership.{n}.service_id');
                    $serviceCondition = ['Service.id' => $connectedServiceIds];
                    $connectedServices = $this->Service->find('all', ['conditions' => $serviceCondition]); // Hosts are checked as well
                    $createdLater = [];
                    foreach ($connectedServices as $connectedService) {
                        if ($start_date - strtotime($connectedService['Service']['created']) > 0) continue;
                        $hostId = $connectedService['Host']['id'];
                        if (!isset($createdLater[$hostId])) {
                            $createdLater[$hostId]['name'] = $connectedService['Host']['name'];
                            $createdLater[$hostId]['Services'] = [];
                            if ($start_date - strtotime($connectedService['Host']['created']) > 0) {
                                $createdLater[$hostId]['createdAfter'] = false;
                            } else {
                                $createdLater[$hostId]['createdAfter'] = true;
                            }
                        }
                        $createdLater[$hostId]['Services'][] = $connectedService['Servicetemplate']['name'];
                    }
                    foreach ($createdLater as $hostId => $servicesLater) {
                        $createdAfterText .= PHP_EOL . ($servicesLater['createdAfter'] ? '-' : '+') . "\t" . $servicesLater['name'];
                        foreach ($servicesLater['Services'] as $serviceLater) {
                            $createdAfterText .= PHP_EOL . "\t\t" . ' - ' . "\t" . $serviceLater;
                        }
                    }

                }

                if (!empty($createdAfterText)) {
                    $this->out('');
                    $this->out("Report $baseFilename cannot be evaluated. Some Hosts/Services were created after the selected starting period $report_start:");
                    if (!empty($all_autoreport_settings['AutoreportSetting']['notification_emails'])) {
                        $this->out('Sending Error emails to ' . $all_autoreport_settings['AutoreportSetting']['notification_emails'] . ' ...');
                        $Email = new CakeEmail();
                        $Email->config('default');
                        $Email->from([$this->_systemsettings['MONITORING']['MONITORING.FROM_ADDRESS'] => $this->_systemsettings['MONITORING']['MONITORING.FROM_NAME']]);
                        $Email->to(explode(';', $all_autoreport_settings['AutoreportSetting']['notification_emails']));
                        $Email->subject(__('Error') . ' - ' . $subject);
                        if ($Email->emailFormat('html')->send(__('Report "%s" cannot be evaluated. The following Hosts/Services were created after the selected starting period "%s":', [$baseFilename, date('d.m.Y', $start_date)]) . $createdAfterText)) {
                            $this->out('Error email was successfully sent', false);
                            $this->out('<green>   Ok</green>');
                        } else {
                            $this->out('Error email could not be sent!', false);
                            $this->out('<red>   Error</red>');
                        }
                    }
                    continue;
                }

                App::uses('CakePdf', 'CakePdf.Pdf');
                App::import('Controller', 'AutoreportModule.Autoreports');
                $AutoreportsController = new AutoreportsController();

                $binary_path = '/usr/bin/wkhtmltopdf';
                if (file_exists('/usr/local/bin/wkhtmltopdf')) {
                    $binary_path = '/usr/local/bin/wkhtmltopdf';
                }

                $pdfFileName = $baseFilename . '.pdf';
                $zipFilename = $baseFilename . '.zip';


                $CakePdf = new CakePdf([
                    'engine'             => 'CakePdf.WkHtmlToPdf',
                    'margin'             => [
                        'bottom' => 15,
                        'left'   => 0,
                        'right'  => 0,
                        'top'    => 15,
                    ],
                    'encoding'           => 'UTF-8',
                    'download'           => false,
                    'binary'             => $binary_path,
                    'orientation'        => 'portrait',
                    'filename'           => $pdfFileName,
                    'no-pdf-compression' => '*',
                    'image-dpi'          => '900',
                    'background'         => true,
                    'no-background'      => false,
                ]);

                $savePath = APP . 'Plugin/AutoreportModule/webroot/tmp/';
                $CakePdf->_engineClass->binary = $binary_path;

                $AutoreportsController->showReport($autoreport_data['Autoreport']['id'], trim($report_start), trim($report_end), true, 'pdf');

                $CakePdf->viewVars($AutoreportsController->viewVars);

                $CakePdf->template('AutoreportModule.show_report', 'AutoreportModule.default');
                //get the pdf string returned
                $pdf = $CakePdf->output();
                //or write it to file directly

                if (!empty($all_autoreport_settings['AutoreportSetting']['store_path'])) {
                    $savePath = $all_autoreport_settings['AutoreportSetting']['store_path'] . DS;
                    $tmp_dir = new Folder($savePath, true);
                }
                $pdf = $CakePdf->write($savePath . $pdfFileName);

                if ($csvSend) {
                    $AutoreportsController->showCsvReport($autoreport_data['Autoreport']['id'], trim($report_start), trim($report_end), false, $savePath, $zipFilename, false);
                }
                //$report_output = $this->requestAction('autoreports/showReport/id:'.$autoreport_data['Autoreport']['id'].'/start:'.trim($report_start).'/end:'.trim($report_start).'/', 'pdf');
                $users_email = Hash::extract($autoreport_data, 'User.{n}.email');
                $Email = new CakeEmail();
                $Email->config('default');

                $attachmentArray = [];
                if ($pdfSend) {
                    $attachmentArray[$pdfFileName] = [
                        'file'     => $savePath . $pdfFileName,
                        'mimetype' => 'application/pdf',
                    ];
                }
                if ($csvSend) {
                    $attachmentArray[$zipFilename] = [
                        'file'     => $savePath . $zipFilename,
                        'mimetype' => 'application/zip',
                    ];
                }
                if ($pdfSend || $csvSend) {
                    foreach ($users_email as $key => $to_mail) {
                        $Email->from([$this->_systemsettings['MONITORING']['MONITORING.FROM_ADDRESS'] => $this->_systemsettings['MONITORING']['MONITORING.FROM_NAME']]);
                        $Email->to($to_mail);
                        $Email->subject($subject);
                        $Email->attachments($attachmentArray);

                        if ($Email->send('Attached you find the automatically generated report: "' . $autoreport_data['Autoreport']['name'] . '" !')) {
                            $this->out('Report "' . $autoreport_data['Autoreport']['name'] . '" sent to mail address "' . $to_mail . '"', false);
                            $this->out('<green>   Ok</green>');
                        } else {
                            $this->out('ERROR sending report  "' . $autoreport_data['Autoreport']['name'] . '" to mail address "' . $to_mail . '" !', false);
                            $this->out('<red>   Error</red>');
                        }

                    }
                }
                $last_send_date = date('o-m-d H:i:s', time());
                $this->out(__('Autoreport successfully updated, last send date is "' . $last_send_date . '"'), false);
                $this->Autoreport->id = $autoreport_data['Autoreport']['id'];
                if ($this->Autoreport->saveField('last_send_date', $last_send_date)) {
                    $this->out('<green>   Ok</green>');
                } else {
                    $this->out('<red>   Error</red>');
                }
            }
        }
        if ($nothing_to_do) {
            $this->out(__(' no reports to send'), false);
            $this->out('<green>   Ok</green>');
        }
    }

    private function getQuarterDates($year, $quarter, $isStart = true) {
        switch ($quarter) {
            case 1:
                return $isStart ? '01.01.' . $year : '31.03.' . $year;
            case 2:
                return $isStart ? '01.04.' . $year : '30.06.' . $year;
            case 3:
                return $isStart ? '01.07.' . $year : '30.09.' . $year;
            case 4:
                return $isStart ? '01.10.' . $year : '31.12.' . $year;
            default:
                return false;

        }
    }

    public function generateQuarterlyReports() {
        App::import('Controller', 'AutoreportModule.Autoreports');
        $AutoreportsController = new AutoreportsController();
        $all_autoreport_settings = $this->AutoreportSetting->currentAutoreportSettings();
        $default_autoreport_settings = $this->AutoreportSetting->defaultAutoreportSettings();

        if (empty($all_autoreport_settings['AutoreportSetting']['store_path'])) {
            $this->out('Autoreports could not be generated! No store path defined!', false);
            $this->out('<red>   Error</red>');
            if (!empty($all_autoreport_settings['AutoreportSetting']['notification_emails'])) {
                $this->out('Sending Error emails to ' . $all_autoreport_settings['AutoreportSetting']['notification_emails'] . ' ...');
                $Email = new CakeEmail();
                $Email->config('default');
                $Email->from([$this->_systemsettings['MONITORING']['MONITORING.FROM_ADDRESS'] => $this->_systemsettings['MONITORING']['MONITORING.FROM_NAME']]);
                $Email->to(explode(';', $all_autoreport_settings['AutoreportSetting']['notification_emails']));
                $Email->subject(__('Error') . ' - ' . __('Cannot generate quarterly autoreports.'));
                if ($Email->emailFormat('html')->send(__('Autoreports could not be generated! No store path defined!'))) {
                    $this->out('Error email was successfully sent', false);
                    $this->out('<green>   Ok</green>');
                } else {
                    $this->out('Error email could not be sent!', false);
                    $this->out('<red>   Error</red>');
                }

            }

            return false;
        }
        if (!empty($all_autoreport_settings['AutoreportSetting']['store_path'])) {
            $savePath = $all_autoreport_settings['AutoreportSetting']['store_path'] . DS;
            $tmp_dir = new Folder($savePath, true);
        }


        $all_reports = [];
        $allTimeperiods = $this->Timeperiod->find('all');
        $validTimeperiods = array_filter($allTimeperiods, function ($elem) {
            return !empty($elem['Timerange']);
        });
        $validTimeperiodIds = Hash::extract($validTimeperiods, '{n}.Timeperiod.id');
        if (!empty($validTimeperiodIds)) {
            $all_reports = $this->Autoreport->find('all', [
                'conditions' => [
                    'Autoreport.timeperiod_id' => $validTimeperiodIds,
                ],
            ]);
        }
        $wasGenerated = [];
        for ($quartLastYear = 1; $quartLastYear <= 4; $quartLastYear++) {
            $wasGenerated[intval(date('Y')) - 1][$quartLastYear]['start'] = $this->getQuarterDates(intval(date('Y')) - 1, $quartLastYear);
            $wasGenerated[intval(date('Y')) - 1][$quartLastYear]['end'] = $this->getQuarterDates(intval(date('Y')) - 1, $quartLastYear, false);
        }
        for ($quarterThisYear = 1; $quarterThisYear < intval(ceil(date('m') / 3)); $quarterThisYear++) {
            $wasGenerated[intval(date('Y'))][$quarterThisYear]['start'] = $this->getQuarterDates(intval(date('Y')), $quarterThisYear);
            $wasGenerated[intval(date('Y'))][$quarterThisYear]['end'] = $this->getQuarterDates(intval(date('Y')), $quarterThisYear, false);
        }

        $checkForNoneExisting = (boolean)($all_autoreport_settings['AutoreportSetting']['configuration_option'] &
            $default_autoreport_settings['AUTOREPORTS_GENERATE_ONLY_FOR_EXISTING']['value']);
        $this->loadModel('Service');
        $createdAfterText = '';

        $this->loadModel('AutoreportQuarterly');
        $myCondition = ['AutoreportQuarterly.year >=' => (intval(date('Y')) - 1)];
        $allQuarterlyAutoreports = $this->AutoreportQuarterly->find('all', ['conditions' => $myCondition]);
        foreach ($wasGenerated as $year => $quartArr) {

            foreach ($quartArr as $quarter => $datesArr) {


                $autoreportIDs = [];
                foreach ($all_reports as $autoreport_data) {

                    $alreadyGenerated = false;
                    foreach ($allQuarterlyAutoreports as $quarterlyAutoreport) {
                        if ($year === intval($quarterlyAutoreport['AutoreportQuarterly']['year']) &&
                            $quarter === intval($quarterlyAutoreport['AutoreportQuarterly']['quarter'])
                        ) {

                            $alreadyGenerated = true;
                            break;
                        }
                    }

                    if ($alreadyGenerated) {
                        continue;
                    }

                    $baseFilename = $autoreport_data['Autoreport']['name'];
                    $areThereExisting = false;
                    if ($checkForNoneExisting) {
                        $connectedServiceIds = Set::extract($autoreport_data, 'AutoreportServiceMembership.{n}.service_id');
                        $serviceCondition = ['Service.id' => $connectedServiceIds];
                        $connectedServices = $this->Service->find('all', ['conditions' => $serviceCondition]); // Hosts are checked as well
                        $createdLater = [];
                        foreach ($connectedServices as $connectedService) {
                            if (strtotime($datesArr['start']) - strtotime($connectedService['Service']['created']) > 0) continue;
                            $areThereExisting = true;
                            $hostId = $connectedService['Host']['id'];
                            if (!isset($createdLater[$hostId])) {
                                $createdLater[$hostId]['name'] = $connectedService['Host']['name'];
                                $createdLater[$hostId]['Services'] = [];
                                if (strtotime($datesArr['start']) - strtotime($connectedService['Host']['created']) > 0) {
                                    $createdLater[$hostId]['createdAfter'] = false;
                                } else {
                                    $createdLater[$hostId]['createdAfter'] = true;
                                }
                            }
                            $createdLater[$hostId]['Services'][] = $connectedService['Servicetemplate']['name'];
                        }
                        $subText = '';
                        foreach ($createdLater as $hostId => $servicesLater) {
                            $subText .= '<br />' . ($servicesLater['createdAfter'] ? '-' : '+') . '&nbsp;' . $servicesLater['name'];
                            foreach ($servicesLater['services'] as $serviceLater) {
                                $subText .= '<br />&nbsp;&nbsp; - &nbsp;' . $serviceLater;
                            }
                        }
                        $createdAfterText = $subText . $createdAfterText;

                    }

                    if ($areThereExisting) {
                        $createdAfterText = "<br /><br />Report $baseFilename cannot be evaluated.<br />Some Hosts/Services were created after the selected starting period {$datesArr['start']}:<br />" . $createdAfterText;
                        $this->out('');
                        $this->out("Report $baseFilename cannot be evaluated. Some Hosts/Services were created after the selected starting period {$datesArr['start']}:");
                        continue;
                    }

                    $autoreportIDs[] = $autoreport_data['Autoreport']['id'];

                }


                $filename = 'Report_Q_' . $quarter . '_' . $year . '.csv';
                if (empty($autoreportIDs)) {
                    $this->out($filename . ' was not generated.');
                } else {
                    $this->out('Generating ' . $filename, false);
                    $fullpath = $savePath . $filename;
                    $AutoreportsController->generateCsvReport($autoreportIDs, $datesArr['start'], $datesArr['end'], $fullpath);
                    $this->out('<green>   Ok</green>');

                    $this->AutoreportQuarterly->create();
                    $this->AutoreportQuarterly->save(['year' => $year, 'quarter' => $quarter, 'filename' => $filename, 'created' => date('Y-m-d H:i:s')]);
                }

            }

        }

        if (!empty($createdAfterText) && !empty($all_autoreport_settings['AutoreportSetting']['notification_emails'])) {
            $this->out('');
            $this->out('Sending Error emails to ' . $all_autoreport_settings['AutoreportSetting']['notification_emails'] . ' ...');
            $Email = new CakeEmail();
            $Email->config('default');
            $Email->from([$this->_systemsettings['MONITORING']['MONITORING.FROM_ADDRESS'] => $this->_systemsettings['MONITORING']['MONITORING.FROM_NAME']]);
            $Email->to(explode(';', $all_autoreport_settings['AutoreportSetting']['notification_emails']));
            $Email->subject(__('Error') . ' - ' . __('Some reports could not be generated.'));
            if ($Email->emailFormat('html')->send(__('Some reports cannot be evaluated.') . $createdAfterText)) {
                $this->out('Error email was successfully sent', false);
                $this->out('<green>   Ok</green>');
            } else {
                $this->out('Error email could not be sent!', false);
                $this->out('<red>   Error</red>');
            }

        }
    }

    public function cleanUp() {
        $savePath = APP . 'Plugin/AutoreportModule/webroot/tmp/';
        if (!is_dir($savePath)) {
            return false;
        }

        $files = [];
        foreach (new DirectoryIterator($savePath) as $fileInfo) {
            if (!$fileInfo->isDot()) {
                $files[] = $fileInfo->getRealPath();
            }
        }
        if (empty($files)) {
            return true;
        }

        $Filesystem = new Filesystem();
        $Filesystem->remove($files);
    }
}
