<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
use itnovum\openITCOCKPIT\Core\DbBackend;
use itnovum\openITCOCKPIT\Core\DowntimeHostConditions;
use itnovum\openITCOCKPIT\Core\DowntimeServiceConditions;
use itnovum\openITCOCKPIT\Core\HoststatusFields;
use itnovum\openITCOCKPIT\Core\NodeJS\ChartRenderClient;
use itnovum\openITCOCKPIT\Core\PerfdataBackend;
use itnovum\openITCOCKPIT\Core\ServicestatusFields;
use itnovum\openITCOCKPIT\Core\StatehistoryHostConditions;
use itnovum\openITCOCKPIT\Core\StatehistoryServiceConditions;
use itnovum\openITCOCKPIT\Core\Views\UserTime;
use itnovum\openITCOCKPIT\Perfdata\PerfdataLoader;

/**
 * Class AutoreportsController
 * @property Servicestatus $Servicestatus
 * @property DbBackend $DbBackend
 * @property PerfdataBackend $PerfdataBackend
 * @property Rrd $Rrd
 */
class AutoreportsController extends AutoreportModuleAppController {
    private $autoreportVar = [];
    private $autoreportDataVar = [];
    private $autoreportSettingsVar = [];
    private $autoreportDefaultSettingsVar = [];

    public $layout = 'Admin.default';
    public $components = [
        'ListFilter.ListFilter',
        'RequestHandler',
        'AdditionalLinks',
        'CustomValidationErrors',
    ];
    public $helpers = [
        'ListFilter.ListFilter',
        'CustomValidationErrors',
        'AutoreportModule.Chart',
    ];


    public $uses = [
        'AutoreportModule.Autoreport',
        'AutoreportModule.AutoreportSetting',
        'AutoreportModule.AutoreportHostMembership',
        'AutoreportModule.AutoreportServiceMembership',
        'AutoreportModule.AutoreportAvailabilityLog',
        'Timeperiod',
        'Container',
        'User',
        'Calendar',
        'Tenant',
        'Host',
        'Service',
        'Systemfailure',
        MONITORING_STATEHISTORY_HOST,
        MONITORING_STATEHISTORY_SERVICE,
        MONITORING_DOWNTIME_HOST,
        MONITORING_DOWNTIME_SERVICE,
        MONITORING_HOSTSTATUS,
        MONITORING_SERVICESTATUS,
        'Rrd'
    ];


    public $listFilters = [
        'index' => [
            'fields' => [
                'Autoreport.name'                  => ['label' => 'Name', 'searchType' => 'wildcard'],
                'Autoreport.description'           => ['label' => 'Description', 'searchType' => 'wildcard'],
                'Autoreport.report_interval'       => ['label' => 'Report interval', 'searchType' => 'wildcard'],
                'Autoreport.report_send_interval'  => ['label' => 'Report send interval', 'searchType' => 'wildcard'],
                'Autoreport.min_availability'      => ['label' => 'Min. availability', 'searchType' => 'wildcard'],
                'Autoreport.max_number_of_outages' => ['label' => 'Max number of outages', 'searchType' => 'wildcard'],
            ],
        ]
    ];

    public function index() {
        $options = [
            'recursive'  => -1,
            'contain'    => [
                'Container',
            ],
            'conditions' => [
                'Container.id' => $this->MY_RIGHTS,
            ],
            'order'      => [
                'Autoreport.name' => 'ASC',
            ],
        ];
        $this->Paginator->settings = Hash::merge($this->Paginator->settings, $options);
        $all_autoreports = $this->Paginator->paginate();
        $this->set(compact(['all_autoreports']));
        $this->set('_serialize', ['all_autoreports']);
    }

    public function add() {
        if ($this->request->ext !== 'json') {
            $this->redirect(['action' => 'addStepOne']);
        }
        if ($this->request->ext == 'json' && !empty($this->request->data)) {
            $this->Autoreport->create();
            if ($this->Autoreport->saveAll($this->request->data)) {
                $this->serializeId(); // REST API ID serialization
            } else {
                $this->serializeErrorMessage();
            }

            return;
        }
    }

    public function addStepOne() {
        $customFieldsToRefill = [
            'Autoreport' => [
                'use_start_time',
                'min_availability_on',
                'max_outages',
                'consider_downtimes',
                'consider_holidays',
            ],
        ];

        $this->CustomValidationErrors->checkForRefill($customFieldsToRefill);

        if ($this->hasRootPrivileges === true) {
            $containerIds = $this->Tree->resolveChildrenOfContainerIds($this->MY_RIGHTS);
            $tenants = $this->Tenant->tenantsByContainerId($containerIds, 'list', 'container_id');
        } else {
            $tenants = $this->Tenant->tenantsByContainerId($this->getWriteContainers(), 'list', 'container_id');
        }

        $timeperiods = [];
        $users = [];
        $calendars = [];


        if ($this->request->is('post') || $this->request->is('put')) {
            $this->request->data['User']['User'] = (isset($this->request->data['Autoreport']['report_user'])) ? $this->request->data['Autoreport']['report_user'] : [];
            if ($this->request->data('Autoreport.min_availability_on')) {
                $this->request->data['Autoreport']['min_availability_percent'] = ($this->request->data('Autoreport.min_availability_on') && $this->request->data('Autoreport.min_availability_in_percent')) ? 1 : 0;
                $this->request->data['Autoreport']['min_availability'] = ($this->request->data('Autoreport.min_availability_on') && $this->request->data('Autoreport.min_availability_in_percent')) ? $this->request->data['Autoreport']['min_availability_in_percent'] : $this->request->data['Autoreport']['min_availability_in_minutes'];
            } else {
                $this->request->data['Autoreport']['min_availability_percent'] = 0;
                $this->request->data['Autoreport']['min_availability'] = null;
            }
            $data_to_save = $this->request->data;
            if (!empty($this->request->data['Autoreport']['report_start_date'])) {
                $data_to_save['Autoreport']['report_start_date'] = date('Y-m-d', strtotime($data_to_save['Autoreport']['report_start_date']));
            }

            if ($this->Autoreport->saveAll($data_to_save)) {
                $this->setFlash(__('Autoreport successfully saved'));
                $this->redirect(['action' => 'addStepTwo/' . $this->Autoreport->id]);
            } else {
                $this->setFlash(__('Could not save data'), false);
                $container_ids = $this->request->data('Autoreport.container_id');
                if ($container_ids > 0) {
                    $container_ids = [ROOT_CONTAINER, $container_ids];
                }

                $timeperiods = $this->Timeperiod->timeperiodsByContainerId($container_ids, 'list');
                $calendars = $this->Calendar->calendarsByContainerId($container_ids, 'list');
                $users = $this->User->usersByContainerId($container_ids, 'list');
            }
        }

        $this->set(compact(['tenants', 'timeperiods', 'users', 'calendars']));
    }

    public function addStepTwo($id = null) {
        if (!$this->Autoreport->exists($id)) {
            throw new NotFoundException(__('Invalid autoreport'));
        }

        if ($id === null && !empty($this->request->params['pass'])) {
            $id = $this->request->params['pass'];
        }
        if (!$this->Autoreport->exists($id)) {
            throw new NotFoundException(__('Invalid autoreport'));
        }
        $autoreport = $this->Autoreport->find('first', [
            'recursiv'   => -1,
            'contain'    => [
                'Container',
                'AutoreportHostMembership',
                'AutoreportServiceMembership',
                'User'
            ],
            'conditions' => [
                'Autoreport.id' => $id
            ]
        ]);

        //redirect: browser back button
        if (!empty($autoreport['AutoreportHostMembership']) || !empty($autoreport['AutoreportServiceMembership'])) {
            $this->redirect(['action' => 'editStepTwo/' . $id]);
        }

        $userId = $this->Auth->user('id');
        $hosts = $this->Host->hostsByContainerId(array_merge([ROOT_CONTAINER, $autoreport['Container']['id']], $this->MY_RIGHTS), 'list');
        $host_and_services = [];

        if (isset($this->request->data['Autoreport']['host_id'])) {
            $host_and_services = $this->Host->find('all', [
                'recursive'  => -1,
                'contain'    => [
                    'Service' => [
                        'Servicetemplate' => [
                            'fields' => [
                                'Servicetemplate.id',
                                'Servicetemplate.name',
                                'Servicetemplate.process_performance_data'
                            ],
                        ],
                        'fields'          => [
                            'Service.id',
                            'Service.name',
                            'Service.process_performance_data',
                        ],
                    ],
                ],
                'fields'     => [
                    'Host.name',
                ],
                'conditions' => [
                    'Host.id' => $this->request->data['Autoreport']['host_id'],
                ],
            ]);
        }

        $host_and_services = $this->Autoreport->prepareDataForView($host_and_services);
        $request_data_for_dynamic_fields = Set::extract($this->request->data, 'Autoreport.Host');
        $this->set(compact(['hosts', 'host_and_services', 'request_data_for_dynamic_fields']));
        if ($this->request->is('post') || $this->request->is('put')) {
            $data_array = Hash::merge(Set::classicExtract($autoreport, '{(Autoreport$|User)}'), $this->Autoreport->prepareDataForSave($this->request->data, $id));
            $customFieldsToRefill = [
                'Autoreport' => [
                    'host_id',
                ],
            ];
            $this->CustomValidationErrors->checkForRefill($customFieldsToRefill);
            if ($this->Autoreport->saveAll(Hash::merge($this->request->data, $data_array))) {
                //set the usage_flag for the host and services which are used in the report
                $hosts = $this->request->data['Autoreport']['Host'];
                $ids = [];
                foreach ($hosts as $hostId => $host) {
                    $ids['hostIds'][] = $hostId;
                    if (!empty($host['Service'])) {
                        foreach ($host['Service'] as $serviceId => $service) {
                            $ids['serviceIds'][] = $serviceId;
                        }
                    }
                }
                $this->Autoreport->setUsageFlags($ids);

                $this->setFlash(__('Autoreport successfully saved'));
                $this->redirect(['action' => 'addStepThree/' . $this->Autoreport->id]);
            } else {
                $this->set('validate_errors', $this->Autoreport->validationErrors);
                $this->setFlash(__('Could not save data'), false);
            }
        }
    }

    public function addStepThree($id = null) {
        if (!$this->Autoreport->exists($id)) {
            throw new NotFoundException(__('Invalid autoreport'));
        }

        $autoreport = $this->Autoreport->find('first', [
            'contain'    => [
                'Container',
                'Timeperiod'                  => [
                    'Timerange',
                ],
                'Calendar',
                'AutoreportHostMembership'    => [
                    'Host' => [
                        'fields' => [
                            'name'
                        ],
                    ],
                ],
                'AutoreportServiceMembership' => [
                    'Service' => [
                        'fields'          => [
                            'name'
                        ],
                        'Servicetemplate' => [
                            'fields' => [
                                'name'
                            ],
                        ],
                        'Host'            => [
                            'fields' => [
                                'name'
                            ],
                        ],
                    ],

                ],
                'User'                        => [
                    'fields' => [
                        'id',
                        'firstname',
                        'lastname'
                    ],
                ],
            ],
            'conditions' => [
                'Autoreport.id' => $id,
            ],
        ]);
        $this->set(compact(['autoreport']));
    }

    public function edit() {
        $this->redirect(['action' => 'editStepOne']);
    }

    public function editStepOne($id = null) {
        if (!$this->Autoreport->exists($id)) {
            throw new NotFoundException(__('Invalid autoreport'));
        }
        $autoreport = $this->Autoreport->find('first', [
            'recursive'  => -1,
            'contain'    => [
                'Container',
                'User'
            ],
            'conditions' => [
                'Autoreport.id' => $id
            ]
        ]);

        if (!$this->allowedByContainerId($autoreport['Container']['id'])) {
            $this->render403();

            return;
        }

        $userId = $this->Auth->user('id');

        $customFieldsToRefill = [
            'Autoreport' => [
                'use_start_time',
                'min_availability_on',
                'max_outages',
                'consider_downtimes',
                'consider_holidays',
            ],
        ];

        $this->CustomValidationErrors->checkForRefill($customFieldsToRefill);

        if ($this->hasRootPrivileges === true) {
            $containerIds = $this->Tree->resolveChildrenOfContainerIds($this->MY_RIGHTS);
            $tenants = $this->Tenant->tenantsByContainerId($containerIds, 'list', 'container_id');
        } else {
            $tenants = $this->Tenant->tenantsByContainerId($this->getWriteContainers(), 'list', 'container_id');
        }
        $timeperiods = $this->Timeperiod->timeperiodsByContainerId([ROOT_CONTAINER, $autoreport['Container']['id']], 'list');
        $users = $this->User->usersByContainerId([ROOT_CONTAINER, $autoreport['Container']['id']], 'list');
        $calendars = $this->Calendar->calendarsByContainerId([ROOT_CONTAINER, $autoreport['Container']['id']], 'list');

        if ($this->request->is('post') || $this->request->is('put')) {
            $this->request->data['User']['User'] = (isset($this->request->data['Autoreport']['report_user'])) ? $this->request->data['Autoreport']['report_user'] : [];
            if ($this->request->data('Autoreport.min_availability_on')) {
                $this->request->data['Autoreport']['min_availability_percent'] = ($this->request->data('Autoreport.min_availability_on') && $this->request->data('Autoreport.min_availability_in_percent')) ? 1 : 0;
                $this->request->data['Autoreport']['min_availability'] = ($this->request->data('Autoreport.min_availability_on') && $this->request->data('Autoreport.min_availability_in_percent')) ? $this->request->data['Autoreport']['min_availability_in_percent'] : $this->request->data['Autoreport']['min_availability_in_minutes'];
            } else {
                $this->request->data['Autoreport']['min_availability_percent'] = 0;
                $this->request->data['Autoreport']['min_availability'] = null;
            }
            $data_to_save = $this->request->data;
            if (!empty($this->request->data['Autoreport']['report_start_date'])) {
                $data_to_save['Autoreport']['report_start_date'] = date('Y-m-d', strtotime($data_to_save['Autoreport']['report_start_date']));
            }
            $this->Autoreport->id = $id;
            if ($this->Autoreport->saveAll($data_to_save)) {
                $this->setFlash(__('Autoreport successfully saved'));
                $this->redirect(['action' => 'editStepTwo/' . $this->Autoreport->id]);
            } else {
                $this->setFlash(__('Could not save data'), false);
                $container_ids = $this->request->data('Autoreport.container_id');
                if ($container_ids > 0 && $container_ids != $autoreport['Container']['id']) {
                    $container_ids = [ROOT_CONTAINER, $container_ids];
                    $timeperiods = $this->Timeperiod->timeperiodsByContainerId($container_ids, 'list');
                    $users = $this->User->usersByContainerId($container_ids, 'list');
                    $calendars = $this->Calendar->calendarsByContainerId($container_ids, 'list');
                }
            }
        } else {
            $autoreport['Autoreport']['report_user'] = Hash::extract($autoreport, 'User.{n}.id');
        }
        $this->request->data = Hash::merge($autoreport, $this->request->data);
        $this->set(compact(['autoreport', 'tenants', 'timeperiods', 'users', 'calendars']));
    }

    public function editStepTwo($id = null) {
        if (!$this->Autoreport->exists($id)) {
            throw new NotFoundException(__('Invalid autoreport'));
        }
        $autoreport = $this->Autoreport->find('first', [
            'recursiv'   => -1,
            'contain'    => [
                'Container',
                'AutoreportHostMembership',
                'AutoreportServiceMembership',
                'User'
            ],
            'conditions' => [
                'Autoreport.id' => $id
            ]
        ]);

        if (!$this->allowedByContainerId($autoreport['Container']['id'])) {
            $this->render403();

            return;
        }
        $userId = $this->Auth->user('id');
        $host_ids = (isset($this->request->data['Autoreport']['host_id'])) ? $this->request->data['Autoreport']['host_id'] : array_unique(Hash::merge(
            Hash::extract($autoreport, 'AutoreportHostMembership.{n}.host_id'),
            Hash::extract($autoreport, 'AutoreportServiceMembership.{n}.host_id')
        ));
        $hosts = $this->Host->hostsByContainerId(
            array_merge([ROOT_CONTAINER, $autoreport['Container']['id']], $this->MY_RIGHTS), 'list', [
                'Host.disabled' => [0, 1]
            ]
        );
        $host_and_services = [];
        if (!empty($host_ids)) {
            $host_and_services = $this->Host->find('all', [
                'recursive'  => -1,
                'contain'    => [
                    'Service' => [
                        'Servicetemplate' => [
                            'fields' => [
                                'Servicetemplate.id',
                                'Servicetemplate.name',
                                'Servicetemplate.process_performance_data'
                            ],
                        ],
                        'fields'          => [
                            'Service.id',
                            'Service.name',
                            'Service.process_performance_data'
                        ],
                    ],
                ],
                'fields'     => [
                    'Host.name',
                ],
                'conditions' => [
                    'Host.id' => $host_ids,
                ],
            ]);
        }
        $host_and_services = $this->Autoreport->prepareDataForView(
            $host_and_services,
            Set::classicExtract(
                $autoreport,
                '{(Autoreport.*Membership)}.{n}.{(host_id|service_id|configuration_option|outage_duration|graph_settings)}'
            )
        );
        $request_data_for_dynamic_fields = Set::extract($this->request->data, 'Autoreport.Host');
        $this->set(compact(['autoreport', 'hosts', 'host_and_services', 'request_data_for_dynamic_fields']));
        if ($this->request->is('post') || $this->request->is('put')) {
            $data_array = Hash::merge(
                Set::classicExtract($autoreport, '{(Autoreport$|User)}'),
                $this->Autoreport->prepareDataForSave($this->request->data, $id, $autoreport)
            );
            $customFieldsToRefill = [
                'Autoreport' => [
                    'host_id',
                ],
            ];
            $host_memberships_to_delete = array_diff(
                Hash::extract($autoreport, 'AutoreportHostMembership.{n}.id'),
                Hash::extract($data_array, 'AutoreportHostMembership.{n}.id')
            );


            $service_memberships_to_delete = array_diff(
                Hash::extract($autoreport, 'AutoreportServiceMembership.{n}.id'),
                Hash::extract($data_array, 'AutoreportServiceMembership.{n}.id')
            );

            $hostIdsToDelete = array_diff(
                Hash::extract($autoreport, 'AutoreportHostMembership.{n}.host_id'),
                Hash::extract($data_array, 'AutoreportHostMembership.{n}.host_id')
            );

            $serviceIdsToDelete = array_diff(
                Hash::extract($autoreport, 'AutoreportServiceMembership.{n}.service_id'),
                Hash::extract($data_array, 'AutoreportServiceMembership.{n}.service_id')
            );

            $hostIds = array_merge(
                Hash::extract($autoreport, 'AutoreportHostMembership.{n}.host_id'),
                Hash::extract($data_array, 'AutoreportHostMembership.{n}.host_id')
            );
            $serviceIds = array_merge(
                Hash::extract($autoreport, 'AutoreportServiceMembership.{n}.service_id'),
                Hash::extract($data_array, 'AutoreportServiceMembership.{n}.service_id')
            );

            $ids = [
                'hostIds'    => $hostIds,
                'serviceIds' => $serviceIds
            ];
            $idsToRemove = [
                'hostIds'    => $hostIdsToDelete,
                'serviceIds' => $serviceIdsToDelete
            ];

            $dataToSave = Hash::merge($this->request->data, $data_array);
            $this->Autoreport->set($dataToSave);
            $this->Autoreport->id = $id;

            if ($this->Autoreport->validates()) {
                /* Delete old host associations */
                foreach ($host_memberships_to_delete as $old_membership_host_id) {
                    $this->AutoreportHostMembership->delete($old_membership_host_id);
                }

                /* Delete old service associations */
                foreach ($service_memberships_to_delete as $service_membership_service_id) {
                    $this->AutoreportServiceMembership->delete($service_membership_service_id);
                }

                $this->CustomValidationErrors->checkForRefill($customFieldsToRefill);
                if ($this->Autoreport->saveAll($dataToSave)) {
                    //add/remove usage flags
                    $this->Autoreport->setUsageFlags($ids, $idsToRemove);
                    $this->setFlash(__('Autoreport successfully saved'));
                    $this->redirect(['action' => 'editStepThree/' . $this->Autoreport->id]);
                } else {
                    $this->set('validate_errors', $this->Autoreport->validationErrors);
                    $this->setFlash(__('Could not save data'), false);
                }
            } else {
                $this->set('validate_errors', $this->Autoreport->validationErrors);
                $this->setFlash(__('Could not save data'), false);
            }
        }
    }

    public function editStepThree($id = null) {
        if (!$this->Autoreport->exists($id)) {
            throw new NotFoundException(__('Invalid autoreport'));
        }

        $autoreport = $this->Autoreport->find('first', [
            'contain'    => [
                'Container',
                'Timeperiod'                  => [
                    'Timerange',
                ],
                'Calendar',
                'AutoreportHostMembership'    => [
                    'Host' => [
                        'fields' => [
                            'name'
                        ],
                    ],
                ],
                'AutoreportServiceMembership' => [
                    'Service' => [
                        'fields'          => [
                            'name'
                        ],
                        'Servicetemplate' => [
                            'fields' => [
                                'name'
                            ],
                        ],
                        'Host'            => [
                            'fields' => [
                                'name'
                            ],
                        ],
                    ],
                ],
                'User'                        => [
                    'fields' => [
                        'id',
                        'firstname',
                        'lastname'
                    ],
                ],
            ],
            'conditions' => [
                'Autoreport.id' => $id,
            ],
        ]);

        if (!$this->allowedByContainerId($autoreport['Container']['id'])) {
            $this->render403();

            return;
        }

        $this->set(compact(['autoreport']));
    }

    public function createReport($id = null) {
        if ($this->request->data('Autoreport.autoreport_id')) {
            $id = $this->request->data('Autoreport.autoreport_id');
        }
        $autoreport = $this->Autoreport->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'Autoreport.id' => $id
            ]
        ]);

        $autoreportTimeperiod = $this->Timeperiod->find('first', [
            'recursive'  => -1,
            'contain'    => [
                'Timerange'
            ],
            'conditions' => [
                'Timeperiod.id' => $autoreport['Autoreport']['timeperiod_id']
            ]
        ]);
        if (!in_array((int)$autoreport['Autoreport']['container_id'], $this->MY_RIGHTS, true)) {
            $this->render403();
            return;
        }
        $all_autoreports = $this->Autoreport->find('list', [
            'order' => [
                'Autoreport.name ASC',
            ],
        ]);

        $this->set(compact(['autoreport', 'all_autoreports']));
        $this->Autoreport->setValidationRules('createReport');
        $this->Autoreport->set($this->request->data);
        $hasErrors = false;
        if ($this->request->is('post') || $this->request->is('put')) {
            $startDate = ($autoreport['Autoreport']['use_start_time'] && strtotime($autoreport['Autoreport']['report_start_date']) < strtotime($this->request->data['Autoreport']['end_date'])) ? date('d.m.Y ', strtotime($autoreport['Autoreport']['report_start_date'] . ' 00:00:00')) : $this->request->data['Autoreport']['start_date'];
            $endDate = $this->request->data['Autoreport']['end_date'];
            $start_date = trim($startDate);
            $end_date = trim($endDate);
            $all_autoreport_settings = $this->AutoreportSetting->currentAutoreportSettings();
            $default_autoreport_settings = $this->AutoreportSetting->defaultAutoreportSettings();
            $checkForNoneExisting = (boolean)($all_autoreport_settings['AutoreportSetting']['configuration_option'] &
                $default_autoreport_settings['AUTOREPORTS_GENERATE_ONLY_FOR_EXISTING']['value']);
            $createdAfterText = '';
            if ($checkForNoneExisting) {
                $connectedServiceIds = Set::extract($autoreport, 'AutoreportServiceMembership.{n}.service_id');
                $serviceCondition = ['Service.id' => $connectedServiceIds];
                $connectedServices = $this->Service->find('all', ['conditions' => $serviceCondition]); // Hosts are checked as well
                $createdLater = [];
                foreach ($connectedServices as $connectedService) {
                    if (strtotime($start_date) - strtotime($connectedService['Service']['created']) > 0) continue;
                    $hostId = $connectedService['Host']['id'];
                    if (!isset($createdLater[$hostId])) {
                        $createdLater[$hostId]['name'] = $connectedService['Host']['name'];
                        $createdLater[$hostId]['Services'] = [];
                        if (strtotime($start_date) - strtotime($connectedService['Host']['created']) > 0) {
                            $createdLater[$hostId]['createdAfter'] = false;
                        } else {
                            $createdLater[$hostId]['createdAfter'] = true;
                        }
                    }
                    $createdLater[$hostId]['Services'][] = $connectedService['Servicetemplate']['name'];
                }

                foreach ($createdLater as $hostId => $servicesLater) {
                    $createdAfterText .= '<br />' . ($servicesLater['createdAfter'] ? '-' : '+') . '&nbsp;' . $servicesLater['name'];
                    foreach ($servicesLater['Services'] as $serviceLater) {
                        $createdAfterText .= '<br />&nbsp;&nbsp; - &nbsp;' . $serviceLater;
                    }
                }

            }

            if (!empty($createdAfterText)) {
                $this->setFlash(__('Report "%s" cannot be evaluated. The following Hosts/Services were created after the selected starting period "%s":' . $createdAfterText, [$autoreport['Autoreport']['name'], $start_date]), false);
            } else if (empty($autoreportTimeperiod['Timerange'])) {
                $this->setFlash(__('Report "%s" cannot be evaluated. Time evaluation report data is not available for the selected period ("%s")', [$autoreport['Autoreport']['name'], $autoreport['Timeperiod']['name']]), false);
            } else {
                if ($this->Autoreport->validates()) {
                    switch ($this->request->data('Autoreport.report_format')) {
                        case 'pdf':
                            $this->redirect([
                                'id'     => $id,
                                'action' => 'showReport',
                                'start'  => $start_date,
                                'end'    => $end_date,
                                'ext'    => 'pdf',
                            ]);
                            break;

                        case 'csv':
                            $this->redirect([
                                'id'     => $id,
                                'action' => 'showCsvReport',
                                'start'  => $start_date,
                                'end'    => $end_date,
                                'ext'    => 'csv',
                            ]);
                            break;

                        default:
                            $this->redirect([
                                'id'     => $id,
                                'action' => 'showReport',
                                'start'  => $start_date,
                                'end'    => $end_date,
                                'html'   => 1 //cake 2.6 requeres a / at the end of request?
                            ]);
                    }

                }
            }
        }
    }

    public function showCsvReport($id = null, $start = null, $end = null, $mail = false, $path = null, $filename = null, $render = true) {
        if (!isset($this->DbBackend)) {
            $this->DbBackend = new DbBackend();
            $this->PerfdataBackend = new PerfdataBackend();
            $this->set('DbBackend', $this->DbBackend);
            $this->set('PerfdataBackend', $this->PerfdataBackend);
        }

        if (is_null($id) && !isset($this->request->params['named']['id'])) {
            throw new NotFoundException(__('Invalid report id'));
        } else if (isset($this->request->params['named']['id'])) {
            $id = $this->request->params['named']['id'];
        }

        if (is_null($start) && isset($this->request->params['named']['start'])) {
            $start = $this->request->params['named']['start'];
        }

        if (is_null($end) && isset($this->request->params['named']['end'])) {
            $end = $this->request->params['named']['end'];
        }

        if (!$this->setReportData($id, $start, $end, $mail)) {

            $this->setFlash($this->autoreportError, false);
            $this->set(compact(['hasErrors']));

        } else {
            $showOutagesInDowntimes = (
                $this->autoreportSettingsVar['AutoreportSetting']['configuration_option'] &
                $this->autoreportDefaultSettingsVar['AUTOREPORTS_SHOW_OUTAGES_IN_DOWNTIME']['value']
            );

            $this->autoRender = false;

            $path = is_null($path) ? (APP . 'Plugin/AutoreportModule/webroot/tmp/') : $path;
            if (!is_dir($path)) {
                mkdir($path, 0755, true);
            }
            $filename = is_null($filename) ? ($this->autoreportVar['Autoreport']['name'] . '.zip') : $filename;
            $reportId = $this->autoreportVar['Autoreport']['id'];

            $zipName = $path . $filename;
            $baseZipName = str_replace('.zip', '', $filename);
            $CSVexport1 = $path . $baseZipName . '_Parameter_' . date('Ymd_His') . '.csv';
            $CSVexport2 = $path . $baseZipName . '_Details_' . date('Ymd_His') . '.csv';

            $weekdaysShort = [
                1 => __('Mo'), // must be __('Monday_short')
                2 => __('Tu'),
                3 => __('We'),
                4 => __('Th'),
                5 => __('Fr'),
                6 => __('Sa'),
                7 => __('Su'),
            ];
            if (isset($this->autoreportVar['Timeperiod']['Timerange']) &&
                count($this->autoreportVar['Timeperiod']['Timerange']) > 0
            ) {
                $timeRangeArr = [];
                foreach ($this->autoreportVar['Timeperiod']['Timerange'] as $timeRangeData) {
                    $timeRangeArr[] = $weekdaysShort[$timeRangeData['day']] . ' ' .
                        $timeRangeData['start'] . ' - ' . $timeRangeData['end'];
                }
                $timeRangeText = implode(' ', $timeRangeArr);
            } else {
                $timeRangeText = '';
            }

            $totalTimeInSec = $this->autoreportVar['total_time'];
            $totalOutageInSec = $this->autoreportVar['total_outage_time'];
            if ($this->autoreportVar['Autoreport']['min_availability_percent']) {
                $minAvailabilityInPercent = floatval($this->autoreportVar['Autoreport']['min_availability']);
                $minAvailabilityInMin = round($totalTimeInSec * $minAvailabilityInPercent / 6000);
            } else {
                $minAvailabilityInMin = $this->autoreportVar['Autoreport']['min_availability'];
                $minAvailabilityInPercent = round($minAvailabilityInMin * 6000 / $totalTimeInSec, 2);
            }

            $availabilityInPercent = strval(round(100 * ($totalTimeInSec - $totalOutageInSec) / $totalTimeInSec, 2));


            $this->Csv = $this->Components->load('Csv');
            $this->Csv->setPath($CSVexport1);
            $this->Csv->addRow([
                __('Report id'),
                __('Report name'),
                __('Time periods'),
                __('Reflection state'),
                __('Downtimes'),
                __('Minimal allowed availability (min)'),
                __('From'),
                __('To'),
                __('Minimal allowed availability (%)'),
                __('Current availability'),
            ]);
            $this->Csv->addRow([
                $reportId,
                $baseZipName,
                $timeRangeText,
                intval($this->autoreportVar['Autoreport']['check_hard_state']),
                intval($this->autoreportVar['Autoreport']['consider_downtimes']),
                $minAvailabilityInMin,
                $start,
                $end,
                sprintf("%01.2f", $minAvailabilityInPercent),
                sprintf("%01.2f", $availabilityInPercent)
            ]);
            $this->Csv->saveFile();


            $this->Csv->setPath($CSVexport2);
            $this->Csv->addRow([
                __('Report id'),
                __('Report name'),
                __('Host name'),
                __('Service name'),
                __('Outage date'),
                __('Start time'),
                __('End time'),
                __('Duration (Minutes)'),
                __('Planned downtime'),
            ]);
            foreach ($this->autoreportDataVar as $hostData) {
                if (isset($hostData['Host']['outages'])) { // host outages
                    foreach ($hostData['Host']['outages'] as $outageData) {
                        $this->Csv->addRow([
                            $reportId,
                            $baseZipName,
                            $hostData['Host']['name'],
                            '',
                            date('d.m.Y', $outageData['start']),
                            date('H:i', $outageData['start']),
                            date('H:i', $outageData['end']),
                            round(($outageData['end'] - $outageData['start']) / 60),
                            0,
                        ]);
                    }
                }
                if (isset($hostData['outages_summarized_in_downtime']) &&
                    $this->autoreportVar['Autoreport']['consider_downtimes'] &&
                    $showOutagesInDowntimes) { // host downtimes
                    foreach ($hostData['outages_summarized_in_downtime'] as $downtimeData) {
                        $this->Csv->addRow([
                            $reportId,
                            $baseZipName,
                            $hostData['Host']['name'],
                            '',
                            date('d.m.Y', $downtimeData['start']),
                            date('H:i', $downtimeData['start']),
                            date('H:i', $downtimeData['end']),
                            round(($downtimeData['end'] - $downtimeData['start']) / 60),
                            1,
                        ]);
                    }
                }
                if (isset($hostData['Services'])) { // services
                    foreach ($hostData['Services'] as $serviceData) {
                        if (isset($serviceData['outages'])) { // service outages
                            foreach ($serviceData['outages'] as $outageData) {
                                $this->Csv->addRow([
                                    $reportId,
                                    $baseZipName,
                                    $hostData['Host']['name'],
                                    $serviceData['Service']['name'],
                                    date('d.m.Y', $outageData['start']),
                                    date('H:i', $outageData['start']),
                                    date('H:i', $outageData['end']),
                                    round(($outageData['end'] - $outageData['start']) / 60),
                                    0,
                                ]);
                            }
                        }
                        if (isset($serviceData['outages_summarized_in_downtime']) &&
                            $this->autoreportVar['Autoreport']['consider_downtimes'] &&
                            $showOutagesInDowntimes) { // service downtimes
                            foreach ($serviceData['outages_summarized_in_downtime'] as $downtimeData) {
                                $this->Csv->addRow([
                                    $reportId,
                                    $baseZipName,
                                    $hostData['Host']['name'],
                                    $serviceData['Service']['name'],
                                    date('d.m.Y', $downtimeData['start']),
                                    date('H:i', $downtimeData['start']),
                                    date('H:i', $downtimeData['end']),
                                    round(($downtimeData['end'] - $downtimeData['start']) / 60),
                                    1,
                                ]);
                            }
                        }
                    }
                }
            }
            $this->Csv->saveFile();

            $this->Csv->readZip($zipName, [$CSVexport1, $CSVexport2], $render);
        }
    }

    public function generateCsvReport($autoreportIDs, $start_date, $end_date, $filename) {
        if (!isset($this->DbBackend)) {
            $this->DbBackend = new DbBackend();
            $this->PerfdataBackend = new PerfdataBackend();
            $this->set('DbBackend', $this->DbBackend);
            $this->set('PerfdataBackend', $this->PerfdataBackend);
        }

        $weekdaysShort = [
            1 => __('Mo'), // must be __('Monday_short')
            2 => __('Tu'),
            3 => __('We'),
            4 => __('Th'),
            5 => __('Fr'),
            6 => __('Sa'),
            7 => __('Su'),
        ];

        $this->Csv = $this->Components->load('Csv');
        $this->Csv->setPath($filename);
        $this->Csv->addRow([
            __('Report id'),
            __('Report name'),
            __('Time periods'),
            __('Reflection state'),
            __('Downtimes'),
            __('Minimal allowed availability (min)'),
            __('From'),
            __('To'),
            __('Minimal allowed availability (%)'),
            __('Current availability'),
        ]);

        foreach ($autoreportIDs as $autoreportID) {
            if (!$this->setReportData($autoreportID, $start_date, $end_date, false)) {
                $this->out($this->autoreportError, false);
            } else {
                if (isset($this->autoreportVar['Timeperiod']['Timerange']) &&
                    count($this->autoreportVar['Timeperiod']['Timerange']) > 0
                ) {
                    $timeRangeArr = [];
                    foreach ($this->autoreportVar['Timeperiod']['Timerange'] as $timeRangeData) {
                        $timeRangeArr[] = $weekdaysShort[$timeRangeData['day']] . ' ' .
                            $timeRangeData['start'] . ' - ' . $timeRangeData['end'];
                    }
                    $timeRangeText = implode(' ', $timeRangeArr);
                } else {
                    $timeRangeText = '';
                }

                $totalTimeInSec = $this->autoreportVar['total_time'];
                $totalOutageInSec = $this->autoreportVar['total_outage_time'];
                if ($this->autoreportVar['Autoreport']['min_availability_percent']) {
                    $minAvailabilityInPercent = floatval($this->autoreportVar['Autoreport']['min_availability']);
                    $minAvailabilityInMin = round($totalTimeInSec * $minAvailabilityInPercent / 6000);
                } else {
                    $minAvailabilityInMin = $this->autoreportVar['Autoreport']['min_availability'];
                    $minAvailabilityInPercent = round($minAvailabilityInMin * 6000 / $totalTimeInSec, 3);
                }

                $availabilityInPercent = strval(round(100 * ($totalTimeInSec - $totalOutageInSec) / $totalTimeInSec, 3));

                $this->Csv->addRow([
                    $autoreportID,
                    $this->autoreportVar['Autoreport']['name'],
                    $timeRangeText,
                    intval($this->autoreportVar['Autoreport']['check_hard_state']),
                    intval($this->autoreportVar['Autoreport']['consider_downtimes']),
                    $minAvailabilityInMin,
                    $start_date,
                    $end_date,
                    sprintf("%01.3f", $minAvailabilityInPercent),
                    sprintf("%01.3f", $availabilityInPercent)
                ]);
            }
        }

        $this->Csv->saveFile();

    }

    public function showReport($id = null, $start = null, $end = null, $mail = false) {
        if (!isset($this->DbBackend)) {
            $this->DbBackend = new DbBackend();
            $this->PerfdataBackend = new PerfdataBackend();
            $this->set('DbBackend', $this->DbBackend);
            $this->set('PerfdataBackend', $this->PerfdataBackend);
        }

        if (is_null($id) && !isset($this->request->params['named']['id'])) {
            throw new NotFoundException(__('Invalid report id'));
        } else if (isset($this->request->params['named']['id'])) {
            $id = $this->request->params['named']['id'];
        }

        if (is_null($start) && isset($this->request->params['named']['start'])) {
            $start = $this->request->params['named']['start'];
        }

        if (is_null($end) && isset($this->request->params['named']['end'])) {
            $end = $this->request->params['named']['end'];
        }


        $isHtml = isset($this->request->params['named']['html']) && $this->request->params['named']['html'] === '1';

        if (!$this->setReportData($id, $start, $end, $mail)) {
            $this->setFlash($this->autoreportError, false);
            $this->set(compact(['hasErrors']));
            //          $this->redirect(['action' => 'createReport/' . $id]);
        } else {
            $GraphImageBlobs = [];
            $configOptionGraph = 1 << 2;
            //Render javascript graphs through NodeJS for PDF and html files
            $ServicestatusFields = new ServicestatusFields($this->DbBackend);
            $ServicestatusFields->perfdata();
            foreach ($this->autoreportVar['AutoreportServiceMembership'] as $serviceMembership) {
                if($serviceMembership['configuration_option'] & $configOptionGraph) {
                    $PerfdataLoader = new PerfdataLoader($this->DbBackend, $this->PerfdataBackend, $this->Servicestatus, $this->Rrd);

                    switch ($serviceMembership['graph_settings']) {
                        case 'MIN':
                        case 'min':
                            $seriesAlgorithm = 'min';
                            break;

                        case 'MAX':
                        case 'max':
                            $seriesAlgorithm = 'max';
                            break;

                        default:
                            $seriesAlgorithm = 'avg';
                            break;
                    }

                    $graphData = $PerfdataLoader->getPerfdataByUuid(
                        $serviceMembership['Service']['Host']['uuid'],
                        $serviceMembership['Service']['uuid'],
                        $this->autoreportVar['evaluation_range']['start'],
                        $this->autoreportVar['evaluation_range']['end'],
                        false,
                        $seriesAlgorithm
                    );

                    if (!empty($graphData)) {
                        $graphServiceName = $serviceMembership['Service']['name'];
                        if (empty($graphServiceName)) {
                            $graphServiceName = $serviceMembership['Service']['Servicetemplate']['name'];
                        }

                        //Render graph data to png image blobs for pdf
                        $NodeJsChartRenderClient = new ChartRenderClient();
                        $NodeJsChartRenderClient->setGraphStartTimestamp($this->autoreportVar['evaluation_range']['start']);
                        $NodeJsChartRenderClient->setGraphEndTimestamp($this->autoreportVar['evaluation_range']['end']);
                        $NodeJsChartRenderClient->setTitle(
                            sprintf(
                                '%s - %s',
                                $serviceMembership['Service']['Host']['name'],
                                $graphServiceName
                            ));

                        //render two gauges per chart
                        foreach (array_chunk($graphData, 2) as $graphDataChunk) {
                            $GraphImageBlobs[$serviceMembership['Service']['Host']['uuid']][$serviceMembership['Service']['uuid']][] = base64_encode($NodeJsChartRenderClient->getAreaChartAsPngStream($graphDataChunk));
                        }
                    }
                }
            }

            $this->set([
                'autoreport'                  => $this->autoreportVar,
                'autoreport_data'             => $this->autoreportDataVar,
                'all_autoreport_settings'     => $this->autoreportSettingsVar,
                'default_autoreport_settings' => $this->autoreportDefaultSettingsVar,
                'GraphImageBlobs'             => $GraphImageBlobs
            ]);
            $this->set('_serialize', ['autoreport', 'autoreport_data']);

            if (!$isHtml) {
                $filename = $this->autoreportVar['Autoreport']['name'];
                $filenameSuffix = '';
                if ($this->autoreportSettingsVar['AutoreportSetting']['configuration_option'] & $this->autoreportDefaultSettingsVar['AUTOREPORTS_USE_CREATE_DATE_IN_FILENAME']['value']) {
                    $filenameSuffix .= '_' . date('dmY', time());
                }
                if ($this->autoreportSettingsVar['AutoreportSetting']['configuration_option'] & $this->autoreportDefaultSettingsVar['AUTOREPORTS_USE_YEAR_IN_FILENAME']['value']) {
                    if (strlen($filenameSuffix) === 0) {
                        $filenameSuffix .= '_' . date('Y', strtotime($start));
                    }
                }
                $filename .= $filenameSuffix . '.pdf';
                $binary_path = '/usr/bin/wkhtmltopdf';
                if (file_exists('/usr/local/bin/wkhtmltopdf')) {
                    $binary_path = '/usr/local/bin/wkhtmltopdf';
                }

                $this->pdfConfig = [
                    'engine'             => 'CakePdf.WkHtmlToPdf',
                    'margin'             => [
                        'bottom' => 15,
                        'left'   => 0,
                        'right'  => 0,
                        'top'    => 15,
                    ],
                    'encoding'           => 'UTF-8',
                    'download'           => true,
                    'binary'             => $binary_path,
                    'orientation'        => 'portrait',
                    'filename'           => $filename,
                    'no-pdf-compression' => '*',
                    'image-dpi'          => '900',
                    'background'         => true,
                    'no-background'      => false,
                ];
            }
        }

    }

    private function setReportData($id, $start, $end, $mail) {
        $this->autoreportVar = [];
        $this->autoreportDataVar = [];
        $this->autoreportSettingsVar = $this->AutoreportSetting->currentAutoreportSettings();
        $this->autoreportDefaultSettingsVar = $this->AutoreportSetting->defaultAutoreportSettings();

        $start_date_sql_format = date('Y-m-d H:i:s', strtotime($start . ' 00:00:00'));
        $end_date_sql_format = date('Y-m-d H:i:s', strtotime($end . ' 23:59:59'));

        $this->autoreportVar = $this->Autoreport->find('first', [
            'contain'    => [
                'Timeperiod'                  => [
                    'Timerange',
                ],
                'Calendar'                    => [
                    'CalendarHoliday' => [
                        'conditions' => [
                            'CalendarHoliday.date BETWEEN ? AND ?' => [
                                $start_date_sql_format,
                                $end_date_sql_format
                            ],
                        ],
                        'fields'     => [
                            'CalendarHoliday.date',
                        ],
                    ],
                    'fields'          => [
                        'Calendar.name',
                    ],
                ],
                'AutoreportHostMembership'    => [
                    'Host' => [
                        'fields' => [
                            'name',
                            'uuid',
                            'description'
                        ],
                    ],
                ],
                'AutoreportServiceMembership' => [
                    'Service' => [
                        'fields'          => [
                            'name',
                            'uuid'
                        ],
                        'Host'            => [
                            'fields' => [
                                'name',
                                'uuid'
                            ],
                        ],
                        'Servicetemplate' => [
                            'fields' => [
                                'name'
                            ],
                        ],
                    ],
                ],
            ],
            'conditions' => [
                'Autoreport.id' => $id,
            ],
        ]);

        if (php_sapi_name() !== 'cli') {
            if (!in_array((int)$this->autoreportVar['Autoreport']['container_id'], $this->MY_RIGHTS, true)) {
                $this->render403();
                return;
            }
        }

        $start_date = $start;
        $end_date = $end;

        if (empty($this->autoreportVar['Timeperiod']['Timerange'])) {
            $this->autoreportError = __(
                'There are no time frames defined.Time evaluation report data is not available for the selected period ("%s")',
                $this->autoreportVar['Timeperiod']['name']
            );
            return false;
        }
        if (strtotime($start . ' 00:00:00') > strtotime($end . ' 00:00:00')) {
            $this->autoreportError = __(
                '"To" %s must not be earlier than "From" %s',
                $start, $end
            );
        }
        $timeSlicesGlobal = Hash::insert(
            $this->Autoreport->createDateRanges(
                $start_date,
                $end_date,
                $this->autoreportVar['Timeperiod']['Timerange']),
            '{n}.is_downtime', false
        );

        //filter holidays if needed
        if ($this->autoreportVar['Autoreport']['consider_holidays'] && $this->autoreportVar['Autoreport']['calendar_id'] > 0 && !empty($this->autoreportVar['Calendar']['CalendarHoliday'])) {
            $holidays = Hash::extract($this->autoreportVar['Calendar']['CalendarHoliday'], '{n}.date');
            foreach ($timeSlicesGlobal as $key => $timeSliceGlobal) {
                if (in_array(date('Y-m-d', $timeSliceGlobal['start']), $holidays)) {
                    unset($timeSlicesGlobal[$key]);
                }
            }
            if (empty($timeSlicesGlobal)) {
                $this->autoreportError = __('Time evaluation report data is not available for the selected period ("%s" with calendar: "%s") ', $this->autoreportVar['Timeperiod']['name'], $this->autoreportVar['Calendar']['name']);

                return false;

            }
        }

        $globalDowntimes = [];
        if ($this->autoreportVar['Autoreport']['consider_downtimes']) {
            $this->Systemfailure->virtualFields['author_name'] = 'CONCAT (User.firstname, " ", User.lastname)';
            $this->Systemfailure->virtualFields['scheduled_start_time'] = 'Systemfailure.start_time';
            $this->Systemfailure->virtualFields['scheduled_end_time'] = 'Systemfailure.end_time';
            $this->Systemfailure->virtualFields['comment_data'] = 'Systemfailure.comment';

            $globalDowntimes = $this->Systemfailure->find('all', [
                'recursive'  => -1,
                'contain'    => [
                    'User' => [
                        'fields' => [
                            'User.firstname',
                            'User.lastname',
                        ],
                    ],
                ],
                'conditions' => [
                    'OR' => [
                        '"' . $start_date_sql_format . '"
                            BETWEEN Systemfailure.start_time
                            AND Systemfailure.end_time',
                        '"' . $end_date_sql_format . '"
                            BETWEEN Systemfailure.start_time
                            AND Systemfailure.end_time',
                        'Systemfailure.start_time BETWEEN "' . $start_date_sql_format . '"
                            AND "' . $end_date_sql_format . '"',
                    ],
                ],
                'fields'     => [
                    'Systemfailure.author_name',
                    'Systemfailure.comment_data',
                    'Systemfailure.scheduled_start_time',
                    'Systemfailure.scheduled_end_time'

                ]
            ]);
            $globalDowntimes = ['Systemfailure' => Hash::extract($globalDowntimes, '{n}.Systemfailure')];
        }
        $total_time = Hash::apply(Hash::map($timeSlicesGlobal, '{n}', ['Autoreport', 'calculateTotalTime']), '{n}', 'array_sum');
        $this->autoreportVar['total_time'] = $total_time;
        $startTimeStamp = strtotime($start_date . ' 00:00:00');
        $endTimeStamp = strtotime($end_date . ' 23:59:59');

        $this->autoreportVar['evaluation_range'] = [
            'start' => $startTimeStamp,
            'end'   => $endTimeStamp
        ];
        foreach ($this->autoreportVar['AutoreportHostMembership'] as $host) {
            $downtimes = [];
            $hostUuid = $host['Host']['uuid'];
            //Process conditions
            $Conditions = new StatehistoryHostConditions();
            $Conditions->setOrder(['StatehistoryHost.state_time' => 'asc']);
            if ($this->autoreportVar['Autoreport']['check_hard_state']) {
                $Conditions->setHardStateTypeAndUpState(true);
            }
            $this->autoreportDataVar[$host['host_id']]['Host'] = [
                'name' => $host['Host']['name']
            ];

            $Conditions->setFrom($startTimeStamp);
            $Conditions->setTo($endTimeStamp);
            $Conditions->setHostUuid($hostUuid);
            $Conditions->setUseLimit(false);

            //Query state history records for hosts
            $query = $this->StatehistoryHost->getQuery($Conditions);
            $statehistories = $this->StatehistoryHost->find('all', $query);
            $stateHistoryWithObject[$hostUuid] = [];
            $record = [];
            foreach ($statehistories as $statehistory) {
                $StatehistoryHost = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryHost($statehistory['StatehistoryHost']);
                $stateHistoryWithObject[$hostUuid]['Statehistory'][] = $StatehistoryHost->toArray();
            }
            if (empty($stateHistoryWithObject[$hostUuid]['Statehistory'])) {
                //Host has no state history record for selected time range
                //Get last available state history record for this host
                $query = $this->StatehistoryHost->getLastRecord($Conditions);
                $record = $this->StatehistoryHost->find('first', $query);
                if (!empty($record)) {
                    $record['StatehistoryHost']['state_time'] = $startTimeStamp;
                    $StatehistoryHost = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryHost($record['StatehistoryHost']);
                    $stateHistoryWithObject[$hostUuid]['Statehistory'][] = $StatehistoryHost->toArray();
                }
            }

            if (empty($stateHistoryWithObject[$hostUuid]['Statehistory']) && empty($record)) {
                $HoststatusFields = new HoststatusFields($this->DbBackend);
                $HoststatusFields->currentState()
                    ->isHardstate()
                    ->lastStateChange()
                    ->lastHardState()
                    ->lastHardStateChange();
                $hoststatus = $this->Hoststatus->byUuid($hostUuid, $HoststatusFields);
                if (!empty($hoststatus['Hoststatus'])) {
                    $Hoststatus = new \itnovum\openITCOCKPIT\Core\Hoststatus($hoststatus['Hoststatus']);
                    if ($Hoststatus->getLastStateChange() <= $startTimeStamp) {
                        $record['StatehistoryHost']['state_time'] = $startTimeStamp;
                        $record['StatehistoryHost']['state'] = $hoststatus['Hoststatus']['current_state'];
                        $record['StatehistoryHost']['last_state'] = $hoststatus['Hoststatus']['current_state'];
                        $record['StatehistoryHost']['last_hard_state'] = $hoststatus['Hoststatus']['last_hard_state'];
                        $record['StatehistoryHost']['state_type'] = $hoststatus['Hoststatus']['state_type'];
                        $StatehistoryHost = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryHost($record['StatehistoryHost']);
                        $stateHistoryWithObject[$hostUuid]['Statehistory'][] = $StatehistoryHost->toArray();
                    }
                }
            }
            if (!empty($stateHistoryWithObject[$hostUuid])) {
                $timeSlices = $timeSlicesGlobal;
                if ($this->autoreportVar['Autoreport']['consider_downtimes']) {
                    //Query downtime records for hosts
                    $DowntimeHostConditions = new DowntimeHostConditions();
                    $DowntimeHostConditions->setOrder(['DowntimeHost.scheduled_start_time' => 'asc']);
                    $DowntimeHostConditions->setFrom($startTimeStamp);
                    $DowntimeHostConditions->setTo($endTimeStamp);
                    $DowntimeHostConditions->setHostUuid($hostUuid);

                    $query = $this->DowntimeHost->getQueryForReporting($DowntimeHostConditions);
                    $downtimes = $this->DowntimeHost->find('all', $query);
                    $stateHistoryWithObject[$hostUuid]['Downtime'] = $downtimes;

                    $downtimes = Hash::sort(
                        Hash::filter(
                            array_merge(
                                $globalDowntimes['Systemfailure'],
                                Hash::extract($downtimes, '{n}.DowntimeHost'))
                        ), '{n}.scheduled_start_time', 'ASC'
                    );
                    if (!empty($downtimes)) {
                        $downtimesFiltered = $this->Autoreport->mergeTimeOverlapping(
                            array_map(
                                function ($downtimes) {
                                    return [
                                        'start_time' => strtotime($downtimes['scheduled_start_time']),
                                        'end_time'   => strtotime($downtimes['scheduled_end_time']),
                                    ];
                                },
                                $downtimes
                            )
                        );
                        $timeSlices = $this->Autoreport->setDowntimesInTimeslices(
                            $timeSlicesGlobal,
                            $downtimesFiltered
                        );
                        unset($downtimesFiltered);
                    }
                }
                $downtimesHost = [];
                if (!empty($stateHistoryWithObject[$hostUuid]['Downtime'])) {
                    $downtimesHost = Hash::extract($stateHistoryWithObject[$hostUuid]['Downtime'], '{n}.DowntimeHost');
                }
                $this->autoreportDataVar[$host['host_id']]['Host'] = array_merge(
                    $this->autoreportDataVar[$host['host_id']]['Host'],
                    $this->Autoreport->generateAutoreportData(
                        $timeSlices,
                        $stateHistoryWithObject[$hostUuid],
                        $this->autoreportVar['Autoreport']['check_hard_state'],
                        true
                    )
                );
                $this->autoreportDataVar[$host['host_id']]['Host']['downtimes'] = $downtimesHost;

            }
        }
        foreach ($this->autoreportVar['AutoreportServiceMembership'] as $service) {
            if (!isset($this->autoreportDataVar[$service['host_id']]['Host'])) {
                $this->autoreportDataVar[$service['host_id']]['Host'] = $service['Service']['Host'];
            }
            $this->autoreportDataVar[$service['host_id']]['Services'][$service['service_id']]['Service'] = [
                'name' => ($service['Service']['name']) ? $service['Service']['name'] : $service['Service']['Servicetemplate']['name'],
                'id'   => $service['service_id']
            ];
            $downtimes = [];
            $record = [];
            $serviceUuid = $service['Service']['uuid'];
            //Process conditions
            $Conditions = new StatehistoryServiceConditions();
            $Conditions->setOrder(['StatehistoryService.state_time' => 'asc']);
            if ($this->autoreportVar['Autoreport']['check_hard_state']) {
                $Conditions->setHardStateTypeAndUpState(true);
            }
            $Conditions->setFrom($startTimeStamp);
            $Conditions->setTo($endTimeStamp);
            $Conditions->setServiceUuid($serviceUuid);
            $Conditions->setUseLimit(false);

            //Query state history records for hosts
            $query = $this->StatehistoryService->getQuery($Conditions);
            $statehistories = $this->StatehistoryService->find('all', $query);
            $stateHistoryWithObject[$serviceUuid] = [];
            foreach ($statehistories as $statehistory) {
                $StatehistoryService = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryService($statehistory['StatehistoryService']);
                $stateHistoryWithObject[$serviceUuid]['Statehistory'][] = $StatehistoryService->toArray();
            }
            if (empty($stateHistoryWithObject[$serviceUuid]['Statehistory'])) {
                //Service has no state history record for selected time range
                //Get last available state history record for this service
                $query = $this->StatehistoryService->getLastRecord($Conditions);
                $record = $this->StatehistoryService->find('first', $query);
                if (!empty($record)) {
                    $record['StatehistoryService']['state_time'] = $startTimeStamp;
                    $StatehistoryService = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryService($record['StatehistoryService']);
                    $stateHistoryWithObject[$serviceUuid]['Statehistory'][] = $StatehistoryService->toArray();
                }
            }

            if (empty($stateHistoryWithObject[$serviceUuid]['Statehistory']) && empty($record)) {
                $ServicestatusFields = new ServicestatusFields($this->DbBackend);
                $ServicestatusFields->currentState()
                    ->isHardstate()
                    ->lastHardState()
                    ->lastStateChange()
                    ->lastHardStateChange();
                $servicestatus = $this->Servicestatus->byUuid($service['Service']['uuid'], $ServicestatusFields);
                if (!empty($servicestatus['Servicestatus'])) {
                    $Servicestatus = new \itnovum\openITCOCKPIT\Core\Servicestatus($servicestatus['Servicestatus']);
                    if ($Servicestatus->getLastStateChange() <= $startTimeStamp) {
                        $record['StatehistoryService']['state_time'] = $startTimeStamp;
                        $record['StatehistoryService']['state'] = $servicestatus['Servicestatus']['current_state'];
                        $record['StatehistoryService']['last_state'] = $servicestatus['Servicestatus']['current_state'];
                        $record['StatehistoryService']['last_hard_state'] = $servicestatus['Servicestatus']['last_hard_state'];
                        $record['StatehistoryService']['state_type'] = $servicestatus['Servicestatus']['state_type'];
                        $StatehistoryService = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryService($record['StatehistoryService']);
                        $stateHistoryWithObject[$serviceUuid]['Statehistory'][] = $StatehistoryService->toArray();
                    }
                }
            }


            if (!empty($stateHistoryWithObject[$serviceUuid])) {
                $timeSlices = $timeSlicesGlobal;
                if ($this->autoreportVar['Autoreport']['consider_downtimes']) {
                    $downtimes = [];
                    //Query downtime records for hosts
                    $DowntimeServiceConditions = new DowntimeServiceConditions();
                    $DowntimeServiceConditions->setOrder(['DowntimeService.scheduled_start_time' => 'asc']);
                    $DowntimeServiceConditions->setFrom($startTimeStamp);
                    $DowntimeServiceConditions->setTo($endTimeStamp);
                    $DowntimeServiceConditions->setServiceUuid($serviceUuid);

                    $query = $this->DowntimeService->getQueryForReporting($DowntimeServiceConditions);
                    $downtimes = $this->DowntimeService->find('all', $query);
                    $stateHistoryWithObject[$serviceUuid]['Downtime'] = $downtimes;
                    $downtimes = Hash::sort(
                        Hash::filter(
                            array_merge(
                                $globalDowntimes['Systemfailure'],
                                Hash::extract($downtimes, '{n}.DowntimeService'))
                        ), '{n}.scheduled_start_time', 'ASC'
                    );
                    if (!empty($downtimes)) {
                        $downtimesFiltered = $this->Autoreport->mergeTimeOverlapping(
                            array_map(
                                function ($downtimes) {
                                    return [
                                        'start_time' => strtotime($downtimes['scheduled_start_time']),
                                        'end_time'   => strtotime($downtimes['scheduled_end_time']),
                                    ];
                                },
                                $downtimes
                            )
                        );
                        $timeSlices = $this->Autoreport->setDowntimesInTimeslices(
                            $timeSlicesGlobal,
                            $downtimesFiltered
                        );
                        unset($downtimesFiltered);
                    }
                }

                $downtimesService = [];
                if (!empty($stateHistoryWithObject[$serviceUuid]['Downtime'])) {
                    $downtimesService = Hash::extract($stateHistoryWithObject[$serviceUuid]['Downtime'], '{n}.DowntimeService');
                }
                $this->autoreportDataVar[$service['host_id']]['Services'][$service['service_id']] = array_merge([
                    'Service' => $this->autoreportDataVar[$service['host_id']]['Services'][$service['service_id']]['Service']
                ],
                    $this->Autoreport->generateAutoreportData(
                        $timeSlices,
                        $stateHistoryWithObject[$serviceUuid],
                        $this->autoreportVar['Autoreport']['check_hard_state'],
                        false
                    )
                );
                $this->autoreportDataVar[$service['host_id']]['Services'][$service['service_id']]['Service']['downtimes'] = $downtimesService;
            }
        }
        $outages_filtered = $this->Autoreport->mergeOutages(
            Hash::sort(
                Hash::merge(
                    Hash::extract($this->autoreportDataVar, '{n}.outages.{n}'),
                    Hash::extract($this->autoreportDataVar, '{n}.Services.{n}.outages.{n}')
                ),
                '{n}.start', 'ASC'
            )
        );

        $total_outage_time = Hash::apply(Hash::map($outages_filtered, '{n}', ['Autoreport', 'calculateTotalTime']), '{n}', 'array_sum');

        $this->autoreportVar['total_outage_time'] = $total_outage_time;
        $this->autoreportVar['outages_filtered'] = $outages_filtered;
        $this->autoreportVar['time_slices'] = $timeSlicesGlobal;

        //update fields after mail reports
        if ($mail) {
            $outages_count = sizeof($outages_filtered);
            $this->Autoreport->id = $this->autoreportVar['Autoreport']['id'];
            $this->Autoreport->saveField('last_number_of_outages', $outages_count);
            $this->Autoreport->saveField('last_percent_value', round($total_outage_time / $total_time * 100), 3);
            $this->Autoreport->saveField('last_absolut_value', $total_outage_time);
            $this->Autoreport->saveField('failure_statistic', ($outages_count > $this->autoreportVar['Autoreport']['last_number_of_outages']) ? 1 : 0);
        }

        return ($this->autoreportTriggered = true);
    }

    public function hostAndServicesByHostId($host_id = null) {
        $this->autoRender = false;
        if ($host_id === 'null') {
            return [];
        }
        if (!$this->request->is('ajax')) {
            throw new MethodNotAllowedException();
        }
        $host_and_services = $this->Host->find('all', [
            'recursive'  => -1,
            'contain'    => [
                'Service' => [
                    'Servicetemplate' => [
                        'fields' => ['Servicetemplate.id, Servicetemplate.name, Servicetemplate.process_performance_data'],
                    ],
                    'fields'          => [
                        'Service.id, Service.name, Service.process_performance_data',
                    ],
                ],
            ],
            'fields'     => [
                'Host.name',
            ],
            'conditions' => [
                'Host.id' => $host_id,
            ],
        ]);

        $this->set('host_and_services', $this->Autoreport->prepareDataForView($host_and_services));
        $this->render('/Elements/load_host_and_services');
    }

    public function delete($id = null) {
        if (!$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }
        if (!$this->Autoreport->exists($id)) {
            throw new NotFoundException(__('Invalid report'));
        }

        $autoreport = $this->Autoreport->find('first', [
            'contain'    => [
                'AutoreportHostMembership',
                'AutoreportServiceMembership'
            ],
            'conditions' => [
                'Autoreport.id' => $id,
            ],
        ]);


        if (!$this->allowedByContainerId($autoreport['Autoreport']['container_id'])) {
            $this->render403();

            return;
        }

        $hostIds = Hash::extract($autoreport, 'AutoreportHostMembership.{n}.host_id');
        $serviceIds = Hash::extract($autoreport, 'AutoreportServiceMembership.{n}.service_id');
        $ids = [
            'hostIds'    => $hostIds,
            'serviceIds' => $serviceIds
        ];

        if ($this->Autoreport->delete($id)) {
            $this->Autoreport->setUsageFlags($ids, $ids);
            $this->setFlash(__('Report deleted'));
            $this->redirect(['action' => 'index']);
        }
        $this->setFlash(__('Could not delete report'), false);
        $this->redirect(['action' => 'index']);
    }

    public function loadElementsByContainerIdStepOne($container_ids = null) {
        $container_ids = [ROOT_CONTAINER, $container_ids];

        $timeperiods = $this->Timeperiod->makeItJavaScriptAble(
            $this->Timeperiod->timeperiodsByContainerId($container_ids, 'list')
        );

        $users = $this->User->makeItJavaScriptAble(
            $this->User->usersByContainerId($container_ids, 'list')
        );

        $calendars = $this->Calendar->makeItJavaScriptAble(
            $this->Calendar->calendarsByContainerId($container_ids, 'list')
        );

        $this->set(compact(['timeperiods', 'users', 'calendars']));
        $this->set('_serialize', ['timeperiods', 'users', 'calendars']);
    }

    public function hostUsedBy($id) {
        $hostIds = func_get_args();
        $this->Host = ClassRegistry::init('Host');
        $this->Service = ClassRegistry::init('Service');
        $this->AutoreportHostMembership = ClassRegistry::init('AutoreportModule.AutoreportHostMembership');
        if (!$this->Host->exists($id)) {
            throw new NotFoundException(__('Invalid host'));
        }
        $autoreportData = $this->Host->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'Host.id' => $hostIds,
            ],
            'fields'     => [
                'Host.id',
                'Host.name',
                'Host.address',
                'Service.id',
                'Service.servicetemplate_id',
                'IF(Service.name IS NULL, Servicetemplate.name, Service.name) AS ServiceName',
                'Autoreport.name',
                'Autoreport.id'
            ],
            'joins'      => [
                [
                    'table'      => 'services',
                    'alias'      => 'Service',
                    'type'       => 'INNER',
                    'conditions' => [
                        'Service.host_id = Host.id'
                    ]
                ],
                [
                    'table'      => 'servicetemplates',
                    'alias'      => 'Servicetemplate',
                    'type'       => 'INNER',
                    'conditions' => [
                        'Servicetemplate.id = Service.servicetemplate_id'
                    ]
                ],
                [
                    'table'      => 'services_to_autoreports',
                    'alias'      => 'ServiceMembership',
                    'type'       => 'INNER',
                    'conditions' => [
                        'ServiceMembership.service_id = Service.id'
                    ]
                ],
                [
                    'table'      => 'autoreports',
                    'alias'      => 'Autoreport',
                    'type'       => 'INNER',
                    'conditions' => [
                        'ServiceMembership.autoreport_id = Autoreport.id'
                    ]
                ],
            ],
        ]);
        $useFallbackData = false;
        if (empty($autoreportData)) {
            //fallback if there is nothing found
            $useFallbackData = true;
            $autoreportData = $this->Host->find('all', [
                'recursive'  => -1,
                'conditions' => [
                    'Host.id' => $hostIds,
                ],
                'fields'     => [
                    'Host.id',
                    'Host.name',
                    'Host.address',
                ]
            ]);
            $newAutoreportData = [];
            foreach ($autoreportData as $hostData) {
                $newAutoreportData['Host'][$hostData['Host']['id']] = $hostData['Host'];
            }
            $autoreportData = $newAutoreportData;
        }
        if (!$useFallbackData) {
            $newAutoreportData = [];
            foreach ($autoreportData as $key => $data) {
                $hostId = $data['Host']['id'];
                if (empty($newAutoreportData['Host'][$hostId])) {
                    $newAutoreportData['Host'][$hostId] = $data['Host'];
                }
                $serviceData = array_merge($data['Service'], $data[0]);
                if (!array_key_exists('Service', $newAutoreportData['Host'][$hostId])) {
                    $newAutoreportData['Host'][$hostId]['Service'] = [];
                }
                $serviceId = $data['Service']['id'];
                if (!array_key_exists($serviceId, $newAutoreportData['Host'][$hostId]['Service'])) {
                    $newAutoreportData['Host'][$hostId]['Service'][$serviceId] = $serviceData;
                    $newAutoreportData['Host'][$hostId]['Service'][$serviceId]['Autoreport'][$data['Autoreport']['id']] = $data['Autoreport'];
                } else {
                    $newAutoreportData['Host'][$hostId]['Service'][$serviceId]['Autoreport'][$data['Autoreport']['id']] = $data['Autoreport'];
                }
            }
            $autoreportData = $newAutoreportData;
        }
        foreach ($autoreportData['Host'] as $hostId => $host) {
            $hostAutoreportData = $this->AutoreportHostMembership->find('all', [
                'recursive'  => -1,
                'conditions' => [
                    'host_id' => $host['id'],
                ],
                'fields'     => [
                    'Autoreport.name',
                    'Autoreport.id'
                ],
                'joins'      => [
                    [
                        'table'      => 'autoreports',
                        'alias'      => 'Autoreport',
                        'type'       => 'LEFT',
                        'conditions' => [
                            'Autoreport.id = AutoreportHostMembership.autoreport_id'
                        ]
                    ],
                ]
            ]);
            if (!empty($hostAutoreportData)) {
                foreach ($hostAutoreportData as $hostAutoreport) {
                    $autoreportData['Host'][$host['id']]['Autoreport'][$hostAutoreport['Autoreport']['id']] = $hostAutoreport['Autoreport'];
                }
            }
        }
        $this->set(compact('autoreportData'));
    }

    public function serviceUsedBy($id) {
        $serviceIds = func_get_args();
        $this->Service = ClassRegistry::init('Service');
        $this->AutoreportServiceMembership = ClassRegistry::init('AutoreportModule.AutoreportServiceMembership');

        if (!$this->Service->exists($id)) {
            throw new NotFoundException(__('Invalid host'));
        }

        $serviceData = $this->Service->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'Service.id' => $serviceIds
            ],
            'fields'     => [
                'IF(Service.name IS NULL, Servicetemplate.name, Service.name) AS ServiceName',
                'Service.id',
                'Host.id',
                'Host.name'
            ],
            'joins'      => [
                [
                    'table'      => 'servicetemplates',
                    'alias'      => 'Servicetemplate',
                    'type'       => 'LEFT',
                    'conditions' => [
                        'Servicetemplate.id = Service.servicetemplate_id'
                    ]
                ],
                [
                    'table'      => 'hosts',
                    'alias'      => 'Host',
                    'type'       => 'INNER',
                    'conditions' => [
                        'Host.id = Service.host_id'
                    ]
                ],
            ]
        ]);

        foreach ($serviceData as $service) {
            $autoreportData[$service['Service']['id']] = $this->AutoreportServiceMembership->find('all', [
                'recursive'  => -1,
                'conditions' => [
                    'service_id' => $service['Service']['id']
                ],
                'fields'     => [
                    'Autoreport.id',
                    'Autoreport.name',
                ],
                'joins'      => [
                    [
                        'table'      => 'autoreports',
                        'alias'      => 'Autoreport',
                        'type'       => 'LEFT',
                        'conditions' => [
                            'Autoreport.id = AutoreportServiceMembership.autoreport_id'
                        ]
                    ],
                ]
            ]);
        }

        $this->set(compact('serviceData', 'autoreportData'));
    }

    public function autoreportWidget() {
        $this->layout = 'blank';

        if (!$this->isApiRequest()) {
            //Only ship HTML template
            return;
        }
        $UserTime = new UserTime($this->Auth->user('timezone'), $this->Auth->user('dateformat'));
        $config = [
            'Autoreport' => [
                'id'                    => 0,
                'in_percent'            => true,
                'set_color_dynamically' => false,
                'show_update_date'      => true
            ],
        ];

        $this->loadModel('Widget');
        if ($this->request->is('get')) {
            $widgetId = (int)$this->request->query('widgetId');
            if (!$this->Widget->exists($widgetId)) {
                throw new RuntimeException('Invalid widget id');
            }

            $widget = $this->Widget->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'Widget.id' => $widgetId
                ],
                'fields'     => [
                    'Widget.json_data'
                ]
            ]);

            if ($widget['Widget']['json_data'] !== null && $widget['Widget']['json_data'] !== '') {
                $json = @json_decode($widget['Widget']['json_data'], true);
                $config = Hash::merge($config, $json);
            }
            $autoreport = $this->Autoreport->find('first', [
                'recursive'  => -1,
                'contain'    => [
                    'AutoreportAvailabilityLog' => [
                        'fields' => [
                            'AutoreportAvailabilityLog.id',
                            'AutoreportAvailabilityLog.last_update',
                            'AutoreportAvailabilityLog.minimal_availability_percent',
                            'AutoreportAvailabilityLog.availability_total_time',
                            'AutoreportAvailabilityLog.determined_availability_time',
                            'AutoreportAvailabilityLog.determined_availability_percent'
                        ],
                        'order'  => [
                            'AutoreportAvailabilityLog.last_update' => 'desc'
                        ],
                        'limit'  => 20
                    ]
                ],
                'fields'     => [
                    'Autoreport.id',
                    'Autoreport.name',
                    'Autoreport.min_availability',
                    'Autoreport.last_send_date',
                    'Autoreport.last_percent_value',
                    'Autoreport.last_absolut_value'
                ],
                'conditions' => [
                    'Autoreport.id' => $config['Autoreport']['id']
                ]
            ]);
            if (!empty($autoreport)) {
                if (!empty($autoreport['AutoreportAvailabilityLog'])) {
                    $autoreport['AutoreportAvailabilityLog'] = Hash::sort(
                        $autoreport['AutoreportAvailabilityLog'],
                        '{n}.id',
                        'asc'
                    );
                    $availableLogCounter = sizeof($autoreport['AutoreportAvailabilityLog']);
                    foreach ($autoreport['AutoreportAvailabilityLog'] as $key => $availabilityLog) {
                        $autoreport['AutoreportAvailabilityLog'][$key]['determined_availability_time_human'] = $UserTime->secondsInHumanShort(
                            $autoreport['AutoreportAvailabilityLog'][$key]['determined_availability_time']
                        );
                        $autoreport['AutoreportAvailabilityLog'][$key]['last_update_human'] = $UserTime->format(
                            $autoreport['AutoreportAvailabilityLog'][$key]['last_update']
                        );
                        //last record
                        if ($key === ($availableLogCounter - 1)) {
                            $autoreport['Autoreport']['min_availability'] = $availabilityLog['minimal_availability_percent'];
                            $autoreport['Autoreport']['last_absolut_value'] = $availabilityLog['determined_availability_time'];
                            $autoreport['Autoreport']['last_percent_value'] = $availabilityLog['determined_availability_percent'];
                            $autoreport['Autoreport']['last_send_date'] = $availabilityLog['last_update'];
                        }
                    }
                }

                $autoreport['Autoreport']['last_absolut_value_human'] = $UserTime->secondsInHumanShort($autoreport['Autoreport']['last_absolut_value']);
                $autoreport['Autoreport']['last_send_date'] = $UserTime->format($autoreport['Autoreport']['last_send_date']);
            } else {
                $autoreport['Autoreport']['id'] = null;
            }

            $acl = [
                'autoreport' => [
                    'createreport' => isset($this->PERMISSIONS['autoreportmodule']['autoreports']['createreport']),
                ]
            ];


            $this->set('autoreport', $autoreport);
            $this->set('config', $config);
            $this->set('ACL', $acl);

            $this->set('_serialize', ['autoreport', 'config', 'ACL']);
            return;
        }

        if ($this->request->is('post')) {
            $config['Autoreport'] = $this->request->data('Autoreport');
            $widgetId = (int)$this->request->data('Widget.id');

            if (!$this->Widget->exists($widgetId)) {
                throw new RuntimeException('Invalid widget id');
            }
            $widget = $this->Widget->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'Widget.id' => $widgetId
                ],
            ]);

            $widget['Widget']['json_data'] = json_encode($config);

            if ($this->Widget->save($widget)) {
                $this->set('config', $config);
                $this->set('_serialize', ['config']);
                return;
            }

            $this->serializeErrorMessageFromModel('Widget');
            return;
        }
        throw new MethodNotAllowedException();
    }

    public function autoreportAvailibilityColors() {
        $this->layout = 'blank';
        //Only ship HTML Template
        return;
    }

    /**
     * @param $id
     * @return array
     * @throws Exception
     */
    public function calculateAvailability($id) {
        $this->autoRender = false;
        if (!$this->Autoreport->exists($id)) {
            throw new NotFoundException(__('Invalid autoreport id'));
        }
        if ($this->DbBackend === null) {
            Configure::load('dbbackend');
            $this->DbBackend = new DbBackend(Configure::read('dbbackend'));
        }
        $allTimeperiods = $this->Timeperiod->find('all');
        $validTimeperiods = array_filter($allTimeperiods, function ($elem) {
            return !empty($elem['Timerange']);
        });
        $calculatedTimeRanges = [];
        $validTimeperiodIds = Hash::extract($validTimeperiods, '{n}.Timeperiod.id');

        if (!empty($validTimeperiodIds)) {
            $report = $this->Autoreport->find('first', [
                'recursive'  => -1,
                'contain'    => [
                    'AutoreportHostMembership'    => [
                        'Host'   => [
                            'fields' => [
                                'Host.uuid'
                            ]
                        ],
                        'fields' => []
                    ],
                    'AutoreportServiceMembership' => [
                        'Service' => [
                            'fields' => [
                                'Service.uuid'
                            ]
                        ],
                        'fields'  => []
                    ]
                ],
                'conditions' => [
                    'Autoreport.id'                       => $id,
                    'Autoreport.timeperiod_id'            => $validTimeperiodIds,
                    'Autoreport.min_availability_percent' => 1 // consider reports with availability in percent
                ],
                'fields'     => [
                    'Autoreport.id',
                    'Autoreport.timeperiod_id',
                    'Autoreport.report_interval',
                    'Autoreport.consider_downtimes',
                    'Autoreport.min_availability_percent',
                    'Autoreport.min_availability',
                    'Autoreport.check_hard_state',
                    'Autoreport.use_start_time',
                    'Autoreport.report_start_date',
                    'Autoreport.consider_holidays',
                    'Autoreport.calendar_id',
                    'Autoreport.max_number_of_outages'
                ]
            ]);
            $hostCounter = sizeof(Hash::extract($report['AutoreportHostMembership'], '{n}.Host.uuid'));
            $serviceCounter = sizeof(Hash::extract($report['AutoreportServiceMembership'], '{n}.Service.uuid'));
            $currentAutoreportSettings = $this->AutoreportSetting->currentAutoreportSettings();
            $defaultAutoreportSettings = $this->AutoreportSetting->defaultAutoreportSettings();

            if (!empty($report)) {
                $firstRun = false;
                $lastReportLogEntryDate = $this->AutoreportAvailabilityLog->find('first', [
                    'conditions' => [
                        'AutoreportAvailabilityLog.autoreport_id' => $id
                    ],
                    'order'      => [
                        'AutoreportAvailabilityLog.id' => 'desc'
                    ],
                    'fields'     => [
                        'AutoreportAvailabilityLog.last_update'
                    ],
                    'limit'      => 1
                ]);
                if (!empty($lastReportLogEntryDate['AutoreportAvailabilityLog']['last_update'])) {
                    $initStartDate = $lastReportLogEntryDate['AutoreportAvailabilityLog']['last_update'];
                } else {
                    $firstRun = true;
                    $initStartDate = $this->Autoreport->reportInitStartTime($report['Autoreport']['report_interval']);
                }
                $dateRanges = $this->Autoreport->createDateSlicesByIntervalAndLastUpdateDate(
                    $initStartDate,
                    $report['Autoreport']['report_interval']
                );
                foreach ($dateRanges as $dateRange) {
                    $availabilityRecord = [];
                    $start = $dateRange['start'];
                    $end = $dateRange['end'];
                    $entryType = $dateRange['entryType'];
                    $lastReportLogEntry = [];
                    if (!$firstRun) {
                        $lastReportLogEntry = $this->AutoreportAvailabilityLog->find('first', [
                            'conditions' => [
                                'AutoreportAvailabilityLog.autoreport_id' => $id
                            ],
                            'order'      => [
                                'AutoreportAvailabilityLog.last_update' => 'desc'
                            ],
                            'limit'      => 1
                        ]);
                    }

                    $autoreportData = [];
                    $start_date_sql_format = date('Y-m-d H:i:s', $start);
                    $end_date_sql_format = date('Y-m-d H:i:s', $end);

                    $query = [
                        'contain'    => [
                            'Timeperiod'                  => [
                                'Timerange',
                            ],
                            'Calendar'                    => [
                                'CalendarHoliday' => [
                                    'conditions' => [
                                        'CalendarHoliday.date BETWEEN ? AND ?' => [
                                            $start_date_sql_format,
                                            $end_date_sql_format
                                        ],
                                    ],
                                    'fields'     => [
                                        'CalendarHoliday.date',
                                    ],
                                ],
                                'fields'          => [
                                    'Calendar.name',
                                ],
                            ],
                            'AutoreportHostMembership'    => [
                                'Host' => [
                                    'fields' => [
                                        'name',
                                        'uuid',
                                        'description',
                                        'created'
                                    ],
                                ],
                            ],
                            'AutoreportServiceMembership' => [
                                'Service' => [
                                    'fields'          => [
                                        'name',
                                        'uuid',
                                        'created'
                                    ],
                                    'Host'            => [
                                        'fields' => [
                                            'name',
                                            'uuid'
                                        ],
                                    ],
                                    'Servicetemplate' => [
                                        'fields' => [
                                            'name'
                                        ],
                                    ],
                                ],
                            ],
                        ],
                        'conditions' => [
                            'Autoreport.id' => $id,
                        ],
                    ];
                    $checkForExisting = $currentAutoreportSettings['AutoreportSetting']['configuration_option'] > 0 &&
                        (boolean)($currentAutoreportSettings['AutoreportSetting']['configuration_option']
                            & $defaultAutoreportSettings['AUTOREPORTS_GENERATE_ONLY_FOR_EXISTING']['value']);
                    if ($checkForExisting) {
                        $query['contain']['AutoreportHostMembership']['Host']['conditions'] = [
                            'Host.created < ' => $start_date_sql_format
                        ];
                        $query['contain']['AutoreportServiceMembership']['Service']['conditions'] = [
                            'Service.created <' => $start_date_sql_format
                        ];
                    }
                    $autoreport = $this->Autoreport->find('first', $query);
                    if ($checkForExisting) {
                        $hostCounterWithCreatedFilter = sizeof(Hash::extract($autoreport['AutoreportHostMembership'], '{n}.Host.uuid'));
                        $serviceCounterWithCreatedFilter = sizeof(Hash::extract($autoreport['AutoreportServiceMembership'], '{n}.Service.uuid'));
                        if($hostCounter > $hostCounterWithCreatedFilter ||
                        $serviceCounter > $serviceCounterWithCreatedFilter){
                            continue;
                        }

                    }
                    if (php_sapi_name() !== 'cli') {
                        if (!in_array((int)$autoreport['Autoreport']['container_id'], $this->MY_RIGHTS, true)) {
                            $this->render403();
                            return [];
                        }
                    }
                    if (empty($autoreport['Timeperiod']['Timerange'])) {
                        $error = __(
                            'There are no time frames defined.Time evaluation report data is not available for the selected period ("%s")',
                            $autoreport['Timeperiod']['name']
                        );
                        return [];
                    }

                    if (empty($autoreport['AutoreportHostMembership']) && empty($autoreport['AutoreportServiceMembership'])) {
                        $error = __(
                            'There are no report elements defined'
                        );
                        return [];
                    }

                    $timeSlicesGlobal = Hash::insert(
                        $this->Autoreport->createDateRanges(
                            date('d.m.Y H:i:s', $start),
                            date('d.m.Y H:i:s', $end),
                            $autoreport['Timeperiod']['Timerange']),
                        '{n}.is_downtime', false
                    );

                    $timeSlicesCounter = sizeof($timeSlicesGlobal);

                    if (!empty($timeSlicesGlobal)) {
                        if ($timeSlicesGlobal[0]['start'] < $start) {
                            $timeSlicesGlobal[0]['start'] = $start;
                        }
                        if ($timeSlicesGlobal[$timeSlicesCounter - 1]['end'] > $end) {
                            $timeSlicesGlobal[$timeSlicesCounter - 1]['end'] = $end;
                        }
                    }

                    //filter holidays if needed
                    if ($autoreport['Autoreport']['consider_holidays'] && $autoreport['Autoreport']['calendar_id'] > 0 && !empty($autoreport['Calendar']['CalendarHoliday'])) {
                        $holidays = Hash::extract($autoreport['Calendar']['CalendarHoliday'], '{n}.date');
                        foreach ($timeSlicesGlobal as $key => $timeSliceGlobal) {
                            if (in_array(date('Y-m-d', $timeSliceGlobal['start']), $holidays)) {
                                unset($timeSlicesGlobal[$key]);
                            }
                        }
                        if (empty($timeSlicesGlobal)) {
                            $error = __(
                                'Time evaluation report data is not available for the selected period ("%s" with calendar: "%s") ',
                                $autoreport['Timeperiod']['name'],
                                $autoreport['Calendar']['name']
                            );
                            return [];

                        }
                    }
                    $globalDowntimes = [];
                    if ($autoreport['Autoreport']['consider_downtimes']) {
                        $this->Systemfailure->virtualFields['author_name'] = 'CONCAT (User.firstname, " ", User.lastname)';
                        $this->Systemfailure->virtualFields['scheduled_start_time'] = 'Systemfailure.start_time';
                        $this->Systemfailure->virtualFields['scheduled_end_time'] = 'Systemfailure.end_time';
                        $this->Systemfailure->virtualFields['comment_data'] = 'Systemfailure.comment';

                        $globalDowntimes = $this->Systemfailure->find('all', [
                            'recursive'  => -1,
                            'contain'    => [
                                'User' => [
                                    'fields' => [
                                        'User.firstname',
                                        'User.lastname',
                                    ],
                                ],
                            ],
                            'conditions' => [
                                'OR' => [
                                    '"' . $start_date_sql_format . '"
                                BETWEEN Systemfailure.start_time
                                AND Systemfailure.end_time',
                                    '"' . $end_date_sql_format . '"
                                BETWEEN Systemfailure.start_time
                                AND Systemfailure.end_time',
                                    'Systemfailure.start_time BETWEEN "' . $start_date_sql_format . '"
                                AND "' . $end_date_sql_format . '"',
                                ],
                            ],
                            'fields'     => [
                                'Systemfailure.author_name',
                                'Systemfailure.comment_data',
                                'Systemfailure.scheduled_start_time',
                                'Systemfailure.scheduled_end_time'

                            ]
                        ]);
                        $globalDowntimes = ['Systemfailure' => Hash::extract($globalDowntimes, '{n}.Systemfailure')];
                    }

                    $total_time = Hash::apply(Hash::map($timeSlicesGlobal, '{n}', ['Autoreport', 'calculateTotalTime']), '{n}', 'array_sum');

                    if ($total_time > 0) {
                        $autoreport['total_time'] = $total_time;
                        $autoreport['evaluation_range'] = [
                            'start' => $start,
                            'end'   => $end
                        ];
                        foreach ($autoreport['AutoreportHostMembership'] as $host) {
                            $downtimes = [];
                            $hostUuid = $host['Host']['uuid'];
                            //Process conditions
                            $Conditions = new StatehistoryHostConditions();
                            $Conditions->setOrder(['StatehistoryHost.state_time' => 'asc']);
                            if ($autoreport['Autoreport']['check_hard_state']) {
                                $Conditions->setHardStateTypeAndUpState(true);
                            }
                            $autoreportData[$host['host_id']]['Host'] = [
                                'name' => $host['Host']['name']
                            ];

                            $Conditions->setFrom($start);
                            $Conditions->setTo($end);
                            $Conditions->setHostUuid($hostUuid);
                            $Conditions->setUseLimit(false);

                            //Query state history records for hosts
                            $query = $this->StatehistoryHost->getQuery($Conditions);
                            $statehistories = $this->StatehistoryHost->find('all', $query);
                            $stateHistoryWithObject[$hostUuid] = [];
                            $record = [];
                            foreach ($statehistories as $statehistory) {
                                $StatehistoryHost = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryHost($statehistory['StatehistoryHost']);
                                $stateHistoryWithObject[$hostUuid]['Statehistory'][] = $StatehistoryHost->toArray();
                            }
                            if (empty($stateHistoryWithObject[$hostUuid]['Statehistory'])) {
                                //Host has no state history record for selected time range
                                //Get last available state history record for this host
                                $query = $this->StatehistoryHost->getLastRecord($Conditions);
                                $record = $this->StatehistoryHost->find('first', $query);
                                if (!empty($record)) {
                                    $record['StatehistoryHost']['state_time'] = $start;
                                    $StatehistoryHost = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryHost($record['StatehistoryHost']);
                                    $stateHistoryWithObject[$hostUuid]['Statehistory'][] = $StatehistoryHost->toArray();
                                }
                            }

                            if (empty($stateHistoryWithObject[$hostUuid]['Statehistory']) && empty($record)) {
                                $HoststatusFields = new HoststatusFields($this->DbBackend);
                                $HoststatusFields->currentState()
                                    ->isHardstate()
                                    ->lastStateChange()
                                    ->lastHardState()
                                    ->lastHardStateChange();
                                $hoststatus = $this->Hoststatus->byUuid($hostUuid, $HoststatusFields);
                                if (!empty($hoststatus['Hoststatus'])) {
                                    $Hoststatus = new \itnovum\openITCOCKPIT\Core\Hoststatus($hoststatus['Hoststatus']);
                                    if ($Hoststatus->getLastStateChange() <= $start) {
                                        $record['StatehistoryHost']['state_time'] = $start;
                                        $record['StatehistoryHost']['state'] = $hoststatus['Hoststatus']['current_state'];
                                        $record['StatehistoryHost']['last_state'] = $hoststatus['Hoststatus']['current_state'];
                                        $record['StatehistoryHost']['last_hard_state'] = $hoststatus['Hoststatus']['last_hard_state'];
                                        $record['StatehistoryHost']['state_type'] = $hoststatus['Hoststatus']['state_type'];
                                        $StatehistoryHost = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryHost($record['StatehistoryHost']);
                                        $stateHistoryWithObject[$hostUuid]['Statehistory'][] = $StatehistoryHost->toArray();
                                    }
                                }
                            }
                            if (!empty($stateHistoryWithObject[$hostUuid])) {
                                $timeSlices = $timeSlicesGlobal;
                                if ($autoreport['Autoreport']['consider_downtimes']) {
                                    //Query downtime records for hosts
                                    $DowntimeHostConditions = new DowntimeHostConditions();
                                    $DowntimeHostConditions->setOrder(['DowntimeHost.scheduled_start_time' => 'asc']);
                                    $DowntimeHostConditions->setFrom($start);
                                    $DowntimeHostConditions->setTo($end);
                                    $DowntimeHostConditions->setHostUuid($hostUuid);

                                    $query = $this->DowntimeHost->getQueryForReporting($DowntimeHostConditions);
                                    $downtimes = $this->DowntimeHost->find('all', $query);
                                    $stateHistoryWithObject[$hostUuid]['Downtime'] = $downtimes;

                                    $downtimes = Hash::sort(
                                        Hash::filter(
                                            array_merge(
                                                $globalDowntimes['Systemfailure'],
                                                Hash::extract($downtimes, '{n}.DowntimeHost'))
                                        ), '{n}.scheduled_start_time', 'ASC'
                                    );
                                    if (!empty($downtimes)) {
                                        $downtimesFiltered = $this->Autoreport->mergeTimeOverlapping(
                                            array_map(
                                                function ($downtimes) {
                                                    return [
                                                        'start_time' => strtotime($downtimes['scheduled_start_time']),
                                                        'end_time'   => strtotime($downtimes['scheduled_end_time']),
                                                    ];
                                                },
                                                $downtimes
                                            )
                                        );
                                        $timeSlices = $this->Autoreport->setDowntimesInTimeslices(
                                            $timeSlicesGlobal,
                                            $downtimesFiltered
                                        );
                                        unset($downtimesFiltered);
                                    }
                                }
                                $downtimesHost = [];
                                if (!empty($stateHistoryWithObject[$hostUuid]['Downtime'])) {
                                    $downtimesHost = Hash::extract($stateHistoryWithObject[$hostUuid]['Downtime'], '{n}.DowntimeHost');
                                }
                                $autoreportData[$host['host_id']]['Host'] = array_merge(
                                    $autoreportData[$host['host_id']]['Host'],
                                    $this->Autoreport->generateAutoreportData(
                                        $timeSlices,
                                        $stateHistoryWithObject[$hostUuid],
                                        $autoreport['Autoreport']['check_hard_state'],
                                        true
                                    )
                                );
                                $autoreportData[$host['host_id']]['Host']['downtimes'] = $downtimesHost;

                            }
                        }
                        foreach ($autoreport['AutoreportServiceMembership'] as $service) {
                            if (!isset($autoreportData[$service['host_id']]['Host'])) {
                                $autoreportData[$service['host_id']]['Host'] = $service['Service']['Host'];
                            }
                            $autoreportData[$service['host_id']]['Services'][$service['service_id']]['Service'] = [
                                'name' => ($service['Service']['name']) ? $service['Service']['name'] : $service['Service']['Servicetemplate']['name'],
                                'id'   => $service['service_id']
                            ];
                            $downtimes = [];
                            $record = [];
                            $serviceUuid = $service['Service']['uuid'];
                            //Process conditions
                            $Conditions = new StatehistoryServiceConditions();
                            $Conditions->setOrder(['StatehistoryService.state_time' => 'asc']);
                            if ($autoreport['Autoreport']['check_hard_state']) {
                                $Conditions->setHardStateTypeAndUpState(true);
                            }
                            $Conditions->setFrom($start);
                            $Conditions->setTo($end);
                            $Conditions->setServiceUuid($serviceUuid);
                            $Conditions->setUseLimit(false);

                            //Query state history records for hosts
                            $query = $this->StatehistoryService->getQuery($Conditions);
                            $statehistories = $this->StatehistoryService->find('all', $query);
                            $stateHistoryWithObject[$serviceUuid] = [];
                            foreach ($statehistories as $statehistory) {
                                $StatehistoryService = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryService($statehistory['StatehistoryService']);
                                $stateHistoryWithObject[$serviceUuid]['Statehistory'][] = $StatehistoryService->toArray();
                            }
                            if (empty($stateHistoryWithObject[$serviceUuid]['Statehistory'])) {
                                //Service has no state history record for selected time range
                                //Get last available state history record for this service
                                $query = $this->StatehistoryService->getLastRecord($Conditions);
                                $record = $this->StatehistoryService->find('first', $query);
                                if (!empty($record)) {
                                    $record['StatehistoryService']['state_time'] = $start;
                                    $StatehistoryService = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryService($record['StatehistoryService']);
                                    $stateHistoryWithObject[$serviceUuid]['Statehistory'][] = $StatehistoryService->toArray();
                                }
                            }

                            if (empty($stateHistoryWithObject[$serviceUuid]['Statehistory']) && empty($record)) {
                                $ServicestatusFields = new ServicestatusFields($this->DbBackend);
                                $ServicestatusFields->currentState()
                                    ->isHardstate()
                                    ->lastHardState()
                                    ->lastStateChange()
                                    ->lastHardStateChange();
                                $servicestatus = $this->Servicestatus->byUuid($service['Service']['uuid'], $ServicestatusFields);
                                if (!empty($servicestatus['Servicestatus'])) {
                                    $Servicestatus = new \itnovum\openITCOCKPIT\Core\Servicestatus($servicestatus['Servicestatus']);
                                    if ($Servicestatus->getLastStateChange() <= $start) {
                                        $record['StatehistoryService']['state_time'] = $start;
                                        $record['StatehistoryService']['state'] = $servicestatus['Servicestatus']['current_state'];
                                        $record['StatehistoryService']['last_state'] = $servicestatus['Servicestatus']['current_state'];
                                        $record['StatehistoryService']['last_hard_state'] = $servicestatus['Servicestatus']['last_hard_state'];
                                        $record['StatehistoryService']['state_type'] = $servicestatus['Servicestatus']['state_type'];
                                        $StatehistoryService = new \itnovum\openITCOCKPIT\Core\Views\StatehistoryService($record['StatehistoryService']);
                                        $stateHistoryWithObject[$serviceUuid]['Statehistory'][] = $StatehistoryService->toArray();
                                    }
                                }
                            }


                            if (!empty($stateHistoryWithObject[$serviceUuid])) {
                                $timeSlices = $timeSlicesGlobal;
                                if ($autoreport['Autoreport']['consider_downtimes']) {
                                    $downtimes = [];
                                    //Query downtime records for hosts
                                    $DowntimeServiceConditions = new DowntimeServiceConditions();
                                    $DowntimeServiceConditions->setOrder(['DowntimeService.scheduled_start_time' => 'asc']);
                                    $DowntimeServiceConditions->setFrom($start);
                                    $DowntimeServiceConditions->setTo($end);
                                    $DowntimeServiceConditions->setServiceUuid($serviceUuid);

                                    $query = $this->DowntimeService->getQueryForReporting($DowntimeServiceConditions);
                                    $downtimes = $this->DowntimeService->find('all', $query);
                                    $stateHistoryWithObject[$serviceUuid]['Downtime'] = $downtimes;
                                    $downtimes = Hash::sort(
                                        Hash::filter(
                                            array_merge(
                                                $globalDowntimes['Systemfailure'],
                                                Hash::extract($downtimes, '{n}.DowntimeService'))
                                        ), '{n}.scheduled_start_time', 'ASC'
                                    );
                                    if (!empty($downtimes)) {
                                        $downtimesFiltered = $this->Autoreport->mergeTimeOverlapping(
                                            array_map(
                                                function ($downtimes) {
                                                    return [
                                                        'start_time' => strtotime($downtimes['scheduled_start_time']),
                                                        'end_time'   => strtotime($downtimes['scheduled_end_time']),
                                                    ];
                                                },
                                                $downtimes
                                            )
                                        );
                                        $timeSlices = $this->Autoreport->setDowntimesInTimeslices(
                                            $timeSlicesGlobal,
                                            $downtimesFiltered
                                        );
                                        unset($downtimesFiltered);
                                    }
                                }

                                $downtimesService = [];
                                if (!empty($stateHistoryWithObject[$serviceUuid]['Downtime'])) {
                                    $downtimesService = Hash::extract($stateHistoryWithObject[$serviceUuid]['Downtime'], '{n}.DowntimeService');
                                }
                                $autoreportData[$service['host_id']]['Services'][$service['service_id']] = array_merge([
                                    'Service' => $autoreportData[$service['host_id']]['Services'][$service['service_id']]['Service']
                                ],
                                    $this->Autoreport->generateAutoreportData(
                                        $timeSlices,
                                        $stateHistoryWithObject[$serviceUuid],
                                        $autoreport['Autoreport']['check_hard_state'],
                                        false
                                    )
                                );
                                $autoreportData[$service['host_id']]['Services'][$service['service_id']]['Service']['downtimes'] = $downtimesService;
                            }
                        }

                        $outages_filtered = $this->Autoreport->mergeOutages(
                            Hash::sort(
                                Hash::merge(
                                    Hash::extract($autoreportData, '{n}.outages.{n}'),
                                    Hash::extract($autoreportData, '{n}.Services.{n}.outages.{n}')
                                ),
                                '{n}.start', 'ASC'
                            )
                        );

                        $total_outage_time = Hash::apply(Hash::map($outages_filtered, '{n}', ['Autoreport', 'calculateTotalTime']), '{n}', 'array_sum');

                        $availabilityInPercent = strval(round(100 * ($total_time - $total_outage_time) / $total_time, 3));
                        $autoreport['total_outage_time'] = $total_outage_time;
                        $autoreport['outages_filtered'] = $outages_filtered;
                        $autoreport['time_slices'] = $timeSlicesGlobal;
                        $availabilityRecord = [
                            'AutoreportAvailabilityLog' => [
                                'autoreport_id'                   => $id,
                                'report_interval'                 => $report['Autoreport']['report_interval'],
                                'last_update'                     => $end,
                                'evaluation_start'                => $start,
                                'evaluation_end'                  => $end,
                                'availability_total_time'         => $total_time,
                                'minimal_availability_time'       => (int)($total_time * $report['Autoreport']['min_availability'] / 100),
                                'determined_availability_time'    => $total_time - $total_outage_time,
                                'minimal_availability_percent'    => $report['Autoreport']['min_availability'],
                                'determined_availability_percent' => $availabilityInPercent,
                                'determined_outage_time'          => $total_outage_time,
                                'maximal_number_outages'          => $report['Autoreport']['max_number_of_outages'],
                                'determined_number_outages'       => sizeof($outages_filtered)
                            ]
                        ];

                        if ($entryType === 'update' && !empty($lastReportLogEntry)) {
                            $availabilityRecord['AutoreportAvailabilityLog']['availability_total_time'] += $lastReportLogEntry['AutoreportAvailabilityLog']['availability_total_time'];
                            $availabilityRecord['AutoreportAvailabilityLog']['minimal_availability_time'] += $lastReportLogEntry['AutoreportAvailabilityLog']['minimal_availability_time'];
                            $availabilityRecord['AutoreportAvailabilityLog']['determined_availability_time'] += $lastReportLogEntry['AutoreportAvailabilityLog']['determined_availability_time'];
                            $availabilityRecord['AutoreportAvailabilityLog']['determined_outage_time'] += $lastReportLogEntry['AutoreportAvailabilityLog']['determined_outage_time'];
                            $availabilityRecord['AutoreportAvailabilityLog']['determined_number_outages'] += $lastReportLogEntry['AutoreportAvailabilityLog']['determined_number_outages'];
                        }
                    }
                    if ($total_time === 0 && !empty($lastReportLogEntry)) {
                        $availabilityRecord = $lastReportLogEntry;
                        $availabilityRecord['AutoreportAvailabilityLog']['last_update'] = $end;
                        $availabilityRecord['AutoreportAvailabilityLog']['evaluation_start'] = $start;
                        $availabilityRecord['AutoreportAvailabilityLog']['evaluation_end'] = $end;
                    }
                    if (!empty($availabilityRecord)) {
                        $this->AutoreportAvailabilityLog->create();
                        if (!$this->AutoreportAvailabilityLog->save($availabilityRecord)) {
                            throw new Exception(__('Cannot save availability log data.'));
                        }
                        $calculatedTimeRanges[] = [
                            'start' => $start,
                            'end'   => $end
                        ];
                    }
                }
            }
        }

        if (!empty($this->request->data) && $this->request->ext == 'json') {
            if ($this->isAngularJsRequest()) {
                $this->serializeId();
                return;
            }
        }
        return $calculatedTimeRanges;
    }
}
