<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement. 
// The license agreement and license key were sent with the order confirmation.


class AutoreportSetting extends AutoreportModuleAppModel {
    public $validate = [
        'store_path'          => [
            'number' => [
                'rule'       => '/^((?:\/[a-zA-Z0-9]+(?:_[a-zA-Z0-9]+)*(?:\-[a-zA-Z0-9]+)*)+)$/',
                'message'    => 'Please enter a valid path',
                'allowEmpty' => true,
            ],
        ],
        'notification_emails' => [
            'slug' => [
                'rule'       => 'validateEmailAddresses',
                'message'    => 'Please enter valid email addresses',
                'allowEmpty' => true,
            ],
        ],
    ];

    public function validateEmailAddresses($check) {
        if (isset($check['notification_emails']) && !empty($check['notification_emails'])) {
            $emails = explode(';', $check['notification_emails']);
            foreach ($emails as $email) {
                if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
                    return false;
                }
            }
        }

        return true;
    }

    public function currentAutoreportSettings() {
        $all_autoreport_settings = $this->find('first');
        if (empty($all_autoreport_settings)) {
            $all_autoreport_settings = [
                'AutoreportSetting' => [
                    'store_path'           => '',
                    'configuration_option' => 0,
                ],
            ];
        }

        return $all_autoreport_settings;
    }

    public function defaultAutoreportSettings() {
        $all_autoreport_settings = [
            'AUTOREPORTS_TENANT_AS_SUBJECT'          => [
                'value' => 1 << 0,
                'info'  => __('Use tenant name instead of report name as subject for automatic generated mails'),
            ],
            'AUTOREPORTS_GENERATE_ONLY_FOR_EXISTING' => [
                'value' => 1 << 11,
                'info'  => __('Generates reports only for Hosts and Services that were created before the start date'),
            ],
            'AUTOREPORTS_SEND_REPORT_AS_PDF'         => [
                'value' => 1 << 1,
                'info'  => __('Send reports in PDF format'),
            ],
            'AUTOREPORTS_SEND_REPORT_AS_CSV'         => [
                'value' => 1 << 9,
                'info'  => __('Send reports in CSV format'),
            ],
            'AUTOREPORTS_GENERATE_QUARTERLY_REPORTS' => [
                'value' => 1 << 10,
                'info'  => __('Send quarterly reports'),
            ],
            'AUTOREPORTS_SHOW_OUTAGES_IN_DOWNTIME'   => [
                'value' => 1 << 2,
                'info'  => __('Shows outages in scheduled downtime'),
            ],
            'AUTOREPORTS_SHOW_DOWNTIMES'             => [
                'value' => 1 << 3,
                'info'  => __('Shows all downtimes'),
            ],
            'AUTOREPORTS_USE_CREATE_DATE_IN_FILENAME'       => [
                'value' => 1 << 12,
                'info'  => __('Use create date in filename. Format: Filename "Reportname_+/- (dmY)..."'),
            ],
            'AUTOREPORTS_USE_YEAR_IN_FILENAME'       => [
                'value' => 1 << 4,
                'info'  => __('Use Year in filename. Format: Filename "Reportname_+/- (Y)..."'),
            ],
            'AUTOREPORTS_SHOW_SLA_GRAPH'             => [
                'value' => 1 << 5,
                'info'  => __('Displays SLA graph'),
            ],
            'AUTOREPORTS_SHOW_SUMMARY_STATISTICS'    => [
                'value' => 1 << 6,
                'info'  => __('Shows summary statistics on first page of report'),
            ],
            'AUTOREPORTS_EXTENDED_HOST_DESC'         => [
                'value' => 1 << 7,
                'info'  => __('Shows host name with current availability'),
            ],
            'AUTOREPORTS_EXTENDED_SERV_DESC'         => [
                'value' => 1 << 8,
                'info'  => __('Shows service description with current availability'),
            ],
            'AUTOREPORTS_ERROR_NOTIFICATION'         => [
                'value' => '',
                'info'  => __('Email addresses which will be notified in case of failure while creating the quarterly reports. Use ";" as email delimiter when sending to multiple recipients'),
            ],
            'AUTOREPORTS_STORE_PATH'                 => [
                'value' => '',
                'info'  => __('Path for Reports files'),
            ],
        ];

        return $all_autoreport_settings;
    }

    public function prepareForSave($request_data) {
        $request_data['AutoreportSetting']['configuration_option'] = 0;
        foreach (Set::classicExtract($request_data, '{(AUTOREPORTS_).*}') as $key => $data_array) {
            foreach ($data_array as $option_key => $option_value) {
                //if checkbox checked => value = '1'
                if ($option_value) {
                    $request_data['AutoreportSetting']['configuration_option'] += $option_key;
                }
            }
        }

        return $request_data;
    }
}
