<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.


/**
 * Class DataExportShell
 * uses CakePHP to get Servicetemplate and command data from itcockpit database
 *
 * usage: oitc data_export_module.data_export
 */
App::uses('Folder', 'Utility');
App::uses('File', 'Utility');

class DataExportShell extends AppShell {

    public $uses = [
        'Servicetemplate',
        'Servicetemplategroup',
        'Hosttemplate',
        'Contact',
        'Command',
        'Commandargument',
        'Timeperiod',
        'Macro',
        'Mkchecks'
    ];

    /** @var array - maps the command_id to a uuid
     *  this is needed for the new command_ids when the commands are saved into a database
     */
    public $mapping = [];

    public $mappingTemplateId = [];

    public $timeperiodId = [1];

    public $servicetemplateData = [];

    public $hosttemplateData = [];

    public $macroData = [];

    public $commandData = [];

    public $servicetemplategroupData = [];

    public $mkcheckData = [];

    public function main() {
        $servicetemplategroupExport = 'n';

        if (!$this->checkNecessaryData()) {
            $this->error('Necessary data was not found');
        }

        //ask for hosttemplate data
        $hosttemplateExport = $this->in('Do you want to export Hosttemplate Data ?', ['y', 'n']);
        if ($hosttemplateExport == 'y') {

            $hosttemplateFilters = [];
            $moreInput = 'y';
            while ($moreInput == 'y') {
                $hosttemplateFilters[] = $this->in('Please choose the Hosttemplate wildcard in SQL Format (%sap% or something like that)');
                $moreInput = $this->in('Do you want to add additional Filters ?', ['y', 'n'], 'n');
            }
            if (empty($hosttemplateFilters)) {
                $this->out('<warning>No Filter given no hosttemplate data will be exported</warning>');
            }
        }

        //ask for servicetemplate data
        $servicetemplateExport = $this->in('Do you want to export Sericetemplate Data ?', ['y', 'n']);
        if ($servicetemplateExport == 'y') {
            $servicetemplateFilters = [];
            $moreInput = 'y';
            while ($moreInput == 'y') {
                $servicetemplateFilters[] = $this->in('Please choose the Servicetemplate wildcard in SQL Format (%sap% or something like that)');
                $moreInput = $this->in('Do you want to add additional Filters ?', ['y', 'n'], 'n');
            }

            if (empty($servicetemplateFilters)) {
                $this->out('<warning>No Filter given no servicetemplate data will be exported</warning>');
            } else {
                //ask for servicetemplategroup data
                $servicetemplategroupExport = $this->in('Do you want to export Sericetemplategroup Data too?', ['y', 'n']);
                if ($servicetemplategroupExport == 'y') {
                    $servicetemplategroupFilters = [];
                    $moreInput = 'y';
                    while ($moreInput == 'y') {
                        $servicetemplategroupFilters[] = $this->in('Please choose the Servicetemplategroup wildcard in SQL Format (%sap% or something like that)');
                        $moreInput = $this->in('Do you want to add additional Filters ?', ['y', 'n'], 'n');
                    }
                    if (empty($servicetemplategroupFilters)) {
                        $this->out('<warning>No Filter given no servicetemplategroup data will be exported</warning>');
                    }
                }
            }

        }

        //ask for macro data
        $macroExport = $this->in('Do you want to export Macro Data ?', ['y', 'n']);
        if ($macroExport == 'y') {
            $macroFilters = [];
            $moreInput = 'y';
            while ($moreInput == 'y') {
                $macroFilters[] = $this->in('Please choose the Macro wildcard in SQL Format it matches the macro description (%sap% or something like that)');
                $moreInput = $this->in('Do you want to add additional Filters ?', ['y', 'n'], 'n');
            }
            if (empty($macroFilters)) {
                $this->out('<warning>No Filter given no macro data will be exported</warning>');
            }
        }

        //ask for mk checks data
        $mkchecksExport = $this->in('Do you want to export MK Checks Data ?', ['y', 'n']);
        if ($mkchecksExport == 'y') {
            $mkchecksFilters = [];
            $moreInput = 'y';
            while ($moreInput == 'y') {
                $mkchecksFilters[] = $this->in('Please choose the MK Check wildcard in SQL Format it matches the macro description (%sap% or something like that)');
                $moreInput = $this->in('Do you want to add additional Filters ?', ['y', 'n'], 'n');
            }
            if (empty($mkchecksFilters)) {
                $this->out('<warning>No Filter given no mk check data will be exported</warning>');
            }
        }

        if ($hosttemplateExport == 'n' && $servicetemplateExport == 'n' && $macroExport == 'n' && $mkchecksExport == 'n') {
            $this->error('You want to do nothing ? okay .. :D');
        }

        if (empty($servicetemplateFilters) && empty($hosttemplateFilters) && empty($macroFilters) && empty($mkchecksFilters)) {
            $this->error('There are no Filters given for any Template type - Exit');
        }

        //ask for filename prefix
        $namePrefix = $this->in('Please Enter a Name for the Files you want to create (e.g. sap or mk)');

        //summary
        $this->out('<info>You Entered:</info>');

        $this->out('Hosttemplate data export <info>' . $hosttemplateExport . '</info>');
        if ($hosttemplateExport == 'y') {
            $this->out('Hosttemplate filters <info>' . implode(', ', $hosttemplateFilters) . '</info>');
        }

        $this->out('Servicetemplate data export <info>' . $servicetemplateExport . '</info>');
        if ($servicetemplateExport == 'y') {
            $this->out('Servicetemplate filters <info>' . implode(', ', $servicetemplateFilters) . '</info>');
        }

        $this->out('Servicetemplategroup data export <info>' . $servicetemplategroupExport . '</info>');
        if ($servicetemplategroupExport == 'y') {
            $this->out('Servicetemplategroup filters <info>' . implode(', ', $servicetemplategroupFilters) . '</info>');
        }

        $this->out('Macro data export <info>' . $macroExport . '</info>');
        if ($macroExport == 'y') {
            $this->out('Macro filters <info>' . implode(', ', $macroFilters) . '</info>');
        }

        $this->out('Mk Checks data export <info>' . $mkchecksExport . '</info>');
        if ($mkchecksExport == 'y') {
            $this->out('Macro filters <info>' . implode(', ', $mkchecksFilters) . '</info>');
        }

        $this->out('Filename prefix <info>' . $namePrefix . '</info>');
        $ack = $this->in('if this looks okay press y', ['y', 'n']);

        //exit if the data looks not okay for the user
        if ($ack == 'n') {
            die('Exit Data Export Shell');
        }

        //hosttemplate data
        if ($hosttemplateExport == 'y') {
            $this->hosttemplateData = $this->getHosttemplateData($hosttemplateFilters);
            $this->out('<info>Start writing Hosttemplate data File</info>');
            $this->writeFilesAsJson($namePrefix . '_hosttemplate_data', $this->hosttemplateData);

            //we also need a contact for hosttemplates
            /* $this->contactData = $this->getContactdata();
            $this->out('<info>Start writing Contact data File for Hosttemplates</info>');
            $this->writeFilesAsJson($namePrefix.'_contact_data',$this->contactData); */
        }

        // servicetemplate data
        if ($servicetemplateExport == 'y') {
            $this->servicetemplateData = $this->getServicetemplateData($servicetemplateFilters);
            $this->out('<info>Start writing Servicetemplate File</info>');
            $this->writeFilesAsJson($namePrefix . '_servicetemplate_data', $this->servicetemplateData);
        }

        // servicetemplategroup data
        if ($servicetemplategroupExport == 'y') {
            $this->servicetemplategroupData = $this->getServicetemplategroupData($servicetemplategroupFilters);
            $this->out('<info>Start writing Servicetemplategroup File</info>');
            $this->writeFilesAsJson($namePrefix . '_servicetemplategroup_data', $this->servicetemplategroupData);
        }

        // macro data
        if ($macroExport == 'y') {
            $this->macroData = $this->getMacroData($macroFilters);
            $this->out('<info>Start writing Macro File</info>');
            $this->writeFilesAsJson($namePrefix . '_macro_data', $this->macroData);
        }

        // mk check data
        if ($mkchecksExport == 'y') {
            $this->mkcheckData = $this->getMkChecksData($mkchecksFilters);
            $this->out('<info>Start writing Mk Checks File</info>');
            $this->writeFilesAsJson($namePrefix . '_mkchecks_data', $this->mkcheckData);
        }

        $this->commandData = $this->getCommandData();
        if (!empty($this->commandData)) {
            $this->writeFilesAsJson($namePrefix . '_command_data', $this->commandData);
        }

        $this->out('<success>json export finished!</success>');
    }


    /**
     * check if the necessary data exists on the system. In this case the must be the Timeperiod with ID 1 or the
     * name 24/7.
     * @return bool
     */
    public function checkNecessaryData() {
        //check if check_period_id and notify_period_id (Timeperiod) 1 exists (should be 24/7)

        $idExists = $this->Timeperiod->exists($this->timeperiodId[0]);

        if (!$idExists) {
            //@TODO test this!!
            $nameExists = $this->Timeperiod->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'Timeperiod.name LIKE' => '%24/7%',
                ]
            ]);
            $this->timeperiodID = Hash::extract($nameExists, 'Timeperiod.id');
            if (empty($nameExists)) {
                return false;
            }
            return true;
        }
        return $idExists;
    }

    /**
     * Get all user defined macros which are matching the filter
     * @return array
     */
    public function getMacroData($filters = []) {
        //get the needed command keys
        $this->out('<info>Get user defined macros</info>');

        $conditions = [];
        $args = [];
        foreach ($filters as $filter) {
            $args[] = [
                'Macro.description LIKE' => $filter
            ];
        }

        $conditions['OR'] = $args;

        $macros = $this->Macro->find('all', [
            'recursive'  => -1,
            'conditions' => [
                $conditions
            ],
            'fields'     => [
                'id',
                'name',
                'value',
                'description',
                'password',
            ]
        ]);

        //remove useless ids
        $macros = Hash::remove($macros, '{n}.Macro.id');

        return $macros;
    }

    /**
     * Get all mkchecks which are matching the filter
     * @return array
     */
    public function getMkChecksData($filters = []) {
        //get the needed command keys
        $this->out('<info>Get mk checks</info>');

        $conditions = [];
        $args = [];
        foreach ($filters as $filter) {
            $args[] = [
                'Mkchecks.name LIKE' => $filter
            ];
        }

        $conditions['OR'] = $args;

        $mkchecks = $this->Mkchecks->find('all', [
            'recursive'  => -1,
            'conditions' => [
                $conditions
            ],
            'fields'     => [
                'id',
                'name',
                'servicetemplate_id',
            ]
        ]);
        //remove useless ids
        $mkchecks = Hash::remove($mkchecks, '{n}.Mkchecks.id');

        $mappedMkChecks = [];
        //replace id of servicetemplate through uuid of servicetemplate for mapping on import
        foreach ($mkchecks as $mkcheck) {
            $id = $mkcheck['Mkchecks']['servicetemplate_id'];
            $templateToMap = $this->Servicetemplate->find('all', [
                'recursive'  => -1,
                'conditions' => [
                    [
                        'Servicetemplate.id' => $id
                    ]
                ],
                'fields'     => [
                    'uuid',
                ]
            ]);
            $mkcheck['Mkchecks']['servicetemplate_id'] = $templateToMap[0]['Servicetemplate']['uuid'];
            array_push($mappedMkChecks, $mkcheck);

        }

        return $mappedMkChecks;
    }

    /**
     * Get all command data which keys exists in the mapping
     * @return array
     */
    public function getCommandData() {
        //get the needed command keys
        $this->out('<info>Get command data</info>');
        $commandKeys = array_keys($this->mapping);
        $commands = $this->Command->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'id' => $commandKeys
            ],
            'fields'     => [
                'id',
                'name',
                'command_line',
                'command_type',
                'human_args',
                'uuid',
                'description',
            ],
            'contain'    => [
                'Commandargument' => [
                    'fields' => [
                        // 'id',
                        // 'command_id',
                        'name',
                        'human_name'
                    ]
                ]
            ]
        ]);

        //replace the uuid of the command with the uuid from mapping
        foreach ($commands as $key => $command) {
            $commands[$key]['Command']['uuid'] = $this->mapping[$command['Command']['id']];
        }

        //remove useless ids
        $commands = Hash::remove($commands, '{n}.Command.id');
        $commands = Hash::remove($commands, '{n}.Commandargument.{n}.command_id');

        return $commands;
    }

    /**
     * @TODO May needed in future versions
     * @return array
     */
    public function getContactdata() {
        $contacts = $this->Contact->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'Contact.name' => 'info'
            ],
        ]);

        foreach ($contacts as $key => $contact) {
            $contacts[$key]['Contact']['uuid'] = $this->mapping[$contact['Contact']['id']];
        }

        //remove useless ids
        $contacts = Hash::remove($contacts, '{n}.Contact.id');

        return $contacts;
    }

    /**
     * @param array $filters
     * @return array
     */
    public function getHosttemplatedata($filters = []) {
        $conditions = [];
        $args = [];
        foreach ($filters as $filter) {
            $args[] = [
                'Hosttemplate.name LIKE' => $filter
            ];
        }

        $conditions['OR'] = $args;

        $this->out('<info>Get Hosttemplate data</info>');
        $ht = $this->Hosttemplate->find('all', [
            'recursive'  => -1,
            'conditions' => [
                $conditions
            ],
            'fields'     => [
                'id',
                'name',
                'hosttemplatetype_id',
                'description',
                'command_id', // must be overridden with uuid and also with the same uuid for uuid in command data
                'check_command_args',
                'check_interval',
                'retry_interval',
                'max_check_attempts',
                'first_notification_delay',
                'notification_interval',
                'notify_on_down',
                'notify_on_unreachable',
                'notify_on_recovery',
                'notify_on_flapping',
                'notify_on_downtime',
                'flap_detection_enabled',
                'flap_detection_on_up',
                'flap_detection_on_down',
                'flap_detection_on_unreachable',
                'low_flap_threshold',
                'high_flap_threshold',
                'process_performance_data',
                'freshness_checks_enabled',
                'freshness_threshold',
                'passive_checks_enabled',
                'event_handler_enabled',
                'active_checks_enabled',
                'retain_status_information',
                'retain_nonstatus_information',
                'notifications_enabled',
                'notes',
                'priority',
                'tags',
                'host_url',
            ],
            'contain'    => [
                //'Contact',
                //'Contactgroup',
                'Hosttemplatecommandargumentvalue' => [
                    'fields' => [
                        'value',
                    ]
                ]
            ]
        ]);

        //remove the useless ids
        $ht = Hash::remove($ht, '{n}.Hosttemplate.id');
        $ht = Hash::remove($ht, '{n}.Hosttemplatecommandargumentvalue.{n}.hosttemplate_id');

        //replace command_id with an uuid which also will inserted as command UUID when we get the command data
        foreach ($ht as $key => $hosttemplate) {
            if (isset($this->mapping[$hosttemplate['Hosttemplate']['command_id']])) {
                //command id has already an uuid in mapping
                $ht[$key]['Hosttemplate']['command_id'] = $this->mapping[$hosttemplate['Hosttemplate']['command_id']];
            } else {
                //no mapping for this command exists so get a new uuid
                $currentUuid = UUID::v4();
                $this->mapping[$hosttemplate['Hosttemplate']['command_id']] = $currentUuid;
                $ht[$key]['Hosttemplate']['command_id'] = $currentUuid;
            }

            /*   if(isset($this->mapping[$hosttemplate['Contact'][0]['id']])){
                   $ht[$key]['Hosttemplate']['Contact'] = $this->mapping[$hosttemplate['Contact'][0]['id']];
                   $ht[$key]['Hosttemplate']['Contactgroup'] = [];
               }else{
                   $currentUuid = UUID::v4();
                   $this->mapping[$hosttemplate['Contact'][0]['id']] = $currentUuid;
                   $ht[$key]['Hosttemplate']['Contact'] = [$currentUuid];
                   $ht[$key]['Hosttemplate']['Contactgroup'] = [];
               }
   */
            //container id is 1 every time
            $ht[$key]['Hosttemplate']['container_id'] = 1;
            //eventhandler_command_id is 0 every time
            $ht[$key]['Hosttemplate']['eventhandler_command_id'] = 0;
            //check_period_id and notify_period is always the 24/7 timeperiod
            $ht[$key]['Hosttemplate']['check_period_id'] = $this->timeperiodId[0];
            $ht[$key]['Hosttemplate']['notify_period_id'] = $this->timeperiodId[0];
            //contact is 1 every time
            //THIS REQUIRES AN INFO CONTACT ON THE MACHINE WHERE THE MODULE SHOULD BE INSTALLED!
            $ht[$key]['Hosttemplate']['Contact'] = [1];
            $ht[$key]['Contact'] = [1];
            //contactgroup is empty
            $ht[$key]['Hosttemplate']['Contactgroup'] = [];
            $ht[$key]['Contactgroup'] = [];
        }

        return $ht;
    }


    /**
     * @param string $filter - name filter from user input
     * @return array
     */
    public function getServicetemplateData($filters = []) {
        $conditions = [];
        $args = [];
        foreach ($filters as $filter) {
            $args[] = [
                'Servicetemplate.template_name LIKE' => $filter
            ];
        }

        $conditions['OR'] = $args;

        $this->out('<info>Get Servicetemplate data</info>');
        $st = $this->Servicetemplate->find('all', [
            'recursive'  => -1,
            'conditions' => [
                $conditions
            ],
            'fields'     => [
                'id',
                'name',
                'uuid',
                'template_name',
                'servicetemplatetype_id',
                'description',
                'command_id', // must be overridden with uuid and also with the same uuid for uuid in command data
                'check_command_args',
                'checkcommand_info',
                'check_interval',
                'retry_interval',
                'max_check_attempts',
                'first_notification_delay',
                'notification_interval',
                'notify_on_warning',
                'notify_on_unknown',
                'notify_on_critical',
                'notify_on_recovery',
                'notify_on_flapping',
                'notify_on_downtime',
                'flap_detection_enabled',
                'flap_detection_on_ok',
                'flap_detection_on_warning',
                'flap_detection_on_unknown',
                'flap_detection_on_critical',
                'low_flap_threshold',
                'high_flap_threshold',
                'process_performance_data',
                'freshness_checks_enabled',
                'freshness_threshold',
                'passive_checks_enabled',
                'event_handler_enabled',
                'active_checks_enabled',
                'retain_status_information',
                'retain_nonstatus_information',
                'notifications_enabled',
                'notes',
                'priority',
                'tags',
                'service_url',
                'is_volatile',
                'check_freshness',
            ],
            'contain'    => [
                'Contact',
                'Contactgroup',
                'Servicetemplatecommandargumentvalue' => [
                    'fields' => [
                        'value',
                    ]
                ]
            ]
        ]);

        //replace command_id with an uuid which also will inserted as command UUID when we get the command data
        foreach ($st as $key => $servicetemplate) {
            if (isset($this->mapping[$servicetemplate['Servicetemplate']['command_id']])) {
                //command id has already an uuid in mapping
                $st[$key]['Servicetemplate']['command_id'] = $this->mapping[$servicetemplate['Servicetemplate']['command_id']];
            } else {
                //no mapping for this command exists so get a new uuid
                $currentUuid = UUID::v4();
                $this->mapping[$servicetemplate['Servicetemplate']['command_id']] = $currentUuid;
                $st[$key]['Servicetemplate']['command_id'] = $currentUuid;
            }
            $this->mappingTemplateId[$st[$key]['Servicetemplate']['id']] = $st[$key]['Servicetemplate']['uuid'];
            //container id is 1 every time
            $st[$key]['Servicetemplate']['container_id'] = 1;
            //eventhandler_command_id is 0 every time
            $st[$key]['Servicetemplate']['eventhandler_command_id'] = 0;
            //check_period_id and notify_period is always the 24/7 timeperiod
            $st[$key]['Servicetemplate']['check_period_id'] = $this->timeperiodId[0];
            $st[$key]['Servicetemplate']['notify_period_id'] = $this->timeperiodId[0];

        }
        //remove the useless ids
        $st = Hash::remove($st, '{n}.Servicetemplate.id');
        $st = Hash::remove($st, '{n}.Servicetemplatecommandargumentvalue.{n}.servicetemplate_id');

        return $st;
    }

    /**
     * @param string $filter - name filter from user input
     * @return array
     */
    public function getServicetemplategroupData($filters = []) {
        $conditions = [];
        $args = [];
        foreach ($filters as $filter) {
            $args[] = [
                'Container.name LIKE' => $filter
            ];
        }

        $conditions['OR'] = $args;

        $this->out('<info>Get Servicetemplategroup data</info>');
        $st = $this->Servicetemplategroup->find('all', [
            'recursive'  => -1,
            'conditions' => [
                $conditions
            ],
            'fields'     => [
                'id',
                'uuid',
                'description',
                'container_id',
            ],
            'contain'    => [
                'Container'       => [
                    'fields' => [
                        'id',
                        'name',
                        'containertype_id',
                        'parent_id'
                    ]
                ],
                'Servicetemplate' => [
                    'fields' => [
                        'id'
                    ]
                ]
            ]
        ]);

        // remove useless data
        $st = Hash::remove($st, '{n}.Servicetemplate.{n}.ServicetemplatesToServicetemplategroup');
        foreach ($st as $keyGroup => $servicetemplategroup) {
            $templateArray = [];
            foreach ($servicetemplategroup['Servicetemplate'] as $keyTemplate => $servicetemplate) {
                if (isset($this->mappingTemplateId[$servicetemplate['id']])) {
                    //command id has already an uuid in mapping
                    $st[$keyGroup]['Servicetemplate'][$keyTemplate]['id'] = $this->mappingTemplateId[$servicetemplate['id']];
                } else {
                    //no mapping for this servicetemplate exists so get a new uuid
                    $currentUuid = UUID::v4();
                    $this->mappingTemplateId[$servicetemplate['id']] = $currentUuid;
                    $st[$keyGroup]['Servicetemplate'][$keyTemplate]['id'] = $currentUuid;
                }
                array_push($templateArray, $st[$keyGroup]['Servicetemplate'][$keyTemplate]['id']);
            }
            $st[$keyGroup]['Servicetemplategroup']['Servicetemplate'] = $templateArray;
            $st[$keyGroup]['Servicetemplate'] = $templateArray;
        }


        //remove the useless ids
        $st = Hash::remove($st, '{n}.Servicetemplategroup.id');
        $st = Hash::remove($st, '{n}.Servicetemplategroup.container_id');
        $st = Hash::remove($st, '{n}.Container.id');

        return $st;
    }


    public function writeFilesAsJson($name, $content) {
        if (empty($content)) {
            $this->out('<error>The ' . $name . ' File could not be written because its empty</error>');
            return false;
        }
        $dir = APP . 'Plugin' . DS . $this->plugin . DS . 'webroot' . DS;
        $folder = 'jsonData';
        if (!is_dir($dir . $folder)) {
            mkdir($dir . $folder, 0755);
        }
        $filename = $name . '.json';
        $file = new File($dir . $folder . DS . $filename, true, 0644);
        if (!$file->exists()) {
            $file->create();
        }
        $content = json_encode($content);
        $return = $file->write($content);
        $file->close();

        if ($return) {
            $this->out('<success>File ' . $filename . ' was sucessfully written to ' . $dir . $folder . '</success>');
        } else {
            $this->out('<error>The File ' . $filename . ' could not be written</error>');
        }
        return $return;
    }
}
