<?php
/**
 * // Copyright (C) <2017>  <it-novum GmbH>
 * reseated* // This file is dual licensed
 * //
 * // 1.
 * //    This program is free software: you can redistribute it and/or modify
 * //    it under the terms of the GNU General Public License as published by
 * //    the Free Software Foundation, version 3 of the License.
 * //
 * //    This program is distributed in the hope that it will be useful,
 * //    but WITHOUT ANY WARRANTY; without even the implied warranty of
 * //    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * //    GNU General Public License for more details.
 * //
 * //    You should have received a copy of the GNU General Public License
 * //    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * //
 * // 2.
 * //    If you purchased an openITCOCKPIT Enterprise Edition you can use this file
 * //    under the terms of the openITCOCKPIT Enterprise Edition license agreement.
 * //    License agreement and license key will be shipped with the order
 * //    confirmation.
 */

/**
 * Class DesignController
 */
class DesignController extends DesignModuleAppController {
    public $layout = 'Admin.default';

    public $defaultDesign = [
        'Design' => [
            'id'                      => '1',
            'primary'                 => '#3a3633',
            'secondary'               => '#474544',
            'navigation-color-fade'   => '93',
            'font-color-nav'          => '#c0bbb7',
            'font-color-nav-hover'    => '#ffffff',
            'font-color'              => '#333333',
            'font-color-header'       => '#333333',
            'body-background'         => '#eeeeef',
            'body-page-headline'      => '#4c4f53',
            'window-background-color' => '#ffffff', // windowHeaderBackground
            'window-border-color'     => '#C2C2C2', // windowBorderColor
            'window-body-color'       => '#ffffff', // windowBodyColor
            'font-color'              => '#333333', //primaryFontColor
            'table-odd-color'         => '#f9f9f9',
            'table-hover-color'       => '#f5f5f5',
            'well-color'              => '0',
            'input-color'             => '0',
            'main-header-background'  => '#eaeaea',
            'main-header-text-color'  => '#6D6A69',
            'logo-in-header'          => '0'
        ]
    ];

    public $components = [
        'RequestHandler',
    ];

    // TODO: more css styles
    public function edit() {
        $design = $this->Design->find('first');
        $design = Hash::merge($this->defaultDesign, $design);

        if ($this->request->is('post') || $this->request->is('put')) {
            Cache::write('design_module_style_time', time(), 'long');
            $this->Design->id = 1;

            //Only Logo in Header enabled?
            $onlyLogoChanged = false;
            $diff = Hash::diff($this->defaultDesign['Design'], $this->request->data['Design']);
            if (sizeof($diff) === 1 && isset($diff['logo-in-header'])) {
                $onlyLogoChanged = true;
            }

            if ($this->Design->saveAll($this->request->data)) {
                $option = $this->request->data;
                $content = require_once APP . 'Plugin' . DS . 'DesignModule' . DS . 'View' . DS . 'Design' . DS . 'css' . DS . 'style.ctp';
                $this->writeStyleFile(APP . 'Plugin' . DS . 'DesignModule' . DS . 'webroot' . DS . 'css' . DS . 'style.css', $content);

                $this->setFlash(__('<a href="/module_design/design/edit">Design</a> successfully saved', 1));
                $this->redirect(['action' => 'edit']);
            } else {
                $this->setFlash(__('Design could not be saved'), false);

            }

            $design = Hash::merge($this->defaultDesign, $this->request->data);
        }

        $this->set('design', $design);
        $this->set('_serialize', ['design']);
    }

    public function export() {
        $design = $this->Design->find('first');
        $design = Hash::merge($this->defaultDesign, $design);
        header("Content-type: text/plain");
        header("Content-Disposition: attachment; filename=openITCOCKPIT-design.json");
        echo json_encode($design, JSON_PRETTY_PRINT);
        die();
    }

    public function import() {
        if (empty($_FILES)) {
            throw new ForbiddenException(__('There is no file to store'));
        }
        $fileExtension = pathinfo($_FILES['file']['name'], PATHINFO_EXTENSION);
        if (mb_strtolower($fileExtension) != 'json') {
            throw new ForbiddenException(__('There is only json allowed'));
        }

        try {
            if (!file_get_contents($_FILES['file']['tmp_name'])) {
                throw new Exception(__('Cannot read uploaded file'));
            }
            $design = json_decode(file_get_contents($_FILES['file']['tmp_name']), true);
            if ($this->Design->saveAll($design)) {
                $option = $design;
                $onlyLogoChanged = false;
                $content = require_once APP . 'Plugin' . DS . 'DesignModule' . DS . 'View' . DS . 'Design' . DS . 'css' . DS . 'style.ctp';
                $this->writeStyleFile(APP . 'Plugin' . DS . 'DesignModule' . DS . 'webroot' . DS . 'css' . DS . 'style.css', $content);
                $this->setFlash(__('<a href="/module_design/design/edit">Design</a> successfully imported', 1));
            } else {
                $this->setFlash(__('Design could not be imported'), false);
            }
        } catch (Exception $e) {
            throw new ForbiddenException($_FILES['file']['tmp_name'] . '.' . $fileExtension . ': ' . $e->getMessage());
        }
        die();
    }

    public function reset($design = null) {
        Cache::write('design_module_style_time', time(), 'long');
        $cssFile = APP . 'Plugin' . DS . 'DesignModule' . DS . 'webroot' . DS . 'css' . DS . 'style.css';
        $cssCtp = APP . 'Plugin' . DS . 'DesignModule' . DS . 'View' . DS . 'Design' . DS . 'css' . DS . 'style.ctp';
        if ($design === null) {
            fclose(fopen($cssFile, 'w+'));
            $this->Design->deleteAll(true);

            $this->setFlash(__('<a href="/module_design/design/edit">Design</a> successfully reseted', 1));
            $this->redirect(['action' => 'edit']);
        }

        $option = [];
        $onlyLogoChanged = false;

        if ($design == 1) {

            $light = [
                'Design' => [
                    'id'                      => '1',
                    'primary'                 => '#f7f7f7',
                    'secondary'               => '#f7f4f3',
                    'navigation-color-fade'   => '70',
                    'font-color-nav'          => '#69737a',
                    'font-color-nav-hover'    => '#000000',
                    'font-color'              => '#333333',
                    'font-color-header'       => '#4c4f53',
                    'body-background'         => '#ffffff',
                    'body-page-headline'      => '#4c4f53',
                    'window-background-color' => '#ffffff',
                    'window-border-color'     => '#cccccc',
                    'window-body-color'       => '#ffffff',
                    'table-odd-color'         => '#f9f9f9',
                    'table-hover-color'       => '#ecf3f8',
                    'well-color'              => '0',
                    'input-color'             => '0',
                    'main-header-background'  => '#2196f3',
                    'main-header-text-color'  => '#ffffff',
                    'logo-in-header'          => '0'
                ]
            ];
            $option = $light;
            $content = require_once $cssCtp;
            $this->writeStyleFile($cssFile, $content);
            $this->Design->saveAll($option);
            $this->setFlash(__('<a href="/module_design/design/edit">Design</a> changed to dark mode', 1));
            $this->redirect(['action' => 'edit']);
        }

        if ($design == 2) {
            $dark = [
                'Design' => [
                    'id'                      => '1',
                    'primary'                 => '#1f1d1d',
                    'secondary'               => '#292929',
                    'navigation-color-fade'   => '56',
                    'font-color-nav'          => '#e3e3e3',
                    'font-color-nav-hover'    => '#cacaca',
                    'font-color'              => '#e3e3e3',
                    'font-color-header'       => '#d8d9da',
                    'body-background'         => '#141414',
                    'body-page-headline'      => '#bdbdbd',
                    'window-background-color' => '#292929',
                    'window-border-color'     => '#4b4a4a',
                    'window-body-color'       => '#292929',
                    'table-odd-color'         => '#383838',
                    'table-hover-color'       => '#616161',
                    'well-color'              => '1',
                    'main-header-background'  => '#1f1d1d',
                    'main-header-text-color'  => '#cac7c7',
                    'input-color'             => '1',
                    'logo-in-header'          => '0'
                ]
            ];
            $option = $dark;
            $content = require_once $cssCtp;
            $this->writeStyleFile($cssFile, $content);
            $this->Design->saveAll($option);
            $this->setFlash(__('<a href="/module_design/design/edit">Design</a> changed to dark mode', 1));
            $this->redirect(['action' => 'edit']);
        }

        if ($design == 3) {
            $darkElegance = [
                'Design' => [
                    'id'                      => '1',
                    'primary'                 => '#1b1e24',
                    'secondary'               => '#22262e',
                    'navigation-color-fade'   => '100',
                    'font-color-nav'          => '#8b91a0',
                    'font-color-nav-hover'    => '#e1eaf1',
                    'font-color'              => '#333333',
                    'font-color-header'       => '#333333',
                    'body-background'         => '#eeeeef',
                    'body-page-headline'      => '#4c4f53',
                    'window-background-color' => '#e6e6e6',
                    'window-border-color'     => '#dddddd',
                    'window-body-color'       => '#ffffff',
                    'table-odd-color'         => '#f9f9f9',
                    'table-hover-color'       => '#f5f5f5',
                    'well-color'              => '0',
                    'main-header-background'  => '#22262e',
                    'main-header-text-color'  => '#bbbbbb',
                    'input-color'             => '0',
                    'logo-in-header'          => '0'
                ]
            ];
            $option = $darkElegance;
            $content = require_once $cssCtp;
            $this->writeStyleFile($cssFile, $content);
            $this->Design->saveAll($option);
            $this->setFlash(__('<a href="/module_design/design/edit">Design</a> changed to Elegance', 1));
            $this->redirect(['action' => 'edit']);
        }


        $this->response->disableCache();
    }

    /**
     * @param string $path
     * @param string $content
     *
     * @return bool
     */
    protected function writeStyleFile($path = '', $content) {
        $handle = fopen($path, 'w+');
        fwrite($handle, $content);
        fclose($handle);

        return true;
    }
}
