<?php
// Copyright (C) <2017>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.

App::uses('AppController', 'Controller');

class NmapConfigurationsController extends AppController {

    public $layout = 'Admin.default';
    public $components = [
        'ListFilter.ListFilter',
        'CustomValidationErrors',
    ];
    public $helpers = [
        'ListFilter.ListFilter',
        'CustomValidationErrors',
    ];

    public $uses = [
        'Servicetemplate',
        'Hosttemplate',
        'DiscoveryModule.NmapMapping',
        'DiscoveryModule.NmapConfiguration'
    ];

    public function index() {

        $allNmapConfigurations = $this->NmapConfiguration->find('all', [
            'recursive' => -1
        ]);

        $this->set(compact('allNmapConfigurations'));

    }

    public function add() {
        if ($this->hasRootPrivileges === true) {
            $container = $this->Tree->easyPath($this->MY_RIGHTS, OBJECT_HOST, [], $this->hasRootPrivileges, [CT_HOSTGROUP]);
        } else {
            $container = $this->Tree->easyPath($this->getWriteContainers(), OBJECT_HOST, [], $this->hasRootPrivileges, [CT_HOSTGROUP]);
        }
        $containerIds = array_keys($container);
        $hosttemplates = $this->Hosttemplate->hosttemplatesByContainerId($containerIds, 'list');
        $servicetemplates = $this->Servicetemplate->servicetemplatesByContainerId($containerIds, 'list');

        $this->set(compact('servicetemplates', 'hosttemplates', 'container'));


        if ($this->request->is('post') || $this->request->is('put')) {

            $configData = [];
            $configData['NmapConfiguration'] = $this->request->data['NmapConfiguration'];

            $this->NmapConfiguration->create();

            $dataToSave = $this->request->data;
            if (isset($this->request->data['NmapConfiguration']['single_host_scan']) && $this->request->data['NmapConfiguration']['single_host_scan'] == true) {
                //set bitmask to 32 if the scan type is Single Host
                $mergedata = [
                    'NmapConfiguration' => [
                        'bitmask' => 32
                    ]
                ];
                $dataToSave = array_merge_recursive($this->request->data, $mergedata);
            }

            //remove empty mappings from the data
            if (!empty($dataToSave['NmapMapping'])) {
                foreach ($dataToSave['NmapMapping'] as $key => $mapping) {
                    if (empty($mapping['port']) && empty($mapping['service_name'])) {
                        unset($dataToSave['NmapMapping'][$key]);
                    }
                }
            } else if (empty($dataToSave['NmapMapping']) && (!isset($this->request->data['NmapConfiguration']['single_host_scan']) || $this->request->data['NmapConfiguration']['single_host_scan'] == false)) {
                $mergedata = [
                    'NmapConfiguration' => [
                        'hosts_scan_without_services' => '1'
                    ]
                ];
                $dataToSave = array_merge_recursive($this->request->data, $mergedata);
            }
            if ($this->NmapConfiguration->saveAll($dataToSave)) {
                $this->setFlash(__('NMAP configuration successfully saved'));
                $this->redirect(['action' => 'index']);
            } else {
                $this->setFlash(__('could not save data'), false);
            }
        }
    }

    public function addMapping() {
        $servicetemplates = $this->Servicetemplate->find('list');
        $this->set(compact('servicetemplates'));
    }

    public function edit($id) {
        if (!$this->NmapConfiguration->exists($id)) {
            throw new NotFoundException(__('Invalid NMAP configuration'));
        }

        $nmapConfig = $this->NmapConfiguration->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'NmapConfiguration.id' => $id
            ],
            'contain'    => [
                'NmapMapping'
            ]
        ]);

        if ($this->hasRootPrivileges === true) {
            $container = $this->Tree->easyPath($this->MY_RIGHTS, OBJECT_HOST, [], $this->hasRootPrivileges, [CT_HOSTGROUP]);
        } else {
            $container = $this->Tree->easyPath($this->getWriteContainers(), OBJECT_HOST, [], $this->hasRootPrivileges, [CT_HOSTGROUP]);
        }

        $containerIds = array_keys($container);
        $hosttemplates = $this->Hosttemplate->hosttemplatesByContainerId($containerIds, 'list');
        $servicetemplates = $this->Servicetemplate->servicetemplatesByContainerId($containerIds, 'list');

        $this->set(compact('nmapConfig', 'servicetemplates', 'hosttemplates', 'container'));

        if ($this->request->is('post') || $this->request->is('put')) {

            $configData = [];
            $configData['NmapConfiguration'] = $this->request->data['NmapConfiguration'];

            $this->NmapConfiguration->create();

            $dataToSave = $this->request->data;
            if (isset($this->request->data['NmapConfiguration']['single_host_scan']) && $this->request->data['NmapConfiguration']['single_host_scan'] == true) {
                //set bitmask to 32 if the scan type is Host Only
                if (!isset($this->request->data['NmapConfiguration']['bitmask'])) {
                    $mergedata = [
                        'NmapConfiguration' => [
                            'bitmask'                     => 32,
                            'hosts_scan_without_services' => '0'
                        ],
                    ];
                } else {
                    $mergedata = [
                        'NmapConfiguration' => [
                            'hosts_scan_without_services' => '0'
                        ],
                    ];
                }

                $dataToSave = array_merge_recursive($this->request->data, $mergedata);
            } else {
                //multihost scan active
                if (empty($dataToSave['NmapMapping'])) {
                    $mergedata = [
                        'NmapConfiguration' => [
                            'hosts_scan_without_services' => '1'
                        ]
                    ];
                } else {
                    $mergedata = [
                        'NmapConfiguration' => [
                            'hosts_scan_without_services' => '0'
                        ]
                    ];
                }
                $dataToSave = array_merge_recursive($this->request->data, $mergedata);
            }

            //remove empty mappings from the data
            if (!empty($dataToSave['NmapMapping'])) {
                foreach ($dataToSave['NmapMapping'] as $key => $mapping) {
                    if (empty($mapping['port']) && empty($mapping['service_name'])) {
                        unset($dataToSave['NmapMapping'][$key]);
                    }
                }
            }

            //delete old mappings
            $this->NmapMapping->deleteAll([
                'NmapMapping.nmap_configuration_id' => $id
            ]);
            if ($this->NmapConfiguration->saveAll($dataToSave)) {
                $this->setFlash(__('NMAP configuration successfully modified'));
                $this->redirect(['action' => 'index']);
            } else {
                $this->setFlash(__('could not save data'), false);
            }
        }
    }

    public function delete($id) {
        if (!$this->NmapConfiguration->exists($id)) {
            throw new NotFoundException('NMAP configuration not found');
        }

        if ($this->NmapConfiguration->delete($id)) {
            $this->setFlash(__('NMAP Configuration deleted'));
            $this->redirect(['action' => 'index']);
        }
        $this->setFlash(__('Could not delete NMAP Configuration'), false);
        $this->redirect(['action' => 'index']);
    }

}
