<?php
// Copyright (C) <2017>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.

App::uses('AppController', 'Controller');
App::uses('UUID', 'Lib');


class NmapController extends AppController {

    public $layout = 'Admin.default';
    public $components = [
        'ListFilter.ListFilter',
        'CustomValidationErrors',
        'GearmanClient',
        'Flash'
    ];
    public $helpers = [
        'ListFilter.ListFilter',
        'CustomValidationErrors',
    ];

    public $uses = [
        'DiscoveryModule.NmapConfiguration',
        'DiscoveryModule.NmapScan',
        'DiscoveryModule.Nmap',
        'Host',
        'Service',
        'Servicetemplate',
    ];

    private $discoveryResultPath = null;
    private $discoveryResultFilename = null;

    public function index() {
        if (!$this->isNmapInstalled()) {
            //throw new CommandNotFoundException;
        }

        $allNmapConfigurations = $this->NmapConfiguration->find('list');

        $this->set(compact('allNmapConfigurations'));
        $this->Frontend->setJson('executing_nmap_discovery', __('Executing NMAP discovery...'));

        if ($this->request->is('post')) {
            $data = $this->request->data;
            $hostSaveFailures = [];
            $hostId = null;

            $containerId = null;
            if (isset($data['container_id'])) {
                $containerId = $data['container_id'];
                unset($data['container_id']);
            }

            foreach ($data as $key => $hostData) {
                $tmpUuid = UUID::v4();
                if ($key != 'hosttemplate_id' && $key != 'default_servicetemplate_id') {
                    $data[$key]['services'][$tmpUuid]['servicetemplate'] = $data['default_servicetemplate_id'];
                }
            }

            $hostAddresses = Hash::extract($data, '{s}.hostaddress');
            $existingHosts = $this->Host->find('all', [
                'recursive'  => -1,
                'conditions' => [
                    'Host.address' => $hostAddresses
                ],
                'fields'     => [
                    'Host.address'
                ]
            ]);
            $existingHosts = Hash::extract($existingHosts, '{n}.Host.address');

            foreach ($data as $key => $hostData) {
                if ($key != 'hosttemplate_id' && $key != 'default_servicetemplate_id') {
                    //check if the host already exists
                    if (in_array($hostData['hostaddress'], $existingHosts)) {
                        //host exists -> skip
                        continue;
                    }

                    //save hosts
                    $hostname = $hostData['hostaddress'];
                    if (!empty($hostData['hostname'])) {
                        $hostname = $hostData['hostname'];
                    }

                    $this->Host->create();
                    $hostDataToSave = [
                        'Host'      => [
                            'uuid'            => UUID::v4(),
                            'name'            => $hostname,
                            'address'         => $hostData['hostaddress'],
                            'hosttemplate_id' => $data['hosttemplate_id'],
                            'container_id'    => (!empty($containerId) ? $containerId : ROOT_CONTAINER),
                            'host_type'       => GENERIC_HOST,
                        ],
                        'Container' => [
                            'Container' => [
                                (!empty($containerId) ? $containerId : ROOT_CONTAINER)
                            ]
                        ]
                    ];

                    if ($this->Host->save($hostDataToSave)) {
                        $hostId = $this->Host->id;
                    } else {
                        $hostSaveFailures[] = [
                            'hostname'         => $hostname,
                            'hostaddress'      => $hostData['hostaddress'],
                            'validationErrors' => $this->Host->validationErrors
                        ];
                    }
                    $serviceDataToSave = [];
                    if (!empty($hostId)) {
                        foreach ($hostData['services'] as $serviceData) {
                            //save services
                            $this->Service->create();
                            $serviceDataToSave[] = [
                                'Service' => [
                                    'uuid'               => UUID::v4(),
                                    'servicetemplate_id' => $serviceData['servicetemplate'],
                                    'host_id'            => $hostId,
                                    'service_type'       => GENERIC_SERVICE,
                                ]
                            ];
                        }
                        if ($this->Service->saveAll($serviceDataToSave)) {

                        } else {
                            $serviceSaveFailures[] = [
                                'hostname'         => $hostname,
                                'hostId'           => $hostId,
                                'validationErrors' => $this->Service->validationErrors,
                            ];
                        }
                    }
                }
            }

            /*
             * handle the host and service failures
             */
            if (empty($hostSaveFailures) && empty($serviceSaveFailures)) {
                $this->Flash->success('Host and Services has been successfully saved', [
                    'key' => 'NmapDiscoveryMsg',
                ]);
                $this->redirect(['action' => 'index']);
            }

            $msg = [];
            if (!empty($hostSaveFailures)) {
                //at lease one host failed to Save
                $msg = [
                    'Hosts' => $hostSaveFailures
                ];
            }

            if (!empty($serviceSaveFailures)) {
                //at least one service failed to Save
                $msg[] = [
                    'Services' => $serviceSaveFailures
                ];
            }

            $this->Flash->error('Could not create ', [
                'key'    => 'NmapDiscoveryMsg',
                'params' => [
                    'failures' => $msg,
                ]
            ]);
            $this->redirect(['action' => 'index']);

        }
    }

    public function discovery($configId) {
        $this->autoRender = false;
        $this->allowOnlyAjaxRequests();
        $config = $this->NmapConfiguration->find('first', [
            'conditions' => [
                'NmapConfiguration.id' => $configId
            ]
        ]);


        //$CIDRAddress = $config['NmapConfiguration']['address'] . '/' . $config['NmapConfiguration']['bitmask'];

        if ($this->request->is('post') && !empty($configId)) {
            $this->discoveryResultPath = '/tmp/';
            $this->discoveryResultFilename = uniqid('nmapScan');

            $scanData = [
                'nmap_configuration_id' => $configId,
                'filename'              => $this->discoveryResultFilename
            ];

            $scanId = null;
            if ($this->NmapScan->save($scanData)) {
                $scanId = $this->NmapScan->id;
            } else {
                die('cant save scan data');
            }


            $payload = [
                'task'                 => 'NmapDiscovery',
                'path'                 => $this->discoveryResultPath,
                'filename'             => $this->discoveryResultFilename,
                'scanId'               => $scanId,
                //'cidrAddress'      => $CIDRAddress,
                'address'              => $config['NmapConfiguration']['address'],
                'bitmask'              => $config['NmapConfiguration']['bitmask'],
                'singleHost'           => $config['NmapConfiguration']['single_host_scan'],
                'hostsWithoutServices' => $config['NmapConfiguration']['hosts_scan_without_services'],
            ];


            Configure::load('gearman');
            $this->Config = Configure::read('gearman');

            $discoveryResult = $this->GearmanClient->client->doBackground("oitc_gearman", Security::cipher(serialize($payload), $this->Config['password']));
            if (!empty($discoveryResult)) {

                echo json_encode($this->discoveryResultFilename);
            }
        }
    }

    public function broadcast($filename = '', $id) {
        $this->autoRender = false;
        $this->discoveryResultPath = '/tmp/';
        $file = $this->discoveryResultPath . $filename;

        $result = $this->NmapScan->find('first', [
            'conditions' => [
                'NmapScan.filename' => $filename,
            ]
        ]);

        if (empty($result)) {
            //stop the ajax requests
            $ret = [
                'stop' => true
            ];
            return json_encode($ret);
        }

        if (file_exists($file) && $result['NmapScan']['successful'] == 1) {
            //scan finished and was successful
            // $xml = simplexml_load_file($file);
            // $beautifiedStructure = $this->Nmap->simplifyNmapResult($xml, $id);
            $ret = [
                'finished' => true
            ];

            echo json_encode($ret);
        } else if ($result['NmapScan']['finished'] == 1 && $result['NmapScan']['successful'] == 0) {
            //Scan finished unsuccessfully
            $ret = [
                'stop' => true
            ];
            echo json_encode($ret);
        } else {
            //scan not yet finished
            $ret = [
                'finished' => false
            ];
            echo json_encode($ret);
        }
    }

    public function discoveryresult($filename = null, $id = null) {
        $discoveryResultPath = '/tmp/';
        $file = $discoveryResultPath . $filename;
        $guessed = true;

        $nmapConfig = $this->NmapConfiguration->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'NmapConfiguration.id' => $id
            ]
        ]);


        $xml = simplexml_load_file($file);
        $discoveryResult = $this->Nmap->simplifyNmapResult($xml, $id, $guessed);

        $hostAddresses = Hash::extract($discoveryResult, 'hosts.{n}.address');
        $existingHosts = $this->Host->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'Host.address' => $hostAddresses
            ],
            'fields'     => [
                'Host.address'
            ]
        ]);
        $existingHosts = Hash::extract($existingHosts, '{n}.Host.address');

        foreach ($discoveryResult['hosts'] as $key => $result) {
            if (in_array($result['address'], $existingHosts)) {
                $discoveryResult['hosts'][$key]['exists'] = 1;
            } else {
                $discoveryResult['hosts'][$key]['exists'] = 0;
            }
        }

        $servicetemplates = $this->Servicetemplate->find('list');

        $this->set(compact('discoveryResult', 'servicetemplates', 'filename', 'discoveryResultPath', 'nmapConfig'));
    }

    public function getServicetemplates() {
        $this->autoRender = false;
        if (!$this->request->is('ajax')) {
            throw new MethodNotAllowedException();
        }
        $servicetemplates = $this->Servicetemplate->find('list');
        echo json_encode($servicetemplates);
    }

    public function isNmapInstalled() {
        $return = shell_exec(sprintf('which %s', escapeshellarg('nmap')));
        return !empty($return);
    }

}
