<?php
// Copyright (C) <2017>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.


class Nmap extends DiscoveryModuleAppModel {


    /**
     * Simplifies the result of the nmap scan, so we get an unified structure
     * @TODO enable guesses - this should automatically choose the servicetemplate by the name or port even its not mapped
     * @TODO this is a feature which can be enabled in the nmap configuration
     *
     * @param array $nmapScanObj
     * @param $id
     * @param bool $guess
     * @return array
     */
    public function simplifyNmapResult($nmapScanObj = [], $id, $guess = false) {
        $mappedTemplates = $this->getMappingTemplates($id);

        //convert the xml object to array for better accessibility
        $nmapScanArr = $this->xml2array($nmapScanObj);

        //for a single Host Scan we need to push it in an array so that our loop is valid
        if (!isset($nmapScanArr['host'][0])) {
            $nmapScanArr['host'] = [$nmapScanArr['host']];
        }

        $hosts = [];
        foreach ($nmapScanArr['host'] as $hostKey => $hostItem) {
            $ports = [];
            if (!empty($hostItem['ports']['port']) && !isset($hostItem['ports']['port'][0])) {
                //nmap doesnt create a numeric array if there is just one port
                $name = $hostItem['ports']['port']['service']['@attributes']['name'];
                $portid = $hostItem['ports']['port']['@attributes']['portid'];

                $matchingTemplate = $this->getMatchingTemplate($mappedTemplates, $name, $portid);
                $ports[0] = [
                    'portid'           => $portid,
                    'name'             => $name,
                    'matchingTemplate' => $matchingTemplate
                ];
            } else {
                if (isset($hostItem['ports']['port'])) {
                    foreach ($hostItem['ports']['port'] as $portKey => $portItem) {
                        $name = $portItem['service']['@attributes']['name'];
                        $portid = $portItem['@attributes']['portid'];

                        $matchingTemplate = $this->getMatchingTemplate($mappedTemplates, $name, $portid);
                        $ports[$portKey] = [
                            'portid'           => $portid,
                            'name'             => $name,
                            'matchingTemplate' => $matchingTemplate
                        ];
                    }
                }
            }

            $hosts[$hostKey] = [
                'address'  => $hostItem['address']['@attributes']['addr'],
                'hostname' => isset($hostItem['hostnames']['hostname']['@attributes']['name']) ? $hostItem['hostnames']['hostname']['@attributes']['name'] : '',
                'ports'    => $ports
            ];

        }

        $runstats = [
            'finished' => [
                'timestr' => $nmapScanArr['runstats']['finished']['@attributes']['timestr'],
                'elapsed' => $nmapScanArr['runstats']['finished']['@attributes']['elapsed'],
                'summary' => $nmapScanArr['runstats']['finished']['@attributes']['summary'],
                'exit'    => $nmapScanArr['runstats']['finished']['@attributes']['exit'],
            ],
            'hosts'    => [
                'up'    => $nmapScanArr['runstats']['hosts']['@attributes']['up'],
                'down'  => $nmapScanArr['runstats']['hosts']['@attributes']['down'],
                'total' => $nmapScanArr['runstats']['hosts']['@attributes']['total'],
            ]
        ];

        $newNmapScanArray = [
            'runinfo' => [
                'nmaprun'  => $nmapScanArr['@attributes'],
                'runstats' => $runstats,
            ],
            'hosts'   => $hosts
        ];
        return $newNmapScanArray;
    }

    function xml2array($xmlObject, $out = []) {
        foreach ((array)$xmlObject as $index => $node)
            $out[$index] = (is_object($node) || is_array($node)) ? $this->xml2array($node) : $node;
        return $out;
    }

    public function getMatchingTemplate($mappedTemplates, $name, $portid) {
        $newMappedTemplates = [];
        foreach ($mappedTemplates as $mappedTemplate) {
            $mappedTemplate = $mappedTemplate['NmapMapping'];
            $newMappedTemplates[$mappedTemplate['servicetemplate_id']] = [
                'port'         => $mappedTemplate['port'],
                'service_name' => strtolower($mappedTemplate['service_name']),
            ];
        }

        foreach ($newMappedTemplates as $templateId => $template) {
            if ($name == $template['service_name'] && $portid == $template['port']) {

                //exact match
                return $templateId;
            }

            if ($name == $template['service_name'] xor $portid == $template['port']) {
                //either service name or port matched
                return $templateId;
            }
        }
        return null;
    }

    public function getMappingTemplates($id) {
        $this->Mapping = ClassRegistry::init('DiscoveryModule.NmapMapping');
        return $this->Mapping->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'nmap_configuration_id' => $id
            ],
            'fields'     => [
                'port',
                'service_name',
                'servicetemplate_id'
            ]
        ]);
    }
}