<?php
// Copyright (C) <2017>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.


class NmapConfiguration extends DiscoveryModuleAppModel {

    public $hasMany = [
        'NmapMapping' => [
            'className'  => 'NmapMapping',
            'foreignKey' => 'nmap_configuration_id',
            'dependent'  => true,
        ],
        'NmapScan'    => [
            'className'  => 'NmapScan',
            'foreignKey' => 'nmap_configuration_id',
            'dependent'  => true,
        ],
    ];

    public $validate = [
        'name'                    => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true,
            ],
        ],
        'address'                 => [
            'notBlank' => [
                'rule'     => ['ip', 'both'], //ip v4 and v6 validation
                'message'  => 'Please supply a valid IP address.',
                'required' => true,
            ],
        ],
        'hosttemplate_id'         => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true,
            ],
            'numeric'  => [
                'rule'    => 'numeric',
                'message' => 'This field needs to be numeric.',
            ],
            'notZero'  => [
                'rule'     => ['comparison', '>', 0],
                'message'  => 'Invalid hosttemplate.',
                'required' => true,
            ],
        ],
        'default_servicetemplate' => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true,
            ],
            'numeric'  => [
                'rule'    => 'numeric',
                'message' => 'This field needs to be numeric.',
            ],
            'notZero'  => [
                'rule'     => ['comparison', '>', 0],
                'message'  => 'Invalid servicetemplate.',
                'required' => true,
            ],
        ],
        'bitmask'                 => [
            'notBlank' => [
                'rule'    => ['validateBitmask'],
                'message' => 'Bitmask cannot be left blank',
            ],
        ],
        'container_id'            => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true,
            ],
            'numeric'  => [
                'rule'    => 'numeric',
                'message' => 'This field needs to be numeric.',
            ],
            'notZero'  => [
                'rule'     => ['comparison', '>', 0],
                'message'  => 'Invalid container.',
                'required' => true,
            ],
        ],
    ];

    public function validateBitmask($data) {
        return !($this->data['NmapConfiguration']['single_host_scan'] == '0' && empty($data['bitmask']));
    }

}