// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.

App.Controllers.NmapIndexController = Frontend.AppController.extend({

    $ajaxLoader: null,
    discoveryId: null,
    intervalID: null,
    filename: null,
    servicetemplates: null,

    _initialize: function(){
        var self = this;
        this.$ajaxLoader = $('#loader');
        $('#executeNmapDiscovery').click(function(){
            self.discoveryId = $('#NmapDiscoveryResultNmapDiscoveryId').val();
            self.executeNmapDiscovery(self.discoveryId);
        });

        /**
         * Disables all form fields which are empty so we dont passing all input vars back to the server on submit
         */
        $('form').submit(function(e){
            e.preventDefault();
            //disable all empty form fields
            $('.discoveryResultFormField').each(function(i, el){
                $el = $(el);
                if($el.hasClass('servicetemplateChosen')){
                    $el.prop('disabled', true);
                }

                if($el.hasClass('SevicetemplateValue')){
                    if($el.val() == null || $el.val() == ''){
                        $el.parents('tr').find(':input').prop('disabled', true);
                    }

                }
                if($el.hasClass('hostname')){
                    if($el.val() == null || $el.val() == ''){
                        $el.prop('disabled', true);
                    }
                }
            });
            //submit the form
            $('form')[0].submit();
        });
    },

    executeNmapDiscovery: function(discoveryId){
        var self = this;
        //clear flash message
        $('#flash-NmapDiscoveryMsg').removeClass().empty();
        this.$ajaxLoader.html('<center><i class="fa fa-cog fa-4x fa-spin"></i><br /><span>' + this.getVar('executing_nmap_discovery') + '</span></center>');
        this.getDiscoveryResult(function(response){
            var data = $.parseJSON(response.responseText);
            self.intervalID = setInterval(self.checkForFinishedNmapDiscovery, 2000, data, discoveryId, self);
        });

    },

    getDiscoveryResult: function(callback){
        $.ajax({
            //url: "/discovery_module/nmap/discovery/" + encodeURIComponent(this.discoveryId) + ".json",
            url: "/discovery_module/nmap/discovery/" + encodeURIComponent(this.discoveryId),
            //data:
            type: "POST",
            cache: false,
            error: function(){
            },
            success: function(){
            },
            complete: callback
        });
    },

    checkForFinishedNmapDiscovery: function(filename, discoveryId, self){
        $.ajax({
            url: "/discovery_module/nmap/broadcast/" + encodeURIComponent(filename) + '/' + discoveryId,
            //data: '',
            type: "POST",
            cache: false,
            error: function(){
            },
            success: function(){
            },
            complete: function(response){
                try{
                    var data = $.parseJSON(response.responseText);
                    if(data.hasOwnProperty('finished') && data.finished == true){
                        clearInterval(self.intervalID);
                        self.getRenderedFormFields(filename, discoveryId, self);
                    }
                    if(data.hasOwnProperty('stop')){
                        clearInterval(self.intervalID);
                    }
                }catch(e){
                    //@TODO hide ajax loader and print message that there were some errors
                    clearInterval(self.intervalID);
                }

            },
        });
    },

    getRenderedFormFields: function(filename, discoveryId, self){
        $.ajax({
            url: "/discovery_module/nmap/discoveryresult/" + encodeURIComponent(filename) + '/' + discoveryId,
            type: "POST",
            cache: false,
            error: function(){
            },
            success: function(){
            },
            complete: function(response){
                //display Discovery ctp response
                $('#loader').html(response.responseText);
                $('#discoveryResultFormActions').show();
                $('.chosen').chosen();
                $('.changeDefault').change(function(){
                    var identifier = $(this).val();
                    var $id = $('.' + identifier);
                    var previousSelectedId = $('.selected_' + identifier).val();
                    if($(this).prop('checked')){
                        //default shall be changed
                        if(self.servicetemplates == null){
                            //get the servicetemplates
                            $.ajax({
                                url: "/discovery_module/nmap/getServicetemplates/",
                                //data: '',
                                type: "POST",
                                cache: false,
                                error: function(){
                                },
                                success: function(){
                                },
                                complete: function(response){
                                    self.servicetemplates = $.parseJSON(response.responseText);
                                    //not cached
                                    self.buildNewChosenOptions(self.servicetemplates, identifier, previousSelectedId);
                                },
                            });
                        }else{
                            //cached
                            self.buildNewChosenOptions(self.servicetemplates, identifier, previousSelectedId);
                        }
                    }else{
                        //take default again
                        $id.val(previousSelectedId);
                        $id.prop('disabled', true);
                        $id.trigger("chosen:updated");
                    }
                });


                $('.existingBlocker').each(function(){
                    if($(this).val() == 1){
                        $(this).parent().find('td').css({
                            'pointer-events': 'none',
                            'filter': 'grayscale(0.5) blur(2px)',
                        });
                    }
                })
            },
        });
    },

    buildNewChosenOptions: function(servicetemplates, id, previousSelectedId){
        var $id = $('.' + id);
        var html = '';
        for(var i in servicetemplates){
            html += '<option value="' + i + '">' + servicetemplates[i] + '</option>';
        }
        $id.html(html);
        $id.val(previousSelectedId);
        $id.prop('disabled', false);
        $id.trigger("chosen:updated");

        $id.change(function(){
            var val = $(this).val();
            $('.SevicetemplateValue_' + id).val(val);
        });
    }
});
