<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement. 
// The license agreement and license key were sent with the order confirmation.
use itnovum\openITCOCKPIT\Satellite\MappingRepository;

class DistributeModuleNagiosExportTask extends AppShell {

    public $uses = [
        'DistributeModule.Satellite',
        'Timeperiod',
        'Host',
        'Service',
        'Servicetemplate'
    ];

    /*
     * Callback from main NagiosExportTask!
     */
    public function beforeExport() {
        $this->init();
        $this->deleteOldConfiguration();

        $this->createMapping();
    }

    /*
     * Callback from main NagiosExportTask!
     */
    public function afterExport() {
        $this->init();
        $this->copyDefaultConfig();

        return true;
    }

    public function init() {
        App::uses('File', 'Utility');
        App::uses('Folder', 'Utility');
        Configure::load('nagios');
        $this->conf = Configure::read('nagios.export');
        $this->Satellites = $this->Satellite->find('all');
    }


    public function deleteOldConfiguration() {
        if (is_dir($this->conf['satellite_path'])) {
            $dir = new Folder($this->conf['satellite_path']);
            if ($dir->delete()) {
                mkdir($this->conf['satellite_path']);

                foreach ($this->Satellites as $satellite) {
                    if (!is_dir($this->conf['satellite_path'] . $satellite['Satellite']['id'])) {
                        mkdir($this->conf['satellite_path'] . $satellite['Satellite']['id']);
                    }

                    if (!is_dir($this->conf['satellite_path'] . $satellite['Satellite']['id'] . DS . $this->conf['config'])) {
                        mkdir($this->conf['satellite_path'] . $satellite['Satellite']['id'] . DS . $this->conf['config']);
                    }
                }

            }
        }
    }

    public function createMapping() {
        $satellites = $this->Satellite->find('all');
        foreach ($satellites as $satellite) {
            if (!is_dir($this->conf['satellite_path'] . $satellite['Satellite']['id'])) {
                mkdir($this->conf['satellite_path'] . $satellite['Satellite']['id']);
            }

            $MappingRepository = new MappingRepository();

            $hosts = $this->Host->find('all', [
                'recursive'  => -1,
                'fields'     => [
                    'Host.id',
                    'Host.uuid',
                    'Host.name'
                ],
                'conditions' => [
                    'Host.satellite_id' => $satellite['Satellite']['id'],
                    'Host.disabled'     => 0
                ]
            ]);

            foreach ($hosts as $host) {
                $MappingRepository->addItem($host['Host']['uuid'], $host['Host']['name'], OBJECT_HOST);
                $services = $this->Service->find('all', [
                    'recursive'  => -1,
                    'contain'    => [
                        'Servicetemplate' => [
                            'fields' => [
                                'Servicetemplate.name'
                            ]
                        ]
                    ],
                    'fields'     => [
                        'Service.name',
                        'Service.uuid'
                    ],
                    'conditions' => [
                        'Service.host_id'  => $host['Host']['id'],
                        'Service.disabled' => 0
                    ]
                ]);
                foreach ($services as $service) {
                    $serviceName = $service['Service']['name'];
                    if ($serviceName === null || $serviceName === '') {
                        $serviceName = $service['Servicetemplate']['name'];
                    }
                    $MappingRepository->addItem($service['Service']['uuid'], $serviceName, OBJECT_SERVICE);
                }
            }

            $timeperiods = $this->Timeperiod->find('all', [
                'recursive' => -1,
                'fields'    => [
                    'Timeperiod.uuid',
                    'Timeperiod.name'
                ]
            ]);
            foreach ($timeperiods as $timeperiod) {
                $MappingRepository->addItem(
                    $timeperiod['Timeperiod']['uuid'],
                    $timeperiod['Timeperiod']['name'],
                    OBJECT_TIMEPERIOD
                );
            }

            $MappingRepository->toFile($this->conf['satellite_path'] . $satellite['Satellite']['id'] . DS . 'mapping.json');

        }
    }

    public function copyDefaultConfig() {
        $dir2copy = [
            'commands',
            'contactgroups',
            'contacts',
            'hosttemplates',
            'servicetemplates',
        ];

        $defaultFolder = new Folder(Configure::read('nagios.basepath') . Configure::read('nagios.etc') . Configure::read('nagios.export.config') . 'defaults/');
        $rolloutFolder = new Folder($this->conf['rollout']);

        foreach ($this->Satellites as $satellite) {
            if (is_dir($this->conf['satellite_path'] . $satellite['Satellite']['id'])) {

                //Copy default settings like generic host, timeperiod and stuff
                $defaultFolder->copy($this->conf['satellite_path'] . $satellite['Satellite']['id'] . DS . $this->conf['config'] . 'defaults/');

                foreach ($dir2copy as $dir) {
                    if (is_dir($this->conf['path'] . $this->conf[$dir])) {
                        $sourceFolder = new Folder($this->conf['path'] . $this->conf[$dir]);
                        if (!is_dir($this->conf['satellite_path'] . $satellite['Satellite']['id'] . DS . $this->conf[$dir])) {
                            mkdir($this->conf['satellite_path'] . $satellite['Satellite']['id'] . DS . $this->conf[$dir]);
                        }
                        $sourceFolder->copy($this->conf['satellite_path'] . $satellite['Satellite']['id'] . DS . $this->conf[$dir]);
                    }
                }
            }

            //Copy files from rolloit folder to sat config folder
            $rolloutFolder->copy($this->conf['satellite_path'] . $satellite['Satellite']['id'] . DS);

        }
    }

}