<?php

use itnovum\openITCOCKPIT\Filter\SatelliteFilter;

/**
 * // Copyright (C) <2017>  <it-novum GmbH>
 * //
 * // This file is dual licensed
 * //
 * // 1.
 * //    This program is free software: you can redistribute it and/or modify
 * //    it under the terms of the GNU General Public License as published by
 * //    the Free Software Foundation, version 3 of the License.
 * //
 * //    This program is distributed in the hope that it will be useful,
 * //    but WITHOUT ANY WARRANTY; without even the implied warranty of
 * //    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * //    GNU General Public License for more details.
 * //
 * //    You should have received a copy of the GNU General Public License
 * //    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * //
 * // 2.
 * //    If you purchased an openITCOCKPIT Enterprise Edition you can use this file
 * //    under the terms of the openITCOCKPIT Enterprise Edition license agreement.
 * //    License agreement and license key will be shipped with the order
 * //    confirmation.
 */
class SatellitesController extends DistributeModuleAppController {
    public $layout = 'Admin.default';

    public $components = [
        'Paginator',
        'RequestHandler',
        'AdditionalLinks',
        'Tree',
        'ListFilter.ListFilter'
    ];

    public $helpers = [
        'ListFilter.ListFilter'
    ];

    public $listFilters = [
        'index' => [
            'fields' => [
                'Satellite.name'    => ['label' => 'Name', 'searchType' => 'wildcard'],
                'Satellite.address' => ['label' => 'Address', 'searchType' => 'wildcard'],
            ],
        ]
    ];

    public function index() {
        $this->layout = 'angularjs';
        if (!$this->isApiRequest()) {
            //Only ship template for AngularJs
            return;
        }
        $SatelliteFilter = new SatelliteFilter($this->request);

        $options = [
            'recursive'  => -1,
            'order'      => $SatelliteFilter->getOrderForPaginator('Satellite.name', 'asc'),
            'conditions' => [
                'Satellite.container_id' => $this->MY_RIGHTS,
                $SatelliteFilter->indexFilter()

            ],
        ];

        $query = Hash::merge($this->Paginator->settings, $options);
        if ($this->isApiRequest()) {
            unset($query['limit']);
            $_all_satellites = $this->Satellite->find('all', $query);
        } else {
            $this->Paginator->settings = Hash::merge($this->Paginator->settings, $options);
            $_all_satellites = $this->Paginator->paginate();
        }

        if ($this->isApiRequest() && !$this->isAngularJsRequest()) {
            unset($options['limit']);
            $_all_satellites = $this->Satellite->find('all', $query);
        } else {
            $this->Paginator->settings = $options;
            $this->Paginator->settings['page'] = $SatelliteFilter->getPage();
            $_all_satellites = $this->Paginator->paginate();
        }


        //Add Container path to the array
        $all_satellites = [];
        foreach ($_all_satellites as $satellite) {
            $satellite['Satellite']['container'] = $this->Tree->treePath($satellite['Satellite']['container_id'], ['delimiter' => '/']);
            $all_satellites[] = $satellite;
        }

        $this->set([
            'satellites' => $all_satellites
        ]);
        $this->set('_serialize', ['satellites', 'paging']);
    }

    public function add() {
        if ($this->request->is('post') || $this->request->is('put')) {
            $isJsonRequest = $this->request->ext === 'json';
            if ($this->Satellite->save($this->request->data)) {
                if ($isJsonRequest) {
                    $this->serializeId();

                    return;
                } else {
                    $this->setFlash(__('<a href="/distribute_module/satellites/edit/%s">Satellite</a> created successfully', $this->Satellite->id));
                    $this->redirect(['action' => 'index']);
                }
            }

            if ($isJsonRequest) {
                $this->serializeErrorMessage();

                return;
            } else {
                $this->setFlash(__('Satellite could not be saved'), false);
            }
        }

        if ($this->hasRootPrivileges === true) {
            $containers = $this->Tree->easyPath($this->MY_RIGHTS, OBJECT_SATELLITE, [], $this->hasRootPrivileges);
        } else {
            $containers = $this->Tree->easyPath($this->getWriteContainers(), OBJECT_SATELLITE, [], $this->hasRootPrivileges);
        }

        $timeZones = $this->getTimeZonesList();

        $this->set('back_url', $this->referer());
        $this->set(compact(['containers', 'timeZones']));
    }

    public function edit($id = null) {
        if (!$this->Satellite->exists($id)) {
            throw new NotFoundException(__('Invalid satellite'));
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            if ($this->Satellite->save($this->request->data)) {
                if ($this->isApiRequest()) {
                    $this->serializeId();

                    return;
                }
                $this->setFlash(__('<a href="/distribute_module/satellites/edit/%s">Satellite</a> successfully saved', $this->Satellite->id));
                $this->redirect(['action' => 'index']);
            }

            if ($this->isApiRequest()) {
                $this->serializeErrorMessage();

                return;
            }
            $this->setFlash(__('Satellite could not be saved'), false);
        }

        $satellite = $this->Satellite->findById($id);

        if (!$this->allowedByContainerId(Hash::extract($satellite, 'Satellite.container_id'))) {
            $this->render403();

            return;
        }

        if ($this->hasRootPrivileges === true) {
            $containers = $this->Tree->easyPath($this->MY_RIGHTS, OBJECT_SATELLITE, [], $this->hasRootPrivileges);
        } else {
            $containers = $this->Tree->easyPath($this->getWriteContainers(), OBJECT_SATELLITE, [], $this->hasRootPrivileges);
        }

        $timeZones = $this->getTimeZonesList();

        $this->set('back_url', $this->referer());
        $this->set(compact([
            'satellite',
            'containers',
            'timeZones',
        ]));
    }

    public function usedBy($id) {
        if (!$this->Satellite->exists($id)) {
            throw new NotFoundException(__('Invalid satellite'));
        }
        $satellite = $this->Satellite->findById($id);
        $this->loadModel('Host');
        $all_hosts = $this->Host->find('all', [
            'recursive'  => -1,
            'contain'    => [],
            'order'      => [
                'Host.name' => 'ASC',
            ],
            'joins'      => [
                [
                    'table'      => 'hosts_to_containers',
                    'alias'      => 'HostsToContainers',
                    'type'       => 'LEFT',
                    'conditions' => [
                        'HostsToContainers.host_id = Host.id',
                    ],
                ],
            ],
            'conditions' => [
                'HostsToContainers.container_id' => $this->MY_RIGHTS,
                'Host.satellite_id'              => $id,
            ],
        ]);
        $this->set(compact([
            'satellite',
            'all_hosts',
        ]));
        $this->set('back_url', $this->referer());
    }

    public function delete($id) {
        $userId = $this->Auth->user('id');
        if (!$this->Satellite->exists($id)) {
            throw new NotFoundException(__('Invalid satellite'));
        }

        if (!$this->request->is('post') && !$this->request->is('delete')) {
            throw new MethodNotAllowedException();
        }

        $this->loadModel('Host');
        $satellite = $this->Satellite->findById($id);
        $hosts = $this->Host->find('all', [
            'conditions' => [
                'Host.satellite_id' => $id,
            ],
        ]);
        $satelliteDataDeleted = $this->Host->__deleteBySatellite($satellite['Satellite']['id'], $userId);
        if ($satelliteDataDeleted['success'] && $this->Satellite->delete($satellite['Satellite']['id'])) {
            $this->setFlash(__('Satellite deleted'));
            $this->redirect(['action' => 'index']);
        } else {
            $this->setFlash($satelliteDataDeleted['success'] ? __('Could not delete satellite') : $satelliteDataDeleted['message']);
            $this->redirect(['action' => 'index']);
        }

    }

    /**
     * @return array list of time zones
     */
    protected function getTimeZonesList() {
        $zones = timezone_identifiers_list();

        foreach ($zones as $zone) {
            $zone = explode('/', $zone); // 0 => Continent, 1 => City

            // Only use "friendly" continent names
            if ($zone[0] == 'Africa' || $zone[0] == 'America' || $zone[0] == 'Antarctica' || $zone[0] == 'Arctic' || $zone[0] == 'Asia' || $zone[0] == 'Atlantic' || $zone[0] == 'Australia' || $zone[0] == 'Europe' || $zone[0] == 'Indian' || $zone[0] == 'Pacific') {
                if (isset($zone[1]) != '') {
                    $locations[$zone[0]][$zone[0] . '/' . $zone[1]] = str_replace('_', ' ', $zone[1]); // Creates array(DateTimeZone => 'Friendly name')
                }
            }
        }

        return $locations;
    }

    public function name() {
        $this->layout = 'blank';

        if (!$this->isAngularJsRequest()) {
            //Only ship template
            return;
        }

        $id = $this->request->query('id');

        if (!$this->Satellite->exists($id)) {
            throw new NotFoundException(__('Invalid satellite'));
        }

        $satellite = $this->Satellite->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'Satellite.id' => $id
            ]
        ]);

        $this->set('satellite', $satellite);
        $this->set('_serialize', ['satellite']);
    }
}
