<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
class Satellite extends DistributeModuleAppModel {
    public $validate = [
        'name'    => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true,
            ],
            'isUnique' => [
                'rule'    => 'isUnique',
                'message' => 'This instance name has already been taken.',
            ],
        ],
        'address' => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true,
            ],
            'url'      => [
                'rule'    => 'url',
                'message' => 'Please enter a valid IPv4 address or hostname',
            ],
        ],
    ];

    public function __delete($satellite, $userId) {
        if (is_numeric($satellite)) {
            $satelliteId = $satellite;
            $satellite = $this->findById($satelliteId);
        } else {
            $satelliteId = $satellite['Satellite']['id'];
        }

        $Container = ClassRegistry::init('Container');
        $Host = ClassRegistry::init('Host');
        if ($this->delete($satelliteId)) {
            //Delete all hosts that are inside of this container
            $hosts = $Host->find('all', [
                'conditions' => [
                    'Host.satellite_id' => $satelliteId,
                ],
            ]);

            foreach ($hosts as $host) {
                $Host->__delete($host, $userId);
            }

            return true;
        }

        return false;
    }

    /**
     * @param array $instaceIds
     */
    public function saveInstancesForConfigSync($instaceIds = []) {
        foreach ($instaceIds as $instanceId) {
            if ($this->exists($instanceId)) {
                $this->id = $instanceId;
                $this->saveField('sync_instance', 1);
            }
        }
    }

    public function disableAllInstanceConfigSyncs() {
        $instacnes = $this->find('all', [
            'recursive' => -1,
            'fields'    => [
                'id',
            ]
        ]);
        foreach ($instacnes as $instace) {
            $this->id = $instace['Satellite']['id'];
            $this->saveField('sync_instance', 0);
        }
    }

}
