<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.


///usr/share/openitcockpit/app/Console/cake EventcorrelationModule.evc_plugin $HOSTNAME$

use itnovum\openITCOCKPIT\Core\DbBackend;
use itnovum\openITCOCKPIT\Core\ServicestatusFields;

class EvcPluginShell extends AppShell {


    public $uses = [
        'EventcorrelationModule.Eventcorrelation',
        'EventcorrelationModule.EventcorrelationSetting',
        'Host',
        'Service',
        'Servicetemplate',
        MONITORING_SERVICESTATUS,
        MONITORING_OBJECTS,
        MONITORING_EXTERNALCOMMAND,
    ];

    const OK = 0;
    const WARNING = 1;
    const CRITICAL = 2;
    const UNKNOWN = 3;

    public function main() {
        $eventcorrelation_settings = $this->EventcorrelationSetting->currentEventcorrelationSettings();
        $default_eventcorrelation_settings = $this->EventcorrelationSetting->defaultEventcorrelationSettings();
        $considerStateType = ($default_eventcorrelation_settings['EVC_CONSIDER_STATETYPE']['value'] & $eventcorrelation_settings['EventcorrelationSetting']['configuration_option']);

        $monitoringSystem = 'naemon';
        if (!isset($this->args[0])) {
            $this->out('parameter uuid is missing');
            exit(2);
        }

        $DbBackend = new DbBackend();

        $longOutputArray = [];
        $evcServices = $this->getEvcTree($this->args[0]);
        $rootServiceState = self::UNKNOWN;
        $rootServiceFullName = null;
        $serviceName = null;

        if (!empty($evcServices)) {
            $evcServices = $this->reorganizeEvcStructure($evcServices);
            $evcServices = array_reverse($evcServices);
            $tmpResult = [];

            $ServicestatusFields = new ServicestatusFields($DbBackend);
            $ServicestatusFields
                ->currentState()
                ->isHardstate()
                ->lastStateChange()
                ->output();

            foreach ($evcServices as $layerNumber => $evcData) {
                foreach ($evcData as $evcId => $evcChilds) {
                    foreach ($evcChilds as $key => $evc) {
                        $serviceOutput = '';
                        if ($layerNumber === 0) {
                            $currentState = self::UNKNOWN;
                            $tmpServicestatus = $this->Servicestatus->byUuid($evc['Service']['uuid'], $ServicestatusFields);
                            if (!empty($tmpServicestatus)) {
                                $Servicestatus = new \itnovum\openITCOCKPIT\Core\Servicestatus($tmpServicestatus['Servicestatus']);
                                $serviceOutput = ' OUTPUT:' . $Servicestatus->getOutput();
                                $currentState = $Servicestatus->currentState();
                                if ($considerStateType && !$Servicestatus->isHardstate()) {
                                    $currentState = $Servicestatus->getLastHardState();
                                }
                            }
                            $tmpResult[$evcId][] = (int)$currentState;
                            if (isset($evc['Service']['Host']['name'])) {
                                $serviceName = $evc['Service']['Host']['name'] . '/' . (($evc['Service']['name']) ? $evc['Service']['name'] : $evc['Service']['Servicetemplate']['name']);
                                $longOutputArray[$evcId][$currentState][] = $serviceName . $serviceOutput;
                            }
                        } else {
                            $evcResult = $this->currentEvcState($evc['Eventcorrelation']['operator'], $tmpResult[$evc['Eventcorrelation']['id']], ($default_eventcorrelation_settings['EVC_CONSIDER_STATE_COUNT']['value']
                                & $eventcorrelation_settings['EventcorrelationSetting']['configuration_option']));
                            $tmpResult[$evcId][] = $evcResult['state'];
                            $tmpServicestatus = $this->Servicestatus->byUuid($evc['Service']['uuid'], $ServicestatusFields);
                            if (!empty($tmpServicestatus)) {
                                $Servicestatus = new \itnovum\openITCOCKPIT\Core\Servicestatus($tmpServicestatus['Servicestatus']);
                                $serviceOutput = ' OUTPUT:' . $Servicestatus->getOutput();
                            }
                            if (isset($evc['Service']['Host']['name'])) {
                                $serviceName = $evc['Service']['Host']['name'] . '/' . (($evc['Service']['name']) ? $evc['Service']['name'] : $evc['Service']['Servicetemplate']['name']);
                                $longOutputArray[$evcId][$evcResult['state']][] = $serviceName . $serviceOutput;
                            }
                            ksort($longOutputArray[$evc['Eventcorrelation']['id']]);
                            $checkData = [
                                'hostUuid'    => $evc['Service']['Host']['uuid'],
                                'serviceUuid' => $evc['Service']['uuid'],
                                'state'       => $evcResult['state'],
                                'comment'     => $evcResult['output'] . $this->parseLongOutput($longOutputArray[$evc['Eventcorrelation']['id']]) . '|state=' . $evcResult['state'] . ';1;2;0;3' . PHP_EOL,
                            ];
                            if ($eventcorrelation_settings['EventcorrelationSetting']['monitoring_system'] === 'naemon') {
                                $command = "#command run ";
                                $command .= '[' . time() . '] PROCESS_SERVICE_CHECK_RESULT;' . implode(';', $checkData);
                                $command .= "\0";
                                $nagiosQH = fsockopen('unix:///opt/openitc/nagios/var/rw/nagios.qh', null, $errno, $errstr, 5);
                                fwrite($nagiosQH, $command);
                                $response = trim(stream_get_contents($nagiosQH));
                                fclose($nagiosQH);
                            } else {
                                $this->Externalcommand->passiveTransferServiceCheckresult($checkData);
                            }
                            $rootServiceState = $evcResult['state'];
                            $rootServiceFullName = $serviceName;

                        }
                    }
                }
            }
            if(($default_eventcorrelation_settings['EVC_REFLECT_CRITICAL_STATE']['value'] & $eventcorrelation_settings['EventcorrelationSetting']['configuration_option'])
                && ($rootServiceState === self::CRITICAL)){
                echo 'EVC Check - '.$rootServiceFullName.' is critical!';
                exit(1);
            }
        }
        echo 'EVC Check ';
        exit(0);
    }

    public function getOptionParser() {
        $parser = parent::getOptionParser();
        /*
        $parser->addArgument('id', [
                'help' => 'Event correlation host id',
                'required' => true
            ]
        )->description(__('This program will correlate service states with different operators'));
        */
        $parser->addOptions([
            'uuid' => ['help' => 'Event correlation host uuid'],
        ]);

        return $parser;
    }

    public function _welcome() {
        //Disable CakePHP welcome messages
    }

    public function getEvcTree($hostUuid = null) {
        $evcHost = $this->Host->find('list', [
            'conditions' => [
                'Host.uuid' => $hostUuid,
            ],
        ]);

        return $this->Eventcorrelation->find(
            'threaded', [
                'recursive'  => -1,
                'contain'    => [
                    'Service' => [
                        'fields'          => [
                            'Service.name',
                            'Service.uuid',
                        ],
                        'Host'            => [
                            'fields' => [
                                'Host.uuid',
                                'Host.name',
                            ],
                        ],
                        'Servicetemplate' => [
                            'fields' => [
                                'Servicetemplate.name',
                            ],
                        ],
                    ],
                ],
                'conditions' => [
                    'Eventcorrelation.host_id' => array_keys($evcHost),
                ],
            ]
        );
    }

    public function reorganizeEvcStructure($evcAsNest) {
        $result = [];
        function flattenElement($children, $layer, &$result) {
            foreach ($children as $element) {
                $data = [
                    'Eventcorrelation' => $element['Eventcorrelation'],
                    'Service'          => $element['Service'],
                ];
                $result[$layer][$element['Eventcorrelation']['parent_id']][] = $data;
            }
            foreach ($children as $element) {
                flattenElement($element['children'], $layer + 1, $result);
            }
        }

        flattenElement($evcAsNest, 1, $result);

        return $result;
    }

    public function currentEvcState($operator, $stateArray = [], $considerStateCount = false) {
        $evcState = [
            'state'  => self::UNKNOWN,
            'output' => __('0 services are OK'),
        ];
        $stateCounter = sizeof($stateArray);
        $stateSummary = array_count_values(Hash::filter($stateArray));
        if (empty($stateSummary)) {
            return $evcState;
        }
        $operatorDetails = (strpos($operator, 'min') === 0) ? array_combine(
            ['operator', 'operatorModifier'],
            array_filter(preg_split('/(\d+)/', $operator, -1, PREG_SPLIT_DELIM_CAPTURE))) : ['operator' => $operator];
        switch ($operatorDetails['operator']) {
            case 'and':
                if (isset($stateSummary[0]) && ($stateSummary[0] === $stateCounter)) {
                    $evcState['state'] = self::OK;

                } else {
                    $evcState['state'] = ($considerStateCount) ? $this->checkResultConsiderStatePriority($stateSummary) : max(array_keys($stateSummary));
                }
                break;
            case 'or':
                if (isset($stateSummary[0])) {
                    $evcState['state'] = self::OK;
                } else {
                    $evcState['state'] = ($considerStateCount) ? $this->checkResultConsiderStatePriority($stateSummary) : max(array_keys($stateSummary));
                }
                break;
            case 'min':
                if (isset($stateSummary[0]) && ($stateSummary[0] >= $operatorDetails['operatorModifier'])) {
                    $evcState['state'] = self::OK;
                } else {
                    $evcState['state'] = ($considerStateCount) ? $this->checkResultConsiderStatePriority($stateSummary) : max(array_keys($stateSummary));
                }
                break;
            case 'eq':
                if ($stateCounter === 1 && !empty($stateSummary)) {
                    $evcState['state'] = $stateArray[0];
                }
                break;
        }
        $evcState['output'] = __(strtoupper($operator) . ' - %s services (%s from %s services are OK)',
            (isset($stateSummary[0])) ? $stateSummary[0] : 0,
            (isset($stateSummary[0])) ? $stateSummary[0] : 0,
            $stateCounter
        );

        return $evcState;
    }

    public function parseLongOutput($stateSummary = []) {
        $longoutput = "";
        $stateArray = ['OK', 'WARNING', 'CRITICAL', 'UNKNOWN'];
        foreach ($stateSummary as $state => $values) {
            $longoutput .= "\n" . $stateArray[$state] . ": " . implode(", ", $values);
        }

        return $longoutput;
    }

    public function checkResultConsiderStatePriority($stateSummary = []) {
        $state = self::UNKNOWN;
        if (empty($stateSummary)) {
            return $state;
        }
        if ((isset($stateSummary[self::CRITICAL], $stateSummary[self::UNKNOWN]) && $stateSummary[self::CRITICAL] >= $stateSummary[self::UNKNOWN]) || !isset($stateSummary[self::UNKNOWN])) {
            $state = self::CRITICAL;
        } else if (isset($stateSummary[self::UNKNOWN], $stateSummary[self::WARNING]) && $stateSummary[self::UNKNOWN] > $stateSummary[self::WARNING]) {
            $state = self::UNKNOWN;
        } else if (isset($stateSummary[self::WARNING], $stateSummary[self::UNKNOWN]) && !isset($stateSummary[self::CRITICAL]) && $stateSummary[self::WARNING] >= $stateSummary[self::UNKNOWN]) {
            $state = self::WARNING;
        }

        return $state;
    }
}
