<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
use itnovum\openITCOCKPIT\Core\ServiceConditions;
use itnovum\openITCOCKPIT\Core\ServicestatusFields;
use itnovum\openITCOCKPIT\Core\Views\UserTime;
use itnovum\openITCOCKPIT\Filter\ServiceFilter;

/**
 * Class EventcorrelationsController
 * @property Eventcorrelation Eventcorrelation
 * @property Host Host
 * @property Service Service
 * @property Container Container
 * @property Hosttemplate Hosttemplate
 * @property Servicetemplate Servicetemplate
 */
class EventcorrelationsController extends EventcorrelationModuleAppController {

    public $layout = 'Admin.default';

    public $components = [
        'ListFilter.ListFilter',
        'RequestHandler',
        'CustomValidationErrors',
        'Tree',
    ];
    public $helpers = [
        'CustomValidationErrors',
        'ListFilter.ListFilter',
        'Status',
    ];

    public $uses = [
        'EventcorrelationModule.Eventcorrelation',
        'EventcorrelationModule.EventcorrelationSetting',
        'Host',
        'Service',
        'Container',
        'Hosttemplate',
        'Servicetemplate',
        MONITORING_SERVICESTATUS,
        MONITORING_OBJECTS,
    ];

    public $listFilters = [
        'index' => [
            'fields' => [
                'Host.name'        => [
                    'label'      => 'Name',
                    'searchType' => 'wildcard',
                ],
                'Host.description' => [
                    'label'      => 'Description',
                    'searchType' => 'wildcard',
                ],
            ],
        ]
    ];

    public function index() {
        $conditions = [
            'Host.host_type'                 => EVK_HOST,
            'Host.disabled'                  => 0,
            'HostsToContainers.container_id' => $this->MY_RIGHTS
        ];
        if (isset($this->Paginator->settings['conditions'])) {
            $conditions = Hash::merge($this->Paginator->settings['conditions'], $conditions);
        }

        $query = [
            'recursive'  => -1,
            'joins'      => [
                [
                    'table'      => 'hosts_to_containers',
                    'alias'      => 'HostsToContainers',
                    'type'       => 'LEFT',
                    'conditions' => [
                        'HostsToContainers.host_id = Host.id',
                    ],
                ],
            ],
            'conditions' => $conditions,
            'fields'     => [
                'Host.id',
                'Host.name',
                'Host.description',
                'Host.container_id'
            ],
            'order'      => [
                'Host.name' => 'asc'
            ]
        ];
        $this->Paginator->settings = array_merge($this->Paginator->settings, $query);
        $all_evc_hosts = $this->Paginator->paginate('Host');


        $firstLayerHosts = $this->Eventcorrelation->find('all', [
            'recursive'  => -1,
            'contain'    => [
                'Service' => [
                    'fields' => [],
                    'Host'   => [
                        'Container' => [
                            'fields' => [
                                'Container.id'
                            ]
                        ],
                        'fields'    => [
                            'Host.id'
                        ]
                    ]
                ]
            ],
            'conditions' => [
                'Eventcorrelation.operator IS NULL'
            ]
        ]);

        $usedContainersByEvcHost = [];
        foreach ($firstLayerHosts as $firstLayerHost) {
            $evcHostId = $firstLayerHost['Eventcorrelation']['host_id'];
            $firstLayerHostId = (int)$firstLayerHost['Service']['host_id'];
            if (isset($firstLayerHost['Service']['Host']['Container'])) {
                foreach (Hash::extract($firstLayerHost['Service']['Host']['Container'], '{n}.id') as $containerId) {
                    $usedContainersByEvcHost[$evcHostId][$firstLayerHostId][$containerId] = $containerId;
                }
            }
        }


        $all_evc_hosts = Hash::combine($all_evc_hosts, '{n}.Host.id', '{n}');
        $allEvcHostsForUsedBy = $this->Host->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'Host.id' =>
                    array_keys($all_evc_hosts)
            ],
            'fields'     => [
                'Host.id',
                'Host.name',
                'Host.description',
            ],
        ]);
        $allEvcHostIds = Hash::extract($allEvcHostsForUsedBy, '{n}.Host.id');
        $all_evc_hosts = Hash::insert($all_evc_hosts, '{n}.Host.usedBy', []);
        $additionalDetails = $this->Eventcorrelation->inUse(Hash::extract($allEvcHostsForUsedBy, '{n}.Host.id'));
        foreach ($additionalDetails as $additionalDetail) {
            if (in_array($additionalDetail['EvcHost']['id'], $allEvcHostIds) && !empty($all_evc_hosts[$additionalDetail['Host']['id']])) {
                $all_evc_hosts[$additionalDetail['Host']['id']]['Host']['usedBy'][$additionalDetail['Eventcorrelation']['host_id']] = $additionalDetail['EvcHost']['name'];
            }
        }
        $this->set(compact('all_evc_hosts', 'usedContainersByEvcHost'));
        $this->set('_serialize', ['all_evc_hosts']);
    }


    public function add() {
        $hosttemplates = $this->Hosttemplate->find('list', [
            'conditions' => [
                'Hosttemplate.hosttemplatetype_id' => EVK_HOSTTEMPLATE,
                'Hosttemplate.container_id'        => $this->MY_RIGHTS,
            ],
        ]);

        $containers = $this->Tree->easyPath($this->MY_RIGHTS, OBJECT_HOST, [], $this->hasRootPrivileges, [CT_HOSTGROUP]);

        $this->set(compact('containers', 'hosttemplates'));
        App::uses('UUID', 'Lib');
        $evkHostPseudoData = [
            'Host' => [
                'address'   => '127.1.2.3',
                'host_type' => EVK_HOST,
                'uuid'      => UUID::v4(),
            ],
        ];
        if ($this->request->is('post') || $this->request->is('put')) {
            $this->request->data = Hash::merge($this->request->data, $evkHostPseudoData);

            $this->request->data['Container']['Container'] = $this->request->data['Host']['container_id'];
            //$dataToSave = $this->Eventcorrelation->prepareForSave($this->request->data);
            $dataToSave = $this->request->data;

            //set usage flag for every new evc host
            $dataToSave['Host']['usage_flag'] = EVENTCORRELATION_MODULE;

            if ($this->Host->save($dataToSave)) {
                $this->setFlash(__('Event correlation successfully saved'));
                $this->redirect(['action' => 'index']);
            } else {
                $this->setFlash(__('could not save data'), false);
            }
        }
    }

    public function view($id = null) {
        $hostCount = $this->Host->find('count', [
            'conditions' => [
                'Host.id'        => $id,
                'Host.host_type' => EVK_HOST,
            ],
        ]);

        if (!$hostCount) {
            throw new NotFoundException(__('Invalid event correlation host'));
        }

        $evcHost = $this->Host->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Host.id',
                'Host.name',
            ],
            'conditions' => [
                'Host.id' => $id
            ]
        ]);

        //Event correlation settings
        $eventcorrelation_settings = $this->EventcorrelationSetting->currentEventcorrelationSettings();
        $default_eventcorrelation_settings = $this->EventcorrelationSetting->defaultEventcorrelationSettings();
        $considerStateType = ($default_eventcorrelation_settings['EVC_CONSIDER_STATETYPE']['value'] & $eventcorrelation_settings['EventcorrelationSetting']['configuration_option']);

        $evcHosts = $this->Eventcorrelation->find('all', [
            'recursive'  => -1,
            'contain'    => [
                'Service' => [
                    'fields' => [
                        'Service.host_id'
                    ]
                ]
            ],
            'conditions' => [
                'Eventcorrelation.host_id' => $id,
            ]
        ]);

        $containerIds = $this->Tree->resolveChildrenOfContainerIds($this->MY_RIGHTS);
        $hosts = $this->Host->hostsByContainerId($containerIds, 'list', [
            'Host.disabled' => [0, 1],
            'Host.id'       => array_unique(Hash::extract($evcHosts, '{n}.Service.host_id'))
        ]);


        $rootContainer = $this->Eventcorrelation->find('first', [
            'recursive'  => -1,
            'contain'    => [
                'Host' => [
                    'fields' => [
                        'Host.id',
                        'Host.name',
                    ],
                ],
            ],
            'conditions' => [
                'Eventcorrelation.host_id' => $id,
            ],
        ]);
        $evcTree = [];
        if (!empty($rootContainer)) {
            $evcTree = $this->Eventcorrelation->getEvcTreeData($id, $hosts);
        }
        $serviceIds = Hash::extract($evcTree, '{n}.{*}.{n}.Service.id');
        //ContainerID => 1 for ROOT Container
        $userContainerIds = $this->Tree->resolveChildrenOfContainerIds($this->MY_RIGHTS);
        $UserTime = new UserTime($this->Auth->user('timezone'), $this->Auth->user('dateformat'));
        $services = $this->Service->find('all', [
            'recursive'  => -1,
            'contain'    => [
                'Host'            => [
                    'fields' => [
                        'Host.id',
                        'Host.name',
                        'Host.uuid'
                    ]
                ],
                'Servicetemplate' => [
                    'fields' => [
                        'Servicetemplate.id',
                        'Servicetemplate.name',
                        'Servicetemplate.description'
                    ]
                ]
            ],
            'conditions' => [
                'Service.id' => $serviceIds,
            ],
            'fields'     => [
                'Service.id',
                'Service.uuid',
                'Service.name',
                'Service.description',
                'Service.service_type',
                'Service.disabled',
                'IF((Service.name IS NULL OR Service.name = ""), Servicetemplate.name, Service.name) AS ServiceName',
                'IF((Service.description IS NULL OR Service.description = ""), Servicetemplate.description, Service.description) AS ServiceDescription'
            ]
        ]);
        $serviceUuids = Hash::extract($services, '{n}.Service.uuid');


        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields
            ->currentState()
            ->lastHardState()
            ->isHardstate()
            ->output()
            ->longOutput()
            ->lastStateChange()
            ->lastCheck()
            ->nextCheck()
            ->currentCheckAttempt()
            ->maxCheckAttempts()
            ->activeChecksEnabled();

        $servicestates = $this->Servicestatus->byUuid($serviceUuids, $ServicestatusFields);
        //@TODO remove vservices from its own correlation (and maybe the real services which are on this correlation)
        //build up a service list for the editor
        $servicesList = [];
        foreach ($services as $service) {
            $currentUuid = $service['Service']['uuid'];
            $Servicestatus = new \itnovum\openITCOCKPIT\Core\Servicestatus([], $UserTime);
            //check if the current service has a state
            if (!empty($servicestates[$currentUuid])) {
                $Servicestatus = new \itnovum\openITCOCKPIT\Core\Servicestatus($servicestates[$currentUuid]['Servicestatus'], $UserTime);
            }
            $currentState = $Servicestatus->currentState();

            if ($considerStateType && !$Servicestatus->isHardState()) {
                $currentState = $Servicestatus->getLastHardState();
            }

            //build service list
            $servicesList[$service['Service']['id']] = [
                'hostId'              => $service['Host']['id'],
                'hostName'            => $service['Host']['name'],
                'hostUuid'            => $service['Host']['uuid'],
                'serviceId'           => $service['Service']['id'],
                'serviceDisabled'     => $service['Service']['disabled'],
                'serviceName'         => $service[0]['ServiceName'],
                'serviceDescription'  => $service[0]['ServiceDescription'],
                'currentState'        => $currentState,
                'output'              => $Servicestatus->getOutput(),
                'longOutput'          => $Servicestatus->getLongOutput(),
                'lastStateChange'     => $UserTime->format($Servicestatus->getLastStateChange()),
                'lastCheck'           => $UserTime->format($Servicestatus->getLastCheck()),
                'nextCheck'           => $UserTime->format($Servicestatus->getNextCheck()),
                'currentCheckAttempt' => $Servicestatus->getCurrentCheckAttempt(),
                'maxCheckAttempts'    => $Servicestatus->getMaxCheckAttempts(),
                'activeChecksEnabled' => $Servicestatus->isActiveChecksEnabled(),
                'serviceUuid'         => $service['Service']['uuid'],
                'serviceType'         => $service['Service']['service_type'],
                'Servicetemplate'     => [
                    'id'   => $service['Servicetemplate']['id'],
                    'name' => $service['Servicetemplate']['name'],
                ],
            ];
        }

        $services = Hash::combine($services, ['%s|%s', '{n}.Host.uuid', '{n}.Service.uuid'], ['%s/%s', '{n}.Host.name', '{n}.{n}.ServiceName'], '{n}.Host.name');
        $servicetemplates = $this->Servicetemplate->servicetemplatesByContainerId($this->MY_RIGHTS, 'list', EVK_SERVICE);
        $evcName = $evcHost['Host']['name'];
        $evcHostId = $evcHost['Host']['id'];
        $this->Frontend->setJson('evcTree', $evcTree);
        $this->Frontend->setJson('hostName', $evcHost['Host']['name']);
        $this->Frontend->setJson('hostId', $evcHost['Host']['id']);
        $this->Frontend->setJson('servicesList', $servicesList);
        $this->set(compact(['services', 'servicetemplates', 'evcName', 'evcHostId']));
        $this->set('_serialize', ['services', 'servicetemplates', 'evcName', 'evcHostId']);
    }

    public function edit($id = null) {
        $userId = $this->Auth->user('id');
        if (!$this->Host->exists($id)) {
            throw new NotFoundException(__('Invalid event correlation'));
        }
        $hosttemplates = $this->Hosttemplate->find('list', [
            'conditions' => [
                'Hosttemplate.hosttemplatetype_id' => EVK_HOSTTEMPLATE,
                'Hosttemplate.container_id'        => $this->MY_RIGHTS,
            ],
        ]);

        $eventCorrelationHost = $this->Host->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'Host.id' => $id
            ]
        ]);
        $containers = $this->Tree->easyPath($this->MY_RIGHTS, OBJECT_HOST, [], $this->hasRootPrivileges, [CT_HOSTGROUP]);
        $this->set(compact('containers', 'hosttemplates', 'eventCorrelationHost'));

        if ($this->request->is('post') || $this->request->is('put')) {
            $this->request->data = Hash::merge($eventCorrelationHost, $this->request->data);
            $this->request->data['Container']['Container'] = $this->request->data['Host']['container_id'];
            $this->Host->id = $id;
            if ($this->Host->save($this->request->data)) {
                $this->setFlash(__('Event correlation successfully saved'));
                $this->redirect(['action' => 'index']);
            } else {
                $this->setFlash(__('could not save data'), false);
            }
        }
    }

    public function editCorrelation($id = null, $highlightedHostId = null) {

        $hostCount = $this->Host->find('count', [
            'conditions' => [
                'Host.id'        => $id,
                'Host.host_type' => EVK_HOST,
            ],
        ]);

        if (!$hostCount) {
            throw new NotFoundException(__('Invalid event correlation host'));
        }
        App::uses('UUID', 'Lib');
        $this->Host->recursive = -1;
        $evcHost = $this->Host->findById($id, [
            'Host.id',
            'Host.name',
        ]);

        //Event correlation settings
        $eventcorrelation_settings = $this->EventcorrelationSetting->currentEventcorrelationSettings();
        $default_eventcorrelation_settings = $this->EventcorrelationSetting->defaultEventcorrelationSettings();
        $considerStateType = ($default_eventcorrelation_settings['EVC_CONSIDER_STATETYPE']['value'] & $eventcorrelation_settings['EventcorrelationSetting']['configuration_option']);

        $containerIds = $this->Tree->resolveChildrenOfContainerIds($this->MY_RIGHTS);
        $hosts = $this->Host->hostsByContainerId($containerIds, 'list', [
            'Host.disabled' => [0, 1]
        ]);

        $evcTree = $this->Eventcorrelation->getEvcTreeData($id, $hosts);

        $rootContainer = $this->Eventcorrelation->find('first', [
            'recursive'  => -1,
            'contain'    => [
                'Host' => [
                    'fields' => [
                        'Host.id',
                        'Host.name',
                    ],
                ],
            ],
            'conditions' => [
                'Eventcorrelation.host_id' => $id,
            ],
        ]);
        $currentEvcConfigServices = (!empty($rootContainer)) ? $this->Eventcorrelation->getCurrentEvcConfig($id) : [];
        $evcTree = [];
        $evcServiceUuids = [];
        $evcFirstLayerServiceIds = [];
        if (!empty($rootContainer)) {
            $evcTree = $this->Eventcorrelation->getEvcTreeData($id, $hosts);
            $evcFirstLayerServiceIds = array_unique(Hash::extract($evcTree[0], '{n}.{n}.Service.id'));
            $evcServiceUuids = Hash::extract($evcTree, '{n}.{*}.{n}.Service.uuid');
        }
        if (!empty($evcTree)) {
            $serviceUsedBy = $this->Eventcorrelation->servicesUsedByList($id, $currentEvcConfigServices);
            if (!empty($serviceUsedBy)) {
                foreach ($evcTree as $layerIndex => $evcLayerObjects) {
                    if ($layerIndex === 0) { //ignore first layer
                        continue;
                    }
                    foreach ($evcLayerObjects as $evcLayer => $evcLayerObjects) {
                        foreach ($evcLayerObjects as $evcKey => $evcObject) {
                            $evcTree[$layerIndex][$evcLayer][$evcKey]['Eventcorrelation']['usedBy'] = '';
                            if (array_key_exists($evcObject['Eventcorrelation']['service_id'], $serviceUsedBy)) {
                                $evcTree[$layerIndex][$evcLayer][$evcKey]['Eventcorrelation']['usedBy'] = implode(', ',
                                    $serviceUsedBy[$evcObject['Eventcorrelation']['service_id']]
                                );
                            }
                        }
                    }
                }
            }
        }


        if ($this->request->is('post') || $this->request->is('put')) {
            $usedServices = [];
            krsort($this->request->data, SORT_NATURAL);
            $parentId = null;
            $parentIds = [];

            $this->Eventcorrelation->Behaviors->attach('Tree', [
                    'scope' => 'Eventcorrelation.host_id = ' . $id,
                ]
            );

            if (!empty($rootContainer)) {
                $this->Eventcorrelation->cleanUpEventCorrelation($id);
            }

            foreach ($this->request->data as $layer => $data) {
                foreach ($data as $key => $value) {
                    //ToDo -> create Service if not exists
                    $currentServiceId = null;
                    if (($this->Eventcorrelation->isNewEvcService($key)) ||
                        (!empty($currentEvcConfigServices[$key]) && !$this->Service->exists($currentEvcConfigServices[$key]))
                    ) {

                        $this->Service->create();
                        //vService is NEW
                        if ($this->Service->save([
                                'Service' => [
                                    'host_id'            => $id,
                                    'uuid'               => UUID::v4(),
                                    'name'               => $value['vServiceName'],
                                    'servicetemplate_id' => $value['vServiceTemplateId'],
                                    'service_type'       => EVK_SERVICE,
                                    'usage_flag'         => EVENTCORRELATION_MODULE
                                ]
                            ]
                        )) {
                            $currentServiceId = $this->Service->id;
                        }
                    } else {
                        if ($this->Service->exists($key) && $this->request->ext == 'json') {
                            $usedServices[] = $key;
                            $currentServiceId = $key;
                        } else if ($this->Service->exists($currentEvcConfigServices[$key])) {
                            $this->Service->id = $currentEvcConfigServices[$key];
                            $this->Service->saveField('name', $value['vServiceName']);
                            $this->Service->saveField('servicetemplate_id', $value['vServiceTemplateId']);
                            $usedServices[] = $currentEvcConfigServices[$key];
                            $currentServiceId = $this->Service->id;
                        }
                    }


                    if ($currentServiceId !== null) {
                        //virtual Services
                        $this->Eventcorrelation->create();
                        if ($this->Eventcorrelation->save([
                            'Eventcorrelation' => [
                                'parent_id'  => ((isset($parentIds[$key]))) ? $parentIds[$key] : null,
                                'host_id'    => $id, //set host id for root element
                                'service_id' => $currentServiceId,
                                'operator'   => $value['operator'],
                            ],
                        ])) {

                            $this->Eventcorrelation->setUsageFlag($currentServiceId);
                            $parentId = $this->Eventcorrelation->id;
                        }
                    }
                    foreach ($value['children'] as $layerServiceId) { //last layer for save has real service id as value
                        //real Services
                        $parentIds[$layerServiceId] = $parentId;
                        if ($layer == 'layer_1') { //first layer in evc array
                            $this->Eventcorrelation->create();
                            $this->Eventcorrelation->save([
                                'Eventcorrelation' => [
                                    'parent_id'  => $parentId,
                                    'host_id'    => $id,
                                    'service_id' => $layerServiceId,
                                    'operator'   => null,
                                ],
                            ]);
                            $this->Eventcorrelation->setUsageFlag($layerServiceId);
                        }
                    }
                }
            }
            $notUsedServices = array_diff($currentEvcConfigServices, $usedServices);
            if (!empty($notUsedServices)) {
                $this->Eventcorrelation->cleanUpNotUsedServices($notUsedServices);
            }
            if ($this->request->ext == 'json') {
                $this->serializeId(); // REST API ID serialization
                return;
            }
            $this->setFlash(__('Event correlation successfully saved.'));
            $this->redirect(['action' => 'index']);
        }
        //ContainerID => 1 for ROOT Container
        $userContainerIds = $this->Tree->resolveChildrenOfContainerIds($this->MY_RIGHTS);

        $services = [];
        if (!empty($evcFirstLayerServiceIds)) {
            $services = $this->Service->servicesByHostContainerIds($userContainerIds, 'all', [
                'Service.disabled' => [0, 1],
                'Host.disabled'    => [0, 1],
                'NOT'              => [
                    'Service.host_id' => $id,
                ],
                'Service.id'       => $evcFirstLayerServiceIds
            ]);
        }


        $evcServices = $this->Service->servicesByHostContainerIds($userContainerIds, 'all', [
            'Service.host_id' => $id,
        ]);

        $serviceUuids = Hash::extract($services, '{n}.Service.uuid');

        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields
            ->currentState()
            ->isHardstate()
            ->lastHardState();
        $servicestates = $this->Servicestatus->byUuid(array_merge($serviceUuids, $evcServiceUuids), $ServicestatusFields);

        //@TODO remove vservices from its own correlation (and maybe the real services which are on this correlation)
        //build up a service list for the editor
        $servicesList = [];
        foreach ($services as $service) {
            $currentState = null;
            $Servicestatus = new \itnovum\openITCOCKPIT\Core\Servicestatus([]);
            //check if the current service has a state
            if (!empty($servicestates[$service['Service']['uuid']])) {
                $Servicestatus = new \itnovum\openITCOCKPIT\Core\Servicestatus($servicestates[$service['Service']['uuid']]['Servicestatus']);
                $currentState = $Servicestatus->currentState();
                if ($considerStateType && !$Servicestatus->isHardState()) {
                    $currentState = $currentState = $Servicestatus->getLastHardState();
                }
            }
            //build service list
            $servicesList[$service['Service']['id']] = [
                'hostId'          => $service['Host']['id'],
                'hostName'        => $service['Host']['name'],
                'hostUuid'        => $service['Host']['uuid'],
                'serviceId'       => $service['Service']['id'],
                'serviceDisabled' => $service['Service']['disabled'],
                'serviceName'     => $service[0]['ServiceDescription'],
                'currentState'    => $currentState,
                'serviceUuid'     => $service['Service']['uuid'],
                'serviceType'     => $service['Service']['service_type'],
                'Servicetemplate' => [
                    'id'   => $service['Servicetemplate']['id'],
                    'name' => $service['Servicetemplate']['name'],
                ],
            ];
        }
        foreach ($evcServices as $evcService) {
            //check if the current service has a state
            $currentState = null;
            if (isset($servicestates[$evcService['Service']['uuid']])) {
                $Servicestatus = new \itnovum\openITCOCKPIT\Core\Servicestatus($servicestates[$evcService['Service']['uuid']]['Servicestatus']);
                $currentState = $Servicestatus->currentState();
                if ($considerStateType && !$Servicestatus->isHardState()) {
                    $currentState = $currentState = $Servicestatus->getLastHardState();
                }
            }
            //build service list
            $servicesList[$evcService['Service']['id']] = [
                'hostId'          => $evcService['Host']['id'],
                'hostName'        => $evcService['Host']['name'],
                'hostUuid'        => $evcService['Host']['uuid'],
                'serviceId'       => $evcService['Service']['id'],
                'serviceDisabled' => $evcService['Service']['disabled'],
                'serviceName'     => $evcService[0]['ServiceDescription'],
                'currentState'    => $currentState,
                'serviceUuid'     => $evcService['Service']['uuid'],
                'serviceType'     => $evcService['Service']['service_type'],
                'Servicetemplate' => [
                    'id'   => $evcService['Servicetemplate']['id'],
                    'name' => $evcService['Servicetemplate']['name'],
                ],
            ];
        }

        $services = Hash::combine(array_slice($services, 0, 10, true), '{n}.Service.id'/* ['%s|%s', '{n}.Host.uuid', '{n}.Service.uuid']*/, ['%s/%s', '{n}.Host.name', '{n}.{n}.ServiceDescription'], '{n}.Host.name');
        $servicetemplates = $this->Servicetemplate->servicetemplatesByContainerId($this->MY_RIGHTS, 'list', EVK_SERVICE);

        $this->Frontend->setJson('highlightedHostId', $highlightedHostId);
        $this->Frontend->setJson('evcTree', $evcTree);
        $this->Frontend->setJson('hostName', $evcHost['Host']['name']);    //Evc Host Name
        $this->Frontend->setJson('hostId', $evcHost['Host']['id']);        //Evc Host Id
        $this->Frontend->setJson('servicesList', $servicesList);
        $this->set(compact(['services', 'servicetemplates', 'evcHost']));
    }


    public function delete($id = null) {
        if (!$this->Host->exists($id)) {
            throw new NotFoundException(__('Invalid event correlation'));
        }
        if (!$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }
        $eventCorrelation = $this->Eventcorrelation->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'Eventcorrelation.host_id' => $id,
            ],
            'fields'     => [
                'Eventcorrelation.id',
            ],
        ]);
        $host = $this->Host->findById($id);
        $containerIdsToCheck = Hash::extract($host, 'Container.{n}.HostsToContainer.container_id');
        $containerIdsToCheck[] = $host['Host']['container_id'];
        if (!$this->allowedByContainerId($containerIdsToCheck)) {
            $this->render403();

            return;
        }

        if (isset($eventCorrelation['Eventcorrelation']['id'])) {
            if (!$this->Eventcorrelation->exists($eventCorrelation['Eventcorrelation']['id'])) {
                throw new NotFoundException(__('Invalid event correlation'));
            }
        }

        if ($this->Host->__delete($host, $this->Auth->user('id'))) {
            if ($this->Eventcorrelation->cleanUpEventCorrelation($id)) {
                $this->setFlash(__('Event correlation deleted'));
                $this->redirect(['action' => 'index']);
            }
        }
        $this->setFlash(__('Could not delete event correlation'), false);
        $this->redirect(['action' => 'index']);
    }

    public
    function usedBy($hostId = null) {
        $conditions = [
            'Host.id'        => $hostId,
            'Host.host_type' => EVK_HOST
        ];
        if (!$this->Host->hasAny($conditions)) {
            throw new NotFoundException(__('Invalid event correlation'));
        }

        if (empty($hostId)) {
            return $hostId;
        }
        $query = [
            'recursive'  => -1,
            'joins'      => [
                [
                    'table'      => 'hosts_to_containers',
                    'alias'      => 'HostsToContainers',
                    'type'       => 'LEFT',
                    'conditions' => [
                        'HostsToContainers.host_id = Host.id',
                    ],
                ],
            ],
            'conditions' => [
                'Host.host_type'                 => EVK_HOST,
                'Host.disabled'                  => 0,
                'HostsToContainers.container_id' => $this->MY_RIGHTS,
            ],
            'fields'     => [
                'Host.id',
                'Host.name',
                'Host.description',
            ],
            'order'      => 'Host.name'
        ];
        $all_evc_hosts = $this->Host->find('all', $query);
        $all_evc_hosts = Hash::combine($all_evc_hosts, '{n}.Host.id', '{n}');
        $all_evc_hosts = Hash::insert($all_evc_hosts, '{n}.Host.usedBy', []);

        $additionalDetails = $this->Eventcorrelation->inUse(Hash::extract($all_evc_hosts, '{n}.Host.id'));
        foreach ($additionalDetails as $additionalDetail) {
            $all_evc_hosts[$additionalDetail['Host']['id']]['Host']['usedBy'][$additionalDetail['Eventcorrelation']['host_id']] = $additionalDetail['EvcHost']['name'];

        }
        $usedBy = $all_evc_hosts[$hostId];
        $this->set(compact('usedBy'));
    }


    public function serviceUsedBy($id) {
        $serviceIds = func_get_args();
        if (!$this->Service->exists($id)) {
            throw new NotFoundException(__('Invalid service'));
        }
        $serviceInfo = $this->Service->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'Service.id' => $serviceIds,
            ],
            'fields'     => [
                'Service.id',
                'IF(Service.name IS NULL, Servicetemplate.name, Service.name) AS ServiceName',
                'Host.id',
                'Host.name'
            ],
            'joins'      => [
                [
                    'table'      => 'servicetemplates',
                    'alias'      => 'Servicetemplate',
                    'type'       => 'LEFT',
                    'conditions' => [
                        'Servicetemplate.id = Service.servicetemplate_id'
                    ]
                ],
                [
                    'table'      => 'hosts',
                    'alias'      => 'Host',
                    'type'       => 'LEFT',
                    'conditions' => [
                        'Host.id = Service.host_id'
                    ]
                ]
            ]
        ]);

        $evcServiceData = [];
        foreach ($serviceInfo as $service) {
            $evcServiceData[$service['Service']['id']] = $this->Eventcorrelation->find('all', [
                'recursive'  => -1,
                'conditions' => [
                    'Eventcorrelation.service_id' => $service['Service']['id']
                ],
                'fields'     => [
                    'Eventcorrelation.host_id',
                ],
            ]);
        }

        $hostData = [];
        if (!empty($evcServiceData)) {
            foreach ($evcServiceData as $serviceId => $serviceData) {
                $hostIds = Hash::extract($serviceData, '{n}.Eventcorrelation.host_id');
                $hostData[$serviceId] = $this->Host->find('all', [
                    'recursive'  => -1,
                    'conditions' => [
                        'Host.id' => $hostIds
                    ],
                    'fields'     => [
                        'Host.name',
                        'Host.id'
                    ]
                ]);
            }

        }
        $this->set(compact('hostData', 'serviceInfo'));
    }

    public function hostUsedBy($id) {
        $hostIds = func_get_args();
        if (!$this->Host->exists($id)) {
            throw new NotFoundException(__('Invalid host'));
        }

        $hostData = $this->Host->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'Host.id' => $hostIds
            ],
            'fields'     => [
                'Host.name',
                'Host.id',
                'Host.address'
            ]
        ]);

        $hostIds = Hash::extract($hostData, '{n}.Host.id');

        $serviceData = [];
        foreach ($hostData as $host) {
            //find all services to the given host
            $serviceData[$host['Host']['id']] = $this->Service->find('all', [
                'recursive'  => -1,
                'conditions' => [
                    'Service.host_id' => $host['Host']['id'],
                    'Service.usage_flag &' . EVENTCORRELATION_MODULE
                ],
                'fields'     => [
                    'Service.id',
                    'IF(Service.name IS NULL, Servicetemplate.name, Service.name) AS ServiceName',
                    'Service.host_id'
                ],
                'joins'      => [
                    [
                        'table'      => 'servicetemplates',
                        'alias'      => 'Servicetemplate',
                        'type'       => 'LEFT',
                        'conditions' => [
                            'Servicetemplate.id = Service.servicetemplate_id'
                        ]
                    ],
                ]
            ]);
        }

        $serviceIds = Hash::extract($serviceData, '{n}.{n}.Service.id');

        //map services to the eventcorrelations (host ids)

        $mapping = [];
        $hostDataMapping = [];
        foreach ($serviceIds as $serviceId) {
            $result = $this->Eventcorrelation->find('all', [
                'recursive'  => -1,
                'conditions' => [
                    'Eventcorrelation.service_id' => $serviceId
                ],
                'fields'     => [
                    'Eventcorrelation.host_id',
                    'Host.name',
                ],
                'joins'      => [
                    [
                        'table'      => 'hosts',
                        'alias'      => 'Host',
                        'type'       => 'LEFT',
                        'conditions' => [
                            'Host.id = Eventcorrelation.host_id'
                        ]
                    ],
                ]
            ]);
            $hostDataMapping[$serviceId] = Hash::combine($result, '{n}.Eventcorrelation.host_id', '{n}.Host.name');
            $hostIds = Hash::extract($result, '{n}.Eventcorrelation.host_id');
            $mapping[$serviceId] = $hostIds;

        }

        $this->set(compact('mapping', 'hostDataMapping', 'serviceData', 'hostData'));
    }

    public function summary_view($id = null) {
        $isWidgetRequest = $this->request->query('widgetRequest') === 'true';

        $hostCount = $this->Host->find('count', [
            'conditions' => [
                'Host.id'        => $id,
                'Host.host_type' => EVK_HOST,
            ],
        ]);

        if (!$hostCount) {
            throw new NotFoundException(__('Invalid event correlation host'));
        }

        $evcHost = $this->Host->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Host.id',
                'Host.name',
            ],
            'conditions' => [
                'Host.id' => $id
            ]
        ]);

        //Event correlation settings
        $eventcorrelation_settings = $this->EventcorrelationSetting->currentEventcorrelationSettings();
        $default_eventcorrelation_settings = $this->EventcorrelationSetting->defaultEventcorrelationSettings();
        $considerStateType = ($default_eventcorrelation_settings['EVC_CONSIDER_STATETYPE']['value'] & $eventcorrelation_settings['EventcorrelationSetting']['configuration_option']);

        $containerIds = $this->Tree->resolveChildrenOfContainerIds($this->MY_RIGHTS);
        $hosts = $this->Host->hostsByContainerId($containerIds, 'list', [
            'Host.disabled' => [0, 1],
        ]);

        $evcTree = $this->Eventcorrelation->getEvcTreeData($id, $hosts);

        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields->wildcard();
        $servicestatus = $this->Servicestatus->byUuid(Hash::extract($evcTree, '{n}.{*}.{n}.Service.uuid'), $ServicestatusFields);
        $evcName = $evcHost['Host']['name'];
        $evcHostId = $evcHost['Host']['id'];
        $this->set(compact(['evcName', 'evcHostId', 'evcTree', 'servicestatus']));
        $this->set('_serialize', ['evcName', 'evcHostId', 'evcTree', 'servicestatus']);

        $binary_path = '/usr/bin/wkhtmltopdf';
        if (file_exists('/usr/local/bin/wkhtmltopdf')) {
            $binary_path = '/usr/local/bin/wkhtmltopdf';
        }

        $this->pdfConfig = [
            'engine'             => 'CakePdf.WkHtmlToPdf',
            'margin'             => [
                'bottom' => 5,
                'left'   => 0,
                'right'  => 0,
                'top'    => 5,
            ],
            'encoding'           => 'UTF-8',
            'download'           => true,
            'binary'             => $binary_path,
            'orientation'        => 'landscape',
            'filename'           => $evcName . '.pdf',
            'no-pdf-compression' => '*',
            'image-dpi'          => '900',
            'background'         => true,
            'no-background'      => false,
        ];

        if ($isWidgetRequest === true) {
            $this->layout = 'blank';
            $this->render('summary_view_widget');
        }

    }

    public function getServiceWithStateById($id) {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }
        if (!$this->Service->exists($id)) {
            throw new NotFoundException('Invalid Service');
        }
        $eventcorrelation_settings = $this->EventcorrelationSetting->currentEventcorrelationSettings();
        $default_eventcorrelation_settings = $this->EventcorrelationSetting->defaultEventcorrelationSettings();
        $considerStateType = ($default_eventcorrelation_settings['EVC_CONSIDER_STATETYPE']['value'] & $eventcorrelation_settings['EventcorrelationSetting']['configuration_option']);

        $tempService = $this->Service->find('first', [
            'recursive'  => -1,
            'contain'    => [
                'Servicetemplate' => [
                    'fields' => [
                        'Servicetemplate.id',
                        'Servicetemplate.name'
                    ],
                ],
                'Host'            => [
                    'fields' => [
                        'Host.id',
                        'Host.name',
                        'Host.uuid'
                    ],
                ],
            ],
            'fields'     => [
                'Service.id',
                'Service.disabled',
                'Service.name',
                'Service.uuid',
                'Service.service_type'
            ],
            'conditions' => [
                'Service.id' => $id,
            ],
        ]);

        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields->wildcard();
        $serviceState = $this->Servicestatus->byUuid($tempService['Service']['uuid'], $ServicestatusFields);

        $currentState = null;
        if (!empty($serviceState)) {
            $Servicestatus = new \itnovum\openITCOCKPIT\Core\Servicestatus($serviceState['Servicestatus']);
            $currentState = $Servicestatus->currentState();
            if ($considerStateType && !$Servicestatus->isHardState()) {
                $currentState = $Servicestatus->getLastHardState();
            }
        }
        $serviceName = $tempService['Service']['name'];
        if (empty($serviceName)) {
            $serviceName = $tempService['Servicetemplate']['name'];
        }

        $service[$tempService['Service']['id']] = [
            'hostId'          => $tempService['Host']['id'],
            'hostName'        => $tempService['Host']['name'],
            'hostUuid'        => $tempService['Host']['uuid'],
            'serviceId'       => $tempService['Service']['id'],
            'serviceDisabled' => $tempService['Service']['disabled'],
            'serviceName'     => $serviceName,
            'currentState'    => $currentState,
            'serviceUuid'     => $tempService['Service']['uuid'],
            'serviceType'     => $tempService['Service']['service_type'],
            'Servicetemplate' => [
                'id'   => $tempService['Servicetemplate']['id'],
                'name' => $tempService['Servicetemplate']['name'],
            ],
        ];

        $this->set('service', $service);
        $this->set('_serialize', ['service']);
    }

    public function loadServicesByString() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }
        $this->Service->virtualFields['servicename'] = 'CONCAT(Host.name,"/",IF((Service.name IS NULL OR Service.name=""), Servicetemplate.name, Service.name))';
        $selected = $this->request->query('selected');
        $ServiceFilter = new ServiceFilter($this->request);


        $ServiceCondition = new ServiceConditions($ServiceFilter->indexFilter());
        $ServiceCondition->setNotConditions([
            'Service.host_id' => $this->request->query('notServiceId')
        ]);

        $ServiceCondition->setContainerIds($this->MY_RIGHTS);
        $ServiceCondition->includeDisabled(true);

        $services = $this->Service->makeItJavaScriptAble(
            $this->Service->getServicesForAngular($ServiceCondition, $selected)
        );


        $this->set(compact(['services']));
        $this->set('_serialize', ['services']);
    }

    public function evcWidget() {
        $this->layout = 'blank';

        if (!$this->isApiRequest()) {
            //Only ship HTML template
            return;
        }

        $this->loadModel('Widget');
        $this->loadModel('Host');

        if ($this->request->is('get')) {
            $widgetId = (int)$this->request->query('widgetId');
            if (!$this->Widget->exists($widgetId)) {
                throw new RuntimeException('Invalid widget id');
            }

            $widget = $this->Widget->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'Widget.id' => $widgetId
                ],
                'fields'     => [
                    'Widget.host_id'
                ]
            ]);

            //Check host permissions
            $host = $this->Host->find('first', [
                'recursive'  => -1,
                'contain'    => [
                    'Container'
                ],
                'fields'     => [
                    'Host.id',
                    'Host.uuid',
                    'Host.name'
                ],
                'conditions' => [
                    'Host.id'       => $widget['Widget']['host_id'],
                    'Host.disabled' => 0
                ]
            ]);

            $hostId = null;
            if (!empty($host)) {
                $hostId = (int)$widget['Widget']['host_id'];
                if ($this->hasRootPrivileges === false) {
                    if (!$this->allowedByContainerId(Hash::extract($host, 'Container.{n}.HostsToContainer.container_id'), false)) {
                        $hostId = null;
                    }
                }
            }

            $acl = [
                'evc' => [
                    'view' => isset($this->PERMISSIONS['eventcorrelationmodule']['eventcorrelations']['view']),
                ]
            ];

            $this->set('host_id', $hostId);
            $this->set('ACL', $acl);

            $this->set('_serialize', ['host_id', 'ACL']);
            return;
        }


        if ($this->request->is('post')) {
            $hostId = (int)$this->request->data('host_id');
            if ($hostId === 0) {
                $hostId = null;
            }

            $widgetId = (int)$this->request->data('Widget.id');

            if (!$this->Widget->exists($widgetId)) {
                throw new RuntimeException('Invalid widget id');
            }
            $widget = $this->Widget->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'Widget.id' => $widgetId
                ],
            ]);

            $widget['Widget']['host_id'] = $hostId;

            if ($this->Widget->save($widget)) {
                $this->set('host_id', $hostId);
                $this->set('_serialize', ['host_id']);
                return;
            }

            $this->serializeErrorMessageFromModel('Widget');
            return;
        }
        throw new MethodNotAllowedException();
    }
}
