<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


App::uses('Host', 'Model');

class Eventcorrelation extends EventcorrelationModuleAppModel {

    public $actsAs = [
        'Tree' => [
            'scope' => 'Eventcorrelation.host_id',
        ],
    ];

    public $belongsTo = [
        'Host',
        'Service',
    ];

    /**
     * get virtual services of >layer1 of an Eventcorrelation
     *
     * @param  Integer $host_id the evc host id
     *
     * @return Array            list of services
     */
    public function getCurrentEvcConfig($host_id) {
        $evcToServicesArray = $this->find('all', [
            'recursive'  => -1,
            'fields'     => [
                'Eventcorrelation.id',
                'Eventcorrelation.service_id',
            ],
            'conditions' => [
                'AND' => [
                    'Eventcorrelation.host_id' => $host_id,
                    'Eventcorrelation.operator IS NOT NULL' //consider only virtual services
                ],

            ],
        ]);

        return Hash::combine($evcToServicesArray, '{n}.Eventcorrelation.id', '{n}.Eventcorrelation.service_id');
    }

    public function isNewEvcService($evcId) {
        return preg_match('/^ui-id-\d+$/', $evcId);
    }

    public function cleanUpEventCorrelation($host_id = null) {
        if ($host_id === null) {
            return false;
        }
        //set usage flags for first layer services before delete the old data
        $fistLayerServices = $this->getFirstLayerServices($host_id);
        foreach ($fistLayerServices as $firstLayerService) {
            $firstLayerService = $firstLayerService['Eventcorrelation']['service_id'];

            $result = $this->find('count', [
                'recursive'  => -1,
                'conditions' => [
                    'Eventcorrelation.service_id' => $firstLayerService,
                    'NOT'                         => [
                        'Eventcorrelation.host_id' => $host_id,
                    ]
                ]
            ]);
            if ($result == 0) {
                //service is not in use
                //reset usage flag
                $this->setUsageFlag($firstLayerService, true);
            }
        }

        return $this->deleteAll(
            [
                'Eventcorrelation.host_id' => $host_id,
            ]
        );
    }

    public function cleanUpNotUsedServices($notUsedEvcServices = []) {
        $servicesToDelete = $this->Service->find('list', [
            'conditions' => [
                'Service.id' => $notUsedEvcServices,
            ],
        ]);
        foreach ($servicesToDelete as $serviceId => $serviceName) {
            $this->Service->delete($serviceId);
        }
    }

    public function servicesUsedByList($ownHostId = null, $serviceIds = []) {
        $usedByList = [];
        $serviceIdsToEvcHostNames = [];
        foreach ($serviceIds as $eventCorrelationId => $serviceId) {
            $usedByList[$serviceId] = $this->find('all', [
                    'recursive'  => -1,
                    'joins'      => [
                        [
                            'table'      => 'hosts',
                            'alias'      => 'Host',
                            'type'       => 'INNER',
                            'conditions' => [
                                'Host.id = Eventcorrelation.host_id'
                            ]
                        ],
                        [
                            'table'      => 'eventcorrelations',
                            'alias'      => 'VirtualService',
                            'type'       => 'INNER',
                            'conditions' => [
                                'AND' => [
                                    'VirtualService.host_id' => $ownHostId,
                                    'VirtualService.service_id = Eventcorrelation.service_id'
                                ]
                            ]
                        ],

                    ],
                    'conditions' => [
                        'NOT'                         => [
                            'Eventcorrelation.host_id' => $ownHostId,
                        ],
                        'Eventcorrelation.service_id' => $serviceId,
                    ],
                    'fields'     => [
                        'Eventcorrelation.id',
                        'Eventcorrelation.service_id',
                        'Host.name',
                        'VirtualService.lft',
                        'VirtualService.rght',
                    ],
                ]
            );
        }
        $usedByList = Hash::filter($usedByList);
        if (!empty($usedByList)) {
            $eventCorrelations = $this->find('all', [
                'recursive'  => -1,
                'conditions' => [
                    'Eventcorrelation.host_id' => $ownHostId,
                    'Eventcorrelation.operator IS NOT NULL'
                ],
                'fields'     => [
                    'Eventcorrelation.service_id',
                    'Eventcorrelation.lft',
                    'Eventcorrelation.rght'
                ],
            ]);
            foreach ($usedByList as $serviceId => $usedEvcServices) {
                foreach ($usedEvcServices as $key => $usedEvcService) {
                    //if(!in_array($serviceIdsToEvcHostNames[$serviceId], $usedEvcService['Host']['name'])){
                    if (!empty($serviceIdsToEvcHostNames[$serviceId]) &&
                        in_array($usedEvcService['Host']['name'], $serviceIdsToEvcHostNames[$serviceId])
                    ) {
                        continue;
                    }
                    $serviceIdsToEvcHostNames[$serviceId][] = $usedEvcService['Host']['name'];
                    foreach ($eventCorrelations as $eventCorrelation) {
                        if ($eventCorrelation['Eventcorrelation']['lft'] > $usedEvcService['VirtualService']['lft'] && $eventCorrelation['Eventcorrelation']['lft'] < $usedEvcService['VirtualService']['rght']) {
                            if ($eventCorrelation['Eventcorrelation']['lft'] > $usedEvcService['VirtualService']['rght']) {
                                break;
                            }
                            if (!empty($serviceIdsToEvcHostNames[$eventCorrelation['Eventcorrelation']['service_id']]) &&
                                in_array($usedEvcService['Host']['name'], $serviceIdsToEvcHostNames[$eventCorrelation['Eventcorrelation']['service_id']])
                            ) {
                                break;
                            }
                            $serviceIdsToEvcHostNames[$eventCorrelation['Eventcorrelation']['service_id']][] = $usedEvcService['Host']['name'];
                        }
                    }
                }
            }
        }
        return $serviceIdsToEvcHostNames;
    }

    public function inUse($hostIds = []) {
        if (empty($hostIds)) {
            return $hostIds;
        }
        return $this->Host->find('all', [
            'recursive' => -1,
            'joins'     => [
                [
                    'table'      => 'services',
                    'alias'      => 'Service',
                    'type'       => 'INNER',
                    'conditions' => [
                        'Service.host_id = Host.id',
                        'Service.service_type' => EVK_SERVICE
                    ],
                ],
                [
                    'table'      => 'eventcorrelations',
                    'alias'      => 'Eventcorrelation',
                    'type'       => 'LEFT',
                    'conditions' => [
                        'AND' => [
                            'Eventcorrelation.service_id = Service.id',
                            'Eventcorrelation.host_id != Service.host_id',
                        ]
                    ],
                ],
                [
                    'table'      => 'hosts',
                    'alias'      => 'EvcHost',
                    'type'       => 'LEFT',
                    'conditions' => [
                        'EvcHost.id = Eventcorrelation.host_id',
                    ],
                ],
            ],

            'conditions' => [
                'Eventcorrelation.host_id' => $hostIds
            ],
            'fields'     => [
                'Host.id',
                'Host.name',
                'Host.description',
                'Service.id',
                'Service.name',
                'Service.service_type',
                'Service.host_id',
                'Eventcorrelation.service_id',
                'Eventcorrelation.host_id',
                'EvcHost.id',
                'EvcHost.name'
            ],
        ]);
    }

    public function setUsageFlag($serviceId, $removeFlag = false) {
        $hasFlag = $this->Service->checkUsageFlag($serviceId, EVENTCORRELATION_MODULE);
        //debug('the service with id '.$serviceId.' has flag='.$hasFlag.' current flag is '.$this->Service->currentUsageFlag);
        if (!$hasFlag && !$removeFlag) {
            //no EVC flag yet
            //only add flag if removeFlag is false
            $newFlag = $this->Service->currentUsageFlag + EVENTCORRELATION_MODULE;
            $this->Service->id = $serviceId;
            $this->Service->saveField('usage_flag', $newFlag);
        } else {
            if ($hasFlag && $removeFlag) {
                $newFlag = $this->Service->currentUsageFlag - EVENTCORRELATION_MODULE;
                $this->Service->id = $serviceId;
                $this->Service->saveField('usage_flag', $newFlag);
            }
        }
    }

    /**
     * check if the service is in use by any other EVC
     * @param $serviceId
     * @return array|null
     */
    public function checkForAdditionalUse($serviceId) {
        $result = $this->find('count', [
            'recursive'  => -1,
            'conditions' => [
                'Eventcorrelation.service_id' => $serviceId
            ],
        ]);
        return $result;
    }

    public function getFirstLayerServices($evcId) {
        $result = $this->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'Eventcorrelation.host_id'  => $evcId,
                'Eventcorrelation.operator' => null,
            ],
            'fields'     => [
                'DISTINCT Eventcorrelation.service_id',
                'Eventcorrelation.host_id',
                'Eventcorrelation.id',
            ]
        ]);
        return $result;
    }

    public function getHosts() {
        $result = $this->find('all', [
            'recursive' => -1,
            'fields'    => [
                'DISTINCT host_id'
            ]
        ]);
        $result = Hash::extract($result, '{n}.Eventcorrelation.host_id');
        return $result;
    }

    public function getServices() {
        $result = $this->find('all', [
            'recursive' => -1,
            'fields'    => [
                'DISTINCT service_id'
            ]
        ]);
        $result = Hash::extract($result, '{n}.Eventcorrelation.service_id');
        return $result;
    }

    public function reorganizeEvcStructure($evcAsNest) {
        $result = [];
        $this->flattenElement($evcAsNest, 1, $result);
        return $result;
    }

    private function flattenElement($children, $layer, &$result) {
        foreach ($children as $element) {
            $data = [
                'Eventcorrelation' => $element['Eventcorrelation'],
                'Service'          => $element['Service'],
            ];
            $result[$layer][$element['Eventcorrelation']['parent_id']][] = $data;
        }
        foreach ($children as $element) {
            $this->flattenElement($element['children'], $layer + 1, $result);
        }
    }

    /**
     * @param int $hostId
     * @param array $hosts
     * @return array
     */
    public function getEvcTreeData($hostId, $hosts) {
        $rootContainer = $this->find('first', [
            'recursive'  => -1,
            'contain'    => [
                'Host' => [
                    'fields' => [
                        'Host.id',
                        'Host.name',
                    ],
                ],
            ],
            'conditions' => [
                'Eventcorrelation.host_id' => $hostId,
            ],
        ]);
        $evcTree = [];
        $conditions = [];
        if (!empty($hosts)) {
            $conditions = [
                'Service.host_id' => array_keys($hosts),
            ];
        }
        if (!empty($rootContainer)) {
            $query = [
                'recursive'  => -1,
                'contain'    => [
                    'Service' => [
                        'fields'          => [
                            'Service.id',
                            'Service.host_id',
                            'Service.name',
                            'Service.service_type',
                            'Service.uuid',
                            'Service.disabled'
                        ],
                        'conditions'      => $conditions,
                        'Servicetemplate' => [
                            'fields' => [
                                'Servicetemplate.name',
                            ],
                        ],
                        'Host'            => [
                            'fields' => [
                                'Host.name',
                            ],
                        ],
                    ],
                ],
                'fields'     => [
                    'Eventcorrelation.id',
                    'Eventcorrelation.parent_id',
                    'Eventcorrelation.host_id',
                    'Eventcorrelation.service_id',
                    'Eventcorrelation.operator',
                ],
                'conditions' => [
                    'Eventcorrelation.host_id' => $hostId,
                ],
            ];

            $childElements = $this->find('threaded', $query);
            $evcTree = $this->reorganizeEvcStructure($childElements);
            $evcTree = array_reverse($evcTree);
        }

        return $evcTree;
    }
}
