<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

class EvcHelper extends AppHelper {

    /**
     * @param $evcTree
     * @param $serviceStatusArray
     * @param null|string $uuidForServiceAlert
     * @return string
     */
    public function getEvcDataAsTable($evcTree, $serviceStatusArray, $uuidForServiceAlert = null) {
        $reFormatData = $this->reformatData($evcTree, $serviceStatusArray);
        if (!$reFormatData) {
            $html = '<span class="text-danger"><i class="fa fa-exclamation-triangle" aria-hidden="true"></i> Invalid configuration</span>';
            if ($uuidForServiceAlert) {
                $html = 'Invalid configuration';
            }
            return $html;
        }
        $htmlTableStructureArray = $this->prepareDataForHtmlCreate($reFormatData);
        $html = $this->createHmtl($htmlTableStructureArray, $uuidForServiceAlert);
        return $html;
    }

    public function reformatData($evcTree, $serviceStatusArray) {
        $tmpParentId = null;
        $tableSettings = [];
        $subCounter = 1;
        $sizeOfLayers = sizeof($evcTree);
        foreach ($evcTree as $layerNumber => $layerData) {
            $tmpLayerNumber = $layerNumber + 1;
            if ($tmpLayerNumber < $sizeOfLayers) {
                $tableSettings[$tmpLayerNumber] = [];
            }
            foreach ($layerData as $parentId => $evcNodeData) {
                foreach ($evcNodeData as $key => $data) {
                    if (!empty($data['Service']['Host']['name'])) {
                        $tableSettings[$tmpLayerNumber - 1][$data['Eventcorrelation']['id']]['hostName'] = $data['Service']['Host']['name'];
                        $tableSettings[$tmpLayerNumber - 1][$data['Eventcorrelation']['id']]['serviceName'] = (($data['Service']['name']) ? $data['Service']['name'] : $data['Service']['Servicetemplate']['name']);
                        $tableSettings[$tmpLayerNumber - 1][$data['Eventcorrelation']['id']]['uuid'] = $data['Service']['uuid'];
                        $tableSettings[$tmpLayerNumber - 1][$data['Eventcorrelation']['id']]['operator'] = $data['Eventcorrelation']['operator'];
                        if (array_key_exists($data['Service']['uuid'], $serviceStatusArray) && $data['Service']['disabled'] == '0') {
                            $tableSettings[$tmpLayerNumber - 1][$data['Eventcorrelation']['id']]['current_state'] = $serviceStatusArray[$data['Service']['uuid']]['Servicestatus']['current_state'];
                        } else if (!array_key_exists($data['Service']['uuid'], $serviceStatusArray) && $data['Service']['disabled'] == '0') {
                            $tableSettings[$tmpLayerNumber - 1][$data['Eventcorrelation']['id']]['current_state'] = '-1';
                        } else {
                            $tableSettings[$tmpLayerNumber - 1][$data['Eventcorrelation']['id']]['current_state'] = '-2';
                        }

                        $tableSettings[$tmpLayerNumber - 1][$data['Eventcorrelation']['id']]['disabled'] = $data['Service']['disabled'];
                        if (!isset($tableSettings[$tmpLayerNumber - 1][$data['Eventcorrelation']['id']]['serviceCounter']) && $layerNumber === 0 && $tmpLayerNumber < $sizeOfLayers) {
                            $tableSettings[$tmpLayerNumber - 1][$data['Eventcorrelation']['id']]['serviceCounter'] = 1;
                        }
                        if (!isset($tableSettings[$tmpLayerNumber][$parentId]) && $tmpLayerNumber < $sizeOfLayers) {
                            $tableSettings[$tmpLayerNumber][$parentId]['serviceCounter'] = 0;
                        }
                        if ($tmpParentId !== $parentId) {
                            $tmpParentId = $parentId;
                            $subCounter = 1;
                        } else {
                            $subCounter++;
                        }
                        if ($layerNumber > 0 && !empty($tableSettings[($tmpLayerNumber - 1)][$data['Eventcorrelation']['id']]) && $tmpLayerNumber < $sizeOfLayers) {
                            $tableSettings[$tmpLayerNumber][$tmpParentId]['serviceCounter'] += $tableSettings[$tmpLayerNumber - 1][$data['Eventcorrelation']['id']]['serviceCounter'];
                        }
                    } else {
                        //if config invalid
                        return false;
                    }
                }
                if ($layerNumber === 0) {
                    $tableSettings[$tmpLayerNumber][$parentId] = [
                        'serviceCounter' => $subCounter
                    ];
                }
            }
        }
        ksort($tableSettings);
        return $tableSettings;
    }

    /**
     * @param array $filteredData
     * @return array
     */
    public function prepareDataForHtmlCreate($filteredData) {
        $htmlTableStructure = [];
        foreach ($filteredData as $layerNumber => $layerDataArray) {
            $rowCounter = 0;
            foreach ($layerDataArray as $id => $layerData) {
                if ($layerNumber === 0) {
                    $htmlTableStructure[$rowCounter][$layerNumber] = $layerData;
                    $rowCounter++;
                }
                if ($layerNumber > 0) {
                    $htmlTableStructure[$rowCounter][$layerNumber] = $layerData;
                    $rowCounter++;
                    for ($i = 1; $i < $layerData['serviceCounter']; $i++) {
                        $rowCounter++;
                    }
                }
            }
        }
        return $htmlTableStructure;
    }

    /**
     * @param array $htmlTableStructureArray
     * @param null|string $uuidForServiceAlert
     * @return string
     */
    public function createHmtl($htmlTableStructureArray, $uuidForServiceAlert = null) {
        $html = '';
        $statusMapping = [
            'ok',
            'warning',
            'critical',
            'unknown'
        ];

        if (!empty($htmlTableStructureArray)) {
            $html = '<table id="evcSummary">';
            foreach ($htmlTableStructureArray as $htmlTableStructure) {
                $html .= '<tr>';
                foreach ($htmlTableStructure as $key => $column) {
                    $html .= '<td ';
                    if ($column['serviceCounter'] > 1) {
                        $html .= ' rowspan="' . $column['serviceCounter'] . '"';
                    }
                    if ($key === 0 && $column['current_state'] >= 0) {
                        $html .= sprintf(
                            ' class="border%sEvcSummary" ',
                            ucfirst($statusMapping[$column['current_state']])
                        );
                    } else if ($key === 0 && $column['current_state'] == '-1') {
                        $html .= ' class="borderNotMonitoredEvcSummary" ';
                    } else if ($key === 0 && $column['current_state'] == '-2') {
                        $html .= ' class="borderDisabledEvcSummary disabledEvcSummary" ';
                    } else {
                        if ($key > 0 && !array_key_exists($column['current_state'], $statusMapping)) {
                            $html .= ' class="notMonitoredEvcSummary no-bordered" ';
                        } else {
                            $html .= sprintf(
                                ' class="%sEvcSummary no-bordered" ',
                                $statusMapping[$column['current_state']]
                            );
                        }
                    }
                    $html .= '>';
                    if ($key === 0) {
                        $html .= sprintf(
                            '<span class="firstLayerServiceSpan" title="%s/%s">%s/%s</span>',
                            h($column['hostName']),
                            h($column['serviceName']),
                            h($column['hostName']),
                            h($column['serviceName'])

                        );
                    } else {
                        $alertMarker = '';
                        if ($uuidForServiceAlert === $column['uuid'] && $column['current_state'] > 0) {
                            $alertMarker = '&#9888;';
                        } else if ($uuidForServiceAlert === $column['uuid'] && $column['current_state'] == 0) {
                            $alertMarker = '&#9432;';
                        }
                        $html .= sprintf(
                            '<div class="gatter txt-color-white">%s</div><span class="txt-color-white virtualLayerService">%s %s</span>',
                            strtoupper($column['operator']),
                            h($column['serviceName']),
                            $alertMarker
                        );
                        h($column['serviceName']);
                    }
                    $html .= '</td>';
                }
                $html .= '</tr>';
            }
            $html .= '</table>';
        }
        return $html;
    }
}
