<?php

class IdoitModuleSchema extends CakeSchema {

    //sudo -g www-data Console/cake schema update --connection default --plugin IdoitModule --file schema.php

    public function before($event = []) {
        $db = ConnectionManager::getDataSource($this->connection);
        $db->cacheSources = false;

        $CronjobModel = ClassRegistry::init('Cronjob');

        //Check if cronjob exists
        $result = $CronjobModel->find('first', [
            'conditions' => [
                'Cronjob.plugin' => 'IdoitModule',
                'Cronjob.task'   => 'Synchronisation',
            ],
        ]);
        if (empty($result)) {
            $data = [
                'Cronjob' => [
                    'task'     => 'Synchronisation',
                    'plugin'   => 'IdoitModule',
                    'interval' => '1440',
                ],
            ];
            $CronjobModel->create();
            $CronjobModel->save($data);
        }

        return true;
    }

    public function after($event = []) {
    }


    public $idoit_configurations = [
        'id'                           => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'primary'],
        'name'                         => ['type' => 'string', 'null' => false, 'default' => null],
        'url'                          => ['type' => 'string', 'null' => false, 'default' => null],
        'apikey'                       => ['type' => 'string', 'null' => false, 'default' => null],
        'host_mapping'                 => ['type' => 'integer', 'null' => false, 'default' => 1],
        'auto_sync'                    => ['type' => 'integer', 'default' => 0],
        'report_id'                    => ['type' => 'integer', 'default' => null],
        'sync_additional_objecttypes'  => ['type' => 'integer', 'default' => 0],
        'additional_objecttypes'       => ['type' => 'integer', 'default' => null],
        'sync_additional_informations' => ['type' => 'integer', 'default' => 0],
        'additional_infos_report_id'   => ['type' => 'integer', 'default' => null],
        'indexes'                      => ['PRIMARY' => ['column' => 'id', 'unique' => 1],],
        'tableParameters'              => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $idoit_objecttypes = [
        'id'               => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'primary'],
        'configuration_id' => ['type' => 'integer'],
        'objecttype_id'    => ['type' => 'integer'],
        'hosttemplate_id'  => ['type' => 'integer'],
        'container_id'     => ['type' => 'integer'],
        'indexes'          => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters'  => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $idoit_linklists = [
        'id'                      => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'primary'],
        'configuration_id'        => ['type' => 'integer'],
        'regex'                   => ['type' => 'string', 'null' => false, 'default' => null],
        'servicetemplategroup_id' => ['type' => 'integer'],
        'indexes'                 => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters'         => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $idoit_hosts = [
        'id'               => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'primary'],
        'configuration_id' => ['type' => 'integer'],
        'objecttype_id'    => ['type' => 'integer'],
        'host_id'          => ['type' => 'integer'],
        'indexes'          => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters'  => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $idoit_objects = [
        'id'               => ['type' => 'integer', 'null' => false, 'key' => 'primary'],
        'configuration_id' => ['type' => 'integer'],
        'objecttype_id'    => ['type' => 'integer'],
        'object_id'        => ['type' => 'integer'],
        'container_id'     => ['type' => 'integer'],
        'hosttemplate_id'  => ['type' => 'integer'],
        'indexes'          => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters'  => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $idoit_mappings = [
        'id'               => ['type' => 'integer', 'null' => false, 'default' => null, 'key' => 'primary'],
        'configuration_id' => ['type' => 'integer'],
        'type'             => ['type' => 'integer'],
        'idoit_object_id'  => ['type' => 'integer'],
        'oitc_object_id'   => ['type' => 'integer'],
        'indexes'          => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters'  => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $idoit_sync_status = [
        'id'              => ['type' => 'integer', 'null' => false, 'key' => 'primary'],
        'task'            => ['type' => 'string', 'null' => false, 'default' => null],
        'status'          => ['type' => 'integer', 'default' => 0],
        'percent'         => ['type' => 'integer', 'default' => 0],
        'group'           => ['type' => 'integer', 'default' => 0],
        'created'         => ['type' => 'datetime', 'null' => false, 'default' => null],
        'indexes'         => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters' => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

}