<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

class ConfigurationsController extends IdoitModuleAppController {
    public $layout = 'Admin.default';
    public $uses = [
        'IdoitModule.Configuration',
        'IdoitModule.Objecttype',
        'IdoitModule.IdoitMapping',
        'IdoitModule.ObjectItem',
        'IdoitModule.Linklist',
        'IdoitModule.JsonRPC',
        'IdoitModule.IdoitSyncStatus',
        'Hosttemplate',
        'Servicetemplategroup',
        'Servicetemplate',
        'Host',
        'Service',
        'Contact',
        'Contactgroup',
        MONITORING_HOSTSTATUS
    ];

    public $components = [
        'GearmanClient',
    ];

    public $maxIdoitVersion = '1.9';

    public function index() {
        $options = [
            'contain' => [],
        ];
        $this->Paginator->settings = Hash::merge($this->Paginator->settings, $options);
        $all_idoits = $this->Paginator->paginate();
        $this->set(compact(['all_idoits']));

    }

    public function add() {
        $additionalObjecttypesMapping = $this->Configuration->setAdditionalObjecttypes();

        if ($this->request->is('post') || $this->request->is('put')) {
            $this->Configuration->create();

            $additionalObjecttypes = 0;
            foreach ($this->request->data['Configuration']['additional_objecttypes'] as $item) {
                $additionalObjecttypes = $additionalObjecttypes | $additionalObjecttypesMapping[$item];
            }
            $this->request->data['Configuration']['additional_objecttypes'] = $additionalObjecttypes;

            if ($this->Configuration->save($this->request->data)) {
                $this->setFlash(__('Configuration saved successfully'));
                $this->redirect(['action' => 'test', $this->Configuration->id]);

                return;
            }
            $this->setFlash(__('Data could not be saved'), false);
        }
    }

    public function edit($id = null) {
        $additionalObjecttypesMapping = $this->Configuration->setAdditionalObjecttypes();

        if (!$this->Configuration->exists($id)) {
            throw new NotFoundException(__('Invalid i-doit configuration'));
        }
        if ($this->request->is('post') || $this->request->is('put')) {
            $additionalObjecttypes = 0;
            if (!empty($this->request->data['Configuration']['additional_objecttypes'])) {

                foreach ($this->request->data['Configuration']['additional_objecttypes'] as $item) {
                    $additionalObjecttypes = $additionalObjecttypes | $additionalObjecttypesMapping[$item];
                }
                $this->request->data['Configuration']['additional_objecttypes'] = $additionalObjecttypes;
            }

            if ($this->Configuration->save($this->request->data)) {
                $this->setFlash(__('Configuration saved successfully'));
                $this->redirect(['action' => 'test', $this->Configuration->id]);

                return;
            } else {
                $this->setFlash(__('Data could not be saved'), false);
            }
        }
        $configuration = $this->Configuration->findById($id);
        $this->request->data = Hash::merge($configuration, $this->request->data);
        $this->set(compact('configuration'));
    }

    public function test($id = null) {
        if (!$this->Configuration->exists($id)) {
            throw new NotFoundException(__('Invalid i-doit configuration'));
        }
        $configuration = $this->Configuration->findById($id);
        $this->JsonRPC->setOptions([
            'url'    => $configuration['Configuration']['url'],
            'apikey' => $configuration['Configuration']['apikey'],
        ]);
        $result = $this->JsonRPC->test();

        $supportedVersion = false;
        $versionResult = $this->JsonRPC->version();
        if ($versionResult['success'] === true) {
            $supportedVersion = version_compare($this->maxIdoitVersion, $versionResult['version'], '>=');
        }

        if (($configuration['Configuration']['sync_additional_objecttypes'] === '1')) {
            $reportObjectsExists = $this->JsonRPC->checkReport($configuration['Configuration']['report_id'], "additional objecttypes");
        } else {
            $reportObjectsExists = [
                'success' => true,
                'message' => 'No Report needed',
            ];
        }

        if (($configuration['Configuration']['sync_additional_informations'] === '1')) {
            $reportInfosExists = $this->JsonRPC->checkReport($configuration['Configuration']['additional_infos_report_id'], "additional informations");
        } else {
            $reportInfosExists = [
                'success' => true,
                'message' => 'No Report needed',
            ];
        }

        $this->set(compact([
            'configuration',
            'result',
            'supportedVersion',
            'reportObjectsExists',
            'reportInfosExists'
        ]));

    }

    public function selectObjects($id = null) {
        if (!$this->Configuration->exists($id)) {
            throw new NotFoundException(__('Invalid i-doit configuration'));
        }

        $configuration = $this->Configuration->findById($id, [
            /*'contain' => [
                'Objecttype'
            ]*/
        ]);
        if ($this->request->is('post') || $this->request->is('put')) {
            $this->request->data['Configuration'] = Hash::merge($configuration['Configuration'], $this->request->data['Configuration']);
            $selectedObjectTypes = [];
            if (isset($this->request->data['Objecttype'])) {
                foreach ($this->request->data['Objecttype'] as $objecttype) {
                    if ($objecttype['import'] == 1) {
                        $selectedObjectTypes[] = [
                            'objecttype_id'   => $objecttype['objecttype_id'],
                            'hosttemplate_id' => $objecttype['hosttemplate_id'],
                            'container_id'    => $objecttype['container_id'],
                        ];
                    }
                }
            }

            $this->request->data['Objecttype'] = $selectedObjectTypes;

            //Delete all old objecttype_ids from database to avoid useless HABTM -.-
            $this->Objecttype->deleteAll(['configuration_id' => $configuration['Configuration']['id']]);

            if ($this->Configuration->saveAll($this->request->data)) {
                $this->redirect(['action' => 'selectItems', $this->Configuration->id]);

                return;
            }
            $this->setFlash(__('Cloud not save object types'), false);
        }

        $this->JsonRPC->init();
        $this->JsonRPC->setOptions([
            'url'    => $configuration['Configuration']['url'],
            'apikey' => $configuration['Configuration']['apikey'],
        ]);
        $data = [
            'method' => 'cmdb.object_types.read',
            'params' => [
                'order_by' => 'title',
            ],
        ];
        $result = $this->JsonRPC->sendRequest($data);

        $objectTypes = [];
        if (isset($result['json']['success']) && $result['json']['success'] === true) {
            foreach ($result['json']['result'] as $objectType) {
                $objectTypes[$objectType->id] = $objectType->title;
            }
        }

        $containers = $this->Tree->easyPath($this->MY_RIGHTS, OBJECT_HOST, [], $this->hasRootPrivileges, [CT_HOSTGROUP]);
        $hosttemplates = $this->Hosttemplate->hosttemplatesByContainerId($this->MY_RIGHTS, 'list');

        $selectedObjectTypesAndHosttemplate = [];
        foreach ($configuration['Objecttype'] as $objecttype) {
            $selectedObjectTypesAndHosttemplate[$objecttype['objecttype_id']] = $objecttype['hosttemplate_id'];
        }

        $this->request->data = Hash::merge($configuration, $this->request->data);
        $this->set(compact([
            'configuration',
            'objectTypes',
            'hosttemplates',
            'selectedObjectTypesAndHosttemplate',
            'containers',
        ]));
    }

    public function selectItems($id = null) {
        // move objects from idoit_hosts to idoit_mappings with type TYPE_HOST if idoit_objects is empty
        $this->IdoitMapping->moveHostsToObjects();

        App::uses('IdoitImport', 'IdoitModule.Lib');
        if (!$this->Configuration->exists($id)) {
            throw new NotFoundException(__('Invalid i-doit configuration'));
        }

        $configuration = $this->Configuration->findById($id);
        if (empty($configuration)) {
            $this->setFlash(__('No object types were selected'), false);

            return $this->redirect(['action' => 'selectObjects', $id]);
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            $this->request->data['Configuration'] = Hash::merge($configuration['Configuration'], $this->request->data['Configuration']);
            $objectsToSave = $objectsToImport = [];
            if (isset($this->request->data['ObjectItem'])) {
                foreach ($this->request->data['ObjectItem'] as $object) {
                    if ($object['import'] === '1') {
                        $objectsToImport[] = [
                            'objecttype_id'   => $object['objecttype_id'],
                            'object_id'       => $object['object_id'],
                            'hosttemplate_id' => $object['hosttemplate_id'],
                            'container_id'    => $object['container_id'],
                            'merge'           => $object['merge']
                        ];

                        foreach ($configuration['Objecttype'] as $objType) {
                            if ($objType['objecttype_id'] == $object['objecttype_id'] &&
                                $objType['hosttemplate_id'] == $object['hosttemplate_id'] &&
                                $objType['container_id'] == $object['container_id']
                            ) continue 2;
                        }
                        $objectsToSave[] = [
                            'objecttype_id'   => $object['objecttype_id'],
                            'object_id'       => $object['object_id'],
                            'hosttemplate_id' => $object['hosttemplate_id'],
                            'container_id'    => $object['container_id'],
                        ];
                    }
                }
            }


            $this->request->data['ObjectItem'] = $objectsToSave;


            //Delete all old objecttype_ids from database to avoid useless HABTM -.-
            $this->ObjectItem->deleteAll(['configuration_id' => $configuration['Configuration']['id']]);

            if ($this->Configuration->saveAll($this->request->data)) {
                $IdoitImport = new IdoitImport($this, $this->DbBackend);
                $IdoitImport->setUserId($this->Auth->user('id'));


                $res = $IdoitImport->runImport(true, $objectsToImport);

                $this->setFlash('i-doit import successfull');

                return $this->redirect([
                    'plugin'     => 'idoit_module',
                    'controller' => 'Synchronisations',
                ]);
            }

            $this->setFlash(__('Cloud not save config items'), false);
        }

        $objectsConf = [];

        $softwareLinkList = $this->Linklist->find('all', [
            'conditions' => [
                'Linklist.configuration_id' => $id,
            ],
            'recursive'  => -1
        ]);

        foreach ($configuration['Objecttype'] as $obType) {
            $objectsConf[$obType['objecttype_id']]['contaner'] = $obType['container_id'];
            $objectsConf[$obType['objecttype_id']]['hosttemplate'] = $obType['hosttemplate_id'];
        }

        $objectTypes = Hash::extract($configuration['Objecttype'], '{n}.objecttype_id');
        $this->JsonRPC->init();
        $this->JsonRPC->setOptions([
            'url'    => $configuration['Configuration']['url'],
            'apikey' => $configuration['Configuration']['apikey'],
        ]);
        $data = [
            'method' => 'cmdb.filter.getValidObjects',
            'params' => [
                'objTypeID' => $objectTypes,
            ],
        ];

        $result = $this->JsonRPC->sendRequest($data);

        $objects = $existingObjects = $checkExistingHosts = $existingObjectIds = [];
        if ($result['json']['success'] === true) {

            $IdoitImport = new IdoitImport($this, $this->DbBackend);
            $idoitStateToImport = [2];
            foreach ($result['json']['result'] as $object) {
                if (!in_array($object->status, $idoitStateToImport)) {
                    continue;
                }

                foreach ($configuration['IdoitHost'] as $existingIdoitHost) {
                    if ($existingIdoitHost['idoit_object_id'] == $object->global->objID) {
                        $existingObjects[] = $this->getHostnameFromIdoitObject($object);
                        $existingObjectIds[] = $existingIdoitHost['oitc_object_id'];
                        continue 2;
                    }
                }
                if ($this->getHostnameFromIdoitObject($object) !== null && strlen($this->getHostnameFromIdoitObject($object)) > 0) {
                    $objectsConf[$object->global->type->id]['title'] = $object->global->type->title;
                    $objects[$object->global->type->id][$object->global->objID]['name'] = $this->getHostnameFromIdoitObject($object);
                    $objects[$object->global->type->id][$object->global->objID]['linklist'] = false;
                    $objects[$object->global->type->id][$object->global->objID]['merge'] = '0';
                    foreach ($object->software as $singleSoftware) {
                        foreach ($softwareLinkList as $softwareLink) {
                            if (preg_match('/' . $softwareLink['Linklist']['regex'] . '/', $singleSoftware->application->title)) {
                                $objects[$object->global->type->id][$object->global->objID]['linklist'] = true;
                                break;
                            }
                        }
                    }
                    $checkExistingHosts[$object->global->type->id][$object->global->objID] = [
                        $this->getIpAddressFromIdoitObject($object),
                        $this->getHostnameFromIdoitObject($object)
                    ];

                    foreach ($configuration['ObjectItem'] as $obItem) {
                        if ($obItem['object_id'] == $object->global->objID) {
                            $objects[$object->global->type->id][$object->global->objID]['container_id'] = $obItem['container_id'];
                            $objects[$object->global->type->id][$object->global->objID]['hosttemplate_id'] = $obItem['hosttemplate_id'];
                            break;
                        }
                    }
                }
            }
        }

        if (!empty($checkExistingHosts)) {
            $conditionHostNames = [];
            $conditionHostAddresses = [];
            foreach ($checkExistingHosts as $checkTypes) {
                foreach ($checkTypes as $checkObjs) {
                    $conditionHostNames[] = $checkObjs[1];
                    $conditionHostAddresses[] = $checkObjs[0];
                }
            }
            $existingHostConditions = ['NOT' => ['Host.id' => $existingObjectIds]];
            switch (intval($configuration['Configuration']['host_mapping'])) {
                case Configuration::MAPPING_IP:
                    $existingHostConditions['Host.address'] = $conditionHostAddresses;
                    break;
                case Configuration::MAPPING_HOSTNAME:
                    $existingHostConditions['Host.name'] = $conditionHostNames;
                    break;
                default:
                    $existingHostConditions['Host.address'] = $conditionHostAddresses;
                    $existingHostConditions['Host.name'] = $conditionHostNames;
            }
            $existingHosts = $this->Host->find('all', [
                'recursive'  => -1,
                'fields'     => [
                    'Host.id',
                    'Host.name',
                    'Host.address',
                ],
                'conditions' => $existingHostConditions,
            ]);

            if (!empty($existingHosts)) {
                foreach ($existingHosts as $existingHost) {
                    foreach ($checkExistingHosts as $typeId => $checkTypes) {
                        foreach ($checkTypes as $objId => $checkObjs) {
                            switch (intval($configuration['Configuration']['host_mapping'])) {
                                case Configuration::MAPPING_IP:
                                    if ($existingHost['Host']['address'] == $checkObjs[0]) {
                                        $objects[$typeId][$objId]['merge'] = $existingHost['Host']['id'];
                                    }
                                    break;
                                case Configuration::MAPPING_HOSTNAME:
                                    if ($existingHost['Host']['name'] == $checkObjs[1]) {
                                        $objects[$typeId][$objId]['merge'] = $existingHost['Host']['id'];
                                    }
                                    break;
                                default:
                                    if ($existingHost['Host']['address'] == $checkObjs[0] && $existingHost['Host']['name'] == $checkObjs[1]) {
                                        $objects[$typeId][$objId]['merge'] = $existingHost['Host']['id'];
                                    }
                            }
                        }
                    }
                }
            }
        }

        $containers = $this->Tree->easyPath($this->MY_RIGHTS, OBJECT_HOST, [], $this->hasRootPrivileges, [CT_HOSTGROUP]);
        $hosttemplates = $this->Hosttemplate->hosttemplatesByContainerId($this->MY_RIGHTS, 'list');

        $this->set(compact([
            'configuration',
            'objects',
            'existingObjects',
            'hosttemplates',
            'containers',
            'objectsConf',
        ]));
    }

    public function delete($id = null) {
        if (!$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }
        if (!$this->Configuration->exists($id)) {
            throw new NotFoundException(__('Invalid i-doit configuration'));
        }

        if ($this->Configuration->delete($id)) {
            $this->setFlash(__('i-doit configuration deleted'));
            $this->redirect(['action' => 'index']);
        }

        $this->setFlash(__('Could not delete configuration'), false);
        $this->redirect(['action' => 'index']);
    }

    /**
     * @param $object
     * @return null|string
     */
    private function getHostnameFromIdoitObject($object) {
        //Workaorund for i-doit version 1.11.1 ITC-1898
        //Old i-doit versions
        $objectHostName = null;
        if (property_exists($object->ip, 'hostname')) {
            $objectHostName = $object->ip->hostname;
        }

        //new i-doit >= 1.1.11
        if (property_exists($object->ip, 'primary_hostname')) {
            $objectHostName = $object->ip->primary_hostname;
        }
        return $objectHostName;
    }

    /**
     * @param $object
     * @return null|string
     */
    private function getIpAddressFromIdoitObject($object) {
        //Workaorund for i-doit version 1.11.1 ITC-1898
        //Old i-doit versions
        $objectIdAddress = null;
        if (property_exists($object->ip, 'hostname')) {
            $objectIdAddress = $object->ip->ipv4_address->ref_title;
        }

        //new i-doit >= 1.1.11
        if (property_exists($object->ip, 'primary_hostname')) {
            $objectIdAddress = $object->ip->primary_hostaddress->ref_title;
        }
        return $objectIdAddress;
    }

}
