<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
class LinklistsController extends IdoitModuleAppController {
    public $layout = 'Admin.default';

    public $uses = [
        'IdoitModule.Linklist',
        'IdoitModule.Configuration',
        'Servicetemplategroup',
    ];

    public $components = [
        'RequestHandler',
        'ListFilter.ListFilter'
    ];

    public $helpers = [
        'ListFilter.ListFilter'
    ];

    public $listFilters = [
        'index' => [
            'fields' => [
                'Linklist.regex' => ['label' => 'Regex', 'searchType' => 'wildcard']
            ]
        ]
    ];

    public function index() {

        $options = [
            'contain' => [
                'Configuration',
                'Servicetemplategroup' => [
                    'Container',
                ],
            ],
            'order'   => [
                'Linklist.regex' => 'asc',
            ],
        ];
        $query = Hash::merge($this->Paginator->settings, $options);

        $all_links = $this->Linklist->find('all', $query);

        $this->set(compact(['all_links']));
    }

    public function add() {
        if ($this->request->is('post') || $this->request->is('put')) {
            $this->Linklist->create();
            if ($this->Linklist->save($this->request->data)) {
                $this->setFlash(__('Software-Link saved successfully'));
                $this->redirect(['action' => 'index']);

                return;
            }
            $this->setFlash(__('Data could not be saved'), false);
        }

        $configurations = $this->Configuration->find('list');
        $servicetemplategroups = $this->Servicetemplategroup->byContainerId(
            $this->MY_RIGHTS,
            'list',
            ['hasRootPrivileges' => $this->hasRootPrivileges],
            'parent_id'
        );

        $this->set(compact([
            'servicetemplategroups',
            'configurations',
        ]));
    }

    public function edit($id) {
        if (!$this->Linklist->exists($id)) {
            throw new NotFoundException(__('Invalid Software-Link'));
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            if ($this->Linklist->save($this->request->data)) {
                $this->setFlash(__('Software-Link saved successfully'));
                $this->redirect(['action' => 'index']);

                return;
            }
            $this->setFlash(__('Data could not be saved'), false);
        }

        $linklist = $this->Linklist->findById($id);
        $configurations = $this->Configuration->find('list');
        $servicetemplategroups = $this->Servicetemplategroup->byContainerId(
            $this->MY_RIGHTS,
            'list',
            ['hasRootPrivileges' => $this->hasRootPrivileges],
            'parent_id'
        );

        $this->request->data = Hash::merge($linklist, $this->request->data);
        $this->set(compact([
            'linklist',
            'servicetemplategroups',
            'configurations',
        ]));
    }

    public function delete($id = null) {
        if (!$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }
        if (!$this->Linklist->exists($id)) {
            throw new NotFoundException(__('Invalid Software-Link'));
        }

        if ($this->Linklist->delete($id)) {
            $this->setFlash(__('Software-Link deleted'));
            $this->redirect(['action' => 'index']);
        }
        $this->setFlash(__('Could not delete Software-Link'), false);
        $this->redirect(['action' => 'index']);
    }
}
