<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
App::uses('CakeTime', 'Utility');

class SynchronisationsController extends IdoitModuleAppController {

    public $layout = 'Admin.default';
    public $components = ['GearmanClient'];
    //Notice, most of this Models are required for the IdoitImport class
    public $uses = [
        'IdoitModule.Linklist',
        'IdoitModule.Configuration',
        'IdoitModule.IdoitMapping',
        'IdoitModule.JsonRPC',
        'IdoitModule.IdoitSyncStatus',
        'Host',
        'Servicetemplategroup',
        'Service',
        'Servicetemplate',
        'User',
        'Changelog',
        'Contact',
        'Contactgroup',
        MONITORING_HOSTSTATUS
    ];
    public $helpers = ['Time'];

    public function index() {
        // move objects from idoit_hosts to idoit_mappings with type TYPE_HOST if idoit_objects is empty
        $this->IdoitMapping->moveHostsToObjects();
    }

    public function sync() {
        $this->autoRender = false;
        Configure::load('gearman');
        $this->Config = Configure::read('gearman');
        $this->GearmanClient->client->doBackground("oitc_gearman", Security::cipher(serialize(['task' => 'idoit_sync', 'isCron' => false, 'authUser' => $this->Auth->user('id')]), $this->Config['password']));
    }

    public function getInfo() {
        $this->autoRender = false;
        $idoitInfo = $this->IdoitSyncStatus->find('first', [
            'order' => ['IdoitSyncStatus.id DESC']
        ]);
        if (!empty($idoitInfo)) {
            $idoitInfo = $this->IdoitSyncStatus->find('all', [
                'conditions' => ['IdoitSyncStatus.group' => $idoitInfo['IdoitSyncStatus']['group']]
            ]);
        }
        $finished = false;
        $result = [];
        if (!empty($idoitInfo)) {
            foreach ($idoitInfo as $idoitItem) {
                $result[] = [
                    'task'    => $idoitItem['IdoitSyncStatus']['task'],
                    'status'  => $idoitItem['IdoitSyncStatus']['status'],
                    'percent' => $idoitItem['IdoitSyncStatus']['percent'],
                    'created' => CakeTime::timeAgoInWords($idoitItem['IdoitSyncStatus']['created']),
                ];
                //CakeTime::format($idoitItem['IdoitSyncStatus']['created'], $this->Auth->user('dateformat'), false, $this->Auth->user('timezone'));
                if ($idoitItem['IdoitSyncStatus']['percent'] == '100') {
                    $finished = true;
                }
            }
        } else {
            $finished = true;
        }
        $resultArr = array_merge($result, ['finished' => $finished]);
        return json_encode($resultArr);
    }

    public function compare() {
        // move objects from idoit_hosts to idoit_mappings with type TYPE_HOST if idoit_objects is empty
        $this->IdoitMapping->moveHostsToObjects();

        $this->set('systemname', $this->systemname);
        $query = [
            'contain'    => [],
            'joins'      => [
                [
                    'table'      => 'idoit_mappings',
                    'type'       => 'LEFT OUTER',
                    'alias'      => 'IdoitMapping',
                    'conditions' => 'IdoitMapping.oitc_object_id = Host.id AND IdoitMapping.type=' . IdoitMapping::TYPE_HOST,
                ],
            ],
            'fields'     => [
                'Host.id',
                'Host.uuid',
                'Host.name',
                'IdoitMapping.id',
            ],
            'conditions' => [
                'IdoitMapping.id IS NULL',
                'Host.host_type' => GENERIC_HOST,
                'Host.disabled'  => 0,
            ],
            'order'      => [
                'Host.name' => 'asc',
            ],
        ];
        $this->Paginator->settings = array_merge($this->Paginator->settings, $query);
        $all_hosts = $this->Paginator->paginate('Host');

        $this->set('all_hosts', $all_hosts);
    }
}
