<?php

use itnovum\openITCOCKPIT\Core\DbBackend;
use itnovum\openITCOCKPIT\Core\HoststatusFields;

class IdoitImport {

    public $Systems = [];
    public $userId = 0;
    public $isCronjob = false;
    public $idoitState = [
        2 => 'ACTIVE',
        3 => 'ARCHIVED',
        4 => 'DELETED',
    ];

    private $Cake = null;
    private $_servicetemplategroupCache = [];
    private $idoitErrors = [];
    private $currentLogGroup;

    /**
     * @var DbBackend
     */
    private $DbBackend;

    public function __construct($cakeThis, DbBackend $DbBackend) {
        App::uses('UUID', 'Lib');
        $this->Cake = $cakeThis;
        $this->DbBackend = $DbBackend;
    }

    public function setUserId($userId) {
        $this->userId = $userId;
    }

    public function isCronjob() {
        $this->isCronjob = true;
    }

    public function getSystems() {
        if ($this->isCronjob === false) {
            return $this->Cake->Configuration->find('all', [
                'contain' => [
                    'Objecttype',
                ],
            ]);
        }

        return $this->Cake->Configuration->find('all', [
            'contain'    => [
                'Objecttype',
            ],
            'conditions' => [
                'Configuration.auto_sync' => 1,
            ],
        ]);
    }

    private function logStatus($task, $percent = 0, $status = 0) {
        $this->Cake->IdoitSyncStatus->updateAll(['IdoitSyncStatus.status' => 1], ['IdoitSyncStatus.status' => 0]);
        $idoitStatusData = [
            'IdoitSyncStatus' => [
                'task'    => $task,
                'status'  => $status,
                'percent' => $percent,
                'group'   => $this->currentLogGroup
            ]
        ];
        $this->Cake->IdoitSyncStatus->create();
        $this->Cake->IdoitSyncStatus->save($idoitStatusData);
    }

    public function runImport($import = false, $selectedItems = []) {
        $processingTasks = $this->Cake->IdoitSyncStatus->find('first', [
            'order' => ['IdoitSyncStatus.id DESC']
        ]);
        if (empty($processingTasks)) {
            $this->currentLogGroup = 1;
        } else {
            if ($processingTasks['IdoitSyncStatus']['status'] === '0') {
                return false;
            } else {
                $this->currentLogGroup = intval($processingTasks['IdoitSyncStatus']['group']) + 1;
            }
        }

        $this->logStatus(__('Initializing ' . ($import ? 'Import' : 'Synchronization')), 1);

        $this->Systems = $this->getSystems();

        $this->logStatus(__('Getting Hosts'), 10);

        $exsitingIdoitHosts = $this->getExistingIdoitHosts();
        $this->logStatus(__('Getting i-doit hosts'), 20);

        $hostsWithSoftware = $this->getHostsWithSoftware($exsitingIdoitHosts, $import, $selectedItems);
        if (!empty($this->idoitErrors)) {
            foreach ($this->idoitErrors as $idoitError) {
                $this->logStatus('<span style="color:red">' . $idoitError . '</span>', 25, 1);
            }
            $this->idoitErrors = [];
        }
        $this->logStatus(__('Building sofware links'), 30);

        $linkList = $this->buildSoftwareLinkList();
        $this->logStatus(__('Matching hosts with software links'), 40);

        $HostsWithServicegroup = $this->matchHostsWithSoftwareLinkList($linkList, $hostsWithSoftware, $exsitingIdoitHosts);
        $this->logStatus(__('Synchronizing hosts with service groups'), 50);

        $importedHosts = $this->createHostWithServices($HostsWithServicegroup, $selectedItems);
        $this->logStatus(__('Synchronizing additional objects'), 80);

        $this->createAdditionalObjects($importedHosts);
        $this->logStatus(__('Disabling service groups'), 90);

        $this->disableHosts($HostsWithServicegroup);

        $this->deleteHosts($HostsWithServicegroup);
        $this->logStatus(__('Done'), 100, 1);
    }

    public function createAdditionalObjects($importedHosts) {
        $contactsCounter = 0;
        $downtimesCounter = 0;
        $locationsCounter = 0;
        $allAdditionalObjecttypes = Configuration::setAdditionalObjecttypes();

        foreach ($importedHosts as $host) {
            $hostId = $host['host_id'];
            $hostUuid = $host['uuid'];
            $hostName = $host['host_name'];
            $objectId = $host['object_id'];
            $configId = $host['config_id'];
            $containerId = $host['container_id'];

            $configuration = $this->Cake->Configuration->findById($configId);
            if ($configuration['Configuration']['sync_additional_objecttypes'] === '0') {
                break;
            }
            $this->Cake->JsonRPC->init();
            $this->Cake->JsonRPC->setOptions([
                'url'    => $configuration['Configuration']['url'],
                'apikey' => $configuration['Configuration']['apikey'],
            ]);

            $currentAdditionalObjecttypes = $configuration['Configuration']['additional_objecttypes'];
            $currentAdditionalObjecttypesValues = [];
            foreach ($allAdditionalObjecttypes as $objecttypeName => $objecttypeCode) {
                if ($currentAdditionalObjecttypes & $objecttypeCode) {
                    $currentAdditionalObjecttypesValues[] = $objecttypeName;
                }
            }

            $additionalObjectsAllHosts = $this->Cake->JsonRPC->readReport($configuration['Configuration']['report_id']);

            if (!in_array($objectId, $allAdditionalObjecttypes)) {
                continue;
            }
            $additionalObjectsCurrentHost = $additionalObjectsAllHosts[$objectId];
            $objectIds = [];

            //Create contacts
            if (in_array("contact", $currentAdditionalObjecttypesValues) && !empty($additionalObjectsCurrentHost['contacts'])) {
                foreach ($additionalObjectsCurrentHost['contacts'] as $id => $contact) {
                    //Check if contact already is synchronised
                    $idoitContactExists = $this->Cake->IdoitMapping->find('all', [
                        'conditions' => [
                            'IdoitMapping.type'            => IdoitMapping::TYPE_CONTACT,
                            'IdoitMapping.idoit_object_id' => $id
                        ]
                    ]);

                    if (!empty($idoitContactExists)) {
                        //When idoit contact was already imported update the contact
                        $existingContact = $this->Cake->Contact->find('first', [
                            'conditions' => [
                                'Contact.id' => $idoitContactExists[0]['IdoitMapping']['oitc_object_id'],
                            ]
                        ]);

                        if ($existingContact['Contact']['name'] != $contact['name']) {
                            $this->Cake->Contact->saveField('name', $contact['name']);
                        }

                        if ($existingContact['Contact']['email'] != $contact['email_address']) {
                            $this->Cake->Contact->saveField('email', $contact['email_address']);
                        }

                        if ($existingContact['Contact']['phone'] != $contact['phone_number']) {
                            $this->Cake->Contact->saveField('phone', $contact['phone_number']);
                        }

                        //Map to host
                        $objectIds[] = $idoitContactExists[0]['IdoitMapping']['oitc_object_id'];
                        continue;
                    }
                    //Check if contact already exists in oitc
                    $oitcContactExists = $this->Cake->Contact->find('first', [
                        'contain'    => [],
                        'conditions' => [
                            'Contact.email' => $contact['email_address'],
                        ]
                    ]);

                    if (!empty($oitcContactExists)) {
                        //Link oitc contact to idoit contact
                        $objectIds[] = $oitcContactExists['Contact']['id'];
                        $this->addIdoitMappingToDatabase($id, $oitcContactExists['Contact']['id'], $configId, IdoitMapping::TYPE_CONTACT);
                        continue;
                    }

                    $contactData = [
                        'Contact'        => [
                            'uuid'                          => UUID::v4(),
                            'name'                          => $contact['name'],
                            'email'                         => $contact['email_address'],
                            'phone'                         => $contact['phone_number'],
                            'host_timeperiod_id'            => '1',
                            'host_notifications_enabled'    => '1',
                            'notify_host_recovery'          => '0',
                            'notify_host_down'              => '1',
                            'notify_host_unreachable'       => '0',
                            'notify_host_flapping'          => '0',
                            'notify_host_downtime'          => '0',
                            'HostCommands'                  => [
                                '1'
                            ],
                            'service_timeperiod_id'         => '1',
                            'service_notifications_enabled' => '1',
                            'notify_service_recovery'       => '0',
                            'notify_service_warning'        => '0',
                            'notify_service_unknown'        => '0',
                            'notify_service_critical'       => '1',
                            'notify_service_flapping'       => '0',
                            'notify_service_downtime'       => '0',
                            'ServiceCommands'               => [
                                '2'
                            ]
                        ],
                        'Container'      => [
                            'Container' => [
                                $containerId
                            ],
                        ],
                        'Customvariable' => []
                    ];
                    $this->Cake->Contact->create();
                    $result = $this->Cake->Contact->save($contactData);
                    $this->addIdoitMappingToDatabase($id, $result['Contact']['id'], $configId, IdoitMapping::TYPE_CONTACT);
                    $objectIds[] = $result['Contact']['id'];
                    $contactsCounter++;
                }
                $this->mapObjectToHost("Contact", $hostId, $objectIds);
            }

            //Create downtimes
            if (in_array("downtime", $currentAdditionalObjecttypesValues) && !empty($additionalObjectsCurrentHost['downtimes'])) {
                foreach ($additionalObjectsCurrentHost['downtimes'] as $id => $downtime) {
                    //Check if downtime already exists in mapping
                    $idoitDowntimeExists = $this->Cake->IdoitMapping->find('all', [
                        'conditions' => [
                            'IdoitMapping.type'            => IdoitMapping::TYPE_DOWNTIME,
                            'IdoitMapping.idoit_object_id' => $id
                        ]
                    ]);

                    if (empty($idoitDowntimeExists)) {
                        $payload = [
                            'hostUuid'     => $hostUuid,
                            'downtimetype' => 0,
                            'start'        => $downtime['starttime'],
                            'end'          => $downtime['endtime'],
                            'comment'      => $downtime['description'],
                            'author'       => 'Imported from i-doit',
                            'task'         => 'createHostDowntime'
                        ];
                        if ($this->checkIfHostIsInMonitoring($hostId)) {
                            $this->addIdoitMappingToDatabase($id, '0', $configId, IdoitMapping::TYPE_DOWNTIME);
                            $this->createDowntime($payload);
                            $downtimesCounter++;
                        } else {
                            $this->logStatus(__('Downtime %s for Host %s could not be imported because the Host is not monitored. Please run an export first.', $downtime['description'], $hostName), 90);
                        }
                    }
                }
            }

        }
        $this->logStatus(__('%s contacts were synchronized', $contactsCounter), 90);
        $this->logStatus(__('%s downtimes were synchronized', $downtimesCounter), 90);
        $this->logStatus(__('%s locations were synchronized', $locationsCounter), 90);
    }

    private function checkIfHostIsInMonitoring($hostId) {
        $hostUuid = $this->Cake->Host->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'Host.id' => $hostId,
            ],
            'fields'     => [
                'Host.uuid',
            ]

        ]);

        $HoststatusFields = new HoststatusFields($this->DbBackend);
        $HoststatusFields->currentState();
        $hoststatus = $this->Cake->Hoststatus->byUuid($hostUuid['Host']['uuid'], $HoststatusFields);
        if (empty($hoststatus)) {
            return false;
        }
        return true;
    }

    private function mapObjectToHost($object, $hostId, $objectIds) {
        $hostToObject = $this->Cake->Host->find('first', [
            'conditions' => [
                'Host.id' => $hostId,
            ]
        ]);
        $objectIds = array_merge($objectIds, $hostToObject[$object]);
        $hostToObject['Host']['own_contacts'] = (string)count($objectIds);
        $hostToObject[$object] = [
            $object => $objectIds
        ];
        $hostToObject['Host']['Hostgroup'] = [];
        $hostToObject['Parenthost'] = [
            'Parenthost' => ''
        ];
        $uuid = $hostToObject['Host']['uuid'];

        $hosttemplate = $this->Cake->Hosttemplate->findById($hostToObject['Host']['hosttemplate_id']);
        $data_to_save = $this->Cake->Host->prepareForSave($this->_diffWithTemplate($hostToObject, $hosttemplate), $hostToObject, 'edit');
        $data_to_save['Host']['uuid'] = $uuid;
        $data_to_save['Host']['disabled'] = '0';
        $data_to_save['Host']['description'] = null;


        $this->Cake->Host->saveAll($data_to_save);
    }

    private function addIdoitMappingToDatabase($idoitObjId, $oitcObjId, $configId, $type) {
        if (!is_null($oitcObjId)) {
            //Create record for idoit => oitc linking
            $data = [
                'IdoitMapping' => [
                    'configuration_id' => $configId,
                    'idoit_object_id'  => $idoitObjId,
                    'oitc_object_id'   => $oitcObjId,
                    'type'             => $type
                ],
            ];
            $this->Cake->IdoitMapping->create();
            $result = $this->Cake->IdoitMapping->save($data);
        }
    }

    //Fetchs all hosts from idoit RPC API
    public function getHostsWithSoftware($exsitingIdoitHosts = null, $import = false, $selectedItems) {
        $results = [];
        $hostsWithSoftware = [];
        $objecttypeToHosttemplate = [];
        $allObjects = 0;

        foreach ($this->Systems as $system) {
            $confId = $system['Configuration']['id'];
            $parsedUrl = parse_url($system['Configuration']['url']);


            foreach ($system['Objecttype'] as $hosttemplate) {
                $objecttypeToHosttemplate[$confId][$hosttemplate['objecttype_id']] = [
                    'hosttemplate_id' => $hosttemplate['hosttemplate_id'],
                    'container_id'    => $hosttemplate['container_id'],
                ];
            }
            $objectTypes = Hash::extract($system['Objecttype'], '{n}.objecttype_id');
            $this->Cake->JsonRPC->setOptions([
                'url'    => $system['Configuration']['url'],
                'apikey' => $system['Configuration']['apikey'],
            ]);
            $data = [
                'method' => 'cmdb.filter.getValidObjects',
                'params' => [
                    'objTypeID' => $objectTypes,
                ],
            ];

            $result = $this->Cake->JsonRPC->sendRequest($data);
            if ($result['json']['success'] === true) {

                $idoitStateToImport = array_keys($this->idoitState);

                foreach ($result['json']['result'] as $object) {

                    //This is a fallback id, but it should never be used!!
                    $hosttemplateId = $objecttypeToHosttemplate[$confId][$object->type]['hosttemplate_id'];
                    $hostContainerId = $objecttypeToHosttemplate[$confId][$object->type]['container_id'];


                    if (in_array($object->status, $idoitStateToImport)) {
                        $objectHostnameToCheck = null;
                        //Workaorund for i-doit version 1.11.1 ITC-1898
                        //Old i-doit versions
                        if (property_exists($object->ip, 'hostname')) {
                            $objectHostnameToCheck = $object->ip->hostname;
                        }

                        //new i-doit >= 1.1.11
                        if (property_exists($object->ip, 'primary_hostname')) {
                            $objectHostnameToCheck = $object->ip->primary_hostname;
                        }

                        if (isset($objectHostnameToCheck) && strlen($objectHostnameToCheck) > 0) {
                            if ($import) {
                                if (isset($exsitingIdoitHosts[$system['Configuration']['id']][$object->global->objID])) continue;
                                $met = false;
                                foreach ($selectedItems as $selectedItem) {
                                    if ($selectedItem['object_id'] == $object->global->objID) {
                                        $met = true;
                                        $hosttemplateId = $selectedItem['hosttemplate_id'];
                                        $hostContainerId = $selectedItem['container_id'];
                                    }
                                }
                                if (!$met) {
                                    continue;
                                }
                            } else {
                                if (!isset($exsitingIdoitHosts[$system['Configuration']['id']][$object->global->objID]))
                                    continue;
                            }
                            $myObjDescription = '';
                            if (strlen($object->ip->description) > 0) {
                                $myObjDescription = str_replace("\n", " ", $object->ip->description);
                                $myObjDescription = strip_tags($myObjDescription);
                                $myObjDescription = html_entity_decode($myObjDescription);
                            }

                            //Workaorund for i-doit version 1.11.1 ITC-1898
                            //Old i-doit versions
                            if (property_exists($object->ip, 'hostname')) {
                                $objectHostName = $object->ip->hostname;
                                $objectIpAddress = $object->ip->ipv4_address->ref_title;
                            }

                            //new i-doit >= 1.1.11
                            if (property_exists($object->ip, 'primary_hostname')) {
                                $objectHostName = $object->ip->primary_hostname;
                                $objectIpAddress = $object->ip->primary_hostaddress->ref_title;
                            }

                            $data = [
                                'Idoit' => [
                                    'objId'            => $object->global->objID,
                                    'status'           => $object->status,
                                    'status_name'      => $this->idoitState[$object->status],
                                    'configuration_id' => $system['Configuration']['id'],
                                ],
                                'Host'  => [
                                    'name'            => $objectHostName,
                                    'address'         => $objectIpAddress,
                                    'description'     => $myObjDescription,
                                    'hosttemplate_id' => $hosttemplateId,
                                    // This is the container_id that should only used to create the host, not for update statements!!!
                                    'container_id'    => $hostContainerId,
                                ],
                            ];
                            $allObjects++;

                            foreach ($object->software as $software) {


                                $data['Software'][] = [
                                    'name' => $software->application->title,
                                    'type' => $software->application->type_title,
                                ];
                            }
                            $hostsWithSoftware[] = $data;
                        }
                    }
                }
            } else {
                $this->idoitErrors[] = 'Connecting to ' . $system['Configuration']['url'] . ' failed: ' . $result['reasonPhrase'];
                error_log(date('m-d-Y H:i:s') . ': i-doit connect to ' . $system['Configuration']['name'] . '[' . $system['Configuration']['url'] . ']: ' . $result['reasonPhrase'] . "\n", 3, '/var/log/nginx/cake/error.log');
            }
        }
        $this->logStatus(__('Received %s objects', $allObjects), 25, 1);

        return $hostsWithSoftware;
    }

    public function buildSoftwareLinkList() {
        $linkList = [];
        foreach ($this->Systems as $system) {
            $linkList[$system['Configuration']['id']] = [];
            $systemLinklist = $this->Cake->Linklist->find('all', [
                'conditions' => [
                    'Linklist.configuration_id' => $system['Configuration']['id'],
                ],
            ]);

            foreach ($systemLinklist as $link) {
                $linkList[$system['Configuration']['id']][] = [
                    $link['Linklist']['regex'] => $link['Servicetemplategroup']['id'],
                ];
            }
        }

        return $linkList;
    }

    //Match the software list of idoit with servicetemplate groups
    public function matchHostsWithSoftwareLinkList($linkList, $hostsWithSoftware, $exsitingIdoitHosts) {

        $HostsWithServicegroup = [
            'ACTIVE'   => [],
            'ARCHIVED' => [],
            'DELETED'  => [],
        ];
        foreach ($hostsWithSoftware as $host) {
            //Check if the host allready exist in openITCOCKPIT by idoits objId
            $configId = $host['Idoit']['configuration_id'];
            if (isset($exsitingIdoitHosts[$configId][$host['Idoit']['objId']])) {
                $host['Host']['id'] = $exsitingIdoitHosts[$configId][$host['Idoit']['objId']];
            }

            switch ($host['Idoit']['status_name']) {
                case 'ACTIVE':
                    if (isset($linkList[$configId])) {
                        $servicetemplategroups = [];

                        foreach ($linkList[$configId] as $link) {
                            foreach ($link as $regex => $servicegroupId) {
                                foreach ($host['Software'] as $software) {
                                    if (preg_match('/' . $regex . '/', $software['name'])) {
                                        //Only add each servicetemplategroup once
                                        if (!in_array($servicegroupId, $servicetemplategroups)) {
                                            $servicetemplategroups[] = $servicegroupId;
                                        }
                                    }
                                }
                            }
                        }

                        //Remove every host, were no matchinf software was found
                        if (!empty($servicetemplategroups)) {
                            $HostsWithServicegroup['ACTIVE'][$configId][] = [
                                'Idoit'                => $host['Idoit'],
                                'Host'                 => $host['Host'],
                                'Servicetemplategroup' => $servicetemplategroups,
                            ];
                        }
                    }
                    break;

                case 'ARCHIVED':
                    $HostsWithServicegroup['ARCHIVED'][$configId][] = [
                        'Host'  => $host['Host'],
                        'Idoit' => $host['Idoit'],
                    ];
                    break;

                case 'DELETED':
                    $HostsWithServicegroup['DELETED'][$configId][] = [
                        'Host'  => $host['Host'],
                        'Idoit' => $host['Idoit'],
                    ];
                    break;
            }
        }

        return $HostsWithServicegroup;
    }

    //Get all hosts that were created by idoit some day...
    public function getExistingIdoitHosts() {
        $IdoitHostsTmp = $this->Cake->IdoitMapping->find('all', [
            'contain'   => [],
            'condition' => [
                'IdoitMapping.type' => IdoitMapping::TYPE_HOST
            ]
        ]);
        $IdoitHosts = [];
        foreach ($IdoitHostsTmp as $host) {
            $configId = $host['IdoitMapping']['configuration_id'];
            $objectId = $host['IdoitMapping']['idoit_object_id'];
            $IdoitHosts[$configId][$objectId] = $host['IdoitMapping']['oitc_object_id'];
        }
        return $IdoitHosts;
    }

    public function createHostWithServices($HostsWithServicegroup, $selectedItems = []) {
        if (!isset($HostsWithServicegroup['ACTIVE'])) return [];
        $allObjects = $syncd = $added = $mapped = 0;
        $importedHosts = [];
        foreach ($HostsWithServicegroup['ACTIVE'] as $configId => $IdoitHosts) {
            foreach ($IdoitHosts as $key => $IdoitHost) {
                $allObjects++;
                $mergeId = '0';
                foreach ($selectedItems as $selectedItem) {
                    if ($selectedItem['object_id'] == $IdoitHost['Idoit']['objId'] && $selectedItem['merge'] !== '0') {
                        $mergeId = $selectedItem['merge'];
                        break;
                    }
                }
                $servicetemplatesToCreate = $this->getServicesFromTemplategroups($IdoitHost['Servicetemplategroup']);
                //Check if the host already exists in openITCOCKPIT
                if (isset($IdoitHost['Host']['id'])) {
                    //Host exists, run update
                    $host = $this->Cake->Host->find('first', [
                        'conditions' => [
                            'Host.id' => $IdoitHost['Host']['id'],
                        ],
                        'contain'    => [
                            'Service' => [
                                'fields' => [
                                    'Service.id',
                                    'Service.servicetemplate_id',
                                ],
                            ],
                        ],
                        'fields'     => [
                            'Host.id',
                            'Host.uuid',
                            'Host.container_id',
                            'Host.name',
                            'Host.address',
                            'Host.description',
                        ],
                    ]);

                    $this->Cake->Host->id = $IdoitHost['Host']['id'];
                    $edited = false;
                    if ($host['Host']['name'] != $IdoitHost['Host']['name']) {
                        $this->Cake->Host->saveField('name', $IdoitHost['Host']['name']);
                        $edited = true;
                    }
                    if ($host['Host']['address'] != $IdoitHost['Host']['address']) {
                        $this->Cake->Host->saveField('address', $IdoitHost['Host']['address']);
                        $edited = true;
                    }
                    if ($host['Host']['description'] != $IdoitHost['Host']['description']) {
                        $this->Cake->Host->saveField('description', $IdoitHost['Host']['description']);
                        $edited = true;
                    }

                    $importedHosts[] = [
                        'object_id'    => $IdoitHost['Idoit']['objId'],
                        'host_id'      => $IdoitHost['Host']['id'],
                        'config_id'    => $configId,
                        'container_id' => $IdoitHost['Host']['container_id'],
                        'uuid'         => $host['Host']['uuid'],
                        'host_name'    => $IdoitHost['Host']['name']
                    ];

                    if ($edited) {
                        $syncd++;
                        $changelog_data = $this->Cake->Changelog->parseDataForChangelog(
                            'edit',
                            'Host',
                            $host['Host']['id'],
                            OBJECT_HOST,
                            $host['Host']['container_id'],
                            $this->userId,
                            $IdoitHost['Host']['name'] . ' (from i-doit)',
                            $IdoitHost,
                            $host
                        );
                        if (!empty($changelog_data)) {
                            CakeLog::write('log', serialize($changelog_data));
                        }
                    }

                    //Remove all servicetemplates that already exists on this host
                    foreach ($host['Service'] as $service) {
                        if (isset($servicetemplatesToCreate[$service['servicetemplate_id']])) {
                            unset($servicetemplatesToCreate[$service['servicetemplate_id']]);
                        }
                    }
                } else {
                    //We need to create the host first, than add the services
                    $hostId = null;
                    if ($mergeId === '0') {
                        $hostData = [
                            'Host'       => [
                                'uuid'              => UUID::v4(),
                                'name'              => $IdoitHost['Host']['name'],
                                'hosttemplate_id'   => $IdoitHost['Host']['hosttemplate_id'],
                                'container_id'      => $IdoitHost['Host']['container_id'],
                                'address'           => $IdoitHost['Host']['address'],
                                'description'       => $IdoitHost['Host']['description'],
                                'host_type'         => GENERIC_HOST,
                                'own_contacts'      => 0,
                                'own_contactgroups' => 0,
                                'Contact'           => [],
                            ],
                            'Container'  => [
                                'id' => $IdoitHost['Host']['container_id'],
                            ],
                            'Parenthost' => [
                                'Parenthost' => [],
                            ],
                            'Hostgroup'  => [
                                'Hostgroup' => [],
                            ],
                        ];
                        $this->Cake->Host->create();
                        $result = $this->Cake->Host->save($hostData);
                        $added++;
                        if (isset($result['Host']['id'])) {
                            $changelog_data = $this->Cake->Changelog->parseDataForChangelog(
                                'add',
                                'Host',
                                $result['Host']['id'],
                                OBJECT_HOST,
                                $IdoitHost['Host']['container_id'],
                                $this->userId,
                                $IdoitHost['Host']['name'] . ' (from i-doit)',
                                ['Host' => ['name' => $IdoitHost['Host']['name'], 'description' => $IdoitHost['Host']['description'], 'address' => $IdoitHost['Host']['address']]]
                            );
                            CakeLog::write('log', serialize($changelog_data));

                            $hostId = $result['Host']['id'];
                            $importedHosts[] = [
                                'object_id'    => $IdoitHost['Idoit']['objId'],
                                'host_id'      => $hostId,
                                'config_id'    => $configId,
                                'container_id' => $IdoitHost['Host']['container_id'],
                                'uuid'         => $result['Host']['uuid'],
                                'host_name'    => $IdoitHost['Host']['name']
                            ];
                        }
                    } else {
                        $hostId = $mergeId;
                    }

                    if (!is_null($hostId)) {
                        $this->addIdoitMappingToDatabase($IdoitHost['Idoit']['objId'], $hostId, $configId, IdoitMapping::TYPE_HOST);
                        if ($mergeId !== '0') {
                            $mapped++;
                        }
                    }
                }
                //Create missing services - if any
                if (!empty($servicetemplatesToCreate) && is_array($servicetemplatesToCreate) && !empty($hostId) && $mergeId === '0') {
                    foreach ($servicetemplatesToCreate as $servicetemplateToCreate) {
                        $service = [];
                        $service['Service']['uuid'] = UUID::v4();
                        $service['Service']['name'] = null;
                        $service['Service']['host_id'] = $hostId;
                        $service['Service']['servicetemplate_id'] = $servicetemplateToCreate['Servicetemplate']['id'];

                        //$service['Service']['Contact'] = $servicetemplateToCreate['Contact'];
                        //$service['Service']['Contactgroup'] = $servicetemplateToCreate['Contactgroup'];

                        $service['Contact']['Contact'] = [];
                        $service['Contactgroup']['Contactgroup'] = [];
                        $service['Servicegroup']['Servicegroup'] = [];

                        $service['Contact']['Contact'] = $service['Contact'];
                        $service['Contactgroup']['Contactgroup'] = $service['Contactgroup'];

                        $service['Service']['own_contacts'] = 0;
                        $service['Service']['own_contactgroups'] = 0;

                        //$data_to_save = $this->Cake->Service->prepareForSave([], $service, 'add');
                        $this->Cake->Service->create();
                        $this->Cake->Service->saveAll($service);
                    }
                }
            }
        }
        if (!empty($syncd)) {
            $this->logStatus(__('%s hosts were synchronized', $syncd . ' / ' . $allObjects), 80);
        }
        if (!empty($added)) {
            $this->logStatus(__('%s hosts were imported', $added . ' / ' . $allObjects), 80);
        }
        if (!empty($mapped)) {
            $this->logStatus(__('%s hosts were mapped', $mapped . ' / ' . $allObjects), 80);
        }
        if (empty($syncd) && empty($added) && empty($mapped)) {
            $this->logStatus(__('No hosts were imported/synchronized'), 80);
        }
        return $importedHosts;
    }

    public function getServicesFromTemplategroups($servicetemplategroups) {
        if (!is_array($servicetemplategroups)) {
            $servicetemplategroups = [$servicetemplategroups];
        }

        $servicetemplates = [];

        foreach ($servicetemplategroups as $servicetemplategroupId) {
            if (!isset($this->_servicetemplategroupCache[$servicetemplategroupId])) {
                $result = $this->Cake->Servicetemplategroup->find('first', [
                    'conditions' => [
                        'id' => $servicetemplategroupId,
                    ],
                    'contain'    => [
                        'Servicetemplate' => [
                            'fields' => [
                                'Servicetemplate.id',
                            ],
                        ],
                    ],
                ]);

                if (isset($result['Servicetemplate']) && !empty($result['Servicetemplate'])) {
                    $result = $this->Cake->Servicetemplate->find('all', [
                        'conditions' => [
                            'Servicetemplate.id' => Hash::extract($result['Servicetemplate'], '{n}.id'),
                        ],
                        'contain'    => [
                            'Contact',
                            'Contactgroup',
                        ],
                    ]);
                    $this->_servicetemplategroupCache[$servicetemplategroupId] = $result;
                }
            }

            foreach ($this->_servicetemplategroupCache[$servicetemplategroupId] as $servicetemplate) {
                if (!isset($servicetemplates[$servicetemplate['Servicetemplate']['id']])) {
                    $servicetemplates[$servicetemplate['Servicetemplate']['id']] = $servicetemplate;
                }
            }
        }

        return $servicetemplates;
    }

    public function disableHosts($HostsWithServicegroup) {
        if (isset($HostsWithServicegroup['ARCHIVED'])) {
            foreach ($HostsWithServicegroup['ARCHIVED'] as $configId => $IdoitHosts) {
                foreach ($IdoitHosts as $IdoitHost) {
                    if (isset($IdoitHost['Host']['id'])) {
                        if ($this->Cake->Host->exists($IdoitHost['Host']['id'])) {
                            $host = $this->Cake->Host->find('first', [
                                'conditions' => [
                                    'Host.id' => $IdoitHost['Host']['id'],
                                ],
                                'fields'     => [
                                    'Host.disabled',
                                ],
                                'contain'    => [],
                            ]);
                            if ($host['Host']['disabled'] == 0) {
                                $this->Cake->Host->id = $IdoitHost['Host']['id'];
                                $this->Cake->Host->saveField('disabled', 1);
                                $this->Cake->Service->updateAll(['Service.disabled' => 1], ['Service.host_id' => $IdoitHost['Host']['id']]);
                            }
                        }
                    }
                }
            }
        }
    }

    public function deleteHosts($HostsWithServicegroup) {
        if (isset($HostsWithServicegroup['DELETED'])) {
            foreach ($HostsWithServicegroup['DELETED'] as $configId => $IdoitHosts) {
                foreach ($IdoitHosts as $IdoitHost) {
                    if (isset($IdoitHost['Host']['id'])) {
                        if ($this->Cake->Host->exists($IdoitHost['Host']['id'])) {
                            $host = $this->Cake->Host->findById($IdoitHost['Host']['id']);
                            $this->Cake->Host->__delete($host, $this->userId);
                        }
                    }
                }
            }
        }
    }

    private function _diffWithTemplate($host, $hosttemplate) {
        $diff_array = [];
        //Host-/Hosttemplate fields
        $fields = [
            'description',
            'command_id',
            'check_interval',
            'retry_interval',
            'max_check_attempts',
            'notification_interval',
            'notify_on_down',
            'notify_on_unreachable',
            'notify_on_recovery',
            'notify_on_flapping',
            'notify_on_downtime',
            'flap_detection_enabled',
            'flap_detection_on_up',
            'flap_detection_on_down',
            'flap_detection_on_unreachable',
            'notes',
            'priority',
            'check_period_id',
            'notify_period_id',
            'tags',
            'active_checks_enabled',
            'host_url'
        ];
        $compare_array = [
            'Host'         => [
                ['Host.{(' . implode('|', array_values(Hash::merge($fields, ['name', 'description', 'address', 'satellite_id', 'host_type']))) . ')}', false],
                ['{^Contact$}.{^Contact$}.{n}', false],
                ['{^Contactgroup$}.{^Contactgroup$}.{n}', false],
                ['{^Hostgroup$}.{^Hostgroup$}.{n}', false],
                ['Hostcommandargumentvalue.{n}.{(commandargument_id|value|id)}', false],
            ],
            'Hosttemplate' => [
                ['Hosttemplate.{(' . implode('|', array_values($fields)) . ')}', false],
                ['{^Contact$}.{n}.id', true],
                ['{^Contactgroup$}.{n}.id', true],
                ['{^Hostgroup$}.{n}.id', true],
                ['Hosttemplatecommandargumentvalue.{n}.{(commandargument_id|value)}', false],
            ],
        ];
        $diff_array = [];
        foreach ($compare_array['Host'] as $key => $data) {
            $extractPath = $compare_array['Hosttemplate'][$key][0];
            if ($data[0] == 'Hostcommandargumentvalue.{n}.{(commandargument_id|value|id)}') {
                if (isset($host['Hostcommandargumentvalue'])) {
                    if (!empty(Hash::diff(Set::classicExtract($host, $data[0]), Set::classicExtract($hosttemplate, $compare_array['Hosttemplate'][$key][0])))) {
                        $diff_data = Set::classicExtract($host, $data[0]);
                        $diff_array['Hostcommandargumentvalue'] = $diff_data;
                    }
                }
            } else {
                //$Key for DiffArray with preg_replace ==>  from 'Customvariable.{n}.{(name|value)}'' to 'Customvariable'
                $possible_key = preg_replace('/(\{.*\})|(\.)/', '', $data[0]);
                $diff_data = $this->Cake->Host->getDiffAsArray($this->Cake->Host->prepareForCompare(Set::classicExtract($host, $data[0]), $data[1]),
                    $this->Cake->Host->prepareForCompare(Set::classicExtract($hosttemplate, $compare_array['Hosttemplate'][$key][0]),
                        $compare_array['Hosttemplate'][$key][1]));
                if (!empty($diff_data)) {
                    $diff_array = Hash::merge($diff_array, (!empty($possible_key)) ? [$possible_key => $diff_data] : $diff_data);
                }
            }
        }
        return $diff_array;
    }

    private function createDowntime($payload) {
        Configure::load('gearman');
        $config = Configure::read('gearman');

        $gearmanClient = new GearmanClient();
        $gearmanClient->addServer($config['address'], $config['port']);
        if ($config['encryption'] === true) {
            $gearmanClient->doBackground('oitc_gearman', Security::cipher(serialize($payload), $config['password']));
        } else {
            $gearmanClient->doBackground('oitc_gearman', serialize($payload));
        }
    }
}
