<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement. 
// The license agreement and license key were sent with the order confirmation.
class Configuration extends IdoitModuleAppModel {

    const MAPPING_IP_HOSTNAME = 1;
    const MAPPING_IP = 2;
    const MAPPING_HOSTNAME = 3;

    const ADDITIONAL_OBJECT_CONTACT = 'contact';
    const ADDITIONAL_OBJECT_DOWNTIME = 'downtime';
    const ADDITIONAL_OBJECT_LOCATION = 'location';

    public $useTable = 'idoit_configurations';
    public $hasMany = [
        'Objecttype'   => [
            'className'  => 'IdoitModule.Objecttype',
            'foreignKey' => 'configuration_id',
            'dependent'  => true,
        ],
        'ObjectItem'   => [
            'className'  => 'IdoitModule.ObjectItem',
            'foreignKey' => 'configuration_id',
            'dependent'  => true,
        ],
        'IdoitMapping' => [
            'className'  => 'IdoitModule.IdoitMapping',
            'foreignKey' => 'configuration_id',
            'dependent'  => true,
        ],
        'IdoitHost'    => [
            'className'  => 'IdoitModule.IdoitMapping',
            'foreignKey' => 'configuration_id',
            'dependent'  => true,
            'conditions' => ['IdoitHost.type' => 1] // TYPE_HOST, cannot refer to const from another model
        ],
        'Linklist'     => [
            'className'  => 'IdoitModule.Linklist',
            'foreignKey' => 'configuration_id',
            'dependent'  => true,
        ],
    ];
    public $validate = [
        'name'                       => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true,
            ],
        ],
        'url'                        => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true,
            ],
            'url'      => [
                'rule'    => ['url', true],
                'message' => 'This field needs to ba a valid URL.',
            ],
        ],
        'apikey'                     => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true,
            ],
        ],
        'report_id'                  => [
            'notBlank' => [
                'rule'     => ['notEmptyReportId', ''],
                'message'  => 'This field cannot be left blank if additional objecttypes should be synced',
                'required' => true,
            ],
        ],
        'additional_objecttypes'     => [
            'notBlank' => [
                'rule'     => ['notEmptyAdditionalObjecttypes', ''],
                'message'  => 'You must select at least one additional objecttype',
                'required' => true,
            ],
        ],
        'additional_infos_report_id' => [
            'notBlank' => [
                'rule'     => ['notEmptyAdditionalInfosReportId', ''],
                'message'  => 'This field cannot be left blank if additional informations should be synced',
                'required' => true,
            ],
        ],
    ];

    public function notEmptyReportId() {
        if (($this->data['Configuration']['sync_additional_objecttypes'] === '1')) {
            if (empty($this->data['Configuration']['report_id'])) {
                return false;
            } else {
                return true;
            }
        } else {
            return true;
        }
    }

    public function notEmptyAdditionalInfosReportId() {
        if (($this->data['Configuration']['sync_additional_informations'] === '1')) {
            if (empty($this->data['Configuration']['additional_infos_report_id'])) {
                return false;
            } else {
                return true;
            }
        } else {
            return true;
        }
    }

    public function notEmptyAdditionalObjecttypes() {
        if ($this->data['Configuration']['sync_additional_objecttypes'] == '1' && empty($this->data['Configuration']['additional_objecttypes'])) {
            return false;
        } else {
            return true;
        }
    }

    public static function getMappingList() {
        return [
            self::MAPPING_IP_HOSTNAME => __('On IP address and Hostname'),
            self::MAPPING_IP          => __('On IP address'),
            self::MAPPING_HOSTNAME    => __('On Hostname')
        ];
    }

    public static function getAdditionalObejctList() {
        return [
            self::ADDITIONAL_OBJECT_CONTACT  => __('Contact'),
            self::ADDITIONAL_OBJECT_DOWNTIME => __('Downtime'),
        ];
    }

    public static function setAdditionalObjecttypes() {
        $additionalObjecttypesMapping = [
            'contact'      => 1 << 0,
            'contactgroup' => 1 << 1,
            'downtime'     => 1 << 2,
            'location'     => 1 << 3
        ];
        return $additionalObjecttypesMapping;
    }
}
