<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement. 
// The license agreement and license key were sent with the order confirmation.
use GuzzleHttp\Client;

class JsonRPC extends IdoitModuleAppModel {

    public $useTable = false;
    public $isInitialized = false;
    public $options = [];
    public $lastError = '';
    public $lastIdoitError = '';

    /**
     * @var Client
     */
    private $Http;

    public function init() {
        $this->Http = new Client([
            'verify' => false
        ]);

        $this->isInitialized = true;
    }

    public function setOptions($options) {
        $this->options = $options;
    }

    public function lastError() {
        return $this->lastError;
    }

    public function lastIdoitError() {
        return $this->lastIdoitError;
    }

    public function version() {
        $data = [
            'method' => 'idoit.version',
            'params' => [],
        ];

        $result = $this->sendRequest($data);

        if ($result['success'] === true && $result['json']['success'] === true) {
            return [
                'success' => true,
                'version' => $result['json']['result']['version'],
            ];
        }

        if ($this->lastError !== '') {
            return [
                'success' => false,
                'message' => $this->lastError,
            ];
        }

        if ($this->lastIdoitError !== '') {
            return [
                'success' => false,
                'message' => $this->lastIdoitError,
            ];
        }

    }

    public function checkReport($reportId, $type) {
        $data = [
            'method' => 'cmdb.reports.read',
            'params' => [
                'id' => $reportId
            ]
        ];
        $result = $this->sendRequest($data);
        if ($result['json']['success'] === true) {
            $reportExists = [
                'success' => true,
                'message' => 'Report could read successfully.',
            ];
        } else {
            $reportExists = [
                'success' => false,
                'message' => 'Report for ' . $type . ' could not be read. Make sure your entered the correct report id.',
            ];
        }
        return $reportExists;
    }

    public function readReport($reportId) {
        $data = [
            'method' => 'cmdb.reports.read',
            'params' => [
                'id' => $reportId
            ]
        ];
        $result = $this->sendRequest($data);
        $resultArray = [];
        foreach ($result['json']['result'] as $idoitObject) {
            $hostId = $idoitObject->host_id;

            $resultArray[$hostId]['hostname'] = $idoitObject->host_name;
            if (empty($idoitObject->contact_id)) {
                $resultArray[$hostId]['contacts'] = [];
            } else {
                $resultArray[$hostId]['contacts'][$idoitObject->contact_id]['name'] = $idoitObject->contact_name;
                $resultArray[$hostId]['contacts'][$idoitObject->contact_id]['email_address'] = strip_tags($idoitObject->contact_email);
                $resultArray[$hostId]['contacts'][$idoitObject->contact_id]['phone_number'] = $idoitObject->contact_phone;
            }
            $locations = [];
            if (!empty($idoitObject->location_id)) {
                $locationPath = strip_tags($idoitObject->location_path);
                $locations = [];
                if (substr($locationPath, 0, 3) === " > ") {
                    $locationNameIdArray = explode("objID=", $locationPath);
                    $locationNameArray = explode(" > ", $locationPath);
                    $cntLocation = 0;

                    foreach ($locationNameIdArray as $locationNameId) {
                        $id = substr($locationNameId, 0, strpos($locationNameId, "'}"));
                        $name = substr($locationNameArray[$cntLocation], 0, strpos($locationNameArray[$cntLocation], "if"));
                        if (!empty($name)) {
                            $locations[$id] = $name;
                        }
                        $cntLocation++;
                    }
                } else {
                    $locations[0] = $locationPath;
                }
            }
            $resultArray[$hostId]['locations'] = $locations;

            if (!empty($idoitObject->downtime_id)) {
                $resultArray[$hostId]['downtimes'][$idoitObject->downtime_id]['description'] = trim(strip_tags($idoitObject->downtime_description));
                $resultArray[$hostId]['downtimes'][$idoitObject->downtime_id]['starttime'] = $idoitObject->downtime_start;
                $resultArray[$hostId]['downtimes'][$idoitObject->downtime_id]['endtime'] = $idoitObject->downtime_end;
            } else {
                $resultArray[$hostId]['downtimes'] = [];
            }
        }
        return $resultArray;
    }

    public function readAdditionalInfosReport($reportId, $idoitId) {
        $data = [
            'method' => 'cmdb.reports.read',
            'params' => [
                'id' => $reportId
            ]
        ];
        $result = $this->sendRequest($data);
        if ($result['json']['success'] === true) {
            foreach ($result['json']['result'] as $idoitObject) {
                if ($idoitObject->ID === $idoitId) {
                    return $idoitObject;
                }
            }
        }

        return null;

    }


    public function test() {
        $data = [
            'method' => 'cmdb.objects.read',
            'params' => [
                'filter' => [
                    'ids' => [1],
                ],
            ],
        ];

        $result = $this->sendRequest($data);

        if ($result['success'] === true && $result['json']['success'] === true) {
            return [
                'success' => true,
                'message' => __('Connection established successfully'),
            ];
        }

        if ($this->lastError !== '') {
            return [
                'success' => false,
                'message' => $this->lastError,
            ];
        }

        if ($this->lastIdoitError !== '') {
            return [
                'success' => false,
                'message' => $this->lastIdoitError,
            ];
        }

    }

    public function sendRequest($data = [], $options = []) {
        if (!$this->isInitialized) {
            $this->init();
        }
        $_data = [
            'version' => '2.0',
            'params'  => [
                'apikey' => $this->options['apikey'],
            ],
            'id' => 1,
        ];

        $data = Hash::merge($_data, $data);

        $_options = [
            'headers'         => [
                'Accept'       => 'application/xml',
                'Content-Type' => 'application/json; charset=UTF-8',
            ],
            'allow_redirects' => true
        ];
        $requestOptions = Hash::merge($_options, $options);

        try {
            $requestOptions['json'] = $data;

            $response = $this->Http->post($this->options['url'], $requestOptions);
            $responseBodyIsEmpty = false;
            if ($response->getStatusCode() == 200) {
                $responseBody = json_decode($response->getBody()->getContents());
                if (!empty($responseBody)) {
                    if (property_exists($responseBody, 'error')) {
                        $json = [
                            'success' => false,
                            'code'    => $responseBody->error->code,
                            'message' => $responseBody->error->message,
                            'data'    => (array)$responseBody->error->data,
                        ];

                        foreach ($json['data'] as $error)
                            $this->lastIdoitError = $error;

                    }

                    if (property_exists($responseBody, 'result')) {
                        $json = [
                            'success' => true,
                            'result'  => (array)$responseBody->result,
                        ];
                    }

                    return [
                        'body'         => $response->getBody()->getContents(),
                        'json'         => $json,
                        'success'      => true,
                        'code'         => $response->getStatusCode(),
                        'reasonPhrase' => $response->getReasonPhrase(),
                    ];
                } else {
                    $responseBodyIsEmpty = true;
                }
            }
            $errorMsg = $response->getStatusCode() . ' ' . $response->getReasonPhrase();

            if ($responseBodyIsEmpty) {
                $errorMsg = $response->getStatusCode() . ' ' . $response->getReasonPhrase() . __(' Something went wrong! Maybe missing URL Parameters ?');
            }

            $this->lastError = $errorMsg;

            return [
                'body'         => $response->getBody()->getContents(),
                'json'         => [
                    'success' => false,
                ],
                'success'      => false,
                'code'         => $response->getStatusCode(),
                'reasonPhrase' => $response->getReasonPhrase(),
            ];

        } catch (Exception $e) {
            $this->lastError = $e->getMessage();

            return [
                'body'         => null,
                'json'         => [
                    'success' => false,
                ],
                'success'      => false,
                'code'         => null,
                'reasonPhrase' => $this->lastError,
            ];
        }

    }

}
