<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.
use itnovum\openITCOCKPIT\ImportTemplates\ImportTemplates;

/**
 * Class PostgresInstallShell
 * uses CakePHP to insert Linux basic monitoring Templates data into the Database
 *
 * usage: oitc LinuxBasicMonitoring_module.LinuxBasicMonitoring_install
 */
class LinuxBasicMonitoringInstallShell extends AppShell {
    public $uses = [
        'Servicetemplate',
        'Servicetemplategroup',
        'Hosttemplate',
        'Contact',
        'Command',
        'Commandargument',
        'Macro',
    ];
    public $path = '';
    private $importer;

    public function main() {
        $this->stdout->styles('success', ['text' => 'green']);
        $this->out('Installing Linux basic monitoring Module');
        $this->path = APP . 'Plugin' . DS . 'LinuxBasicMonitoringModule' . DS . 'webroot' . DS;
        $jsonDataPath = 'jsonData' . DS;
        $importer = new ImportTemplates($this->Servicetemplate, $this->Servicetemplategroup, $this->Hosttemplate, $this->Contact, $this->Command, $this->Commandargument, $this->Macro);
        try {
            $this->out('<info>Checking dependencies for installing Templates</info>');
            $importer->checkDependencies();
            $this->out('<success>done</success>');
            $iterator = new DirectoryIterator($this->path . $jsonDataPath);
            $files = [];
            foreach ($iterator as $file) {
                if (!$file->isDot()) {
                    $filename = $file->getFilename();
                    if (preg_match('/.+_command_data.json/', $filename)) {
                        $key = 'command';
                    } else if (preg_match('/.+_servicetemplate_data.json/', $filename)) {
                        $key = 'servicetemplate';
                    } else if (preg_match('/.+_servicetemplategroup_data.json/', $filename)) {
                        $key = 'servicetemplategroup';
                    } else {
                        continue;
                    }
                    $files[$key] = $file->getFilename();
                }
            }
        } catch (Exception $e) {
            $this->hr(1);
            $this->error($e->getMessage());
        }
        if (empty($files['command'])) {
            $this->error('no Commands found');
        }

        if (empty($files['servicetemplate'])) {
            $this->error('no Template data found');
        }

        if (empty($files['servicetemplategroup'])) {
            $this->error('no Servicetemplategroup data found');
        }

        //save the data into the DB
        $jsonFiles = [
            'Command'              => $this->path . $jsonDataPath . $files['command'],
            'Servicetemplate'      => $this->path . $jsonDataPath . $files['servicetemplate'],
            'Servicetemplategroup' => $this->path . $jsonDataPath . $files['servicetemplategroup']
        ];

        $this->out('<info>Start installing Templates</info>');
        $importer->startInstall($jsonFiles);
        $this->out('<success>Successfully finished installing Templates</success>');
    }
}