<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.


/*
 * oitc MassenversandDeModule.notifyByMassenversandDe -r 00491712345678 -s it-novum --id yourAccountId --pw yourAccountPass
 *
 * /usr/share/openitcockpit/app/Console/cake massenversand_de_module.notify_by_massenversand_de -r 00491712345678 -s it-novum --id yourAccountId --pw yourAccountPass
 *
 * Host notification
 * /usr/share/openitcockpit/app/Console/cake massenversand_de_module.notify_by_massenversand_de -r $CONTACTPAGER$ -s it-novum --id $USER2$ --pw $USER3$ --type "host" --hostuuid "$HOSTNAME$" --hoststate "$HOSTSTATE$"
 *
 * Service notification
 * /usr/share/openitcockpit/app/Console/cake massenversand_de_module.notify_by_massenversand_de -r $CONTACTPAGER$ -s it-novum --id $USER2$ --pw $USER3$ --type "service" --serviceuuid "$SERVICEDESC$" --servicestate "$SERVICESTATE$"
 */

App::uses('HttpSocket', 'Network/Http');

class NotifyByMassenversandDeShell extends AppShell {

    public $uses = [
        'Proxy',
        'Host',
        'Service',
    ];

    public function main() {
        //check if there are proxy settings
        $proxySettings = $this->getProxySettings();

        try {
            $params = $this->params;
            if (empty($params['type'])) {
                throw new Exception('type has not been set');
            }
            $type = $params['type'];
            $host = [];
            $service = [];
            switch ($type) {
                case 'host':
                    if (empty($params['hostuuid'])) {
                        throw new Exception('hostuuid is not set');
                    }
                    $host = $this->getHostname($params['hostuuid']);
                    break;
                case 'service':
                    if (empty($params['serviceuuid'])) {
                        throw new Exception('serviceuuid is not set');
                    }
                    $service = $this->getServicename($params['serviceuuid']);
                    break;
            }

            if (empty($params['date'])) {
                $params['date'] = date("Y-m-d H:i:s");
            }

            //build message
            if ($type == 'host') {
                if (!empty($host['hostname'])) {
                    $params['msg'] = 'Host ' . $host['hostname'] . ' is ' . $params['hoststate'] . ' since ' . $params['date'];
                } else {
                    throw new Exception('Hostuuid could not be found');
                }
            } else {
                if (!empty($service['hostname']) && !empty($service['servicedesc'])) {
                    $params['msg'] = 'Service ' . $service['servicedesc'] . ' of ' . $service['hostname'] . ' is ' . $params['servicestate'] . ' since ' . $params['date'];
                } else {
                    throw new Exception('Hostuuid and/or Serviceuuid could not be found');
                }
            }
            $this->send($params, $proxySettings);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }


    public function getOptionParser() {
        $parser = parent::getOptionParser();
        $parser->addOptions([
            'receiver'     => ['short' => 'r', 'help' => __d('oitc_console', 'The Receiver of the message in international format eg. 0049123456789')],
            'sender'       => ['short' => 's', 'help' => __d('oitc_console', 'The Sender of the message')],
            //'msg' => ['help' => __d('oitc_console', 'The Message to send')],
            'id'           => ['help' => __d('oitc_console', 'The ID of your massenversand.de account')],
            'pw'           => ['help' => __d('oitc_console', 'The Password of your massenversand.de account')],
            'sendDate'     => ['help' => __d('oitc_console', 'The time to send the Message 0 = now or hhmmddmmyyyy')],
            'msgtype'      => ['help' => __d('oitc_console', 't = text sms, c = oversize sms, f = flash sms, utf8 = utf8 sms 70 digits per sms ')],
            'test'         => ['help' => __d('oitc_console', '1 = no SMS will be created but the server will respond like it would be')],
            'type'         => ['help' => __d('oitc_console', 'host or service')],
            'date'         => ['help' => __d('oitc_console', 'The date and time the problem occoured')],
            'hostuuid'     => ['help' => __d('oitc_console', 'The Host UUID')],
            'hoststate'    => ['help' => __d('oitc_console', 'State of the Host')],
            'serviceuuid'  => ['help' => __d('oitc_console', 'The Service UUID')],
            'servicestate' => ['help' => __d('oitc_console', 'State of the Service')],
        ]);

        return $parser;
    }

    public function getProxySettings() {
        return $this->Proxy->find('first', [
            'conditions' => [
                'Proxy.enabled' => 1,
            ],
        ]);
    }

    /**
     * Will send the message to the Massenversand.de server
     *
     * @param $param
     *              MANDATORY
     *              receiver     international format eg. "00491712345678"
     *              sender       11 digits alphnumeric or 16 digits numeric, url-encoded
     *              msg          ISO 8859-1/15 url-encoded
     *              id           massenversand.de account ID
     *              pw           massenversand.de account Pass url-encoded when using specialchars
     *              time         0 = send immediately
     *              hhmmddmmyyyy = send sms on this date/time
     *              msgtype      t = text sms - will be cut after 160 digits
     *              c = oversize sms
     *              f = flash sms
     *              utf8 = utf8 sms. 70 digits per sms
     *              OPTIONAL
     *              userAccount  cost center of the customer
     *              tarif        MA = Platin Tarif
     *              PM = High Quality Tarif
     *              test         1 = no SMS will be created but the server will respond like it would be
     *              getID        1 = get the message ID
     *              countMsg     1 = get number of charged messages
     *              getLimit     1 = get the residual volume
     *              getStatus    1 = get a statereport (DLR) for the message
     *
     * @return      Boolean true on success
     */
    private function send($param, $proxy) {
        $url1 = 'https://gate1.goyyamobile.com/sms/sendsms.asp'; //load balanced, SSL, both allow POST and GET
        $url2 = 'https://gate2.goyyamobile.com/sms/sendsms.asp'; //not load balanced, SSL, use if url1 is not working

        try {
            if (empty($param['receiver']) && !is_string($param['receiver']) && !$this->isValidPhoneNumber($param['receiver'])) {
                throw new Exception('Please check the receiver number!');
            }

            if (empty($param['sender']) && !$this->isValidSender($param['sender'])) {
                throw new Exception('Please check the Sender value!');
            }

            if (empty($param['msg'])) {
                throw new Exception('Message is empty!');
            }

            if (empty($param['sendDate'])) {
                $param['sendDate'] = 0;
            }

            if (empty($param['msgtype'])) {
                $param['msgtype'] = 'c';
            }

            if (empty($param['test'])) {
                $param['test'] = 0;
            }

            $config = [
                'receiver' => $param['receiver'],
                'sender'   => (string)$param['sender'],
                'msg'      => (string)$param['msg'],
                'id'       => $param['id'],
                'pw'       => (string)$param['pw'],
                'time'     => $param['sendDate'],
                'msgtype'  => $param['msgtype'],
                'test'     => $param['test'],
            ];

            $HttpSocket = new HttpSocket();
            if (!empty($proxy)) {
                $HttpSocket->configProxy($proxy['Proxy']['ipaddress'], $proxy['Proxy']['port'], 'Basic');
            }

            if ($HttpSocket->post($url1, $config)) {
                return true;
            } else {
                //first url doesnt work so lets try the second
                if ($HttpSocket->post($url2, $config)) {
                    return true;
                } else {
                    //no url is working so the notification by massenversand will not work
                    throw new Exception('can not reach massenversand.de servers');
                }
            }
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function getHostname($hostUuid) {
        $host = $this->Host->find('first', [
            'recurisve'  => -1,
            'conditions' => [
                'Host.uuid' => $hostUuid,
            ],
            'fields'     => [
                'Host.id', 'Host.name', 'Host.uuid',
            ],
            'contain'    => [],
        ]);

        return ['hostUuid' => $host['Host']['uuid'], 'hostname' => $host['Host']['name']];
    }

    public function getServicename($serviceUuid) {
        $service = $this->Service->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'Service.uuid' => $serviceUuid,
            ],
            'fields'     => [
                'Service.id',
                'Service.uuid',
                'Service.name',
            ],
            'contain'    => [
                'Servicetemplate' => [
                    'fields' => [
                        'Servicetemplate.id',
                        'Servicetemplate.name',
                    ],
                ],
                'Host'            => [
                    'fields' => [
                        'Host.id',
                        'Host.uuid',
                        'Host.name',
                    ],
                ],
            ],
        ]);

        if ($service['Service']['name'] === null || $service['Service']['name'] === '') {
            $service['Service']['name'] = $service['Servicetemplate']['name'];
        }

        return ['serviceUuid' => $service['Service']['uuid'], 'servicedesc' => $service['Service']['name'], 'hostname' => $service['Host']['name']];
    }

    private function isValidSender($sender) {
        //check if the sender string is 1-11 digits alphanumeric or 1-16 digits numeric
        preg_match('/^([a-zA-Z0-9]{1,11}|[0-9]{1,16})$/', $sender, $match);

        if (!empty($match)) {
            return true;
        }

        return false;
    }

    private function isValidPhoneNumber($number) {
        $pattern = '/^011(999|998|997|996|995|994|993|992|991|990|979|978|977|976|975|974|973|972|971|970|969|968|967|966|965|964|963|962|961|960|899|898|897|896|895|894|893|892|891|890|889|888|887|886|885|884|883|882|881|880|879|878|877|876|875|874|873|872|871|870|859|858|857|856|855|854|853|852|851|850|839|838|837|836|835|834|833|832|831|830|809|808|807|806|805|804|803|802|801|800|699|698|697|696|695|694|693|692|691|690|689|688|687|686|685|684|683|682|681|680|679|678|677|676|675|674|673|672|671|670|599|598|597|596|595|594|593|592|591|590|509|508|507|506|505|504|503|502|501|500|429|428|427|426|425|424|423|422|421|420|389|388|387|386|385|384|383|382|381|380|379|378|377|376|375|374|373|372|371|370|359|358|357|356|355|354|353|352|351|350|299|298|297|296|295|294|293|292|291|290|289|288|287|286|285|284|283|282|281|280|269|268|267|266|265|264|263|262|261|260|259|258|257|256|255|254|253|252|251|250|249|248|247|246|245|244|243|242|241|240|239|238|237|236|235|234|233|232|231|230|229|228|227|226|225|224|223|222|221|220|219|218|217|216|215|214|213|212|211|210|98|95|94|93|92|91|90|86|84|82|81|66|65|64|63|62|61|60|58|57|56|55|54|53|52|51|49|48|47|46|45|44|43|41|40|39|36|34|33|32|31|30|27|20|7|1)[0-9]{0,14}$/';
        preg_match($pattern, $number, $match);
        if (!empty($match)) {
            return true;
        }

        return false;
    }
}