<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.

class MaximoownergroupsController extends MaximoModuleAppController {

    public $layout = 'Admin.default';

    public $uses = [
        'MaximoModule.Maximoownergroup',
    ];

    public $components = [
        'ListFilter.ListFilter'
    ];

    public $helpers = [
        'ListFilter.ListFilter',
    ];

    public $listFilters = [
        'index' => [
            'fields' => [
                'Maximoownergroup.name'        => [
                    'label'      => 'Name',
                    'searchType' => 'wildcard'
                ],
                'Maximoownergroup.description' => [
                    'label'      => 'Description',
                    'searchType' => 'wildcard'
                ],
            ],
        ]
    ];

    public function index() {
        $query = [
            'order' => [
                'Maximoownergroup.name' => 'asc'
            ]
        ];

        $this->Paginator->settings = array_merge($this->Paginator->settings, $query);
        $allMaximoownergroups = $this->Paginator->paginate();
        $ownergroupIds = Hash::extract($allMaximoownergroups, '{n}.Maximoownergroup.id');

        $inUseIds = [];
        foreach ($ownergroupIds as $ownergroupId) {
            $result = $this->Maximoownergroup->inUse($ownergroupId);
            if (!empty($result)) {
                $inUseIds[$ownergroupId] = $result;
            }
        }

        $this->set(compact('allMaximoownergroups', 'inUseIds'));
    }

    public function add() {
        if ($this->request->is('post') || $this->request->is('put')) {
            if ($this->Maximoownergroup->save($this->request->data)) {
                $this->setFlash(__('Maximo Ownergroup successfully saved'));
                $this->redirect(['action' => 'index']);
            } else {
                $this->setFlash(__('could not save data'), false);
            }
        }
    }

    public function edit($id) {
        if (!$this->Maximoownergroup->exists($id)) {
            throw new NotFoundException(__('Invalid Maximo Ownergroup'));
        }

        $maximoOwnergroup = $this->Maximoownergroup->find('first', [
            'conditions' => [
                'Maximoownergroup.id' => $id
            ]
        ]);

        $this->set(compact('maximoOwnergroup'));

        if ($this->request->is('post') || $this->request->is('put')) {
            if ($this->Maximoownergroup->save($this->request->data)) {
                $this->setFlash(__('Maximo Ownergroup successfully Edited'));
                $this->redirect(['action' => 'index']);
            } else {
                $this->setFlash(__('could not save data'), false);
            }
        }
    }

    public function delete($id) {
        if (!$this->Maximoownergroup->exists($id)) {
            throw new NotFoundException(__('Invalid Maximo Ownergroup'));
        }
        //check if its in use
        if (!$this->Maximoownergroup->inUse($id)) {
            if ($this->Maximoownergroup->delete($id, true)) {
                $this->setFlash(__('Maximo Ownergroup deleted'));
                $this->redirect(['action' => 'index']);
            }
        } else {
            $this->setFlash(__('Could not delete Maximo Ownergroup'), false);
            $this->redirect(['action' => 'index']);
        }
    }

    public function mass_delete() {
        $ownergroups = func_get_args();
        $ownergroupAmount = sizeof($ownergroups);
        foreach ($ownergroups as $ownergroup) {
            if (!$this->Maximoownergroup->exists($ownergroup)) {
                throw new NotFoundException(__('Invalid Maximo Ownergroup'));
            }

            if (!$this->Maximoownergroup->inUse($ownergroup)) {
                if ($this->Maximoownergroup->delete($ownergroup)) {
                    $ownergroupAmount -= 1;
                }
            }
        }
        if ($ownergroupAmount == 0) {
            $this->setFlash(__('Ownergroups deleted'));
            $this->redirect(['action' => 'index']);
        }
        $this->setFlash(__('Could not delete all selected Maximo Ownergroups'), false);
        $this->redirect(['action' => 'index']);
    }

    public function usedBy($id) {
        if (!$this->Maximoownergroup->exists($id)) {
            throw new NotFoundException(__('Invalid Maximo Ownergroup'));
        }

        $ownergroup = $this->Maximoownergroup->find('first', [
            'conditions' => [
                'Maximoownergroup.id' => $id
            ]
        ]);

        $usedBy = $this->Maximoownergroup->inUse($id);
        $this->set(compact('usedBy', 'ownergroup'));
        $this->set('back_url', $this->referer());
    }
}