<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.

class MaximoservicesController extends MaximoModuleAppController {

    public $layout = 'Admin.default';

    public $uses = [
        'MaximoModule.Maximoservice'
    ];

    public $components = [
        'ListFilter.ListFilter',
    ];

    public $helpers = [
        'ListFilter.ListFilter',
    ];

    public $listFilters = [
        'index' => [
            'fields' => [
                'Maximoservice.name' => [
                    'label'      => 'Name',
                    'searchType' => 'wildcard'
                ],
            ],
        ]
    ];

    public function index() {
        $query = [
            'order' => [
                'Maximoservice.name' => 'asc'
            ]
        ];

        if (!isset($this->Paginator->settings['conditions'])) {
            $this->Paginator->settings['conditions'] = [];
        }

        $this->Paginator->settings = array_merge($this->Paginator->settings, $query);
        $allMaximoservices = $this->Paginator->paginate();
        $serviceIds = Hash::extract($allMaximoservices, '{n}.Maximoservice.id');

        $inUseIds = [];
        foreach ($serviceIds as $serviceId) {
            $result = $this->Maximoservice->inUse($serviceId);
            if (!empty($result)) {
                $inUseIds[$serviceId] = $result;
            }
        }

        $this->set(compact('allMaximoservices', 'inUseIds'));
    }

    public function add() {
        if ($this->request->is('post') || $this->request->is('put')) {
            if ($this->Maximoservice->save($this->request->data)) {
                $this->setFlash(__('Maximo Service successfully saved'));
                $this->redirect(['action' => 'index']);
            } else {
                $this->setFlash(__('could not save data'), false);
            }
        }
    }

    public function edit($id) {
        if (!$this->Maximoservice->exists($id)) {
            throw new NotFoundException(__('Invalid Maximo Service'));
        }

        $maximoService = $this->Maximoservice->find('first', [
            'conditions' => [
                'Maximoservice.id' => $id
            ]
        ]);

        $this->set(compact('maximoService'));

        if ($this->request->is('post') || $this->request->is('put')) {
            if ($this->Maximoservice->save($this->request->data)) {
                $this->setFlash(__('Maximo Service successfully edited'));
                $this->redirect(['action' => 'index']);
            } else {
                $this->setFlash(__('could not save data'), false);
            }
        }
    }

    public function delete($id) {
        if (!$this->Maximoservice->exists($id)) {
            throw new NotFoundException(__('Invalid Maximo Service'));
        }

        if ($this->Maximoservice->delete($id, true)) {
            $this->setFlash(__('Maximo Service deleted'));
            $this->redirect(['action' => 'index']);
        }

        $this->setFlash(__('Could not delete Maximo Service'), false);
        $this->redirect(['action' => 'index']);
    }

    public function mass_delete() {
        $services = func_get_args();
        $serviceAmount = sizeof($services);
        foreach ($services as $service) {
            if (!$this->Maximoservice->exists($service)) {
                throw new NotFoundException(__('Invalid Maximo Service'));
            }

            if (!$this->Maximoservice->inUse($service)) {
                if ($this->Maximoservice->delete($service)) {
                    $serviceAmount -= 1;
                }
            }
        }
        if ($serviceAmount == 0) {
            $this->setFlash(__('Services deleted'));
            $this->redirect(['action' => 'index']);
        }
        $this->setFlash(__('Could not delete all selected Maximo Services'), false);
        $this->redirect(['action' => 'index']);
    }

    public function usedBy($id) {
        if (!$this->Maximoservice->exists($id)) {
            throw new NotFoundException(__('Invalid Maximo Service'));
        }

        $service = $this->Maximoservice->find('first', [
            'conditions' => [
                'Maximoservice.id' => $id
            ]
        ]);

        $usedBy = $this->Maximoservice->inUse($id);
        $this->set(compact('usedBy', 'service'));
        $this->set('back_url', $this->referer());
    }
}