<?php

App::uses('AppModel', 'Model');

class MaximoModuleAppModel extends AppModel {


    public function resolveObjectNames($serviceIds, $hostIds) {
        $serviceNames = [];
        $hostNames = [];
        if (!empty($serviceIds)) {
            $serviceData = $this->Service->find('all', [
                'recursive'  => -1,
                'conditions' => [
                    'Service.id' => $serviceIds
                ],
                'joins'      => [
                    [
                        'table'      => 'hosts',
                        'alias'      => 'Host',
                        'type'       => 'LEFT OUTER',
                        'conditions' => [
                            'Host.id = Service.host_id'
                        ],
                    ],
                    [
                        'table'      => 'servicetemplates',
                        'type'       => 'INNER',
                        'alias'      => 'Servicetemplate',
                        'conditions' => [
                            'Servicetemplate.id = Service.servicetemplate_id',
                        ]
                    ],
                ],
                'fields'     => [
                    'Service.id',
                    'Host.name',
                    'IF(Service.name IS NULL, Servicetemplate.name, Service.name) AS ServiceName',
                ]
            ]);

            $newServiceData = [];
            foreach ($serviceData as $key => $service) {
                $newServiceData[$service['Service']['id']] = $service['Host']['name'] . '/' . $service[0]['ServiceName'];
            }
            $serviceData = $newServiceData;
            if (!empty($serviceData)) {
                $serviceNames = $serviceData;
            }
        }
        if (!empty($hostIds)) {
            $hostData = $this->Host->find('all', [
                'recursive'  => -1,
                'conditions' => [
                    'Host.id' => $hostIds
                ],
                'fields'     => [
                    'Host.id',
                    'Host.name'
                ]
            ]);

            $hostData = Hash::combine($hostData, '{n}.Host.id', '{n}.Host.name');
            if (!empty($hostData)) {
                $hostNames = $hostData;
            }
        }

        return [
            'Hosts'    => $hostNames,
            'Services' => $serviceNames,
        ];
    }
}
