<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.

class Maximoconfiguration extends AppModel {

    public $belongsTo = [
        'Host'             => [
            'className'  => 'Host',
            'foreignKey' => 'element_id'
        ],
        'Service'          => [
            'className'  => 'Service',
            'foreignKey' => 'element_id'
        ],
        'Maximoservice'    => [
            'className'  => 'MaximoModule.Maximoservice',
            'foreignKey' => 'maximo_service_id'
        ],
        'Maximoownergroup' => [
            'className'  => 'MaximoModule.Maximoownergroup',
            'foreignKey' => 'maximo_ownergroup_id'
        ],
    ];

    public $validate = [
        'impact_level'         => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true
            ],
            'numeric'  => [
                'rule'    => 'numeric',
                'message' => 'This field needs to be numeric.'
            ],
            'notZero'  => [
                'rule'     => ['comparison', '>', 0],
                'message'  => 'Invalid Impact level.',
                'required' => true,
            ],
        ],
        'urgency_level'        => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true
            ],
            'numeric'  => [
                'rule'    => 'numeric',
                'message' => 'This field needs to be numeric.'
            ],
            'notZero'  => [
                'rule'     => ['comparison', '>', 0],
                'message'  => 'Invalid Urgency level.',
                'required' => true,
            ],
        ],
        'maximo_ownergroup_id' => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true
            ],
            'numeric'  => [
                'rule'    => 'numeric',
                'message' => 'This field needs to be numeric.'
            ],
            'notZero'  => [
                'rule'     => ['comparison', '>', 0],
                'message'  => 'Invalid Maximo Ownergroup.',
                'required' => true,
            ],
        ],
        'maximo_service_id'    => [
            'notBlank' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true
            ],
            'numeric'  => [
                'rule'    => 'numeric',
                'message' => 'This field needs to be numeric.'
            ],
            'notZero'  => [
                'rule'     => ['comparison', '>', 0],
                'message'  => 'Invalid Maximo Service.',
                'required' => true,
            ],
        ],
    ];

    public function getMaximoconfiguration($type, $elementId) {
        return $this->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'type'       => $type,
                'element_id' => $elementId
            ]
        ]);
    }

    public function hasMaximoConfiguration($type, $id) {
        $maximoConfig = $this->getMaximoconfiguration($type, $id);
        debug($maximoConfig);
        if (!empty($maximoConfig)) {
            return $maximoConfig['Maximoconfiguration']['id'];
        }
        return false;
    }
}