<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

use itnovum\openITCOCKPIT\Core\CustomMacroReplacer;
use itnovum\openITCOCKPIT\Core\HostMacroReplacer;
use itnovum\openITCOCKPIT\Core\UserDefinedMacroReplacer;

class MkModuleNagiosExportTask extends AppShell {

    public $uses = [
        'Systemsetting',
        MONITORING_CORECONFIG_MODEL,
        'MkModule.Mkservicedata',
        'Host',
        'Servicetemplate',
        'Service',
        'Macro',
        'MkModule.Mksnmp',
        'MkModule.Mkagent',
        'MkModule.LastUsedMkagent',
    ];

    /*
     * Callback from main NagiosExportTask!
     */
    public function beforeExport() {
        $this->init();
        $this->createMainMk();
        $this->cleanUp();
        $this->createConfigFiles(null, [], true);
        $this->precompileHostchecks();

        if (CakePlugin::loaded('DistributeModule')) {
            $this->replaceNameWithUuid();
        }

    }

    /*
     * Callback from main NagiosExportTask!
     */
    public function afterExport() {
        return true;
    }

    /**
     * like __construct in a php class to set all required variables
     * @return    void
     * @author    Daniel Ziegler <daniel.ziegler@it-novum.com>
     * @since     3.0.0
     */
    public function init() {
        App::uses('File', 'Utility');
        App::uses('UUID', 'Lib');
        Configure::load('cmddaemon');

        $this->oitc_cmd = Configure::read('pipe');
        $this->mkModel = 'pipe'; // values: 'pipe' or 'file' to use nagios checkresults spool dir

        $this->_systemsetting = $this->Systemsetting->findAsArray();
        $this->MkActiveTemplateId = $this->Servicetemplate->findByName($this->_systemsetting['CHECK_MK']['CHECK_MK.ACTIVE_CHECK'])['Servicetemplate']['id'];
        $this->hostHasMkActiveCache = [];
        $this->hostMkSnmpConfigCache = [];
    }


    /**
     * Create Check_mk's main.mk file (overwrite existing file on every export!)
     * @return    void
     * @author    Daniel Ziegler <daniel.ziegler@it-novum.com>
     * @since     3.0.0
     */
    public function createMainMk() {
        $mainMk = fopen($this->_systemsetting['CHECK_MK']['CHECK_MK.ETC'] . 'main.mk', 'w+');
        fwrite($mainMk, '# -*- coding: utf-8 -*-');
        fwrite($mainMk, 'all_hosts = []' . PHP_EOL);
        fwrite($mainMk, 'ipaddresses = {}' . PHP_EOL);
        fwrite($mainMk, 'snmp_communities = []' . PHP_EOL);
        fwrite($mainMk, 'bulkwalk_hosts = []' . PHP_EOL);
        fwrite($mainMk, 'nagios_illegal_chars = \'' . addslashes($this->Coreconfig->_read('illegal_object_name_chars')) . '\'' . PHP_EOL);
        fwrite($mainMk, 'nagios_command_pipe_path = \'' . $this->oitc_cmd . '\'' . PHP_EOL);
        fwrite($mainMk, 'check_submission = \'' . $this->mkModel . '\'' . PHP_EOL);
        fclose($mainMk);
    }

    /**
     * Create one or all .mk config files for every host
     * ### Options
     * - `for_snmp_scan`            If the config is only created for a SNMP scan (default false)
     * - `snmp_version`                The version SNMP
     * - `v3`                        Array of different SNMP V3 options
     * --- `security_level`            Integer value: 1=authPriv 2=authNoPriv 3=noAuthnoPriv
     * --- `hash_algorithm`            Integer value: 1=MD5, 2=SHA
     * --- `username`                String: SNMP Username
     * --- `password`                String:-SNMP Password
     * --- `encryption_algorithm`    Integer value: 1=AES 2=DES
     * --- `encryption_password`    String: Password for data encryption
     *
     * @param    string    hostUuid - if null we create a configs, if a uuid we create only the config for this host
     * @param    arrray    a aray of different options
     * @param    bool      Are we in the Export or just Check_MK Discovery?
     *
     * @return    void
     * @author    Daniel Ziegler <daniel.ziegler@it-novum.com>
     * @since     3.0.0
     */
    public function createConfigFiles($hostUuid = null, $options = [], $isExport = false) {

        $_options = [
            'for_snmp_scan'  => false,
            'snmp_version'   => 0,
            'snmp_community' => '',
            'v3'             => [
                'security_level'       => '',
                'hash_algorithm'       => '',
                'username'             => '',
                'password'             => '',
                'encryption_algorithm' => '',
                'encryption_password'  => '',
            ],
        ];

        $options = Hash::merge($_options, $options);

        $exportAll = true;

        $hashAlgorithm = [
            1 => 'MD5',
            2 => 'SHA',
        ];

        $securityLevel = [
            1 => 'authPriv',
            2 => 'authNoPriv',
            3 => 'noAuthnoPriv',
        ];

        $encryptionAlgorithm = [
            1 => 'AES',
            2 => 'DES',
        ];


        $conditions = [
            'Service.disabled' => 0,
        ];
        if ($hostUuid !== null) {
            $exportAll = false;
            $conditions = Hash::merge($conditions, [
                'Host.uuid' => $hostUuid,
            ]);
        }
        $Mkservicedata = $this->Mkservicedata->find('all', [
            'recursive'  => -1,
            'contain'    => [
                'Host'    => ['fields' => ['id', 'uuid', 'address', 'satellite_id']],
                'Service' => [
                    'Servicetemplate'             => [
                        'fields'                              => ['id', 'name', 'uuid'],
                        'Servicetemplatecommandargumentvalue' => [
                            'fields' => [
                                'value',
                            ],
                        ],
                    ],
                    'Servicecommandargumentvalue' => [
                        'fields' => [
                            'value',
                        ],
                    ],
                    'fields'                      => ['id', 'uuid', 'servicetemplate_id', 'disabled'],
                ]
            ],
            'conditions' => $conditions,
        ]);
        $forConfig = [];

        if ($exportAll === true) {
            $this->buildSnmpConfigCache();
        }
        foreach ($Mkservicedata as $data) {
            if (!isset($forConfig[$data['Host']['id']])) {
                $forConfig[$data['Host']['id']] = [];
                $forConfig[$data['Host']['id']]['Host'] = $data['Host'];
            }
            //$forConfig[$data['Host']['id']]['Mkservicedata'][] = $data['Mkservicedata'];
            //$forConfig[$data['Host']['id']]['Service'][] = $data['Service'];

            if ($data['Host']['uuid'] != null && $data['Host']['uuid'] != "") {
                $forConfig[$data['Host']['id']]['mkService'][] = [
                    'Mkservicedata' => $data['Mkservicedata'],
                    'Service'       => $data['Service'],
                ];
            }
        }

        foreach ($forConfig as $host_id => $config) {
            if ($exportAll === true) {
                $options = $this->getSnmpConfigFromCacheIfExists($host_id, $options);
            }

            $file = new File($this->_systemsetting['CHECK_MK']['CHECK_MK.ETC'] . 'conf.d/' . $config['Host']['uuid'] . '.mk');
            if (!$file->exists()) {
                $file->create();
            }

            $configFileHeader = '# -*- coding: utf-8 -*-' . PHP_EOL;

            switch ($options['snmp_version']) {
                case 1:
                    //wie behindert ist die check_mk syntax - mal erlich jetzt WHAT THE FUCK
                    if ($options['for_snmp_scan'] == true) {
                        $content = 'all_hosts.append("' . $config['Host']['uuid'] . '|snmp")' . PHP_EOL;
                    } else {
                        $content = 'all_hosts.append("' . $config['Host']['uuid'] . '")' . PHP_EOL;
                    }
                    $content .= 'snmp_communities.append(("' . $options['snmp_community'] . '" ,["' . $config['Host']['uuid'] . '"]),)' . PHP_EOL;
                    break;

                case 2:
                    //wie behindert ist die check_mk syntax - mal erlich jetzt WHAT THE FUCK
                    if ($options['for_snmp_scan'] == true) {
                        //Funzt bei SNMP Scan -II -v $hostUUid und -D $hostUuid
                        $content = 'all_hosts.append("' . $config['Host']['uuid'] . '|snmp")' . PHP_EOL;
                    } else {
                        //Funzt nicht wenn check_mk die .py datie ausführt weil:
                        //WARN - Missing agent sections: cpu - execution time 11.2 sec
                        $content = 'all_hosts.append("' . $config['Host']['uuid'] . '")' . PHP_EOL;
                    }

                    $content .= 'bulkwalk_hosts.append((["' . $config['Host']['uuid'] . '|snmp"],))' . PHP_EOL;
                    $content .= 'snmp_communities.append(("' . $options['snmp_community'] . '" ,["' . $config['Host']['uuid'] . '"]),)' . PHP_EOL;
                    break;

                case 3:
                    if ($options['for_snmp_scan'] == true) {
                        $content = 'all_hosts.append("' . $config['Host']['uuid'] . '|snmp|v3")' . PHP_EOL;
                    } else {
                        $content = 'all_hosts.append("' . $config['Host']['uuid'] . '")' . PHP_EOL;
                    }
                    $readableV3SNMPConfig = [
                        $securityLevel[$options['v3']['security_level']],
                        $hashAlgorithm[$options['v3']['hash_algorithm']],
                        $options['v3']['username'],
                        $options['v3']['password'],
                        $encryptionAlgorithm[$options['v3']['encryption_algorithm']],
                        $options['v3']['encryption_password'],
                    ];
                    $content .= 'snmp_communities.append((("' . implode('","', $readableV3SNMPConfig) . '") ,["' . $config['Host']['uuid'] . '"]),)' . PHP_EOL;
                    break;

                default:
                    $content = 'all_hosts.append("' . $config['Host']['uuid'] . '")' . PHP_EOL;
                    break;
            }

            $content .= 'ipaddresses["' . $config['Host']['uuid'] . '"]="' . $config['Host']['address'] . '"' . PHP_EOL;

            $lastSelectedAgent = $this->LastUsedMkagent->findByHostId($config['Host']['id']);
            if (!empty($lastSelectedAgent) && !in_array($lastSelectedAgent['LastUsedMkagent']['agent_id'], [-1, 0])) {

                $host = $this->Host->find('first', [
                    'recursive'  => -1,
                    'conditions' => [
                        'Host.uuid' => $config['Host']['uuid']
                    ],
                    'contain'    => [
                        'Hosttemplate' => [
                            'Customvariable'
                        ],
                        'Customvariable',
                    ]
                ]);

                $userMacros = $this->Macro->find('all');

                $customVariableMerger = new \itnovum\openITCOCKPIT\Core\CustomVariableMerger(
                    $host['Customvariable'],
                    $host['Hosttemplate']['Customvariable']
                );
                $mergedCustomVariables = $customVariableMerger->getCustomVariablesMergedAsRepository();
                //replace hosttemplate, host, user and custom macros
                $HosttemplateCustomMacroReplacer = new CustomMacroReplacer($mergedCustomVariables, OBJECT_HOSTTEMPLATE);
                $replacedHosttemplateMacroString = $HosttemplateCustomMacroReplacer->replaceAllMacros($lastSelectedAgent['Mkagent']['command_line']);
                $HostMacroReplacer = new HostMacroReplacer($host);
                $replacedHostMacroString = $HostMacroReplacer->replaceBasicMacros($replacedHosttemplateMacroString);
                $CustomMacroReplacer = new CustomMacroReplacer($host['Customvariable'], OBJECT_HOST);
                $replacedCustomMacroString = $CustomMacroReplacer->replaceAllMacros($replacedHostMacroString);
                $UserDefinedMacroReplacer = new UserDefinedMacroReplacer($userMacros);
                $resolvedCommandline = $UserDefinedMacroReplacer->replaceMacros($replacedCustomMacroString);

                $content .= 'datasource_programs.append(("' . addslashes($resolvedCommandline) . '",["' . $config['Host']['uuid'] . '"]))' . PHP_EOL;
            }

            $content .= 'checks +=[' . PHP_EOL;

            foreach ($config['mkService'] as $Mkservicedata) {
                if (!empty($Mkservicedata['Service']['Servicecommandargumentvalue'])) {
                    $checkCommandArgs = $Mkservicedata['Service']['Servicecommandargumentvalue'][0]['value'];
                } else {
                    $checkCommandArgs = '';
                    if (isset($Mkservicedata['Service']['Servicetemplate']['Servicetemplatecommandargumentvalue'][0])) {
                        $checkCommandArgs = $Mkservicedata['Service']['Servicetemplate']['Servicetemplatecommandargumentvalue'][0]['value'];
                    }

                }

                if ($checkCommandArgs == '') {
                    $checkCommandArgs = 'None';
                }

                switch ($Mkservicedata['Mkservicedata']['check_name']) {
                    case 'ps':
                        $checkCommandArgs = preg_replace('/_PROC_/', '"' . $Mkservicedata['Mkservicedata']['check_item'] . '"', $checkCommandArgs);
                        $content .= '    ("' . $config['Host']['uuid'] . '","' . $Mkservicedata['Mkservicedata']['check_name'] . '","' . str_replace('~', '', $Mkservicedata['Mkservicedata']['check_item']) . '", ' . $checkCommandArgs . '),' . PHP_EOL;
                        break;

                    case 'oracle_tablespaces':
                        $checkItem = preg_replace('/ Tablespace/', '', $Mkservicedata['Mkservicedata']['check_item']);
                        $content .= '    ("' . $config['Host']['uuid'] . '","' . $Mkservicedata['Mkservicedata']['check_name'] . '",\'' . $checkItem . '\',' . $checkCommandArgs . '),' . PHP_EOL;
                        break;

                    default:
                        if ($Mkservicedata['Mkservicedata']['check_item'] == 'None') {
                            $content .= '    ("' . $config['Host']['uuid'] . '","' . $Mkservicedata['Mkservicedata']['check_name'] . '",' . $Mkservicedata['Mkservicedata']['check_item'] . ',' . $checkCommandArgs . '),' . PHP_EOL;
                        } else {
                            $content .= '    ("' . $config['Host']['uuid'] . '","' . $Mkservicedata['Mkservicedata']['check_name'] . '",\'' . $Mkservicedata['Mkservicedata']['check_item'] . '\',' . $checkCommandArgs . '),' . PHP_EOL;
                        }

                        break;
                }
            }

            $content .= ']' . PHP_EOL;

            //Merge $configFileHeader and $content to one string and write to -mk file
            $file->write(sprintf('%s%s', $configFileHeader, $content));
            $file->close();

            if ($config['Host']['satellite_id'] !== '0' && $isExport === false) {
                $this->rsyncConfigFileToSat($file->name, $file->path, $config['Host']['satellite_id']);
            }

            if ($exportAll === true) {
                $options = [
                    'for_snmp_scan'  => false,
                    'snmp_version'   => 0,
                    'snmp_community' => '',
                    'v3'             => [
                        'security_level'       => '',
                        'hash_algorithm'       => '',
                        'username'             => '',
                        'password'             => '',
                        'encryption_algorithm' => '',
                        'encryption_password'  => '',
                    ],
                ];
            }

            //Create active check_mk check if not exists
            if ($this->_hasHostActiveServiceFromCache($host_id) == false) {
                // We are in the beforeExport callback, so we can create services and they will be exported :-)

                $data = [
                    'Service' => [
                        'uuid'               => UUID::v4(),
                        'servicetemplate_id' => $this->MkActiveTemplateId,
                        'host_id'            => $host_id,
                        'own_contacts'       => 0,
                        'own_contactgroups'  => 0,
                        'service_type'       => MK_SERVICE,
                    ],
                ];
                $this->Service->create();
                if ($this->Service->save($data)) {
                    $this->_updateHasHostActiveServiceCache($host_id, true);
                }

            }

        }

        //this will be executed when we press the Execute discovery button
        //If a host has no mk service, we need to create a empty config, that we can run the SNMP scan!

        if (empty($forConfig) && $options['for_snmp_scan'] === true) {
            $file = new File($this->_systemsetting['CHECK_MK']['CHECK_MK.ETC'] . 'conf.d/' . $hostUuid . '.mk');
            if (!$file->exists()) {
                $file->create();
            }

            switch ($options['snmp_version']) {
                case 1:
                    $content = 'all_hosts.append("' . $hostUuid . '|snmp")' . PHP_EOL;
                    $content .= 'snmp_communities.append(("' . $options['snmp_community'] . '" ,["' . $hostUuid . '"]),)' . PHP_EOL;
                    break;

                case 2:
                    $content = 'all_hosts.append("' . $hostUuid . '|snmp")' . PHP_EOL;
                    $content .= 'bulkwalk_hosts.append((["' . $hostUuid . '|snmp"],))' . PHP_EOL;
                    $content .= 'snmp_communities.append(("' . $options['snmp_community'] . '" ,["' . $hostUuid . '"]),)' . PHP_EOL;
                    break;

                case 3:
                    $content = 'all_hosts.append("' . $hostUuid . '|snmp|v3")' . PHP_EOL;
                    $readableV3SNMPConfig = [
                        $securityLevel[$options['v3']['security_level']],
                        $hashAlgorithm[$options['v3']['hash_algorithm']],
                        $options['v3']['username'],
                        $options['v3']['password'],
                        $encryptionAlgorithm[$options['v3']['encryption_algorithm']],
                        $options['v3']['encryption_password'],
                    ];
                    $content .= 'snmp_communities.append((("' . implode('","', $readableV3SNMPConfig) . '") ,["' . $hostUuid . '"]),)' . PHP_EOL;
                    break;

                default:
                    $content = 'all_hosts.append("' . $hostUuid . '")' . PHP_EOL;
                    break;
            }

            $content .= 'ipaddresses["' . $hostUuid . '"]="' . $options['host_address'] . '"' . PHP_EOL;

            $host = $this->Host->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'Host.uuid' => $hostUuid
                ],
                'contain'    => [
                    'Hosttemplate' => [
                        'Customvariable'
                    ],
                    'Customvariable',
                ]
            ]);

            $host_id = $host['Host']['id'];
            if (!empty($host_id)) {
                $lastSelectedAgent = $this->LastUsedMkagent->findByHostId($host_id);

                $userMacros = $this->Macro->find('all');

                $customVariableMerger = new \itnovum\openITCOCKPIT\Core\CustomVariableMerger(
                    $host['Customvariable'],
                    $host['Hosttemplate']['Customvariable']
                );
                $mergedCustomVariables = $customVariableMerger->getCustomVariablesMergedAsRepository();

                //replace hosttemplate, host, user and custom macros
                $HosttemplateCustomMacroReplacer = new CustomMacroReplacer($mergedCustomVariables->getAllCustomVariablesAsArray(), OBJECT_HOSTTEMPLATE);
                $replacedHosttemplateMacroString = $HosttemplateCustomMacroReplacer->replaceAllMacros($lastSelectedAgent['Mkagent']['command_line']);
                $HostMacroReplacer = new HostMacroReplacer($host);
                $replacedHostMacroString = $HostMacroReplacer->replaceBasicMacros($replacedHosttemplateMacroString);
                $CustomMacroReplacer = new CustomMacroReplacer($host['Customvariable'], OBJECT_HOST);
                $replacedCustomMacroString = $CustomMacroReplacer->replaceAllMacros($replacedHostMacroString);
                $UserDefinedMacroReplacer = new UserDefinedMacroReplacer($userMacros);
                $resolvedCommandline = $UserDefinedMacroReplacer->replaceMacros($replacedCustomMacroString);

                if (!empty($lastSelectedAgent) && !in_array($lastSelectedAgent['LastUsedMkagent']['agent_id'], [-1, 0])) {
                    $content .= 'datasource_programs.append(("' . addslashes($resolvedCommandline) . '",["' . $hostUuid . '"]))' . PHP_EOL;
                }
            }

            $file->write($content);
            $file->close();

            if ($host['Host']['satellite_id'] !== '0' && $isExport === false) {
                $this->rsyncConfigFileToSat($file->name, $file->path, $host['Host']['satellite_id']);
            }

        }
        $this->wipeCaches();
    }

    /**
     * Rsync the configuration file to Satellite
     * @return    void
     */
    public function rsyncConfigFileToSat($configFileName, $configFilePath, $satelliteId) {
        $SatelliteModel = ClassRegistry::init('DistributeModule.Satellite', 'Model');
        $satellite = $SatelliteModel->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'Satellite.id' => $satelliteId
            ]
        ]);
        if (empty($satellite)) {
            return false;
        }

        Configure::load('after_export');
        $this->conf = Configure::read('after_export');
        //Generate the arguments for rsync
        $commandArgs = [
            $this->conf['SSH']['private_key'],
            $configFilePath,
            $this->conf['SSH']['username'],
            $satellite['Satellite']['address'],
            str_replace($configFileName, '', $configFilePath),
        ];
        //Define the rsync commando
        $commandTemplate = "rsync -e 'ssh -C -ax -i %s -o StrictHostKeyChecking=no' -avm --timeout=10 --delete %s %s@%s:%s";
        //Merge template and arguments
        $command = vsprintf($commandTemplate, $commandArgs);
        //Preform the transfer via rsync

        exec($command);

    }

    /**
     * Execute the Check_MK command, to compile all checks ./check_mk -C
     * @return    void
     * @author    Daniel Ziegler <daniel.ziegler@it-novum.com>
     * @since     3.0.0
     */
    public function precompileHostchecks() {
        //Compile the python files
        exec($this->_systemsetting['CHECK_MK']['CHECK_MK.BIN'] . ' -C');
        $mkPrecompiled = $this->_systemsetting['CHECK_MK']['CHECK_MK.VAR'] . 'precompiled/';
        $oitcPrecompiled = $this->_systemsetting['CHECK_MK']['CHECK_MK.VAR'] . 'oitc_precompiled/';
        if (!is_dir($oitcPrecompiled)) {
            mkdir($oitcPrecompiled);
        }

        exec(sprintf('rsync -avm --timeout=10 --delete %s %s', $mkPrecompiled, $oitcPrecompiled));
    }

    /**
     * Delete all *.mk config files out of 3rd/check_mk/etc/conf.d and 3rd/check_mk/var/precompiled/*
     * @return    void
     * @author    Daniel Ziegler <daniel.ziegler@it-novum.com>
     * @since     3.0.0
     */
    public function cleanUp() {
        $mkConfigFiles = glob($this->_systemsetting['CHECK_MK']['CHECK_MK.ETC'] . 'conf.d/*.mk');
        foreach ($mkConfigFiles as $mkConfigFile) {
            unlink($mkConfigFile);
        }

        $precompiledChecks = glob($this->_systemsetting['CHECK_MK']['CHECK_MK.VAR'] . 'precompiled/*');
        foreach ($precompiledChecks as $precompiledCheck) {
            unlink($precompiledCheck);
        }
    }

    private function _hasHostActiveServiceFromCache($host_id) {
        if (!isset($this->hostHasMkActiveCache[$host_id])) {
            $this->hostHasMkActiveCache[$host_id] = $this->Service->hostHasServiceByServicetemplateId($host_id, $this->MkActiveTemplateId);
        }

        return $this->hostHasMkActiveCache[$host_id];
    }

    private function _updateHasHostActiveServiceCache($host_id, $value) {
        $this->hostHasMkActiveCache[$host_id] = $value;
    }

    /**
     * Build a cache with every Check_MK SNMP Configuration for every host
     * @return    void
     * @author    Daniel Ziegler <daniel.ziegler@it-novum.com>
     * @since     3.0.1
     */
    public function buildSnmpConfigCache() {
        $MkSnmpConfigs = $this->Mksnmp->find('all');
        $cache = [];
        foreach ($MkSnmpConfigs as $MkSnmpConfig) {
            $MkSnmpConfig['Mksnmp']['version'] = (int)$MkSnmpConfig['Mksnmp']['version'];

            if ($MkSnmpConfig['Mksnmp']['version'] < 3) {
                $this->hostMkSnmpConfigCache[$MkSnmpConfig['Mksnmp']['host_id']] = [
                    'snmp_version'   => $MkSnmpConfig['Mksnmp']['version'],
                    'snmp_community' => $MkSnmpConfig['Mksnmp']['community'],
                    'v3'             => [],
                ];
            } else {
                $this->hostMkSnmpConfigCache[$MkSnmpConfig['Mksnmp']['host_id']] = [
                    'snmp_version'   => $MkSnmpConfig['Mksnmp']['version'],
                    'snmp_community' => $MkSnmpConfig['Mksnmp']['community'],
                    'v3'             => [
                        'security_level'       => $MkSnmpConfig['Mksnmp']['security_level'],
                        'hash_algorithm'       => $MkSnmpConfig['Mksnmp']['hash_algorithm'],
                        'username'             => $MkSnmpConfig['Mksnmp']['username'],
                        'password'             => $MkSnmpConfig['Mksnmp']['password'],
                        'encryption_algorithm' => $MkSnmpConfig['Mksnmp']['encryption_algorithm'],
                        'encryption_password'  => $MkSnmpConfig['Mksnmp']['encryption_password'],
                    ],
                ];
            }
        }
    }

    /**
     * Execute the Check_MK command, to compile all checks ./check_mk -C
     *
     * @param    int        host_id the id of the host you want to lookup SNMP config in the cache
     * @param    array      the options array from $this->createConfigFiles (will be merged in return result)
     *
     * @return    array    $options with all the options + SNMP config
     * @author    Daniel Ziegler <daniel.ziegler@it-novum.com>
     * @since     3.0.0
     */
    public function getSnmpConfigFromCacheIfExists($host_id, $options) {
        if (isset($this->hostMkSnmpConfigCache[$host_id])) {
            return Hash::merge($options, $this->hostMkSnmpConfigCache[$host_id]);
        }

        return $options;
    }

    /**
     * Wipe / reset every cache variable to free memory
     * Remember, we run inside of a while(true)
     * @return    void
     * @author    Daniel Ziegler <daniel.ziegler@it-novum.com>
     * @since     3.0.1
     */
    public function wipeCaches() {
        $this->hostHasMkActiveCache = [];
        $this->hostMkSnmpConfigCache = [];
    }

    /**
     * Replace the servicedescriptions with the UUIDs
     */
    public function replaceNameWithUuid() {
        //Die oitc.cmd muss noch mit der nagios cmd ersetzt werden in der main.mk. Wirklich? Warum? Geht auch so.
        //Search all hosts which are sat systems
        $hosts = $this->Host->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'satellite_id >' => 0,
                'disabled'       => 0,
            ],
            'contain'    => [
                'Service' => [
                    'conditions' => [
                        'Service.disabled' => 0,
                        'not'              => ['Service.name' => null],
                    ],
                    'fields'     => [
                        'Service.uuid',
                        'Service.name',
                    ],
                ],
            ],
            'fields'     => [
                'Host.uuid',
                'Host.satellite_id',
            ],
        ]);
        //Read the folder for check_mk from the systemsettings and generate the path to the py files
        $this->_systemsetting = $this->Systemsetting->findAsArray();
        $pathToPythonFiles = $this->_systemsetting['CHECK_MK']['CHECK_MK.VAR'] . 'precompiled/';
        $tmpPathToPythonFilesForSats = "/usr/share/openitcockpit/app/tmp/precompiled_for_sat/";
        //Before generating the files delete all old files in directory
        if (is_dir($tmpPathToPythonFilesForSats) && sizeof(scandir($tmpPathToPythonFilesForSats)) > 2) {
            exec("/bin/rm -rf " . $tmpPathToPythonFilesForSats . "*");
        }
        //Read all files from the path
        $pyFiles = scandir($pathToPythonFiles);
        foreach ($pyFiles as $pyFile) {
            //Check if the file is a py file
            if ($pyFile != '.' && $pyFile != '..' && strstr($pyFile, '.py')) {
                foreach ($hosts as $host) {
                    //Check if a py file exists to the found sat hosts
                    if (strstr($pyFile, $host['Host']['uuid'])) {
                        //For the first use the directory precompiled_for_sat must be generated
                        if (!is_dir($tmpPathToPythonFilesForSats)) {
                            mkdir($tmpPathToPythonFilesForSats, 0755);
                        }
                        //Define path for the satellite python files.
                        $pathToEditedPyFiles = $tmpPathToPythonFilesForSats . $host['Host']['satellite_id'] . "/";
                        //Check if the local directory exists , if not create it
                        if (!is_dir($pathToEditedPyFiles)) {
                            mkdir($pathToEditedPyFiles, 0755);
                        }
                        //Read the content of the current file
                        $inhalt = file_get_contents($pathToPythonFiles . $pyFile);
                        foreach ($host['Service'] as $service) {
                            //Replace the service name with the service uuid
                            $fixedServiceName = str_replace('\\', '\\\\', $service['name']);
                            $inhalt = str_replace("'" . $fixedServiceName . "'", "'" . $service['uuid'] . "'", $inhalt);
                            $inhalt = str_replace("'mrpe', '" . $service['uuid'] . "'", "'mrpe', '" . $fixedServiceName . "'", $inhalt);
                            $inhalt = str_replace("'local', '" . $service['uuid'] . "'", "'local', '" . $fixedServiceName . "'", $inhalt);
                        }
                        //Write the edited content back to the py file
                        file_put_contents($pathToEditedPyFiles . $pyFile, $inhalt);
                    }
                }

            }
        }
        //Transfer the edited py files to the sat system
        $this->transferConfigFilesToSat($pathToPythonFiles);
    }


    /**
     * Transfer the config files to the sat sytem with rsync.
     * The folder precompiled_for_sat must be created manuel.
     * Is it possible to do this over rsync???
     */
    public function transferConfigFilesToSat($pathToPythonFiles) {
        if (is_dir(APP . 'Plugin' . DS . 'DistributeModule')) {
            $SatelliteModel = ClassRegistry::init('DistributeModule.Satellite', 'Model');

            //Read the config from after export to get the connection informations
            Configure::load('after_export');
            $this->conf = Configure::read('after_export');


            $monitoringSystemsettings = $this->Systemsetting->findAsArraySection('MONITORING');
            if ($monitoringSystemsettings['MONITORING']['MONITORING.SINGLE_INSTANCE_SYNC'] == 1) {
                $satellites = $SatelliteModel->find('all', [
                    'recursive'  => -1,
                    'conditions' => [
                        'Satellite.sync_instance' => 1,
                    ]
                ]);
            } else {
                $satellites = $SatelliteModel->find('all', [
                    'recursive' => -1
                ]);
            }


            foreach ($satellites as $satellite) {
                if (is_dir("/usr/share/openitcockpit/app/tmp/precompiled_for_sat/" . $satellite['Satellite']['id'])) {
                    //Generate the arguments for rsync
                    $commandArgs = [
                        $this->conf['SSH']['private_key'],
                        "/usr/share/openitcockpit/app/tmp/precompiled_for_sat/" . $satellite['Satellite']['id'],
                        $this->conf['SSH']['username'],
                        $satellite['Satellite']['address'],
                        $pathToPythonFiles,
                    ];
                    //Define the rsync commando
                    $commandTemplate = "rsync -e 'ssh -C -ax -i %s -o StrictHostKeyChecking=no' -avm --timeout=10 --delete %s/ %s@%s:%s";
                    //Merge template and arguments
                    $command = vsprintf($commandTemplate, $commandArgs);
                    //Preform the transfer via rsync
                    exec($command);
                }
            }

        }
    }
}

