<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement. 
// The license agreement and license key were sent with the order confirmation.
class MkParserComponent extends Component {


    /* Parse the output of ./check_mk -D $hostUuid into a array
     *
     */
    public function parseMkDumpOutput($output = []) {
        /* check_mk -D $hostUuid output looks like this: (RAW untouched)
        f9ccaac1-ca81-4ace-9222-0c492ec80284 (172.16.101.55)                           
        Tags:                   snmp
        Host groups:            
        Contact groups:         
        Type of agent:          SNMP (community: 'public', bulk walk: yes, port: default, inline: no)
        Is aggregated:          no
        Services:
          checktype    item              params                                                                                                                description            groups summarized to groups
          ------------ ----------------- --------------------------------------------------------------------------------------------------------------------- ---------------------- ------ ------------- ------
          ucd_cpu_load None              (5, 10)                                                                                                               CPU load                                          
          ucd_cpu_util None              {}                                                                                                                    CPU utilization                                   
          if64         2                 {'state': ['1'], 'errors': (0.01, 0.1), 'speed': 0}                                                                   Interface 2                                       
          hr_mem       None              (150.0, 200.0)                                                                                                        Memory used                                       
          snmp_info    None              None                                                                                                                  SNMP Info                                         
          snmp_uptime  None              {}                                                                                                                    Uptime                                            
          hr_fs        /                 {'levels_low': (50.0, 60.0), 'magic_normsize': 20, 'levels': (80.0, 90.0), 'trend_range': 24, 'trend_perfdata': True} fs_/                                              
          hr_fs        /boot             {'levels_low': (50.0, 60.0), 'magic_normsize': 20, 'levels': (80.0, 90.0), 'trend_range': 24, 'trend_perfdata': True} fs_/boot                                          
          hr_fs        /mnt/perfdata     {'levels_low': (50.0, 60.0), 'magic_normsize': 20, 'levels': (80.0, 90.0), 'trend_range': 24, 'trend_perfdata': True} fs_/mnt/perfdata                                  
          ps           /usr/sbin/apache2 ('/usr/sbin/apache2', ('/usr/sbin/apache2', 1, 1, 98, 99))                                                            proc_/usr/sbin/apache2 
         
        
        So first of all, we trim() every line untile we get to the -------- ------ ... line.
        Than we split this line by space into a array.
        Out array looks now like this:
        Array
        (
            [0] => ------------
            [1] => -----------------
            [2] => ---------------------------------------------------------------------------------------------------------------------
            [3] => ----------------------
            [4] => ------
            [5] => -------------
            [6] => ------
        )
        we now we know the length of "checktype" [0], "item" [1], "params"[2], "description"[3], "groups"[4] and so on to parse the table witout regex and sutff like this.
        */

        $startParsing = false;
        $return = [];
        foreach ($output as $line) {
            if (preg_match('/---+/', $line)) {
                $lens = explode(' ', trim($line));
                $startParsing = true;
                continue;
            }

            if ($startParsing) {
                //ltrim() to remove the spaces on the left hand site of the string "  "if64
                $line = ltrim($line);
                $return[] = [
                    // Why strlen($lens[0]) + 1 ? Because we explode by space, because it is the delimiter. So the space is missing now in our strlen() and we needto add it :)
                    'checktype'   => trim(substr($line, 0, strlen($lens[0]))),
                    'item'        => trim(substr($line, strlen($lens[0]) + 1, strlen($lens[1]))),
                    'params'      => trim(substr($line, strlen($lens[0]) + 1 + strlen($lens[1]) + 1, strlen($lens[2]))),
                    'description' => trim(substr($line, strlen($lens[0]) + 1 + strlen($lens[1]) + 1 + strlen($lens[2]) + 1, strlen($lens[3]))),
                    'groups'      => '',
                    'summarized'  => '',
                    'to_groups'   => '',
                ];
            }
        }

        return $return;
    }

    public function parseMkListChecks($output = []) {
        $return = [
            'tcp'    => [],
            'snmp'   => [],
            'active' => [],
        ];

        /*$output = [
            '3ware_disks                                  tcp    State of disks of 3ware ATA RAID Controller',
            '3ware_info                                   tcp    General information about 3ware ATA RAID Controller',
            '3ware_units                                  tcp    State of units of 3ware ATA RAID Controller',
            'ad_replication                               tcp    State of Microsoft Active Directory Replication between DCs',
            'adva_fsp_current                             snmp   Adva FSP 3000 Optical Transport: Current of Power Supplies'
        ];*/

        foreach ($output as $line) {
            $line = trim($line); //remove spaces - if any
            $checktype = explode(' ', $line, 2); // 3ware_disks, 3ware_info etc...
            //debug($checktype);
            if (count($checktype) > 1) {
                if (!is_null($checktype[1])) {
                    $type = explode(' ', trim($checktype[1]), 2); //tcp, snmp or active
                    $description = trim($type[1]); // State of disks of 3ware ATA RAID Controller

                    $return[$type[0]][] = [
                        'checktype'   => $checktype[0],
                        'type'        => $type[0],
                        'description' => $description,
                    ];
                    unset($checktype, $type, $description);
                }
            }


        }

        return $return;
    }

    public function compareDumpWithList($MkDumpAsAray, $MkCheckListAsArray, $type = 'tcp', $ignore = []) {
        $return = [];
        $MkCheckNames = Hash::extract($MkCheckListAsArray, $type . '.{n}.checktype');
        foreach ($MkDumpAsAray as $MkArray) {
            if (in_array($MkArray['checktype'], $MkCheckNames)) {
                if (in_array($MkArray['checktype'], $ignore)) {
                    continue;
                }
                $return[] = $MkArray;
            }
        }

        return $return;
    }

    public function parseMkRawAgentOutput($processesResultRaw = [], $illegal_object_name_chars, $systemsettings = []) {
        //Build illegal chars RegEx
        $illegalCharsAsRegEx = '/[\\' . implode('\\', str_split($illegal_object_name_chars, 1)) . ']/';
        $MkProcesses = [];
        $MkProgressConfig = [];

        $return = [
            'ps'       => [],
            'services' => [],
        ];

        if (sizeof($processesResultRaw) > 0) {
            foreach ($processesResultRaw as $key => $value) {
                preg_match('/\<\<\<([^\]]*)\>\>\>/', $value, $matches);
                if (sizeof($matches) > 0 && isset($matches[1])) {
                    $process_list_key = $matches[1];
                    $real_processes = true;
                    continue;
                }
                if ($real_processes) {
                    if (strpos($value, '[') === 0) {
                        unset($processesResultRaw[$key]);
                        continue;
                    }

                    //ps on linux, ps:sep(0) on windows
                    if ($process_list_key == 'ps') {
                        $regex_value = trim(preg_replace('/\((.*)\)\s/', '', $value));

                        if (preg_match('/(?:\/){0,1}' . $systemsettings['CHECK_MK']['CHECK_MK.MATCH'] . '\s/', $regex_value)) {
                            $value_arr[0] = $regex_value;
                        } else {
                            $value_arr = explode(' ', $regex_value);
                        }

                        $MkProcesses[$process_list_key][] = preg_replace($illegalCharsAsRegEx, '', $value_arr[0]);

                    } else if ($process_list_key == 'services') {
                        $value_arr = explode(' ', $value);
                        $regex_value = trim(preg_replace('/\((.*)\)\s/', '', $value_arr[0]));
                        $MkProcesses[$process_list_key][] = preg_replace($illegalCharsAsRegEx, '', $regex_value);

                    } else if (preg_match('/^ps:sep\(\d+\)$/', $process_list_key)) {
                        if (preg_match('/^\(.*\)\s*(.*)$/', $value, $pmatch)) {
                            $pvalue = $pmatch[1];
                        } else {
                            $pvalue = $value;
                        }
                        $MkProcesses['ps'][] = preg_replace($illegalCharsAsRegEx, '', $pvalue);
                    }
                }
            }

            foreach ($MkProcesses as $key => $process_value) {
                foreach ($process_value as $processData) {
                    switch ($key) {
                        case 'ps':
                            $return[$key][md5($processData)] = [
                                'checktype'   => $key,
                                'item'        => (preg_match('/(?:\/){0,1}' . $systemsettings['CHECK_MK']['CHECK_MK.MATCH'] . '\s/', $processData)) ? "~" . $processData : $processData,
                                'description' => 'proc_' . $processData,
                            ];
                            break;

                        case 'services':
                            $return[$key][md5($processData)] = [
                                'checktype'   => $key,
                                'item'        => (preg_match('/(?:\/){0,1}' . $systemsettings['CHECK_MK']['CHECK_MK.MATCH'] . '\s/', $processData)) ? "~" . $processData : $processData,
                                'description' => 'service_' . $processData,
                            ];
                            break;
                    }

                }
            }

        }

        return $return;
    }
}
