<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

use itnovum\openITCOCKPIT\Core\HoststatusFields;

class MkLogfileController extends MkModuleAppController {
    public $layout = 'Admin.default';

    public $components = [
        'ListFilter.ListFilter',
        'RequestHandler',
        'AdditionalLinks',
    ];
    public $helpers = ['ListFilter.ListFilter', 'Status', 'Monitoring'];

    public $uses = ['Servicetemplate', 'Service', MONITORING_HOSTSTATUS];

    public function showCheckMKLogfile($id = null) {
        if ($this->Service->exists($id)) {
            $service = $this->Service->find('first', [
                'fields'     => [
                    'Service.id',
                    'Service.name',
                    'Host.uuid',
                    'Host.id',
                    'Host.name',
                    'Host.address',
                ],
                'conditions' => [
                    'Service.id' => $id,
                ],
                'contain'    => [
                    'Host' => [
                    ],
                ],
            ]);
        }
        $filterFrom = date('Y-m-d H:i:s', strtotime(' -1 hour'));
        $filterTo = date('Y-m-d H:i:s');
        if ($this->request->is('post') || $this->request->is('put')) {
            $dateStart = DateTime::createFromFormat('Y-m-d H:i:s', $this->request->data['from']);
            if ($dateStart && $dateStart->format('Y-m-d H:i:s') === $this->request->data['from']) {
                $filterFrom = $this->request->data['from'];
            }
            $dateEnd = DateTime::createFromFormat('Y-m-d H:i:s', $this->request->data['to']);
            if ($dateEnd && $dateEnd->format('Y-m-d H:i:s') === $this->request->data['to']) {
                $filterTo = $this->request->data['to'];
            }
        }

        $HoststatusFields = new HoststatusFields($this->DbBackend);
        $HoststatusFields->currentState();
        $hoststatus = $this->Hoststatus->byUuid($service['Host']['uuid'], $HoststatusFields);
        $logfileContent = $this->loadCheckMkLogfile($service['Host']['uuid'], $service['Service']['name'], $filterFrom, $filterTo);
        if (is_null($logfileContent)) {
            $this->setFlash(__('MkLogfile is empty or does not exist.'), false);
            $this->redirect('/services/browser/' . $id);
        }
        $this->set(compact(['logfileContent', 'service', 'hoststatus', 'filterFrom', 'filterTo']));

    }


    public function modifyCheckMkLogfile() {
        $this->autoRender = false;
        $hostUuid = $this->request->data['host_uuid'];
        $serviceName = $this->request->data['service_name'];
        $arrayKeysToDelete = $this->request->data['arrayKeys'];
        $FileAsArray = $this->loadCheckMkLogfile($hostUuid, $serviceName);

        foreach ($arrayKeysToDelete as $keyToDelete) {
            unset($FileAsArray[$keyToDelete]);
        }

        return $this->updateCheckMkLogfile($hostUuid, $serviceName, $FileAsArray);

    }

    private function loadCheckMkLogfile($hostUuid, $serviceName, $from = null, $to = null) {
        $logtype = explode(' ', $serviceName);
        $logFilename = $this->getMkLogBasePath() . $hostUuid . '/' . $logtype[1];
        if (!file_exists($logFilename)) {
            return null;
        }
        $file = new File($logFilename);
        $logContents = $file->read();
        $logArr = explode("\n", $logContents);

        $FileAsArray = [];
        $LastMatch = null;
        foreach ($logArr as $logRow) {
            preg_match('/<<<([^>]*)>>>/', $logRow, $matches);
            if (sizeof($matches) > 0) {
                $LastMatch = trim($matches[1]);
            } else if (!is_null($LastMatch)) {
                $FileAsArray[$LastMatch][] = $logRow;
            }
        }
        if (!is_null($from) && !is_null($to)) {
            foreach ($FileAsArray as $timeD => $infoArr) {
                if ($timeD < $from || $timeD > $to) {
                    unset($FileAsArray[$timeD]);
                }
            }
        }

        return ($FileAsArray);
    }

    private function updateCheckMkLogfile($hostUuid, $serviceName, $FileAsArray) {
        $logtype = explode(' ', $serviceName);
        $logFilename = $this->getMkLogBasePath() . $hostUuid . '/' . $logtype[1];
        if (!file_exists($logFilename)) {
            return null;
        }

        $file = new File($logFilename);
        $contents = '';
        foreach ($FileAsArray as $key => $value) {
            $contents .= "<<<$key>>>\n";
            if (is_array($value)) {
                foreach ($value as $val) {
                    $contents .= "$val\n";
                }
            } else {
                $contents .= "$value\n";
            }
        }

        return $file->write($contents);

    }

    private function getMkLogBasePath() {
        return '/opt/openitc/nagios/3rd/check_mk/var/logwatch/';
    }

}
