<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement. 
// The license agreement and license key were sent with the order confirmation.
class ScansController extends MkModuleAppController {
    public $layout = 'Admin.default';

    public $uses = ['Host', 'Systemsetting', 'MkModule.Mkcheck', 'MkModule.Mkservicedata', 'Service', 'MkModule.Mkagent', 'MkModule.LastUsedMkagent'];
    public $components = [MONITORING_CORECONFIG, 'MkModule.MkParser', 'GearmanClient'];

    public function index($host_id = null) {
        if (!$this->Host->exists($host_id)) {
            throw new NotFoundException(__('Invalid host'));
        }

        $_host = $this->Host->find('first', [
            'conditions' => [
                'Host.id' => $host_id,
            ],
            'fields'     => [
                'Host.id',
                'Host.uuid',
                'Host.name',
                'Host.address',
            ],
            'contain'    => [],
        ]);

        $this->set('host', $_host);
        $this->set('back_url', $this->referer());
        $this->Frontend->setJson('executing_remote_discovery', __('Executing remote discovery...'));
        $this->set('host_id', $host_id);

        $allMkAgents = $this->Mkagent->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'Mkagent.container_id' => $this->MY_RIGHTS,
            ],
        ]);
        $mkAgentsList = [
            0  => __('Execute remote discovery'),
            -1 => __('Execute SNMP discovery')
        ];
        foreach ($allMkAgents as $mkAgent) {
            $mkAgentsList[$mkAgent['Mkagent']['id']] = $mkAgent['Mkagent']['name'];
        }
        $lastSelectedAgent = $this->LastUsedMkagent->findByHostId($host_id);
        $selectedAgent = 0;
        if (!empty($lastSelectedAgent)) {
            $selectedAgent = $lastSelectedAgent['LastUsedMkagent']['agent_id'];
        }

        $mkCheckCount = $this->Mkcheck->find('count');
        if ($mkCheckCount == 0) {
            $this->setFlash(__('No MKChecks defined! Check_MK will not work.'), false);
        }

        //Create services after submit
        if ($this->request->is('post') || $this->request->is('put')) {
            App::uses('UUID', 'Lib');
            //$mkChecks = $this->Mkcheck->find('all');
            $mkSession = $this->Session->read('mk');
            foreach ($this->request->data['Scan'] as $healthPsOrServiceKey => $serviceHashesAsArray) {
                if (is_array($serviceHashesAsArray)) {
                    foreach ($serviceHashesAsArray as $serviceHash) {
                        switch ($healthPsOrServiceKey) {
                            case 'health':
                                if (!isset($mkSession['health']['result'][$serviceHash])) {
                                    continue;
                                }
                                $check = $mkSession['health']['result'][$serviceHash];
                                $is_process = 0;
                                break;

                            case 'ps':
                                if (!isset($mkSession['ps'][$serviceHash])) {
                                    continue;
                                }
                                $check = $mkSession['ps'][$serviceHash];
                                $is_process = 1;
                                break;

                            case 'services':
                                if (!isset($mkSession['services'][$serviceHash])) {
                                    continue;
                                }
                                $check = $mkSession['services'][$serviceHash];
                                $is_process = 1;
                                break;
                        }

                        $data = [
                            'Service' => [
                                'uuid'               => UUID::v4(),
                                'name'               => $check['description'],
                                'servicetemplate_id' => $check['servicetemplate_id'],
                                'host_id'            => $host_id,
                                'own_contacts'       => 0,
                                'own_contactgroups'  => 0,
                                'service_type'       => MK_SERVICE,
                            ],
                        ];
                        //Create the service
                        $this->Service->create();
                        if ($this->Service->save($data)) {
                            $this->Mkservicedata->create();
                            $this->Mkservicedata->save([
                                'service_id' => $this->Service->id,
                                'host_id'    => $host_id,
                                'is_process' => $is_process,
                                'check_name' => $check['checktype'],
                                'check_item' => $check['item'],
                            ]);
                        }
                    }
                }
            }
            //Redirect to service list:
            $this->setFlash(__('MK service created successfully'));
            $this->redirect('/services/serviceList/' . $host_id);
        }
        $this->Frontend->setJson('hostId', $host_id);
        $this->set('allMkAgents', $mkAgentsList);
        $this->set('selectedAgent', $selectedAgent);
    }

    public function ajaxHealthList($host_id) {
        $agentId = null;
        if ($this->request->is('post') && isset($this->request->data['agentId'])) {
            $agentId = $this->request->data['agentId'];
            $thisLastUsedMkagent = $this->LastUsedMkagent->findByHostId($host_id);
            if (empty($thisLastUsedMkagent)) {
                $this->LastUsedMkagent->save([
                    'LastUsedMkagent' => [
                        'host_id'  => $host_id,
                        'agent_id' => $agentId
                    ]
                ]);
            } else if ($thisLastUsedMkagent['LastUsedMkagent']['agent_id'] != $agentId) {
                $thisLastUsedMkagent['LastUsedMkagent']['agent_id'] = $agentId;
                $this->LastUsedMkagent->save($thisLastUsedMkagent);
            }
        } else {
            $this->LastUsedMkagent->deleteAll(['LastUsedMkagent.host_id' => $host_id]);
        }
        $this->allowOnlyAjaxRequests();
        $discoveryResult = $this->_fetchHealthList($host_id);

        $this->Session->write('mk.health', $discoveryResult);

        $this->set(compact(['discoveryResult']));
        $this->set('_serialize', ['discoveryResult']);
    }

    private function _fetchHealthList($host_id) {
        if (!$this->Host->exists($host_id)) {
            throw new NotFoundException(__('Invalid host'));
        }

        $host = $this->Host->find('first', [
            'conditions' => [
                'Host.id' => $host_id,
            ],
            'fields'     => [
                'Host.id',
                'Host.uuid',
                'Host.address',
                'Host.satellite_id',
            ],
            'contain'    => [],
        ]);

        $discoveryResultRaw = $this->GearmanClient->send('CheckMkDiscovery', [
            'hostUuid'     => $host['Host']['uuid'],
            'hostaddress'  => $host['Host']['address'],
            'satellite_id' => $host['Host']['satellite_id'],
        ]);

        $mkListRaw = $this->GearmanClient->send('CheckMKListChecks', [
            'satellite_id' => $host['Host']['satellite_id'],
        ]);


        $MkCheckList = $this->MkParser->parseMkListChecks($mkListRaw);
        $discoveryResult = $this->MkParser->parseMkDumpOutput($discoveryResultRaw);
        $discoveryResult = $this->MkParser->compareDumpWithList($discoveryResult, $MkCheckList, 'tcp', ['ps', 'service']);

        $_MkChecks = $this->Mkcheck->find('all', [
            'conditions' => [
                'Mkcheck.servicetemplate_id IS NOT NULL',
            ],
            'contain'    => [],
            'fields'     => [
                'name',
                'servicetemplate_id',
            ],
        ]);

        $MkChecks = [];
        foreach ($_MkChecks as $mkcheck) {
            $MkChecks[$mkcheck['Mkcheck']['name']] = $mkcheck['Mkcheck']['servicetemplate_id'];
        }

        $return = [
            'result'           => [],
            'missing_template' => [],
        ];

        $existingServices = $this->_getExistingServices($host_id);
        foreach ($discoveryResult as $result) {
            //Filter exsisting services
            if (in_array($result['description'], $existingServices)) {
                continue;
            }
            if (isset($MkChecks[$result['checktype']])) {
                // We can create this services, because we have a service template + MK Check for it
                $return['result'][md5($result['description'])] = [
                    'checktype'          => $result['checktype'],
                    'item'               => $result['item'],
                    'params'             => $result['params'],
                    'description'        => $result['description'],
                    'servicetemplate_id' => $MkChecks[$result['checktype']],
                ];
            } else {
                // Missing MK Check or service template
                $return['missing_template'][md5($result['description'])] = [
                    'checktype'          => $result['checktype'],
                    'item'               => $result['item'],
                    'params'             => $result['params'],
                    'description'        => $result['description'],
                    'servicetemplate_id' => null,
                ];
            }
        }

        return $return;
    }

    public function ajaxProcessList($host_id) {
        $this->allowOnlyAjaxRequests();

        $_MkChecks = $this->Mkcheck->find('all', [
            'conditions' => [
                'Mkcheck.servicetemplate_id IS NOT NULL',
            ],
            'contain'    => [],
            'fields'     => [
                'name',
                'servicetemplate_id',
            ],
        ]);

        $MkChecks = [];
        foreach ($_MkChecks as $mkcheck) {
            $MkChecks[$mkcheck['Mkcheck']['name']] = $mkcheck['Mkcheck']['servicetemplate_id'];
        }


        $_MkProcesses = $this->_fetchProcesses($host_id);

        $MkProcesses = [];
        //debug($MkChecks);
        $existingServices = $this->_getExistingServices($host_id);
        foreach ($_MkProcesses as $key => $MkProcessAsArray) {
            if (!empty($MkProcessAsArray)) {
                foreach ($MkProcessAsArray as $serviceHash => $MkProcess) {
                    //Filter existing mk services
                    if (!in_array($MkProcess['description'], $existingServices)) {
                        //Is there an MK Servicetemplate for given check?
                        if (isset($MkChecks[$key])) {
                            $MkProcesses[$key][$serviceHash] = Hash::merge($MkProcess, ['servicetemplate_id' => $MkChecks[$key]]);
                        }
                    }
                }

            } else {
                $MkProcesses[$key] = [];
            }
        }

        //No ps key, because of missing template?
        if (!isset($MkProcesses['ps'])) {
            $MkProcesses['ps'] = [];
        }

        //No services key, because of missing template?
        if (!isset($MkProcesses['services'])) {
            $MkProcesses['services'] = [];
        }

        $this->Session->write('mk.ps', $MkProcesses['ps']);
        $this->Session->write('mk.services', $MkProcesses['services']);

        //With this trick, we dont need to create an ajax view template ;)
        $this->set(compact(['MkProcesses']));
        $this->set('_serialize', ['MkProcesses']);
    }


    private function _fetchProcesses($host_id) {
        if (!$this->Host->exists($host_id)) {
            throw new NotFoundException(__('Invalid host'));
        }

        $host = $this->Host->find('first', [
            'conditions' => [
                'Host.id' => $host_id,
            ],
            'fields'     => [
                'Host.id',
                'Host.address',
                'Host.uuid',
                'Host.satellite_id',
            ],
            'contain'    => [],
        ]);

        $processesResultRaw = $this->GearmanClient->send('CheckMKProcesses', [
            'hostaddress'  => $host['Host']['address'],
            'hostUuid'     => $host['Host']['uuid'],
            'satellite_id' => $host['Host']['satellite_id'],
        ]);

        $processesResult = $this->MkParser->parseMkRawAgentOutput($processesResultRaw, $this->CoreConfig->read('illegal_object_name_chars'), $this->Systemsetting->findAsArray());

        return $processesResult;
    }

    private function _getExistingServices($host_id) {
        $services = $this->Service->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'Service.host_id'                       => $host_id,
                'Service.service_type'                  => MK_SERVICE,
                'Servicetemplate.active_checks_enabled' => 0,
            ],
            'contain'    => [
                'Servicetemplate' => [
                    'fields' => [
                        'Servicetemplate.name',
                        'Servicetemplate.active_checks_enabled',
                    ],
                ],
            ],
            'fields'     => [
                'Service.name',
                'Service.servicetemplate_id',
            ],
        ]);
        $existingServices = [];
        foreach ($services as $service) {
            $existingServices[] = $service['Service']['name'];

            if ($service['Service']['name'] == '' || $service['Service']['name'] == null) {
                $existingServices[] = $service['Servicetemplate']['name'];
            }
        }

        return $existingServices;
    }

}
