<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement. 
// The license agreement and license key were sent with the order confirmation.
class SnmpController extends MkModuleAppController {
    public $layout = 'Admin.default';

    public $uses = ['Host', 'Systemsetting', 'MkModule.Mkcheck', 'MkModule.Mkservicedata', 'Service', 'MkModule.Mksnmp', 'MkModule.LastUsedMkagent'];
    public $components = [MONITORING_CORECONFIG, 'MkModule.MkParser', 'GearmanClient'];

    public function index($host_id) {
        if (!$this->Host->exists($host_id)) {
            throw new NotFoundException(__('Invalid host'));
        }


        $this->set('back_url', $this->referer());

        $Mksnmp = $this->Mksnmp->find('first', [
            'conditions' => [
                'Mksnmp.host_id' => $host_id,
            ],
        ]);

        $this->Frontend->setJson('clearUserPassword', !isset($Mksnmp['Mksnmp']['username']) || empty($Mksnmp['Mksnmp']['username']));

        if ($this->request->is('post') || $this->request->is('put')) {
            $this->request->data['Mksnmp']['host_id'] = $host_id;

            $this->LastUsedMkagent->deleteAll(['LastUsedMkagent.host_id' => $host_id]);
            $this->LastUsedMkagent->save([
                'LastUsedMkagent' => [
                    'host_id'  => $host_id,
                    'agent_id' => -1
                ]
            ]);
            if (empty($Mksnmp)) {
                $this->Mksnmp->create();
            } else {
                $this->request->data['Mksnmp']['id'] = $Mksnmp['Mksnmp']['id'];
            }

            if ($this->Mksnmp->save($this->request->data)) {
                return $this->redirect([
                    'plugin'     => 'mk_module',
                    'controller' => 'snmp',
                    'action'     => 'execute',
                    $host_id,
                ]);
            } else {
                $this->setFlash(__('Cloud not save data'), false);
            }
        }
        $snmpVersions = [
            1 => __('SNMP V 1'),
            2 => __('SNMP V 2c'),
            3 => __('SNMP V 3'),
        ];

        $hashAlgorithm = [
            1 => 'MD5',
            2 => 'SHA',
        ];

        $securityLevel = [
            1 => 'authPriv',
            2 => 'authNoPriv',
            3 => 'noAuthnoPriv',
        ];

        $encryptionAlgorithm = [
            1 => 'AES',
            2 => 'DES',
        ];


        $host = $this->Host->find('first', [
            'conditions' => [
                'Host.id' => $host_id,
            ],
            'fields'     => [
                'id',
                'uuid',
                'container_id',
                'name',
                'address',
            ],
            'contain'    => [],
        ]);

        //Refil fields in view
        $this->request->data = $Mksnmp;

        $this->set(compact(['host', 'snmpVersions', 'hashAlgorithm', 'securityLevel', 'encryptionAlgorithm']));

    }

    public function execute($host_id = null) {
        if (!$this->Host->exists($host_id)) {
            throw new NotFoundException(__('Invalid host'));
        }

        $_host = $this->Host->find('first', [
            'conditions' => [
                'Host.id' => $host_id,
            ],
            'fields'     => [
                'Host.id',
                'Host.uuid',
                'Host.name',
                'Host.address',
            ],
            'contain'    => [],
        ]);

        $this->set('host', $_host);

        if ($this->request->is('post') || $this->request->is('put')) {
            App::uses('UUID', 'Lib');
            $SnmpScan = $this->Session->read('mk.snmp');

            if (isset($this->request->data['Mksnmp']['service_hash'])) {
                foreach ($this->request->data['Mksnmp']['service_hash'] as $hash) {
                    if (isset($SnmpScan['result'][$hash])) {
                        $rawService = $SnmpScan['result'][$hash];

                        //Create the service
                        $data = [
                            'Service' => [
                                'uuid'               => UUID::v4(),
                                'name'               => $rawService['description'],
                                'servicetemplate_id' => $rawService['servicetemplate_id'],
                                'host_id'            => $host_id,
                                'own_contacts'       => 0,
                                'own_contactgroups'  => 0,
                                'service_type'       => MK_SERVICE,
                            ],
                        ];

                        $this->Service->create();
                        if ($this->Service->save($data)) {
                            // Create Mkservicedata record to create mk config files in MkModuleNagiosExportTask.php
                            $this->Mkservicedata->create();
                            $this->Mkservicedata->save([
                                'service_id' => $this->Service->id,
                                'host_id'    => $host_id,
                                'is_process' => 0,
                                'check_name' => $rawService['checktype'],
                                'check_item' => $rawService['item'],
                            ]);
                        }
                    }

                }
            }
            //Redirect to service list:
            $this->setFlash(__('MK service created successfully'));
            $this->redirect('/services/serviceList/' . $host_id);

            return;
        }

        $Mksnmp = $this->Mksnmp->find('first', [
            'conditions' => [
                'Mksnmp.host_id' => $host_id,
            ],
        ]);

        if (empty($Mksnmp)) {
            throw new NotFoundException(__('SNMP configuration not found!'));
        }


        $this->set('host_id', $host_id);
        $this->set('back_url', $this->referer());
        $this->Frontend->setJson('hostId', $host_id);
    }

    public function executeSnmpDiscoveryByAjax($host_id = null) {
        $this->allowOnlyAjaxRequests();
        if (!$this->Host->exists($host_id)) {
            throw new NotFoundException(__('Invalid host'));
        }

        $snmpResult = $this->_fetchData($host_id);

        $this->Session->write('mk.snmp', $snmpResult);
        //With this trick, we dont need to create an ajax view template ;)
        $this->set(compact(['snmpResult']));
        $this->set('_serialize', ['snmpResult']);
    }

    private function _fetchData($host_id = null) {
        $Mksnmp = $this->Mksnmp->find('first', [
            'conditions' => [
                'Mksnmp.host_id' => $host_id,
            ],
        ]);

        $host = $this->Host->find('first', [
            'conditions' => [
                'Host.id' => $host_id,
            ],
            'fields'     => [
                'id',
                'uuid',
                'container_id',
                'name',
                'address',
                'satellite_id'
            ],
            'contain'    => [],
        ]);


        $Mksnmp['Mksnmp']['version'] = (int)$Mksnmp['Mksnmp']['version'];

        $payload = [
            'hostUuid'       => $host['Host']['uuid'],
            'hostuuid'       => $host['Host']['uuid'],
            'host_address'   => $host['Host']['address'],
            'satellite_id'   => $host['Host']['satellite_id'],
            'snmp_version'   => $Mksnmp['Mksnmp']['version'],
            'snmp_community' => $Mksnmp['Mksnmp']['community'],
        ];

        if ($Mksnmp['Mksnmp']['version'] == 3) {
            $payload['v3'] = [
                'security_level'       => $Mksnmp['Mksnmp']['security_level'],
                'hash_algorithm'       => $Mksnmp['Mksnmp']['hash_algorithm'],
                'username'             => $Mksnmp['Mksnmp']['username'],
                'password'             => $Mksnmp['Mksnmp']['password'],
                'encryption_algorithm' => $Mksnmp['Mksnmp']['encryption_algorithm'],
                'encryption_password'  => $Mksnmp['Mksnmp']['encryption_password'],
            ];
        }

        $processesResultRaw = $this->GearmanClient->send('CheckMKSNMP', $payload);
        unset($payload);
        //debug($processesResultRaw);

        $mkListRaw = $this->GearmanClient->send('CheckMKListChecks', [
            'satellite_id' => $host['Host']['satellite_id'],
        ]);

        $MkCheckList = $this->MkParser->parseMkListChecks($mkListRaw);
        $scanResult = $this->MkParser->parseMkDumpOutput($processesResultRaw);

        $scanResult = $this->MkParser->compareDumpWithList($scanResult, $MkCheckList, 'snmp');

        $_MkChecks = $this->Mkcheck->find('all', [
            'conditions' => [
                'Mkcheck.servicetemplate_id IS NOT NULL',
            ],
            'contain'    => [],
            'fields'     => [
                'name',
                'servicetemplate_id',
            ],
        ]);

        $MkChecks = [];
        foreach ($_MkChecks as $mkcheck) {
            $MkChecks[$mkcheck['Mkcheck']['name']] = $mkcheck['Mkcheck']['servicetemplate_id'];
        }

        $return = [
            'result'           => [],
            'missing_template' => [],
        ];

        $existingServices = $this->_getExistingServices($host_id);

        foreach ($scanResult as $result) {

            if (isset($MkChecks[$result['checktype']])) {
                // We can create this services, because we have a service template + MK Check for it

                //Filter already esisting services
                if (!in_array($result['description'], $existingServices)) {
                    $return['result'][md5($result['description'])] = [
                        'checktype'          => $result['checktype'],
                        'item'               => $result['item'],
                        'params'             => $result['params'],
                        'description'        => $result['description'],
                        'servicetemplate_id' => $MkChecks[$result['checktype']],
                    ];
                }

            } else {
                // Missing MK Check or service template
                $return['missing_template'][md5($result['description'])] = [
                    'checktype'          => $result['checktype'],
                    'item'               => $result['item'],
                    'params'             => $result['params'],
                    'description'        => $result['description'],
                    'servicetemplate_id' => null,
                ];
            }


        }

        return $return;
    }

    private function _getExistingServices($host_id) {
        $services = $this->Service->find('all', [
            'recursive'  => -1,
            'conditions' => [
                'Service.host_id'                       => $host_id,
                'Service.service_type'                  => MK_SERVICE,
                'Servicetemplate.active_checks_enabled' => 0,
            ],
            'contain'    => [
                'Servicetemplate' => [
                    'fields' => [
                        'Servicetemplate.name',
                        'Servicetemplate.active_checks_enabled',
                    ],
                ],
            ],
            'fields'     => [
                'Service.name',
                'Service.servicetemplate_id',
            ],
        ]);
        $existingServices = [];
        foreach ($services as $service) {
            $existingServices[] = $service['Service']['name'];

            if ($service['Service']['name'] == '' || $service['Service']['name'] == null) {
                $existingServices[] = $service['Servicetemplate']['name'];
            }
        }

        return $existingServices;
    }

}