<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.
use itnovum\openITCOCKPIT\Core\Views\UserTime;


/**
 * Class OpenstreetmapController
 * @property Openstreetmap $Openstreetmap
 */
class OpenstreetmapController extends OpenstreetmapModuleAppController {

    public $uses = [
        'Location',
        'Container',
        'OpenstreetmapModule.Openstreetmap',
        'OpenstreetmapModule.OpenstreetmapSetting',
        'Host',
        'Service',
        MONITORING_HOSTSTATUS,
        MONITORING_SERVICESTATUS,
    ];

    public $layout = 'angularjs';

    public $defaultStates = [
        'not_monitored'       => 1,
        'up_ok'               => 1,
        'warning'             => 1,
        'down_critical'       => 1,
        'unreachable_unknown' => 1
    ];

    public $filterStates = [];
    public $hideEmptyLocations = 1;

    public function index() {
        if ($this->isAngularJsRequest()) {
            if (!empty($this->request->query('filterStates'))) {
                $this->filterStates = array_map('intval', $this->request->query('filterStates'));
            } else {
                $this->filterStates = $this->OpenstreetmapSetting->getSettings()['OpenstreetmapSetting']['state_filter'];
            }

            if (!empty($this->request->query('hideEmptyLocations')) || $this->request->query('hideEmptyLocations') === "0") {
                $this->hideEmptyLocations = (int)$this->request->query('hideEmptyLocations');
            } else {
                $this->hideEmptyLocations = (int)$this->OpenstreetmapSetting->getSettings()['OpenstreetmapSetting']['hide_empty_locations'];
            }
            list($locations, $emptyLocations) = $this->buildLocations($this->filterStates);
            foreach ($locations as $location) {
                $locationPoints[] = [$location['Location']['latitude'], $location['Location']['longitude']];
            }

            $hideEmptyLocations = $this->hideEmptyLocations;
            $filterStates = $this->filterStates;
            $unfilteredLocationsAvailable = 0;
            if (sizeof($this->buildLocations($this->defaultStates)[0]) > 0) {
                $unfilteredLocationsAvailable = 1;
            }
            $this->set(compact(['locations', 'locationPoints', 'emptyLocations', 'filterStates', 'hideEmptyLocations', 'unfilteredLocationsAvailable']));
            $this->set('_serialize', ['locations', 'locationPoints', 'emptyLocations', 'filterStates', 'hideEmptyLocations', 'unfilteredLocationsAvailable']);
        }
    }

    /**
     * @param $filterStates
     * @return array
     */
    public function buildLocations($filterStates) {
        $includedLocationIds = $this->request->query('includedLocationId');

        $query = [
            'recursive'  => -1,
            'contain'    => [
                'Container'
            ],
            'conditions' => [
                'Location.latitude IS NOT NULL',
                'Location.longitude IS NOT NULL',
                'Container.id' => $this->MY_RIGHTS
            ],
            'fields'     => [
                'Location.id',
                'Location.latitude',
                'Location.longitude',
                'Location.description',
                'Container.name'
            ]
        ];
        if (!empty($includedLocationIds)) {
            $query['conditions']['Location.id'] = $includedLocationIds;
        }

        $filtered_locations = [];
        $empty_locations = [];
        $locations = $this->Location->find('all', $query);
        foreach ($locations as $key => $location) {
            $hostsCounter = 0;
            $containerIds = $this->Tree->removeRootContainer(
                $this->Tree->resolveChildrenOfContainerIds($location['Container']['id'], false, [
                    CT_NODE,
                    CT_TENANT,
                    CT_LOCATION
                ])
            );
            if (!empty($containerIds)) {
                $hostsCounter = $this->Host->find('count', [
                    'conditions' => [
                        'Host.container_id' => $containerIds
                    ]
                ]);
            }

            if ($hostsCounter > 0 || $this->hideEmptyLocations == 0) {
                $colorCode = $this->Openstreetmap->getColorCodeForStateByContainerIds(
                    $containerIds,
                    $this->Host,
                    $this->Hoststatus,
                    $this->Service,
                    $this->Servicestatus
                );
                if ($this->colorCodeToState(intval($colorCode)) != '' && $filterStates[$this->colorCodeToState(intval($colorCode))] == 1) {
                    $locations[$key]['Location']['colorcode'] = $colorCode;
                    $filtered_locations[] = $locations[$key];
                }
            } else {
                $empty_locations[] = $locations[$key];
            }
        }

        return [
            $filtered_locations,
            $empty_locations
        ];
    }

    public function colorCodeToState($colorCode) {
        if ($colorCode == -1) return 'not_monitored';
        if ($colorCode == 0) return 'up_ok';
        if ($colorCode == 1) return 'warning';
        if ($colorCode == 2) return 'down_critical';
        if ($colorCode == 3) return 'unreachable_unknown';
        return '';
    }

    public function getAclAndSettings() {
        if ($this->isAngularJsRequest()) {
            $acl = [
                'hosts'     => [
                    'index' => isset($this->PERMISSIONS['hosts']['index'])
                ],
                'services'  => [
                    'index' => isset($this->PERMISSIONS['services']['index'])
                ],
                'locations' => [
                    'edit' => isset($this->PERMISSIONS['locations']['edit'])
                ]
            ];
            $settings = $this->OpenstreetmapSetting->find('first', [
                'fields' => [
                    'OpenstreetmapSetting.server_url',
                    'OpenstreetmapSetting.reload_interval'
                ]
            ]);

            $settings['OpenstreetmapSetting']['titleResetButton'] = __('Reset map');
            $this->set('settings', $settings['OpenstreetmapSetting']);
            $this->set('acl', $acl);
            $this->set('_serialize', ['acl', 'settings']);
        }
    }

    public function openstreetmapsummary() {
        $this->layout = 'blank';
        if (!$this->isApiRequest()) {
            return;
        }
        $UserTime = new UserTime($this->Auth->user('timezone'), $this->Auth->user('dateformat'));
        $containerIds = $this->request->query('containerIds');
        $summary = [];
        if (empty($containerIds)) {
            throw new RuntimeException('No container defined');
        }
        $locationContainers = $this->Container->find('all', [
            'recursive'  => -1,
            'joins'      => [
                [
                    'table'      => 'locations',
                    'alias'      => 'Location',
                    'type'       => 'INNER',
                    'conditions' => [
                        'Container.id = Location.container_id'
                    ]
                ]
            ],
            'conditions' => [
                'Container.id' => $containerIds
            ],
            'order'      => [
                'Container.name' => 'asc'
            ],
            'fields'     => [
                'Container.id',
                'Container.name',
                'Location.id'
            ]
        ]);
        foreach ($locationContainers as $key => $locationContainer) {
            $noObjectsInMonitoring = true;
            $containerIds = $this->Tree->removeRootContainer(
                $this->Tree->resolveChildrenOfContainerIds($locationContainer['Container']['id'], false, [
                    CT_NODE,
                    CT_TENANT,
                    CT_LOCATION
                ])
            );
            $hosts = $this->Host->find('all', [
                'recursive'  => -1,
                'contain'    => [
                    'Container',
                    'Service' => [
                        'conditions'      => [
                            'Service.disabled' => 0
                        ],
                        'fields'          => [
                            'Service.uuid',
                            'Service.id',
                            'Service.name'
                        ],
                        'Servicetemplate' => [
                            'fields' => [
                                'Servicetemplate.name'
                            ]
                        ]
                    ]
                ],
                'conditions' => [
                    'Host.container_id' => $containerIds,
                    'Host.disabled'     => 0
                ],
                'fields'     => [
                    'Host.id',
                    'Host.uuid',
                    'Host.name'
                ]
            ]);

            $services = [];
            if (!empty($hosts)) {
                $noObjectsInMonitoring = false;
                if ($this->hasRootPrivileges === false) {
                    if (!$this->allowedByContainerId(Hash::extract($hosts, '{n}.Container.{n}.HostsToContainer.container_id'))) {
                        break;
                    }
                }
                foreach ($hosts as $host) {
                    foreach ($host['Service'] as $serviceData) {
                        $services[$serviceData['id']] = [
                            'Service' => $serviceData,
                            'Host'    => [
                                'name' => $host['Host']['name']
                            ]
                        ];
                    }
                }
                $summary[$locationContainer['Location']['id']] = $this->Openstreetmap->getMapSummary(
                    $this->Host,
                    $this->Hoststatus,
                    $this->Service,
                    $this->Servicestatus,
                    $hosts,
                    $services,
                    $UserTime
                );
            }
            $summary[$locationContainer['Location']['id']]['Location'] = $locationContainer;
            $summary[$locationContainer['Location']['id']]['Location']['noObjectsInMonitoring'] = $noObjectsInMonitoring;

        }
        $this->set('summary', $summary);
        $this->set('_serialize', ['summary']);
    }
}

