<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

use itnovum\openITCOCKPIT\Core\HoststatusFields;
use itnovum\openITCOCKPIT\Core\ServicestatusFields;
use itnovum\openITCOCKPIT\Core\Views\UserTime;


class Openstreetmap extends OpenstreetmapModuleAppModel {

    /**
     * @param $containerIds
     * @param Host $Host
     * @param Hoststatus $Hoststatus
     * @param Service $Service
     * @param Servicestatus $Servicestatus
     * @return int|mixed
     */
    public function getColorCodeForStateByContainerIds($containerIds, Host $Host, Hoststatus $Hoststatus, Service $Service, Servicestatus $Servicestatus) {
        $colorCode = -1;

        /**
         * color codes for OSM Legend
         * -1 => blue 0 => green 1 => yellow 2 => red 3 => gray
         */

        $HoststatusFields = new HoststatusFields($this->DbBackend);
        $HoststatusFields->currentState();

        $hostsUuids = $Host->find('list', [
            'recursive'  => -1,
            'fields'     => [
                'Host.uuid'
            ],
            'conditions' => [
                'Host.container_id' => $containerIds
            ]
        ]);

        $hoststatus = $Hoststatus->byUuid($hostsUuids, $HoststatusFields);
        if (!empty($hoststatus)) {
            $cumulatedState = (int)Hash::apply($hoststatus, '{s}.Hoststatus.current_state', 'max');
            $colorCode = $cumulatedState;
            if ($cumulatedState > 0) {
                //cast for openstreetmap colors array 2 (2 is red if (host)down or (service)critical) => Host Down or Service Critical
                if ($cumulatedState === 1) {
                    return 2;
                }
                //cast for openstreetmap colors array 3 (3 is grey if (host)unreachable or (service)unknown)=> Host Unreachable or Service Unknown
                if ($cumulatedState === 2) {
                    return 3;
                }
            }
        }

        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields->currentState();

        $serviceUuids = $Service->find('list', [
            'recursive'  => -1,
            'joins'      => [
                [
                    'table'      => 'hosts',
                    'alias'      => 'Host',
                    'type'       => 'INNER',
                    'conditions' => [
                        'Host.id = Service.host_id'
                    ]
                ]
            ],
            'fields'     => [
                'Service.uuid'
            ],
            'conditions' => [
                'Host.container_id' => $containerIds
            ]
        ]);
        $servicestatus = $Servicestatus->byUuid($serviceUuids, $ServicestatusFields);

        if (!empty($servicestatus)) {
            //service state cast for openstreetmap colors is unnecessary
            $colorCode = Hash::apply($servicestatus, '{s}.Servicestatus.current_state', 'max');
        }
        return $colorCode;
    }

    /**
     * @param Model $Host
     * @param Model $Hoststatus
     * @param Model $Service
     * @param Model $Servicestatus
     * @param $hosts
     * @param $services
     * @param UserTime $UserTime
     * @return array
     */
    public function getMapSummary(Model $Host, Model $Hoststatus, Model $Service, Model $Servicestatus, $hosts, $services, UserTime $UserTime) {
        $cumulatedHostState = null;
        $cumulatedServiceState = null;
        $hostIdsGroupByState = [
            0 => [],
            1 => [],
            2 => []
        ];
        $serviceIdsGroupByState = [
            0 => [],
            1 => [],
            2 => [],
            3 => []
        ];
        $HoststatusFields = new HoststatusFields($this->DbBackend);
        $HoststatusFields
            ->currentState()
            ->isHardstate()
            ->scheduledDowntimeDepth()
            ->problemHasBeenAcknowledged();

        $hostUuids = Hash::extract($hosts, '{n}.Host.uuid');
        $hoststatusByUuids = $Hoststatus->byUuid($hostUuids, $HoststatusFields);
        $hostStateSummary = $Host->getHostStateSummary($hoststatusByUuids, false);
        $ServicestatusFieds = new ServicestatusFields($this->DbBackend);
        $ServicestatusFieds
            ->currentState()
            ->isHardstate()
            ->scheduledDowntimeDepth()
            ->problemHasBeenAcknowledged();

        $servicesUuids = Hash::extract($services, '{n}.Service.uuid');
        $servicestatusResults = $Servicestatus->byUuid($servicesUuids, $ServicestatusFieds);
        $serviceStateSummary = $Service->getServiceStateSummary($servicestatusResults, false);

        if (!empty($hoststatusByUuids)) {
            $worstHostState = array_values(
                $hoststatusByUuids = Hash::sort($hoststatusByUuids, '{s}.Hoststatus.current_state', 'desc')
            );
            $cumulatedHostState = (int)$worstHostState[0]['Hoststatus']['current_state'];
            $hosts = Hash::combine($hosts, '{n}.Host.uuid', '{n}');
            foreach ($hoststatusByUuids as $hostUuid => $hoststatusByUuid) {
                $hostStatus = new \itnovum\openITCOCKPIT\Core\Hoststatus($hoststatusByUuid['Hoststatus'], $UserTime);
                $currentHostState = $hostStatus->currentState();
                $hostIdsGroupByState[$currentHostState][] = $hosts[$hostUuid]['Host']['id'];
            }

        }

        if (!empty($servicestatusResults)) {
            $worstServiceState = array_values(
                $servicestatusResults = Hash::sort($servicestatusResults, '{s}.Servicestatus.current_state', 'desc')
            );
            $cumulatedServiceState = (int)$worstServiceState[0]['Servicestatus']['current_state'];
            $services = Hash::combine($services, '{n}.Service.uuid', '{n}');
            foreach ($servicestatusResults as $serviceUuid => $servicestatusByUuid) {
                $serviceStatus = new \itnovum\openITCOCKPIT\Core\Servicestatus($servicestatusByUuid['Servicestatus'], $UserTime);
                $currentServiceState = $serviceStatus->currentState();
                $serviceIdsGroupByState[$currentServiceState][] = $services[$serviceUuid]['Service']['id'];
            }

        }
        $CumulatedHostStatus = new \itnovum\openITCOCKPIT\Core\Hoststatus([
            'current_state' => $cumulatedHostState
        ]);
        $CumulatedHumanState = $CumulatedHostStatus->toArray()['humanState'];
        if (($cumulatedHostState === 0 || is_null($cumulatedHostState)) && !is_null($cumulatedServiceState)) {
            $CumulatedServiceStatus = new \itnovum\openITCOCKPIT\Core\Servicestatus([
                'current_state' => $cumulatedServiceState
            ]);
            $CumulatedHumanState = $CumulatedServiceStatus->toArray()['humanState'];
        }

        return [
            'HostSummary'            => $hostStateSummary,
            'ServiceSummary'         => $serviceStateSummary,
            'CumulatedHumanState'    => $CumulatedHumanState,
            'HostIdsGroupByState'    => $hostIdsGroupByState,
            'ServiceIdsGroupByState' => $serviceIdsGroupByState
        ];
    }
}
