<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


class OpenstreetmapSetting extends OpenstreetmapModuleAppModel {
    public $validate = [
        'server_url'      => [
            'notBlank' => [
                'rule'       => 'notBlank',
                'message'    => 'This field cannot be left blank.',
                'required'   => true,
                'allowEmpty' => false
            ],
        ],
        'reload_interval' => [
            'allowEmpty' => [
                'rule'     => 'notBlank',
                'message'  => 'This field cannot be left blank.',
                'required' => true,
            ],
            'numeric'    => [
                'rule'    => 'numeric',
                'message' => 'This field need to be numeric.',
            ],
            'comparison' => [
                'rule'    => ['comparison', '>=', 15],
                'message' => 'This value need to be at least 15',
            ],
        ],
    ];

    /**
     * @return array
     */
    public function getSettings() {
        $defaultSettings = [
            'OpenstreetmapSetting' => [
                'server_url'           => 'https://openstreetmap.openitcockpit.io/osm_tiles/{z}/{x}/{y}.png',
                'reload_interval'      => 15,
                'state_filter'         => 31,
                'hide_empty_locations' => 1
            ]
        ];
        $settings = Hash::merge($defaultSettings, $this->find('first'));
        $settings['OpenstreetmapSetting']['state_filter'] = $this->bitOperatorToFilteredStatesArray($settings['OpenstreetmapSetting']['state_filter']);
        $settings['OpenstreetmapSetting']['reload_interval'] = (int)$settings['OpenstreetmapSetting']['reload_interval'];
        return $settings;
    }

    /**
     * @param $statesArray
     * @return int
     */
    public function filteredStatesToBitOperator($statesArray) {
        $filteredStateOperator = 0;

        if($statesArray["not_monitored"] == 1){
            $filteredStateOperator += 1 << 0;
        }
        if($statesArray["up_ok"] == 1){
            $filteredStateOperator += 1 << 1;
        }
        if($statesArray["warning"] == 1){
            $filteredStateOperator += 1 << 2;
        }
        if($statesArray["down_critical"] == 1){
            $filteredStateOperator += 1 << 3;
        }
        if($statesArray["unreachable_unknown"] == 1){
            $filteredStateOperator += 1 << 4;
        }

        return $filteredStateOperator;
    }

    /**
     * @param $filteredStateOperator
     * @return array
     */
    public function bitOperatorToFilteredStatesArray($filteredStateOperator) {
        $statesArray = [
            'not_monitored' => 0,
            'up_ok' => 0,
            'warning' => 0,
            'down_critical' => 0,
            'unreachable_unknown' => 0
        ];

        if (1 << 0 & $filteredStateOperator) {
            $statesArray['not_monitored'] = 1;
        }
        if (1 << 1 & $filteredStateOperator) {
            $statesArray['up_ok'] = 1;
        }
        if (1 << 2 & $filteredStateOperator) {
            $statesArray['warning'] = 1;
        }
        if (1 << 3 & $filteredStateOperator) {
            $statesArray['down_critical'] = 1;
        }
        if (1 << 4 & $filteredStateOperator) {
            $statesArray['unreachable_unknown'] = 1;
        }

        return $statesArray;
    }
}
