angular.module("openITCOCKPIT").controller("OpenstreetmapIndexController", function($scope, $q, $http, $timeout, $interval) {
    $scope.mapRefIntervalSeconds = 15000;
    $scope.mapMinimumIntervalSeconds = 15000;
    $scope.mapURL = "";
    $scope.showFilter = false;
    $scope.parametersMissing = null;
    $scope.initParametersMissing = null;
    $scope.mapViewOptionsHasBeenChanged = false;
    $scope.unfilteredLocationsAvailable = 0;
    $scope.locations = [];
    $scope.emptyLocations = [];
    $scope.init = false;
    $scope.mapControlInit = false;
    $scope.mapOptions = {
        center: {
            lat: 0,
            lng: 0
        },
        zoom: 0,
        fullscreenControl: true
    };
    var defaultFilter = function() {
        $scope.filterStates = {
            not_monitored: 1,
            up_ok: 1,
            warning: 1,
            down_critical: 1,
            unreachable_unknown: 1
        };
        $scope.included = {
            Location: {
                id: []
            }
        };
        $scope.hideEmptyLocations = 1;
    };
    $scope.allLocations = [];

    $scope.stateFilterOptions = [
        {
            state: -1,
            description: "Not monitored"
        },
        {
            state: 0,
            description: "Up / Ok"
        },
        {
            state: 1,
            description: "Warning"
        },
        {
            state: 2,
            description: "Down / Critical"
        },
        {
            state: 3,
            description: "Unreachable / Unknown"
        }
    ];

    $scope.map = L.map("map", $scope.mapOptions);

    var intervalId = null;

    $scope.loadLocationsInMap = function(includeLocations) {
        if (includeLocations === true) {
            $scope.included.Location.id = [];
            /**
             * Filter locations if out of view
             */
            var bounds = $scope.map.getBounds();
            for (key in $scope.allLocations) {
                if (bounds.contains(new L.LatLng($scope.allLocations[key].Location.latitude, $scope.allLocations[key].Location.longitude))) {
                    $scope.included.Location.id.push($scope.allLocations[key].Location.id);
                }
            }
            if ($scope.included.Location.id.length === 0) {
                return;
            }
            for (key in $scope.emptyLocations) {
                $scope.included.Location.id.push($scope.emptyLocations[key].Location.id);
            }
        }
        let indexParams = {
            "angular": true,
            'includedLocationId[]': $scope.included.Location.id,
            'filterStates[not_monitored]': $scope.filterStates.not_monitored,
            'filterStates[up_ok]': $scope.filterStates.up_ok,
            'filterStates[warning]': $scope.filterStates.warning,
            'filterStates[down_critical]': $scope.filterStates.down_critical,
            'filterStates[unreachable_unknown]': $scope.filterStates.unreachable_unknown,
            'hideEmptyLocations': $scope.hideEmptyLocations
        };
        if ($scope.init === false) {
            indexParams = {
                "angular": true,
                'includedLocationId[]': []
            }
        }
        $q.all([$http.get("/openstreetmap_module/openstreetmap/getAclAndSettings.json", {
            params: {
                "angular": true
            }
        }), $http.get("/openstreetmap_module/openstreetmap/index.json", {
            params: indexParams
        })]).then(function(results) {
            $scope.acl = results[0].data.acl;
            $scope.mapRefIntervalSeconds = parseInt(results[0].data.settings.reload_interval, 10) * 1000; // * 1000 in milliseconds
            $scope.mapURL = results[0].data.settings.server_url;
            $scope.titleResetButton = results[0].data.settings.titleResetButton;

            if ($scope.init === false) {
                $scope.allLocations = results[1].data.locations;
                $scope.locationPoints = results[1].data.locationPoints;
                $scope.filterStates = results[1].data.filterStates;
                $scope.hideEmptyLocations = results[1].data.hideEmptyLocations;
            }
            $scope.locations = results[1].data.locations;
            $scope.emptyLocations = results[1].data.emptyLocations;
            $scope.unfilteredLocationsAvailable = parseInt(results[1].data.unfilteredLocationsAvailable);
            $scope.parametersMissing = true;
            if ($scope.init === false) $scope.initParametersMissing = true;
            if ($scope.mapURL && $scope.unfilteredLocationsAvailable > 0) {
                $scope.parametersMissing = false;
                if (results[1].data.locationPoints) loadMap(results[1].data.locationPoints);
                else loadMap([]);
                if ($scope.init === false) $scope.initParametersMissing = false;
                $scope.init = true;
            }
            if (!$scope.parametersMissing && $scope.mapRefIntervalSeconds >= $scope.mapMinimumIntervalSeconds) {
                // minimum seconds for refresh
                if (intervalId === null) {
                    intervalId = $interval(function() {
                        $scope.loadLocationsInMap(true);
                    }, $scope.mapRefIntervalSeconds);
                }
            }
        });
    };

    $scope.triggerFilter = function() {
        if ($scope.showFilter === true) {
            $scope.showFilter = false;
        } else {
            $scope.showFilter = true;
        }
    };

    $scope.problemsOnly = function() {
        defaultFilter();
        $scope.filterStates.up_ok = 0;
    };

    $scope.resetFilter = function() {
        $scope.init = false;
        $scope.loadLocationsInMap(false);
    };

    defaultFilter();
    $scope.loadLocationsInMap(false);

    $scope.$watch('filterStates | json', function() {
        if ($scope.init === true) {
            $scope.loadLocationsInMap(false);
        }
    }, true);
    $scope.$watch('hideEmptyLocations', function() {
        if ($scope.init === true) {
            $scope.loadLocationsInMap(false);
        }
    }, true);

    var hasChanges = function hasChanges (mapOptions) {
        if (mapOptions.zoom === 0) {
            return false;
        }
        if (mapOptions.zoom !== $scope.map.getZoom()) {
            return true;
        }
        if (mapOptions.center.lat !== $scope.map.getCenter()['lat'] && mapOptions.center.lng !== $scope.map.getCenter()['lng']) {
            return true;
        }
        return false;
    };

    var newCustomControl = L.Control.extend({
        options: {
            position: "topleft"
        },
        onAdd: function onAdd () {
            var container = L.DomUtil.create("div", "leaflet-bar leaflet-control leaflet-control-custom");
            container.title = $scope.titleResetButton;
            container.onclick = function() {
                $scope.mapOptions.zoom = 0;
                $scope.resetFilter();
                $scope.loadLocationsInMap(false);
            };
            return container;
        }
    });

    var loadMap = function loadMap (locationPoints) {
        if (typeof locationPoints === "undefined") {
            return;
        }
        if (!hasChanges($scope.mapOptions) && locationPoints.length > 0) {
            $scope.map.fitBounds(new L.LatLngBounds(locationPoints));
            $scope.mapOptions.zoom = $scope.map.getZoom();
            $scope.mapOptions.center = $scope.map.getCenter();
        }
        L.tileLayer($scope.mapURL, {
            maxNativeZoom: 18,
            maxZoom: 20,
            attribution: "&copy; <a target='_blank' href='http://openstreetmap.org/copyright'>OpenStreetMap</a> contributors | <a target='_blank' href='https://it-novum.com'>it-novum.com</a>",
            noWrap: false
        }).addTo($scope.map);

        var options = {
            click: function click (data) {
                if ($scope.map.isFullscreen() === true) {
                    return;
                }
                var containerIds = [];
                data.forEach(function(point) {
                    containerIds.push(point.o.id);
                });
                $scope.showOpenstreetMapSummaryState(containerIds, $scope.acl);
            }
        };

        var cells = _.chain($scope.locations).groupBy(function(location) {
            return location.Container.id;
        }).map(function(values) {
            return {
                latitude: Number(values[0].Location.latitude),
                longitude: Number(values[0].Location.longitude),
                id: values[0].Container.id,
                name: values[0].Container.name,
                colorcode: values[0].Location.colorcode
            };
        }).value();

        L.hexbinLayer(options).addTo($scope.map).data(cells);
        if ($scope.mapControlInit === false) {
            $scope.map.addControl(new newCustomControl());
            $scope.mapControlInit = true;
        }
    };
});