<?php

class PagerdutyModuleSchema extends CakeSchema {

    public function before($event = []) {
        $db = ConnectionManager::getDataSource($this->connection);
        $db->cacheSources = false;
        $CronjobModel = ClassRegistry::init('Cronjob');
        //Check if cronjob exists
        $result = $CronjobModel->find('first', [
            'conditions' => [
                'Cronjob.plugin' => 'PagerdutyModule',
                'Cronjob.task'   => 'PagerdutySendqueue',
            ],
        ]);
        if (empty($result)) {
            $data = [
                'Cronjob' => [
                    'task'     => 'PagerdutySendqueue',
                    'plugin'   => 'PagerdutyModule',
                    'interval' => '1',
                ],
            ];
            $CronjobModel->create();
            $CronjobModel->save($data);
        }
        return true;
    }

    public function after($event = []) {


        App::uses('UUID', 'Lib');
        $CommandModel = ClassRegistry::init('Command');

        //Check if pagerduty commands exists
        $result = $CommandModel->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Command.id',
                'Command.name',
            ],
            'conditions' => [
                'Command.command_type' => NOTIFICATION_COMMAND,
                'Command.name'         => 'host-notifiy-by-pagerduty'
            ]
        ]);
        if (empty($result)) {
            $CommandModel->create();
            $data = [
                'Command' => [
                    'name'         => 'host-notifiy-by-pagerduty',
                    'command_line' => '/usr/share/openitcockpit/app/Console/cake PagerdutyModule.pagerduty_notification --type Host --notificationtype $NOTIFICATIONTYPE$ --hostname "$HOSTNAME$" --hoststate "$HOSTSTATEID$" --output "$HOSTOUTPUT$"',
                    'command_type' => NOTIFICATION_COMMAND,
                    'human_args'   => null,
                    'uuid'         => UUID::v4(),
                    'description'  => 'Send host notifications to PagerDuty'
                ]
            ];
            $CommandModel->save($data);
        }

        $result = $CommandModel->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Command.id',
                'Command.name',
            ],
            'conditions' => [
                'Command.command_type' => NOTIFICATION_COMMAND,
                'Command.name'         => 'service-notify-by-pagerduty'
            ]
        ]);
        if (empty($result)) {
            $CommandModel->create();
            $data = [
                'Command' => [
                    'name'         => 'service-notify-by-pagerduty',
                    'command_line' => '/usr/share/openitcockpit/app/Console/cake PagerdutyModule.pagerduty_notification --type Service --notificationtype $NOTIFICATIONTYPE$ --hostname "$HOSTNAME$" --hoststate "$HOSTSTATEID$" --servicedesc "$SERVICEDESC$" --servicestate "$SERVICESTATEID$" --output "$SERVICEOUTPUT$"',
                    'command_type' => NOTIFICATION_COMMAND,
                    'human_args'   => null,
                    'uuid'         => UUID::v4(),
                    'description'  => 'Send service notifications to PagerDuty'
                ]
            ];
            $CommandModel->save($data);
        }

        return true;
    }

    public $pagerduty_queues = [
        'id'               => ['type' => 'integer', 'unsigned' => true, 'key' => 'primary'],
        'host_uuid'        => ['type' => 'string', 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'host_state'       => ['type' => 'integer'],
        'service_uuid'     => ['type' => 'string', 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'service_state'    => ['type' => 'integer'],
        'notificationtype' => ['type' => 'string', 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'type'             => ['type' => 'integer', 'unsigned' => true],
        'output'           => ['type' => 'string', 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'created'          => ['type' => 'datetime', 'null' => false, 'default' => null],
        'modified'         => ['type' => 'datetime', 'null' => false, 'default' => null],
        'indexes'          => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters'  => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

    public $pagerduty_settings = [
        'id'              => ['type' => 'integer', 'unsigned' => true, 'key' => 'primary'],
        'integration_key' => ['type' => 'string', 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'api_key'         => ['type' => 'string', 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'api_url'         => ['type' => 'string', 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'two_way'         => ['type' => 'boolean', 'null' => false, 'default' => '0'],
        'use_proxy'       => ['type' => 'boolean', 'null' => false, 'default' => '0'],
        'created'         => ['type' => 'datetime', 'null' => false, 'default' => null],
        'modified'        => ['type' => 'datetime', 'null' => false, 'default' => null],
        'indexes'         => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters' => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

}
