<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

class PagerdutyNotificationShell extends AppShell {

    public $uses = [
        'PagerdutyModule.PagerdutyQueue'
    ];

    public function main() {


        if (array_key_exists('type', $this->params)) {
            if ($this->params['type'] === 'Host') {
                $notification = [
                    'PagerdutyQueue' => [
                        'host_uuid'        => $this->params['hostname'],
                        'host_state'       => (int)$this->params['hoststate'],
                        'notificationtype' => $this->params['notificationtype'],
                        'type'             => OBJECT_HOST,
                        'output'           => $this->params['output'],
                    ]
                ];
            }

            if ($this->params['type'] === 'Service') {
                $notification = [
                    'PagerdutyQueue' => [
                        'host_uuid'        => $this->params['hostname'],
                        'host_state'       => (int)$this->params['hoststate'],
                        'service_uuid'     => $this->params['servicedesc'],
                        'service_state'    => $this->params['servicestate'],
                        'notificationtype' => $this->params['notificationtype'],
                        'type'             => OBJECT_SERVICE,
                        'output'           => $this->params['output'],
                    ]
                ];
            }

            $this->queueNotification($notification);

        }


    }

    public function getOptionParser() {
        $parser = parent::getOptionParser();
        $parser->addOptions([
            'type'             => ['short' => 't', 'help' => __d('oitc_console', 'Type of the notification host or service')],
            'notificationtype' => ['help' => __d('oitc_console', 'Notification type of monitoring engine')],
            'hostname'         => ['help' => __d('oitc_console', 'Host uuid you want to send a notification')],
            'hoststate'        => ['help' => __d('oitc_console', 'current host state')],
            'output'           => ['help' => __d('oitc_console', 'output')],
            'servicedesc'      => ['help' => __d('oitc_console', 'Service uuid you want to notify')],
            'servicestate'     => ['help' => __d('oitc_console', 'service state')],
        ]);

        return $parser;
    }

    /**
     * @param array $notification
     */
    public function queueNotification($notification) {
        $this->PagerdutyQueue->create();
        $this->PagerdutyQueue->save($notification);
    }


    public function _welcome() {

    }

}