<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

use GuzzleHttp\Client;
use itnovum\openITCOCKPIT\Core\Interfaces\CronjobInterface;
use itnovum\openITCOCKPIT\Core\Views\HoststatusIcon;
use itnovum\openITCOCKPIT\Core\Views\ServicestatusIcon;
use itnovum\openITCOCKPIT\Exceptions\HostNotFoundException;
use itnovum\openITCOCKPIT\Exceptions\ServiceNotFoundException;

class PagerdutySendqueueTask extends AppShell implements CronjobInterface {

    public $uses = [
        'Host',
        'Service',
        'Proxy',
        'Systemsetting',
        'PagerdutyModule.PagerdutyQueue',
        'PagerdutyModule.PagerdutySettings',
    ];

    private $proxySettings;
    private $pagerdutySettings;
    private $systemAddress;
    private $systemName;

    public function execute($quiet = false) {
        $this->params['quiet'] = $quiet;

        $this->stdout->styles('green', ['text' => 'green']);
        $this->stdout->styles('red', ['text' => 'red']);
        $this->out('Push notifications to PagerDuty', false);
        $this->out('<green>   Ok</green>');
        $this->processQueue();
        $this->hr();

    }

    private function processQueue() {
        $this->proxySettings = $this->Proxy->getSettings();
        $this->pagerdutySettings = $this->PagerdutySettings->find('first');

        $this->systemAddress = $this->Systemsetting->find('first', [
            'conditions' => ['key' => 'SYSTEM.ADDRESS'],
        ])['Systemsetting']['value'];

        $this->systemName = $this->Systemsetting->find('first', [
            'conditions' => ['key' => 'FRONTEND.SYSTEMNAME'],
        ])['Systemsetting']['value'];


        $records = $this->PagerdutyQueue->find('all');
        $ids = [];
        foreach ($records as $record) {
            //https://v2.developer.pagerduty.com/docs/events-api-v2
            if ($this->isHostNotification($record['PagerdutyQueue']['type'])) {
                try {
                    $host = $this->getHost($record['PagerdutyQueue']['host_uuid']);
                    $HoststatusIcon = new HoststatusIcon($record['PagerdutyQueue']['host_state']);
                    $event = [
                        /*
                        'payload'      => [
                            'summary'        => sprintf('[%s] %s', $host['Host']['name'], $record['PagerdutyQueue']['output']),
                            //'timestamp'      => date('c'),
                            'source'         => $record['PagerdutyQueue']['host_uuid'],
                            'severity'       => 'info',
                            'class'          => $HoststatusIcon->getHumanState(),
                            'custom_details' => [
                                'host_uuid' => $record['PagerdutyQueue']['host_uuid'],
                            ]
                        ],
                        'routing_key'  => $this->pagerdutySettings['PagerdutySettings']['integration_key'],
                        'dedup_key'    => $record['PagerdutyQueue']['host_uuid'],
                        'dedup_key'    => $record['PagerdutyQueue']['host_uuid'],
                        'alert_key'    => $record['PagerdutyQueue']['host_uuid'],
                        'event_action' => 'trigger',
                        'client'       => 'openITCOCKPIT',
                        'client_url'   => sprintf(
                            'https://%s/hosts/browser/%s',
                            $this->systemAddress,
                            $host['Host']['id']
                        )
                        */


                        'service_key'  => $this->pagerdutySettings['PagerdutySettings']['integration_key'],
                        'event_type'   => $this->getEventType($record['PagerdutyQueue']['notificationtype']),
                        'description'  => sprintf('[%s] %s', $host['Host']['name'], $record['PagerdutyQueue']['output']),
                        'incident_key' => $record['PagerdutyQueue']['host_uuid'],
                        'dedup_key'    => $record['PagerdutyQueue']['host_uuid'],
                        'details'      => [
                            'output'    => $record['PagerdutyQueue']['output'],
                            'host_name' => $host['Host']['name'],
                            'host_uuid' => $record['PagerdutyQueue']['host_uuid'],
                            'hoststate' => $HoststatusIcon->getHumanState()
                        ],

                        'client'     => $this->systemName,
                        'client_url' => sprintf(
                            'https://%s/hosts/browser/%s',
                            $this->systemAddress,
                            $host['Host']['id']
                        ),


                    ];
                } catch (HostNotFoundException $e) {
                    error_log(sprintf('Host with uuid %s not found', $record['PagerdutyQueue']['host_uuid']));
                    continue;
                }
            } else {
                try {
                    $service = $this->getService($record['PagerdutyQueue']['service_uuid']);
                    $HoststatusIcon = new HoststatusIcon($record['PagerdutyQueue']['host_state']);
                    $ServicestatusIcon = new ServicestatusIcon($record['PagerdutyQueue']['service_state']);
                    $event = [
                        'service_key'  => $this->pagerdutySettings['PagerdutySettings']['integration_key'],
                        'event_type'   => $this->getEventType($record['PagerdutyQueue']['notificationtype']),
                        'description'  => sprintf(
                            '[%s/%s] %s',
                            $service['Host']['name'],
                            $service['Service']['name'],
                            $record['PagerdutyQueue']['output']
                        ),
                        'incident_key' => $record['PagerdutyQueue']['service_uuid'],
                        'dedup_key'    => $record['PagerdutyQueue']['service_uuid'],
                        'details'      => [
                            'output'       => $record['PagerdutyQueue']['output'],
                            'host_name'    => $service['Host']['name'],
                            'service_name' => $service['Service']['name'],
                            'host_uuid'    => $record['PagerdutyQueue']['host_uuid'],
                            'service_uuid' => $record['PagerdutyQueue']['service_uuid'],
                            'hoststate'    => $HoststatusIcon->getHumanState(),
                            'servicestate' => $ServicestatusIcon->getHumanState()
                        ],
                        'client'       => $this->systemName,
                        'client_url'   => sprintf(
                            'https://%s/services/browser/%s',
                            $this->systemAddress,
                            $service['Service']['id']
                        ),
                    ];
                } catch (ServiceNotFoundException $e) {
                    error_log(sprintf('Service with uuid %s not found', $record['PagerdutyQueue']['service_uuid']));
                    continue;
                }
            }

            $ids[] = $record['PagerdutyQueue']['id'];
            $this->sendRequest($event);
        }

        if (!empty($ids)) {
            $this->PagerdutyQueue->delete($ids);
        }
    }

    private function sendRequest($event) {
        $client = new Client();

        $options = [
            'json'    => $event,
            'headers' => [
                'Accept'        => 'application/vnd.pagerduty+json;version=2',
                'Authorization' => sprintf('Token token=%s', $this->pagerdutySettings['PagerdutySettings']['api_key']),
            ]
        ];

        if ($this->pagerdutySettings['PagerdutySettings']['use_proxy']) {
            $options['proxy'] = [
                'http'  => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port']),
                'https' => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port'])
            ];
        } else {
            $options['proxy'] = [
                'http'  => false,
                'https' => false
            ];
        }

        $response = $client->post($this->pagerdutySettings['PagerdutySettings']['api_url'], $options);

        $body = $response->getBody();
        //print_r($body->getContents());
    }

    private function isHostNotification($type) {
        return $type == OBJECT_HOST;
    }

    /**
     * @param string $notificationtype
     * @return string
     */
    private function getEventType($notificationtype) {
        switch ($notificationtype) {
            case 'PROBLEM':
                return 'trigger';

            case 'ACKNOWLEDGEMENT':
                return 'acknowledge';

            case 'RECOVERY':
                return 'resolve';

            default:
                return 'trigger';
        }
    }

    /**
     * @param string $hostUuid
     * @return array
     * @throws HostNotFoundException
     */
    private function getHost($hostUuid) {
        $host = $this->Host->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Host.id',
                'Host.name',
                'Host.address'
            ],
            'conditions' => [
                'Host.uuid' => $hostUuid
            ]
        ]);

        if (empty($host)) {
            throw new HostNotFoundException();
        }
        return $host;
    }

    /**
     * @param string $serviceUuid
     * @return array
     * @throws ServiceNotFoundException
     */
    private function getService($serviceUuid) {
        $service = $this->Service->find('first', [
            'recurisve'  => -1,
            'conditions' => [
                'Service.uuid' => $serviceUuid,
            ],
            'fields'     => [
                'Service.id',
                'Service.uuid',
                'Service.name',
            ],
            'contain'    => [
                'Servicetemplate' => [
                    'fields' => [
                        'Servicetemplate.id',
                        'Servicetemplate.name',
                    ],
                ],
                'Host'            => [
                    'fields' => [
                        'Host.id',
                        'Host.uuid',
                        'Host.name',
                    ],
                ],
            ],
        ]);

        if (empty($service)) {
            throw new ServiceNotFoundException();
        }

        //$this->dump($service);
        if ($service['Service']['name'] === null || $service['Service']['name'] === '') {
            $service['Service']['name'] = $service['Servicetemplate']['name'];
        }

        return $service;
    }

}
