<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.


class AcknowledgeController extends PagerdutyModuleAppController {

    public $uses = [
        'Host',
        'Service',
        'Proxy',
        'PagerdutyModule.PagerdutySettings',
    ];

    public $components = [
        'GearmanClient'
    ];

    public function submit() {
        $data = $this->getPostData();

        foreach ($data['messages'] as $message) {
            if ($message['event'] !== 'incident.acknowledge') {
                $this->set('message', __('Wrong incident type'));
                $this->set('_serialize', ['message']);
                return;
            }
            $incident = $message['incident'];
            $incidentKey = $incident['incident_key'];

            foreach ($incident['acknowledgements'] as $acknowledger) {
                if ($acknowledger['acknowledger']['type'] == 'api') {
                    $authorName = 'API';
                    $authorUrl = null;
                } else {
                    $authorName = $acknowledger['acknowledger']['summary'];
                    $authorUrl = $acknowledger['acknowledger']['html_url'];
                    break;
                }
            }

            $ackUrl = $authorName;
            if ($authorUrl) {
                $ackUrl = sprintf('[url=%s]%s[/url]', $authorUrl, $authorName);
            }

            $host = $this->getHost($incidentKey);
            if ($host) {
                $command = 'ACKNOWLEDGE_HOST_PROBLEM';

                $parameters = [
                    $host['Host']['uuid'],
                    1, //sticky
                    0, //notify - do not enable - loop!
                    1, //persistent
                    $authorName,
                    sprintf(
                        'Issue %s was acknowledged by %s due to PagerDuty.',
                        sprintf('[url=%s]%s[/url]', $incident['html_url'], $incident['id']),
                        $ackUrl
                    )
                ];
                $this->GearmanClient->sendBackground('cmd_external_command', [
                    'command'     => $command,
                    'parameters'  => $parameters,
                    'satelliteId' => $host['Host']['satellite_id']
                ]);
                $this->set('message', __('Command added successfully to queue'));
                $this->set('_serialize', ['message']);
                return;
            }

            $service = $this->getService($incidentKey);
            if ($service) {
                $command = 'ACKNOWLEDGE_SVC_PROBLEM';
                $parameters = [
                    $service['Host']['uuid'],
                    $service['Service']['uuid'],
                    1, //sticky
                    0, //notify - do not enable - loop!
                    1, //persistent
                    $authorName,
                    sprintf(
                        'Issue %s was acknowledged by %s due to PagerDuty.',
                        sprintf('[url=%s]%s[/url]', $incident['html_url'], $incident['id']),
                        sprintf('[url=%s]%s[/url]', $authorUrl, $authorName)
                    )
                ];
                $this->GearmanClient->sendBackground('cmd_external_command', [
                    'command'     => $command,
                    'parameters'  => $parameters,
                    'satelliteId' => $service['Host']['satellite_id']
                ]);
                $this->set('message', __('Command added successfully to queue'));
                $this->set('_serialize', ['message']);
                return;
            }

        }
        $this->set('message', __('Unknown'));
        $this->set('_serialize', ['message']);
    }

    /**
     * @param string $hostUuid
     * @return bool|array
     */
    private function getHost($hostUuid) {
        $host = $this->Host->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Host.id',
                'Host.uuid',
                'Host.name',
                'Host.address',
                'Host.satellite_id'
            ],
            'conditions' => [
                'Host.uuid' => $hostUuid
            ]
        ]);

        if (empty($host)) {
            return false;
        }

        return $host;
    }

    /**
     * @param string $serviceUuid
     * @return bool|array
     */
    private function getService($serviceUuid) {
        $service = $this->Service->find('first', [
            'recurisve'  => -1,
            'conditions' => [
                'Service.uuid' => $serviceUuid,
            ],
            'fields'     => [
                'Service.id',
                'Service.uuid',
                'Service.name',
            ],
            'contain'    => [
                'Host' => [
                    'fields' => [
                        'Host.id',
                        'Host.uuid',
                        'Host.name',
                        'Host.satellite_id'
                    ],
                ],
            ],
        ]);

        if (empty($service)) {
            return false;
        }

        return $service;
    }

    /**
     * @return mixed
     */
    private function getPostData() {
        $data = file_get_contents('php://input');
        if (strlen($data)) {
            return json_decode(file_get_contents('php://input'), true);
        }
        throw new BadRequestException('Empty post data');
    }

}

