<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, version 3 of the License.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//	If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//	under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//	License agreement and license key will be shipped with the order
//	confirmation.


/**
 * Class SAPInstallShell
 * uses CakePHP to insert SAP Template data into the Database
 */
class SAPInstallShell extends AppShell {


    public $uses = [
        'Servicetemplate',
        'Command',
        'Commandargument',
    ];

    public $mapping = [];

    //root path for json and check files
    public $path = '';

    public function main() {
        $this->stdout->styles('success', ['text' => 'green']);
        $this->out('Installing SAP Module');
        $this->path = APP . 'Plugin' . DS . 'SAPModule' . DS . 'webroot' . DS;
        $jsonDataPath = 'jsonData' . DS;

        //save the data into the DB
        //@TODO sequential saving is ugly may we find a better way to do this
        //commands
        $commandData = $this->readJsonFile($this->path . $jsonDataPath . 'sap_commands.json');
        $this->saveData($commandData);

        //servicetemplates
        $stData = $this->readJsonFile($this->path . $jsonDataPath . 'sap_servicetemplates.json');
        $servicetemplateData = $this->modifyServictemplatedata($stData);
        $this->saveData($servicetemplateData);
        //currently not needed cuz it will be done by Package
        /*$this->out('<info>moving check scripts</info>');
		$this->moveCheckScripts($perlChecks);
		*/
    }

    /**
     * saves the SAP Data into the Database
     *
     * @param $dataToSave - the array data to save
     *
     * @return bool
     */
    private function saveData($dataToSave) {
        try {
            $this->out('Saving data to DB');
            $model = key($dataToSave[0]);
            foreach ($dataToSave as $data) {
                //calling model->create() so we can use save in a loop
                $this->{$model}->create();
                //check if the record already exists
                $existingData = $this->{$model}->find('first', [
                    'recursive'  => -1,
                    'conditions' => [
                        $model . '.name' => $data[$model]['name']
                    ]
                ]);
                if (!empty($existingData)) {
                    //the data already exists. Skip the loop for this record
                    $this->out('<info>' . $data[$model]['name'] . ' already exists. Skipping</info>');
                    if ($model == 'Command') {
                        //add mapping for commands with the already existing record from the database
                        $this->mapping['Command'][$data['Command']['uuid']] = $existingData['Command']['id'];
                        $this->mapCommandArgs($existingData['Command']['id']);
                    }
                    continue;
                }
                //@TODO Transactions would be much better
                //$datasource = $this->{$model}->getDatasource();
                //$datasource->begin(); etc..
                if ($model == 'Command') {
                    $data = Hash::remove($data, 'Commandargument.{n}.id');
                }

                if ($this->{$model}->saveAll($data)) {
                    if ($model == 'Command') {
                        $this->out('<info>Add Command ' . $data['Command']['name'] . ' to Mapping</info>');
                        $this->mapping['Command'][$data['Command']['uuid']] = $this->{$model}->id;
                        $this->mapCommandArgs($this->{$model}->id);
                    }
                    $this->out('<success>Data for ' . $model . ' ' . $data[$model]['name'] . ' sucessfully saved!</success>');
                } else {
                    print_r($data);
                    print_r($this->{$model}->validationErrors);
                    throw new Exception('the data for ' . $model . ' ' . $data[$model]['name'] . ' could not be saved');
                }
            }
        } catch (Exception $e) {
            echo $e->getMessage();
            echo "\n";
        }

        return false;
    }

    /**
     * reads a json file and return the json representation as php array
     *
     * @param $file - the json file to read
     *
     * @return array - the php array representation of the json
     */
    private function readJsonFile($file) {
        try {
            $this->out('Reading json file ' . $file);
            if (!file_exists($file)) {
                throw new Exception($file . ' could not be found');
            }

            $fileData = file_get_contents($file);

            $data = json_decode($fileData, true);
            $jsonErr = json_last_error();

            if ($jsonErr != JSON_ERROR_NONE) {
                throw  new Exception($jsonErr);
            }

            return $data;
        } catch (Exception $e) {
            echo $e->getMessage();
            exit();
        }
    }

    /**
     * @param $servicetemplateData
     *
     * @return array
     */
    private function modifyServictemplatedata($servicetemplateData) {
        $this->out('<info>Replacing IDs </info>');
        if (!empty($this->mapping['Commandarguments'])) {
            foreach ($servicetemplateData as $key => $servicetemplate) {
                $commandId = $this->mapping['Command'][$servicetemplate['Servicetemplate']['command_id']];
                //replacing the command uuid with the command id
                $servicetemplateData[$key]['Servicetemplate']['command_id'] = $commandId;
                //insert a new UUID for the Servicetemplates
                $servicetemplateData[$key]['Servicetemplate']['uuid'] = UUID::v4();

                $i = 0;
                //go through the servicetemplatecommandarguments
                if (!empty($servicetemplate['Servicetemplatecommandargumentvalue'])) {
                    foreach ($servicetemplate['Servicetemplatecommandargumentvalue'] as $stcavKey => $stcav) {
                        //replace the commandargument ids from the servicetemplatecommandarguments
                        // with the commandargument ids (the id they have after save)
                        if (array_key_exists($commandId, $this->mapping['Commandarguments']) && !empty($this->mapping['Commandarguments'][$commandId])) {
                            if(isset($this->mapping['Commandarguments'][$commandId][$i])){
                                $servicetemplateData[$key]['Servicetemplatecommandargumentvalue'][$stcavKey]['commandargument_id'] = $this->mapping['Commandarguments'][$commandId][$i];
                            }
                            $i++;
                        }
                    }
                }
            }
        } else {
            $this->out('<error>There is no mapping for Commandarguments so the servicetemplates cant be Saved!</error>');
        }

        return $servicetemplateData;
    }

    /**
     * maps the command ids with the commandargument ids
     *
     * @param $commandId
     *
     * @return void
     */
    private function mapCommandArgs($commandId) {
        $commands = $this->Command->find('all', [
            'conditions' => [
                'id' => $commandId,
            ],
        ]);
        $commandargIds = Hash::filter(Hash::extract($commands, '{n}.Commandargument.{n}.id'));
        $this->mapping['Commandarguments'][$commandId] = $commandargIds;
    }


    /**
     * Moves the check files from the SAPModule webroot folder to the libexec folder
     *
     * @param $files - the files to move
     */
    private function moveCheckScripts($files) {
        $libexecPath = DS . 'opt' . DS . 'openitc' . DS . 'nagios' . DS . 'libexec' . DS;
        if (is_array($files) && !empty($files)) {
            foreach ($files as $file) {
                if (file_exists($libexecPath . $file)) {
                    $selection = $this->in('The File ' . $file . ' already exists in ' . $libexecPath . ' Do you want to override it ?', ['y', 'n'], 'y');
                    if ($selection == 'y') {
                        //user want to override the existing file
                        $state = copy($this->path . $file, $libexecPath . $file);
                        if ($state) {
                            chmod($libexecPath . $file, 0755);
                            $this->out('<success>' . $file . ' overridden </success>');
                        } else {
                            $this->out('<error>' . $file . ' cant be overridden</error>');
                        }
                    } else {
                        $this->out('keeping the existing file ' . $file);
                    }
                } else {
                    $state = copy($this->path . $file, $libexecPath . $file);
                    if ($state) {
                        chmod($libexecPath . $file, 0755);
                        $this->out('<success>' . $file . ' copied </success>');
                    } else {
                        $this->out('<error>' . $file . ' cant be copied </error>');
                    }
                }
            }
        }
    }
}