<?php
// Copyright (C) <2019>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use itnovum\openITCOCKPIT\Exceptions\HostNotFoundException;
use itnovum\openITCOCKPIT\Exceptions\ServiceNotFoundException;

class ServicenowNotificationShell extends AppShell {

    public $uses = [
        'Host',
        'Service',
        'Systemsetting',
        'ServicenowModule.ServicenowSettings',
        'ServicenowModule.ServicenowHostspecificSettings',
        'ServicenowModule.ServicenowServicespecificSettings',
        'ServicenowModule.ServicenowIncident',
        'Proxy'
    ];

    private $servicenowSettings;
    private $servicenowSpecificSettings;
    private $incident;
    private $create_new_incident = false;
    private $send_notification = true;

    public function main() {

        $this->proxySettings = $this->Proxy->getSettings();
        $this->servicenowSettings = $this->ServicenowSettings->find('first');

        if (array_key_exists('type', $this->params)) {
            if ($this->params['type'] === 'Host') {
                $notification = [
                    'host_uuid'    => $this->params['hostname'],
                    'host_state'   => (int)$this->params['hoststate'],
                    'host_address' => $this->params['hostaddress'],
                    'type'         => $this->params['type'],
                    'output'       => $this->params['output'],
                    'comment'       => $this->params['comment'],
                ];
            }

            if ($this->params['type'] === 'Service') {
                $notification = [
                    'host_uuid'     => $this->params['hostname'],
                    'host_state'    => (int)$this->params['hoststate'],
                    'host_address'  => $this->params['hostaddress'],
                    'service_uuid'  => $this->params['servicedesc'],
                    'service_state' => $this->params['servicestate'],
                    'type'          => $this->params['type'],
                    'output'        => $this->params['output'],
                    'comment'       => $this->params['comment'],
                ];
            }
        }

        $systemAddress = $this->Systemsetting->find('first', [
            'conditions' => ['key' => 'SYSTEM.ADDRESS'],
        ])['Systemsetting']['value'];

        if ($notification['type'] == "Host") {

            $this->incident = $this->ServicenowIncident->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'ServicenowIncident.host_uuid' => $notification['host_uuid']
                ]
            ]);

            $hostSpecificSettings = $this->ServicenowHostspecificSettings->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'ServicenowHostspecificSettings.host_uuid' => $notification['host_uuid']
                ]
            ]);

            if (isset($hostSpecificSettings['ServicenowHostspecificSettings'])) {
                $this->servicenowSpecificSettings = Hash::merge($this->servicenowSettings['ServicenowSettings'], array_filter($hostSpecificSettings['ServicenowHostspecificSettings']));
            } else {
                $this->servicenowSpecificSettings = $this->servicenowSettings['ServicenowSettings'];
            }

            $host = $this->getHost($notification['host_uuid']);

            if ($notification['host_state'] == "0")
                $state = "UP";
            else if ($notification['host_state'] == "1")
                $state = "DOWN";
            else
                $state = "UNREACHABLE";

            /*if (isset($host['Host']['tags'])) {
                $tags = str_replace(',', ' ', $host['Host']['tags']);
            }*/

            $hosturl = sprintf('https://%s/hosts/browser/%s', $systemAddress, $host['Host']['id']);
            $serviceurl = '';
            $comment = 'URL: [code]<a href="' . $hosturl . '">' . $hosturl . '</a>[/code]';

            $short_description = sprintf("Host %s is %s", $host['Host']['name'], $state);

            $description = sprintf("Time: %s\n", date('H:i:s - d.m.Y T'));
            $description .= sprintf("Host Name: %s\n", $host['Host']['name']);
            $description .= sprintf("Hostaddress: %s\n", $notification['host_address']);
            $description .= sprintf("State: %s\n", $state);
            $description .= sprintf("Output: %s\n", $notification['output']);
        }

        if ($notification['type'] == "Service") {

            $this->incident = $this->ServicenowIncident->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'ServicenowIncident.service_uuid' => $notification['service_uuid']
                ]
            ]);

            $serviceSpecificSettings = $this->ServicenowServicespecificSettings->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'ServicenowServicespecificSettings.service_uuid' => $notification['service_uuid']
                ]
            ]);

            if (isset($serviceSpecificSettings['ServicenowServicespecificSettings'])) {
                $this->servicenowSpecificSettings = Hash::merge($this->servicenowSettings['ServicenowSettings'], array_filter($serviceSpecificSettings['ServicenowServicespecificSettings']));
            } else {
                $this->servicenowSpecificSettings = $this->servicenowSettings['ServicenowSettings'];
            }
            print_r($this->servicenowSpecificSettings);

            $host = $this->getHost($notification['host_uuid']);
            $service = $this->getService($notification['service_uuid']);

            /*if (isset($service['Service']['tags'])) {
                $tags = str_replace(',', ' ', $service['Service']['tags']);
            }*/

            $hosturl = sprintf('https://%s/hosts/browser/%s', $systemAddress, $host['Host']['id']);
            $serviceurl = sprintf('https://%s/services/browser/%s', $systemAddress, $service['Service']['id']);
            $comment = 'Host URL: [code]<a href="' . $hosturl . '">' . $hosturl . '</a><br>[/code]';
            $comment .= 'Service URL: [code]<a href="' . $serviceurl . '">' . $serviceurl . '</a>[/code]';

            if ($notification['service_state'] == "0")
                $state = "OK";
            else if ($notification['service_state'] == "1")
                $state = "WARNING";
            else if ($notification['service_state'] == "2")
                $state = "CRITICAL";
            else
                $state = "UNKNOWN";

            $short_description = sprintf("Service %s on Host %s is %s", $service['Service']['name'], $host['Host']['name'], $state);

            $description = sprintf("Time: %s\n", date('H:i:s - d.m.Y T'));
            $description .= sprintf("Service Name: %s\n", $service['Service']['name']);
            $description .= sprintf("Hostaddress: %s\n", $notification['host_address']);
            $description .= sprintf("State: %s\n", $state);
            $description .= sprintf("Output: %s\n", $notification['output']);
        }


        if ($state == "UP" || $state == "OK") {
            $incident_active_state = 'false';
            $incident_state = $this->servicenowSettings['ServicenowSettings']['recovery_state'];
        }
        if ($state == "CRITICAL" || $state == "DOWN") {
            $incident_active_state = 'true';
            $incident_state = '1';
        }
        if ($state == "WARNING") {
            $incident_active_state = 'true';
            $incident_state = '1';
        }
        if ($state == "UNKNOWN" || $state == "UNREACHABLE") {
            $incident_active_state = 'true';
            $incident_state = '1';
        }

        if (empty($this->incident['ServicenowIncident'])) {  //create new incident
            $this->create_new_incident = true;
            $data = [
                'short_description' => $short_description,
                'description'       => $description,
                'comments'          => $comment,
                'active'            => $incident_active_state,
                'category'          => 'openITCOCKPIT',
                'urgency'           => $this->servicenowSpecificSettings['urgency'], //see: https://docs.servicenow.com/bundle/london-it-service-management/page/product/incident-management/reference/r_PrioritizationOfIncidents.html?cshalt=yes
                'impact'            => $this->servicenowSpecificSettings['impact'],
                //'knowledge' => 'true',
                'u_host_url'        => $hosturl,
                'u_service_url'     => $serviceurl,
                //'sys_tags'          => $tags,
                'caller_id'         => $this->servicenowSettings['ServicenowSettings']['user_id']
            ];
        } else {    //update existing incident
            $data = [
                'short_description' => $short_description,
                'description'       => $description,
                'active'            => $incident_active_state,
                'state'             => $incident_state, //1 = "New"; 2 = "In Progress"; 3 = "On Hold"; 4 = ""; 5 = ""; 6 = "Resolved"; 7 = "Closed"; 8 = "Canceled";
                'category'          => 'openITCOCKPIT',
                'urgency'           => $this->servicenowSpecificSettings['urgency'], //see: https://docs.servicenow.com/bundle/london-it-service-management/page/product/incident-management/reference/r_PrioritizationOfIncidents.html?cshalt=yes
                'impact'            => $this->servicenowSpecificSettings['impact'],
                'caller_id'         => $this->servicenowSettings['ServicenowSettings']['user_id']
            ];

        }

        if(isset($notification['comment']) && !empty($notification['comment']) && !$this->create_new_incident){
            $data['comments'] = $notification['comment'];
        }

        if ($state == "UP" || $state == "OK") {
            $data['close_code'] = 'Closed/Resolved by Caller';
            $data['close_notes'] = $notification['output'];

            if ($this->create_new_incident) {
                $this->send_notification = false;
            }
        }

        if (!empty($this->servicenowSpecificSettings['assignment_group'])) {
            $data['assignment_group'] = $this->servicenowSpecificSettings['assignment_group'];
        }
        if (!empty($this->servicenowSpecificSettings['assigned_to'])) {
            $data['assigned_to'] = $this->servicenowSpecificSettings['assigned_to'];
        }
        if (!empty($this->servicenowSpecificSettings['business_service'])) {
            $data['business_service'] = $this->servicenowSpecificSettings['business_service'];
        }

        $guzzleclient = new Client();

        $options = [
            'json'    => $data,
            'auth'    => [$this->servicenowSettings['ServicenowSettings']['user_id'], $this->servicenowSettings['ServicenowSettings']['user_password']],
            'headers' => [
                'Accept'       => 'application/json',
                'Content-Type' => 'application/json'
            ]
        ];

        if ($this->servicenowSettings['ServicenowSettings']['use_proxy']) {
            $options['proxy'] = [
                'http'  => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port']),
                'https' => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port'])
            ];
        } else {
            $options['proxy'] = [
                'http'  => false,
                'https' => false
            ];
        }

        $serviceNowUrl = sprintf('%s/api/now/table/incident', $this->servicenowSettings['ServicenowSettings']['url']);
        if ($this->send_notification) {
            try {
                if ($this->create_new_incident) {
                    $response = $guzzleclient->post($serviceNowUrl, $options);
                } else {
                    if ($incident_active_state == 'true' || ($incident_active_state == 'false' && $this->incident['ServicenowIncident']['state'] != 7)) {
                        try {
                            $response = $guzzleclient->patch($serviceNowUrl . '/' . $this->incident['ServicenowIncident']['sys_id'], $options);
                        } catch (RequestException $e) {
                            echo $e->getMessage();
                        }
                    }
                }

                if (isset($response)) {
                    $result = json_decode($response->getBody()->getContents(), true)['result'];
                    $incident_id = $result['number'];
                    $sys_id = $result['sys_id'];
                }

                if ($incident_active_state == 'true' && isset($response) && $incident_id != null) {   //insert or update incident entry
                    if ($this->create_new_incident) { //insert
                        $this->incident['ServicenowIncident'] = [
                            'incident_id'  => $incident_id,
                            'sys_id'       => $sys_id,
                            'host_uuid'    => $notification['host_uuid'],
                            'service_uuid' => isset($notification['service_uuid']) ? $notification['service_uuid'] : '',
                        ];
                        if (!$this->ServicenowIncident->save($this->incident)) {
                            throw new ErrorException();
                        }
                    } else {    //update
                        $this->incident['ServicenowIncident']['state'] = intval($incident_state);
                        $this->incident['ServicenowIncident']['incident_id'] = $incident_id;
                        $this->incident['ServicenowIncident']['sys_id'] = $sys_id;
                        $this->incident['ServicenowIncident']['modified'] = date("Y-m-d H:i:s");
                        if (!$this->ServicenowIncident->save($this->incident)) {
                            throw new ErrorException();
                        }
                    }
                } else if ($incident_active_state != 'true') {    //delete incident entry
                    $this->ServicenowIncident->delete($this->incident['ServicenowIncident']['id']);
                }

            } catch (RequestException $e) {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param string $serviceUuid
     * @return bool|array
     */
    private function getHost($hostUuid) {
        $host = $this->Host->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Host.id',
                'Host.name',
                'Host.address'
            ],
            'conditions' => [
                'Host.uuid' => $hostUuid
            ]
        ]);

        if (empty($host)) {
            throw new HostNotFoundException();
        }
        return $host;
    }

    /**
     * @param string $serviceUuid
     * @return bool|array
     */
    private function getService($serviceUuid) {
        $service = $this->Service->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'Service.uuid' => $serviceUuid,
            ],
            'fields'     => [
                'Service.id',
                'Service.uuid',
                'Service.name',
            ],
            'contain'    => [
                'Servicetemplate' => [
                    'fields' => [
                        'Servicetemplate.id',
                        'Servicetemplate.name',
                    ],
                ],
                'Host'            => [
                    'fields' => [
                        'Host.id',
                        'Host.uuid',
                        'Host.name',
                    ],
                ],
            ],
        ]);

        if (empty($service)) {
            throw new ServiceNotFoundException();
        }

        if ($service['Service']['name'] === null || $service['Service']['name'] === '') {
            $service['Service']['name'] = $service['Servicetemplate']['name'];
        }

        return $service;
    }

    public function getOptionParser() {
        $parser = parent::getOptionParser();
        $parser->addOptions([
            'type'         => ['short' => 't', 'help' => __d('oitc_console', 'Type of the notification host or service')],
            'hostname'     => ['help' => __d('oitc_console', 'Host uuid you want to send a notification')],
            'hoststate'    => ['help' => __d('oitc_console', 'current host state')],
            'hostaddress'  => ['help' => __d('oitc_console', 'host address')],
            'output'       => ['help' => __d('oitc_console', 'output')],
            'servicedesc'  => ['help' => __d('oitc_console', 'Service uuid you want to notify')],
            'servicestate' => ['help' => __d('oitc_console', 'service state')],
            'comment'      => ['help' => __d('oitc_console', 'comment')],
        ]);

        return $parser;
    }

    public function _welcome() {

    }

}