<?php
// Copyright (C) <2019>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

use GuzzleHttp\Client;

class ServicenowAcknowledgeController extends ServicenowModuleAppController {

    public $uses = [
        'Host',
        MONITORING_HOSTSTATUS,
        'Service',
        MONITORING_SERVICESTATUS,
        'ServicenowModule.ServicenowSettings',
        'ServicenowModule.ServicenowIncident',
        'Proxy'
    ];

    public $components = [
        'GearmanClient'
    ];

    private $servicenowStateList = [
        1 => 'New',
        2 => 'In Progress',
        3 => 'On Hold',
        6 => 'Resolved',
        7 => 'Closed',
        8 => 'Canceled'
    ];

    public function submit() {

        $this->proxySettings = $this->Proxy->getSettings();
        $this->servicenowSettings = $this->ServicenowSettings->find('first');

        if (!$this->request->is('post') || $this->servicenowSettings['ServicenowSettings']['two_way'] != 1) {
            throw new MethodNotAllowedException();
        }

        $this->autoRender = false;

        $incident_id = $this->request->data['incident_id'];
        $state_number = intval($this->request->data['state']);
        $state = $this->servicenowStateList[$state_number];
        $resolution_notes = $this->request->data['close_notes'];
        $resolution_code = $this->request->data['close_code'];
        $sys_resolved_by = $this->request->data['sys_resolved_by'];
        $updated_by = $this->request->data['updated_by'];

        $this->incident = $this->ServicenowIncident->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'ServicenowIncident.incident_id' => $incident_id
            ]
        ]);

        if (empty($this->incident['ServicenowIncident'])) {
            throw new NotFoundException();
        }

        if ($sys_resolved_by != null && !empty($sys_resolved_by)) {
            $guzzleclient = new Client();
            $options = [
                'auth'    => [$this->servicenowSettings['ServicenowSettings']['user_id'], $this->servicenowSettings['ServicenowSettings']['user_password']],
                'headers' => [
                    'Accept'       => 'application/json',
                    'Content-Type' => 'application/json'
                ]
            ];
            if ($this->servicenowSettings['ServicenowSettings']['use_proxy']) {
                $options['proxy'] = [
                    'http'  => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port']),
                    'https' => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port'])
                ];
            } else {
                $options['proxy'] = [
                    'http'  => false,
                    'https' => false
                ];
            }
            $serviceNowUrl = sprintf('%s/api/now/table/sys_user', $this->servicenowSettings['ServicenowSettings']['url']);
            $response = $guzzleclient->get($serviceNowUrl . '/' . $sys_resolved_by, $options);
            $result = json_decode($response->getBody()->getContents(), true)['result'];
            $resolved_by_name = $result['name'];
        }

        $type = '';
        if (!empty($this->incident['ServicenowIncident']['host_uuid'])) {
            $type = 'Host';
            if (!empty($this->incident['ServicenowIncident']['service_uuid'])) {
                $type = 'Service';
            }
        }


        $ack_message = sprintf(
            'ServiceNow Incident set to [b]%s[/b] by [b]%s[/b] due to ServiceNow.',
            $state,
            isset($resolved_by_name) ? $resolved_by_name : $updated_by
        );

        if ($state_number == 6 || $state_number == 7) {   // 6 = resolved; 7 = closed;
            $this->incident['ServicenowIncident']['resolution_code'] = $resolution_code;
            $this->incident['ServicenowIncident']['resolution_notes'] = $resolution_notes;
        }

        $incidentURL = sprintf('%s/nav_to.do?uri=/incident.do?sys_id=%s', $this->servicenowSettings['ServicenowSettings']['url'], $this->incident['ServicenowIncident']['sys_id']);
        $ack_message .= sprintf('\nServiceNow Ticket: [url=%s]%s[/url]', $incidentURL, $incident_id);

        if ($type == "Host") {
            $host = $this->getHost($this->incident['ServicenowIncident']['host_uuid']);
            $command = 'ACKNOWLEDGE_HOST_PROBLEM';

            $parameters = [
                $host['Host']['uuid'],
                1, //sticky
                0, //notify - do not enable - loop!
                1, //persistent
                $updated_by,
                $ack_message
            ];
            $this->GearmanClient->sendBackground('cmd_external_command', [
                'command'     => $command,
                'parameters'  => $parameters,
                'satelliteId' => $host['Host']['satellite_id']
            ]);
        }

        if ($type == "Service") {
            $service = $this->getService($this->incident['ServicenowIncident']['service_uuid']);
            $command = 'ACKNOWLEDGE_SVC_PROBLEM';

            $parameters = [
                $this->incident['ServicenowIncident']['host_uuid'],
                $service['Service']['uuid'],
                1, //sticky
                0, //notify - do not enable - loop!
                1, //persistent
                $updated_by,
                $ack_message
            ];
            $this->GearmanClient->sendBackground('cmd_external_command', [
                'command'     => $command,
                'parameters'  => $parameters,
                'satelliteId' => $service['Host']['satellite_id']
            ]);
        }

        $this->incident['ServicenowIncident']['state'] = $state_number;
        if (!$this->ServicenowIncident->save($this->incident)) {
            throw new ErrorException();
        }
        return;
    }

    /**
     * @param string $hostUuid
     * @return bool|array
     */
    private function getHost($hostUuid) {
        $host = $this->Host->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Host.id',
                'Host.uuid',
                'Host.name',
                'Host.address',
                'Host.satellite_id'
            ],
            'conditions' => [
                'Host.uuid' => $hostUuid
            ]
        ]);

        if (empty($host)) {
            return false;
        }

        return $host;
    }

    /**
     * @param string $serviceUuid
     * @return bool|array
     */
    private function getService($serviceUuid) {
        $service = $this->Service->find('first', [
            'recursive'  => -1,
            'conditions' => [
                'Service.uuid' => $serviceUuid,
            ],
            'fields'     => [
                'Service.id',
                'Service.uuid',
                'Service.name',
            ],
            'contain'    => [
                'Host' => [
                    'fields' => [
                        'Host.id',
                        'Host.uuid',
                        'Host.name',
                        'Host.satellite_id'
                    ],
                ],
            ],
        ]);

        if (empty($service)) {
            return false;
        }

        return $service;
    }

}

