<?php
// Copyright (C) <2019>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

class ServicenowEditController extends ServicenowModuleAppController {

    public $layout = 'angularjs';

    public $uses = [
        'ServicenowModule.ServicenowSettings',
        'ServicenowModule.ServicenowHostspecificSettings',
        'ServicenowModule.ServicenowServicespecificSettings',
        'ServicenowModule.ServicenowIncident'
    ];

    private $servicenowStateList = [
        1 => 'New',
        2 => 'In Progress',
        3 => 'On Hold',
        6 => 'Resolved',
        7 => 'Closed',
        8 => 'Canceled'
    ];

    public function index() {
        if (!$this->isAngularJsRequest()) {
            //Only ship html template
            return;
        }

        $defaultId = 1;
        $defaults = [
            'ServicenowSettings' => [
                'url'              => null,
                'user_id'          => null,
                'user_password'    => null,
                'assignment_group' => null,
                'assigned_to'      => null,
                'business_service' => null,
                'impact'           => "1",
                'urgency'          => "1",
                'recovery_state'   => "6",
                'two_way'          => false,
                'use_proxy'        => false
            ]
        ];

        if ($this->request->is('post') || $this->request->is('put')) {
            $this->request->data['ServicenowSettings']['id'] = $defaultId;
            $this->request->data['ServicenowSettings']['modified'] = date("Y-m-d H:i:s");
            $this->request->data['ServicenowSettings']['url'] = $this->getValidURL($this->request->data['ServicenowSettings']['url']);
            if (!$this->ServicenowSettings->save($this->request->data)) {
                throw new ErrorException();
            }
        }

        $settings = Hash::merge($defaults, $this->ServicenowSettings->find('first'));
        $this->set('settings', $settings);
        $this->set('_serialize', ['settings']);
    }

    public function hostspecific($uuid = null) {
        if (!$this->isAngularJsRequest()) {
            //Only ship html template
            return;
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            $this->request->data['ServicenowHostspecificSettings']['modified'] = date("Y-m-d H:i:s");
            if (!$this->ServicenowHostspecificSettings->save($this->request->data)) {
                throw new ErrorException();
            }

            $this->set('id', $this->ServicenowHostspecificSettings->getInsertId());
            $this->set('_serialize', ['id']);
            return;
        }

        if ($uuid !== null) {
            $defaults = [
                'ServicenowHostspecificSettings' => [
                    'host_uuid'        => null,
                    'assignment_group' => null,
                    'assigned_to'      => null,
                    'business_service' => null,
                    'impact'           => null,
                    'urgency'          => null,
                ]
            ];

            $settings = Hash::merge($defaults, $this->ServicenowHostspecificSettings->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'ServicenowHostspecificSettings.host_uuid' => $uuid
                ]
            ]));
            $incident = $this->ServicenowIncident->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'ServicenowIncident.host_uuid'    => $uuid,
                    'ServicenowIncident.service_uuid' => ''
                ]
            ]);

            $globalsettings = $this->ServicenowSettings->find('first');
            if (isset($globalsettings['ServicenowSettings']) && $globalsettings['ServicenowSettings']['impact'] != null && $settings['ServicenowHostspecificSettings']['impact'] == null) {
                $settings['ServicenowHostspecificSettings']['impact'] = $globalsettings['ServicenowSettings']['impact'];
            }
            if (isset($globalsettings['ServicenowSettings']) && $globalsettings['ServicenowSettings']['urgency'] != null && $settings['ServicenowHostspecificSettings']['urgency'] == null) {
                $settings['ServicenowHostspecificSettings']['urgency'] = $globalsettings['ServicenowSettings']['urgency'];
            }
            if (isset($incident['ServicenowIncident']) && $incident['ServicenowIncident']['state'] != null) {
                $incident['ServicenowIncident']['state'] = $this->servicenowStateList[$incident['ServicenowIncident']['state']];
            }

            $this->set('globalsettings', $globalsettings);
            $this->set('settings', $settings);
            $this->set('incident', $incident);
            $this->set('_serialize', ['settings', 'incident', 'globalsettings']);
        }
    }

    public function servicespecific($uuid = null) {
        if (!$this->isAngularJsRequest()) {
            //Only ship html template
            return;
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            $this->request->data['ServicenowServicespecificSettings']['modified'] = date("Y-m-d H:i:s");
            if (!$this->ServicenowServicespecificSettings->save($this->request->data)) {
                throw new ErrorException();
            }

            $this->set('id', $this->ServicenowServicespecificSettings->getInsertId());
            $this->set('_serialize', ['id']);
            return;
        }

        if ($uuid !== null) {
            $defaults = [
                'ServicenowServicespecificSettings' => [
                    'service_uuid'     => null,
                    'assignment_group' => null,
                    'assigned_to'      => null,
                    'business_service' => null,
                    'impact'           => null,
                    'urgency'          => null,
                ]
            ];

            $settings = Hash::merge($defaults, $this->ServicenowServicespecificSettings->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'ServicenowServicespecificSettings.service_uuid' => $uuid
                ]
            ]));
            $incident = $this->ServicenowIncident->find('first', [
                'recursive'  => -1,
                'conditions' => [
                    'ServicenowIncident.service_uuid' => $uuid
                ]
            ]);

            $globalsettings = $this->ServicenowSettings->find('first');
            if (isset($globalsettings['ServicenowSettings']) && $globalsettings['ServicenowSettings']['impact'] != null && $settings['ServicenowServicespecificSettings']['impact'] == null) {
                $settings['ServicenowServicespecificSettings']['impact'] = $globalsettings['ServicenowSettings']['impact'];
            }
            if (isset($globalsettings['ServicenowSettings']) && $globalsettings['ServicenowSettings']['urgency'] != null && $settings['ServicenowServicespecificSettings']['urgency'] == null) {
                $settings['ServicenowServicespecificSettings']['urgency'] = $globalsettings['ServicenowSettings']['urgency'];
            }

            if (isset($incident['ServicenowIncident']) && $incident['ServicenowIncident']['state'] != null) {
                $incident['ServicenowIncident']['state'] = $this->servicenowStateList[$incident['ServicenowIncident']['state']];
            }

            $this->set('globalsettings', $globalsettings);
            $this->set('settings', $settings);
            $this->set('incident', $incident);
            $this->set('_serialize', ['settings', 'incident', 'globalsettings']);
        }
    }

    private function getValidURL($url) {
        if (!$this->startsWith($url, 'http') && strpos($url, '.service-now.com') !== false) {
            $url = 'https://' . $url;
        }
        return $url;
    }

    private function startsWith($haystack, $needle) {
        return substr_compare($haystack, $needle, 0, strlen($needle)) === 0;
    }
}
