<?php

class SlackModuleSchema extends CakeSchema {

    public function before($event = []) {
        $db = ConnectionManager::getDataSource($this->connection);
        $db->cacheSources = false;
        return true;
    }

    public function after($event = []) {


        App::uses('UUID', 'Lib');
        $CommandModel = ClassRegistry::init('Command');

        //Check if slack commands exists
        $result = $CommandModel->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Command.id',
                'Command.name',
            ],
            'conditions' => [
                'Command.command_type' => NOTIFICATION_COMMAND,
                'Command.name'         => 'host-notifiy-by-slack'
            ]
        ]);
        if (empty($result)) {
            $CommandModel->create();
            $data = [
                'Command' => [
                    'name'         => 'host-notifiy-by-slack',
                    'command_line' => '/usr/share/openitcockpit/app/Console/cake SlackModule.slack_notification --type Host --notificationtype $NOTIFICATIONTYPE$ --hostname "$HOSTNAME$" --hostaddress "$HOSTADDRESS$" --hoststate "$HOSTSTATEID$" --output "$HOSTOUTPUT$"',
                    'command_type' => NOTIFICATION_COMMAND,
                    'human_args'   => null,
                    'uuid'         => UUID::v4(),
                    'description'  => 'Send host notifications to Slack'
                ]
            ];
            $CommandModel->save($data);
        }

        $result = $CommandModel->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Command.id',
                'Command.name',
            ],
            'conditions' => [
                'Command.command_type' => NOTIFICATION_COMMAND,
                'Command.name'         => 'service-notify-by-slack'
            ]
        ]);
        if (empty($result)) {
            $CommandModel->create();
            $data = [
                'Command' => [
                    'name'         => 'service-notify-by-slack',
                    'command_line' => '/usr/share/openitcockpit/app/Console/cake SlackModule.slack_notification --type Service --notificationtype $NOTIFICATIONTYPE$ --hostname "$HOSTNAME$" --hostaddress "$HOSTADDRESS$" --hoststate "$HOSTSTATEID$" --servicedesc "$SERVICEDESC$" --servicestate "$SERVICESTATEID$" --output "$SERVICEOUTPUT$"',
                    'command_type' => NOTIFICATION_COMMAND,
                    'human_args'   => null,
                    'uuid'         => UUID::v4(),
                    'description'  => 'Send service notifications to Slack'
                ]
            ];
            $CommandModel->save($data);
        }

        return true;
    }

    public $slack_settings = [
        'id'                 => ['type' => 'integer', 'unsigned' => true, 'key' => 'primary'],
        'api_url'            => ['type' => 'string', 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'oauth_access_token' => ['type' => 'string', 'collate' => 'utf8_swedish_ci', 'charset' => 'utf8'],
        'two_way'            => ['type' => 'boolean', 'null' => false, 'default' => '0'],
        'use_proxy'          => ['type' => 'boolean', 'null' => false, 'default' => '0'],
        'created'            => ['type' => 'datetime', 'null' => false, 'default' => null],
        'modified'           => ['type' => 'datetime', 'null' => false, 'default' => null],
        'indexes'            => [
            'PRIMARY' => ['column' => 'id', 'unique' => 1],
        ],
        'tableParameters'    => ['charset' => 'utf8', 'collate' => 'utf8_swedish_ci', 'engine' => 'InnoDB'],
    ];

}
