<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

use GuzzleHttp\Client;

class SlackNotificationShell extends AppShell {

    public $uses = [
        'Host',
        'Service',
        'Proxy',
        'Systemsetting',
        'SlackModule.SlackSettings'
    ];

    private $slackSettings;

    public function main() {

        $this->proxySettings = $this->Proxy->getSettings();
        $this->slackSettings = $this->SlackSettings->find('first');

        if (array_key_exists('type', $this->params)) {
            if ($this->params['type'] === 'Host') {
                $notification = [
                    'host_uuid'        => $this->params['hostname'],
                    'host_state'       => (int)$this->params['hoststate'],
                    'host_address'     => $this->params['hostaddress'],
                    'notificationtype' => $this->params['notificationtype'],
                    'type'             => $this->params['type'],
                    'output'           => $this->params['output'],
                ];
            }

            if ($this->params['type'] === 'Service') {
                $notification = [
                    'host_uuid'        => $this->params['hostname'],
                    'host_state'       => (int)$this->params['hoststate'],
                    'host_address'     => $this->params['hostaddress'],
                    'service_uuid'     => $this->params['servicedesc'],
                    'service_state'    => $this->params['servicestate'],
                    'notificationtype' => $this->params['notificationtype'],
                    'type'             => $this->params['type'],
                    'output'           => $this->params['output'],
                ];
            }
        }

        $acknowledgeValues = 'acknowledged||' . $notification['type'] . '||' . $notification['host_uuid'];

        $systemAddress = $this->Systemsetting->find('first', [
            'conditions' => ['key' => 'SYSTEM.ADDRESS'],
        ])['Systemsetting']['value'];

        $output = sprintf("Output: %s\n", $notification['output']);

        if ($notification['type'] == "Host") {
            $host = $this->getHost($notification['host_uuid']);

            if ($notification['host_state'] == "0")
                $state = "UP";
            else if ($notification['host_state'] == "1")
                $state = "DOWN";
            else if ($notification['host_state'] == "2")
                $state = "UNREACHABLE";
            else
                $state = "Unknown";

            $hosturl = sprintf('https://%s/hosts/browser/%s', $systemAddress, $host['Host']['id']);
            $notificationText = sprintf("\n\nHost <%s|*%s*> is *%s*\n\n", $hosturl, $host['Host']['name'], $state);

            if ($state == "UP" || $this->slackSettings['SlackSettings']['two_way'] == "0") {
                //$notificationText = sprintf("\nHost *%s* is *%s*\n\n",$host['Host']['name'],$state);
                $notificationText .= $output;
            }

            //$notificationText .= sprintf("Time: %s\n",date('H:i:s - d.m.Y T'));
            //$notificationText .= sprintf("Host Name: *%s*\n",$host['Host']['name']);
            //$notificationText .= sprintf("Hostaddress: %s\n",$notification['host_address']);
            //$notificationText .= sprintf("State: *%s*\n",$state);
        }

        if ($notification['type'] == "Service") {
            $acknowledgeValues .= '||' . $notification['service_uuid'];
            $host = $this->getHost($notification['host_uuid']);
            $service = $this->getService($notification['service_uuid']);

            $hosturl = sprintf('https://%s/hosts/browser/%s', $systemAddress, $host['Host']['id']);
            $serviceurl = sprintf('https://%s/services/browser/%s', $systemAddress, $service['Service']['id']);

            if ($notification['service_state'] == "0")
                $state = "OK";
            else if ($notification['service_state'] == "1")
                $state = "WARNING";
            else if ($notification['service_state'] == "2")
                $state = "CRITICAL";
            else
                $state = "UNKNOWN";

            $notificationText = sprintf("\n\nService <%s|*%s*> on Host <%s|*%s*> is *%s*\n\n", $serviceurl, $service['Service']['name'], $hosturl, $host['Host']['name'], $state);

            if ($state == "OK" || $this->slackSettings['SlackSettings']['two_way'] == "0") {
                //$notificationText = sprintf("\nService *%s* on Host *%s* is *%s*\n\n",$service['Service']['name'],$host['Host']['name'],$state);
                $notificationText .= $output;
            }

            //$notificationText .= sprintf("Time: %s\n",date('H:i:s - d.m.Y T'));
            //$notificationText .= sprintf("Service Name: *%s*\n",$service['Service']['name']);
            //$notificationText .= sprintf("Hostaddress: %s\n",$notification['host_address']);
            //$notificationText .= sprintf("State: *%s*\n",$state);
        }


        if ($state == "UP" || $state == "OK") {
            $notificationcolor = "#5cb85c";
        }
        if ($state == "CRITICAL" || $state == "DOWN") {
            $notificationcolor = "#d9534f";
        }
        if ($state == "WARNING") {
            $notificationcolor = "#f0ad4e";
        }
        if ($state == "UNKNOWN" || $state == "UNREACHABLE") {
            $notificationcolor = "#adadad";
        }

        if ($state != "UP" && $state != "OK" && $this->slackSettings['SlackSettings']['two_way'] == "1") {
            $data_string = [
                'text'        => $notificationText,
                'attachments' => [
                    [
                        'text'            => $output,
                        'fallback'        => 'No output',
                        'color'           => $notificationcolor,
                        'callback_id'     => 'host_output',
                        'attachment_type' => 'default',
                    ],
                    [
                        'text'            => "Click to be the first acknowledger for this",
                        'fallback'        => 'You are unable to acknowledge this',
                        'color'           => $notificationcolor,
                        'callback_id'     => 'ack_host',
                        'attachment_type' => 'default',
                        'actions'         => [
                            [
                                'name'  => 'ack_button',
                                'text'  => 'Acknowledge',
                                'style' => 'danger',
                                'type'  => 'button',
                                'value' => $acknowledgeValues,
                            ]
                        ]
                    ]
                ]
            ];
        } else {
            $data_string = [
                'text'        => "",
                'attachments' => [
                    [
                        'text'            => $notificationText,
                        "mrkdwn_in"       => ["text"],
                        'color'           => $notificationcolor,
                        'callback_id'     => 'info',
                        'attachment_type' => 'default'
                    ]
                ]
            ];
        }


        $guzzleclient = new Client();

        $options = [
            'json'    => $data_string,
            'headers' => [
                'Accept' => 'application/json'
            ]
        ];

        if ($this->slackSettings['SlackSettings']['use_proxy']) {
            $options['proxy'] = [
                'http'  => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port']),
                'https' => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port'])
            ];
        } else {
            $options['proxy'] = [
                'http'  => false,
                'https' => false
            ];
        }

        $guzzleclient->post($this->slackSettings['SlackSettings']['api_url'], $options);

    }

    public function getOptionParser() {
        $parser = parent::getOptionParser();
        $parser->addOptions([
            'type'             => ['short' => 't', 'help' => __d('oitc_console', 'Type of the notification host or service')],
            'notificationtype' => ['help' => __d('oitc_console', 'Notification type of monitoring engine')],
            'hostname'         => ['help' => __d('oitc_console', 'Host uuid you want to send a notification')],
            'hoststate'        => ['help' => __d('oitc_console', 'current host state')],
            'hostaddress'      => ['help' => __d('oitc_console', 'host address')],
            'output'           => ['help' => __d('oitc_console', 'output')],
            'servicedesc'      => ['help' => __d('oitc_console', 'Service uuid you want to notify')],
            'servicestate'     => ['help' => __d('oitc_console', 'service state')],
        ]);

        return $parser;
    }

    /**
     * @param string $serviceUuid
     * @return bool|array
     */
    private function getHost($hostUuid) {
        $host = $this->Host->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Host.id',
                'Host.name',
                'Host.address'
            ],
            'conditions' => [
                'Host.uuid' => $hostUuid
            ]
        ]);

        if (empty($host)) {
            throw new HostNotFoundException();
        }
        return $host;
    }

    /**
     * @param string $serviceUuid
     * @return bool|array
     */
    private function getService($serviceUuid) {
        $service = $this->Service->find('first', [
            'recurisve'  => -1,
            'conditions' => [
                'Service.uuid' => $serviceUuid,
            ],
            'fields'     => [
                'Service.id',
                'Service.uuid',
                'Service.name',
            ],
            'contain'    => [
                'Servicetemplate' => [
                    'fields' => [
                        'Servicetemplate.id',
                        'Servicetemplate.name',
                    ],
                ],
                'Host'            => [
                    'fields' => [
                        'Host.id',
                        'Host.uuid',
                        'Host.name',
                    ],
                ],
            ],
        ]);

        if (empty($service)) {
            throw new ServiceNotFoundException();
        }

        //$this->dump($service);
        if ($service['Service']['name'] === null || $service['Service']['name'] === '') {
            $service['Service']['name'] = $service['Servicetemplate']['name'];
        }

        return $service;
    }

    public function _welcome() {

    }

}