<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

use GuzzleHttp\Client;
use itnovum\openITCOCKPIT\Core\HoststatusFields;
use itnovum\openITCOCKPIT\Core\ServicestatusFields;

class AcknowledgeController extends SlackModuleAppController {

    public $uses = [
        'Host',
        MONITORING_HOSTSTATUS,
        'Service',
        MONITORING_SERVICESTATUS,
        'Proxy',
        'SlackModule.SlackSettings',
    ];

    public $components = [
        'GearmanClient'
    ];

    public function submit() {

        $this->proxySettings = $this->Proxy->getSettings();
        $this->slackSettings = $this->SlackSettings->find('first');

        $this->autoRender = false;

        $getArray = json_decode($_POST['payload'], true);
        $actionsData = $getArray['actions'][0];
        $userId = $getArray['user']['id'];
        $OAuthAccessToken = $this->slackSettings['SlackSettings']['oauth_access_token'];
        $userInfoUrl = sprintf('https://slack.com/api/users.info?token=%s&user=%s&pretty=1', $OAuthAccessToken, $userId);


        $guzzleclient = new Client();

        $options['headers'] = [
            'Accept' => 'application/json'
        ];
        if ($this->slackSettings['SlackSettings']['use_proxy']) {
            $options['proxy'] = [
                'http'  => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port']),
                'https' => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port'])
            ];
        } else {
            $options['proxy'] = [
                'http'  => false,
                'https' => false
            ];
        }

        $response = $guzzleclient->post($userInfoUrl, $options);
        $userJson = $response->getBody();


        $userRealName = json_decode($userJson)->user->real_name;
        $date = new DateTime();
        $values = explode('||', $actionsData['value']);

        /*
            $values[0]="acknowledged"
            $values[1]=notification type
            $values[2]=host-uuid
            $values[3]=service-uuid if notification type == Service
        */

        if (strpos($getArray['original_message'][attachments][1][text], '*Already acknowledged* by') !== false) {
            $acknowledgeText = $getArray['original_message'][attachments][1][text] . ", " . $userRealName;
            $acknowledgeColor = $getArray['original_message'][attachments][1][color];
        } else {
            $acknowledgeText = '*Already acknowledged* by ' . $userRealName;
            $acknowledgeColor = $getArray['original_message'][attachments][1][color];
        }
        $output = $getArray['original_message'][attachments][0][text];

        $acknowledgeTime = $date->getTimestamp();
        $acknowledgeButtonType = 'button';
        $HoststatusFields = new HoststatusFields($this->DbBackend);
        $HoststatusFields->currentState();
        if ($values[1] == "Host" && $this->Hoststatus->byUuid($values[2], $HoststatusFields)['Hoststatus']['current_state'] == "0") {
            $acknowledgeTime = $getArray['original_message'][attachments][1][ts];
            $acknowledgeText = $getArray['original_message'][attachments][1][text] . "\n\n*" . $values[1] . " is back!*";
            $acknowledgeButtonType = 'disabled';
            $acknowledgeColor = '#5cb85c';
        }
        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields->currentState();
        if ($values[1] == "Service" && $this->Servicestatus->byUuid($values[3], $ServicestatusFields)['Servicestatus']['current_state'] == "0") {
            $acknowledgeTime = $getArray['original_message'][attachments][1][ts];
            $acknowledgeText = $getArray['original_message'][attachments][1][text] . "\n\n*" . $values[1] . " is back!*";
            $acknowledgeButtonType = 'disabled';
            $acknowledgeColor = '#5cb85c';
        }

        $acknowledgeValues = $values[0] . '||' . $values[1] . '||' . $values[2];
        if ($values[1] == "Service") {
            $acknowledgeValues .= '||' . $values[3];
        }

        $newJsonArray = [
            'response_type'    => 'ephemeral',
            'replace_original' => true,
            'text'             => $getArray['original_message'][text],
            'attachments'      => [
                [
                    'text'            => $output,
                    'fallback'        => 'No output',
                    'color'           => $acknowledgeColor,
                    'callback_id'     => 'host_output',
                    'attachment_type' => 'default',
                ],
                [
                    'text'            => $acknowledgeText,
                    "mrkdwn_in"       => ["text"],
                    'callback_id'     => 'ack_host',
                    'color'           => $acknowledgeColor,
                    'attachment_type' => 'default',
                    'actions'         => [
                        [
                            'name'  => 'ack_button',
                            'text'  => 'Acknowledge too',
                            'style' => 'primary',
                            'type'  => $acknowledgeButtonType,
                            'value' => $acknowledgeValues,
                        ]
                    ],
                    'footer'          => 'Last acknowledged',
                    'footer_icon'     => 'https://pbs.twimg.com/profile_images/692366534623256576/158ywUkg.jpg',
                    "ts"              => $acknowledgeTime,
                ]
            ],
        ];

        $this->response->type('json');
        $this->response->body(json_encode($newJsonArray));

        if ($values[1] == "Host") {
            $host = $this->getHost($values[2]);
            $command = 'ACKNOWLEDGE_HOST_PROBLEM';

            $parameters = [
                $host['Host']['uuid'],
                1, //sticky
                0, //notify - do not enable - loop!
                1, //persistent
                $userRealName,
                sprintf(
                    'Issue was acknowledged by %s due to Slack.',
                    $userRealName
                )
            ];
            $this->GearmanClient->sendBackground('cmd_external_command', [
                'command'     => $command,
                'parameters'  => $parameters,
                'satelliteId' => $host['Host']['satellite_id']
            ]);
            return;
        }

        if ($values[1] == "Service") {
            $service = $this->getService($values[3]);
            $command = 'ACKNOWLEDGE_SVC_PROBLEM';

            $parameters = [
                $values[2], //host uuid
                $service['Service']['uuid'], //service uuid
                1, //sticky
                0, //notify - do not enable - loop!
                1, //persistent
                $userRealName,
                sprintf(
                    'Issue was acknowledged by %s due to Slack.',
                    $userRealName
                )
            ];
            $this->GearmanClient->sendBackground('cmd_external_command', [
                'command'     => $command,
                'parameters'  => $parameters,
                'satelliteId' => $service['Host']['satellite_id']
            ]);
            return;
        }
    }

    /**
     * @param string $hostUuid
     * @return bool|array
     */
    private function getHost($hostUuid) {
        $host = $this->Host->find('first', [
            'recursive'  => -1,
            'fields'     => [
                'Host.id',
                'Host.uuid',
                'Host.name',
                'Host.address',
                'Host.satellite_id'
            ],
            'conditions' => [
                'Host.uuid' => $hostUuid
            ]
        ]);

        if (empty($host)) {
            return false;
        }

        return $host;
    }

    /**
     * @param string $serviceUuid
     * @return bool|array
     */
    private function getService($serviceUuid) {
        $service = $this->Service->find('first', [
            'recurisve'  => -1,
            'conditions' => [
                'Service.uuid' => $serviceUuid,
            ],
            'fields'     => [
                'Service.id',
                'Service.uuid',
                'Service.name',
            ],
            'contain'    => [
                'Host' => [
                    'fields' => [
                        'Host.id',
                        'Host.uuid',
                        'Host.name',
                        'Host.satellite_id'
                    ],
                ],
            ],
        ]);

        if (empty($service)) {
            return false;
        }

        return $service;
    }

}

